// AccountsAndPositionsDlg.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include "Common.h"
#include "AccountsAndPositions.h"
#include "AccountsAndPositionsDlg.h"

const CString CAccountsAndPositionsDlg::m_N_A = _T("N/A");
const int CAccountsAndPositionsDlg::m_stSummaryColWidth = 130;

// CAccountsAndPositionsDlg dialog

CAccountsAndPositionsDlg::CAccountsAndPositionsDlg(CWnd* pParent /*=NULL*/)
   : CDialog(CAccountsAndPositionsDlg::IDD, pParent)
{
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

//////////////////////////////////////////////////////////////////////////
//////////////////// Protected methods
//////////////////////////////////////////////////////////////////////////
void CAccountsAndPositionsDlg::DoDataExchange(
                                    CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_ST_DATA_CONNECTION, m_lblDataConnection);
   DDX_Control(pDX, IDC_ST_GW_CONNECTION, m_lblGWConnection);
   DDX_Control(pDX, IDC_CMB_ACCOUNTS, m_cmbAccounts);
   DDX_Control(pDX, IDC_CMB_REPORT_DAYS, m_cmbReportDays);
   DDX_Control(pDX, IDC_CMB_CURRENCIES, m_cmbCurrencies);
   DDX_Control(pDX, IDC_LSV_ACCOUNT_SUMMARY, m_lvAccSummary);
   DDX_Control(pDX, IDC_LSV_ACCOUNT_PROPS, m_lvAccProperties);
   DDX_Control(pDX, IDC_LSV_POSITIONS, m_lvPositions);
}

BEGIN_MESSAGE_MAP(CAccountsAndPositionsDlg, CDialog)
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_CBN_SELCHANGE(IDC_CMB_ACCOUNTS, OnCbnSelChangeCmbAccounts)
   ON_CBN_SELCHANGE(IDC_CMB_REPORT_DAYS, OnCbnSelchangeCmbReportDays)
   ON_CBN_SELCHANGE(IDC_CMB_CURRENCIES, OnCbnSelchangeCmbCurrencies)
   ON_WM_DESTROY()
   ON_WM_CONTEXTMENU()
   ON_COMMAND(ID_POPUP_NOMARGIN, OnPopupNomargin)
   ON_COMMAND(ID_POPUP_MARGINSNAPSHOT, OnPopupMarginsnapshot)
   ON_COMMAND(ID_POPUP_MARGINSNAPSHOTANDUPDATES, OnPopupMarginsnapshotandupdates)
END_MESSAGE_MAP()


BOOL CAccountsAndPositionsDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   // Set the icon for this dialog.  The framework does this automatically
   //  when the application's main window is not a dialog
   SetIcon(m_hIcon, TRUE);       // Set big icon
   SetIcon(m_hIcon, FALSE);      // Set small icon

   try
   {
      InitializeControls();
      InitializeCQGCEL();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
}

void CAccountsAndPositionsDlg::OnDestroy()
{
   try
   {
      if (m_spCQGCEL)
      {
         HRESULT hr = ICQGCELDispEventImpl::DispEventUnadvise(m_spCQGCEL);
         assert(SUCCEEDED(hr));

         hr = m_spCQGCEL->Shutdown();
         AssertCOMError(m_spCQGCEL, hr);
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}

void CAccountsAndPositionsDlg::OnOK()
{
   // Override to prevent dialog closing on Enter
}

void CAccountsAndPositionsDlg::OnPaint() 
{
   if (IsIconic())
   {
      CPaintDC dc(this); // device context for painting

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      // Center icon in client rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      // Draw the icon
      dc.DrawIcon(x, y, m_hIcon);
   }
   else
   {
      CDialog::OnPaint();
   }
}

HCURSOR CAccountsAndPositionsDlg::OnQueryDragIcon()
{
   return static_cast<HCURSOR>(m_hIcon);
}

void CAccountsAndPositionsDlg::OnCbnSelChangeCmbAccounts()
{
   try
   {
      ShowSelectedAccountData();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CAccountsAndPositionsDlg::OnCbnSelchangeCmbReportDays()
{
   try
   {
      ShowSummaries(GetSelectedAccount());
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CAccountsAndPositionsDlg::OnCbnSelchangeCmbCurrencies()
{
   try
   {
      ShowSummaries(GetSelectedAccount());
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CAccountsAndPositionsDlg::OnContextMenu(CWnd* /*pWnd*/, CPoint point)
{
   CMenu mnuPopupSubmit;
   mnuPopupSubmit.LoadMenu(IDR_MARGIN_DET_CONTEXT);

   CRect rectSubmitButton;
   m_lvAccProperties.GetWindowRect(&rectSubmitButton);

   CMenu *mnuPopupMenu = mnuPopupSubmit.GetSubMenu(0);
   ASSERT(mnuPopupMenu);

   POSITION pos = m_lvAccProperties.GetFirstSelectedItemPosition();
   if( rectSubmitButton.PtInRect(point) && pos && m_lvAccProperties.GetNextSelectedItem(pos) == aprMarginDetailingLevel)
   {
      mnuPopupMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this);
   }
}

void CAccountsAndPositionsDlg::SetCurrentAccountLevel(eAccountMarginDetailing detailingLevel)
{
   ATL::CComPtr<ICQGAccount> spAccount = GetSelectedAccount();
   if (spAccount == NULL)
   {
      return;
   }
   HRESULT hr = spAccount->put_AccountMarginDetailing(detailingLevel);
   AssertCOMError(spAccount, hr);
   m_lvAccProperties.SetItemText(aprMarginDetailingLevel, apcValue, AccountMarginDetailingToString(detailingLevel));
}

void CAccountsAndPositionsDlg::OnPopupNomargin()
{
   SetCurrentAccountLevel(amdNoMargin);
}

void CAccountsAndPositionsDlg::OnPopupMarginsnapshot()
{
   SetCurrentAccountLevel(amdMarginSnapshot);
}

void CAccountsAndPositionsDlg::OnPopupMarginsnapshotandupdates()
{
   SetCurrentAccountLevel(amdMarginSnapshotAndUpdate);
}
