using System.Diagnostics;
using System;
using System.Windows.Forms;
using CQG;

namespace AutoTrader
{
   public class AutoTrader
   {
      private CQGCEL m_CEL;
      private ListView m_lvAutoTrades;
      private ListView m_lvOrders;
      private string m_Symbol;
      private CQGInstrument m_Instrument;
      private int m_OpenPositions;
      private string m_Name;
      private CQGAccount m_Account;
      private CQGTradingSystem m_TradingSystem;
      private bool m_IsNameSelected;
      private bool m_IsAccountSelected;
      private bool m_IsTSResolved;
      private bool m_IsSymbolResolved;
      private bool m_IsActivated;
      private bool m_IsSignalModeSelected;
      private int m_Row;
      private TradesRowData m_tradeRowData;

      private enum eAutoTradeColumns
      {
         atcTurnedOnAt = 1,
         atcOpenPositions,
         atcWorkingMode,
         atcName,
         atcTradingSystem,
         atcSymbol,
         atcAccountID,
         atcAccountName,
         atcFullName,
         atcPreferences,
         atcParameters
      }

      public AutoTrader(CQGCEL CEL, ListView lvAT, ListView lvO)
      {
         m_CEL = CEL;
         m_lvAutoTrades = lvAT;
         m_lvOrders = lvO;

         m_IsNameSelected = false;
         m_IsAccountSelected = false;
         m_IsTSResolved = false;
         m_IsSymbolResolved = false;
         m_IsActivated = false;
         m_IsSignalModeSelected = false;

         m_OpenPositions = int.MinValue;
         m_tradeRowData = new TradesRowData();

         ListViewItem item = m_lvAutoTrades.Items.Add(new ListViewItem());
         for (int i = System.Convert.ToInt32(eAutoTradeColumns.atcTurnedOnAt); i <= System.Convert.ToInt32(eAutoTradeColumns.atcParameters); i++)
         {
            item.SubItems.Add("");
         }
      }

      public string GetPreferences()
      {
         string sPref = "";

         if (m_TradingSystem != null)
         {
            if (m_TradingSystem.Request.HistoricalPeriod != eHistoricalPeriod.hpUndefined)
            {
               sPref = HistoricalPeriodToString(m_TradingSystem.Request.HistoricalPeriod) + ", ";
            }
            else
            {
               sPref = m_TradingSystem.Request.IntradayPeriod + " Min, ";
            }

            sPref += "CNT:" + ContinuationTypeToString(m_TradingSystem.Request.Continuation) + ", ";
            sPref += "DBE:" + m_TradingSystem.Request.DaysBeforeExpiration + ", ";
            sPref += "ECL:" + m_TradingSystem.Request.EqualizeCloses + ", ";
            sPref += "SFL:" + m_TradingSystem.Request.SessionsFilter.ToString();
         }
         return sPref;
      }

      public string GetParameters()
      {
         string sParams = "";
         string sParamName;

         if (m_TradingSystem != null)
         {
            for (int i = 0; i <= m_TradingSystem.Request.Definition.ParameterDefinitions.Count - 1; i++)
            {
               sParamName = m_TradingSystem.Request.Definition.ParameterDefinitions[i].Name;
               sParams += sParamName + ":" + m_TradingSystem.Request.get_Parameter(sParamName).ToString() + ", ";
            }
         }
         return sParams;
      }

      public string Name
      {
         set
         {
            m_Name = value;
            m_IsNameSelected = value.Trim().Length > 0;
            m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcName)].Text = value;
         }
      }

      public string TradingSystemDefName
      {
         set
         {
            m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcTradingSystem)].Text = value;
         }
      }

      public CQGAccount Account
      {
         set
         {
            m_Account = value;
            m_IsAccountSelected = value != null;
            if (m_IsAccountSelected)
            {
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcAccountID)].Text = value.GWAccountID.ToString();
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcAccountName)].Text = value.GWAccountName;
            }
            else
            {
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcAccountID)].Text = "";
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcAccountName)].Text = "";
            }
         }
      }

      public CQGTradingSystem TradingSystem
      {
         set
         {
            m_TradingSystem = value;
            m_IsTSResolved = value != null;
            if (m_IsTSResolved)
            {
               m_tradeRowData.SetData(m_TradingSystem.TradesRows[m_TradingSystem.TradesRows.Count - 1]);
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcPreferences)].Text = GetPreferences();
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcParameters)].Text = GetParameters();
            }
            else
            {
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcPreferences)].Text = "";
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcParameters)].Text = "";
            }
         }
      }

      public string Symbol
      {
         get
         {
            return m_Symbol;
         }
         set
         {
            m_Symbol = value;
            m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcSymbol)].Text = value;
            m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcFullName)].Text = "";

            if (value.Trim().Length > 0)
            {
               m_CEL.NewInstrument(value);
            }
         }
      }

      public CQGInstrument Instrument
      {
         set
         {
            m_Instrument = value;
            m_IsSymbolResolved = (value != null);
            if (m_IsSymbolResolved)
            {
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcFullName)].Text = m_Instrument.FullName;
            }
            else
            {
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcFullName)].Text = modCommon.N_A;
            }
         }
      }

      public int Row
      {
         set
         {
            m_Row = value;
         }
      }

      private bool GetReadyStatus()
      {
         bool readyStatus = m_IsNameSelected && m_IsAccountSelected && m_IsTSResolved && m_IsSymbolResolved;
         return (readyStatus && (m_IsSignalModeSelected || m_OpenPositions != int.MinValue));
      }

      public bool IsSymbolResolved
      {
         get
         {
            return m_IsSymbolResolved;
         }
      }

      public bool IsSignalModeSelected
      {
         get
         {
            return m_IsSignalModeSelected;
         }

         set
         {
            m_IsSignalModeSelected = value;
            m_lvAutoTrades.Items[m_Row].SubItems[(int)eAutoTradeColumns.atcWorkingMode].Text
                                   = m_IsSignalModeSelected ? "By Signals" : "By Statistics";
         }
      }

      public TradesRowData TradeRowData
      {
         get
         {
            return m_tradeRowData;
         }
         set
         {
            m_tradeRowData = value;
         }
      }

      public void Activate(ref CheckState newVal)
      {
         if (newVal == CheckState.Checked)
         {
            if (GetReadyStatus())
            {
               m_IsActivated = true;
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcTurnedOnAt)].Text = DateTime.Now.ToString();
            }
            else
            {
               newVal = CheckState.Unchecked;
            }
         }
         else
         {
            m_IsActivated = false;
            m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcTurnedOnAt)].Text = "";
         }

      }

      public void PositionsChanged(CQGTradingSystem tradingSystem)
      {
         if (tradingSystem != null)
         {
            int newVal = System.Convert.ToInt32(tradingSystem.Statistics[tradingSystem.Statistics.Count - 1].OpenPosition);
            bool isValid = m_CEL.IsValid(newVal);
            if (isValid)
            {
               m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcOpenPositions)].Text = newVal.ToString();
            }
            else
            {
               if (m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcOpenPositions)].Text.Trim().Length == 0)
               {
                  m_lvAutoTrades.Items[m_Row].SubItems[System.Convert.ToInt32(eAutoTradeColumns.atcOpenPositions)].Text = modCommon.N_A;
               }

               if (!m_IsSignalModeSelected)
               {
                   return;
               }
            }
            int delta = 0;
            if (m_OpenPositions != int.MinValue)
            {
                delta = newVal - m_OpenPositions;
            }
            m_OpenPositions = newVal;

            if (m_IsActivated)
            {
                if (!m_IsSignalModeSelected && isValid)
                {
                    PlaceOrderByStatistcs(delta);
                }
                else if (m_IsSignalModeSelected)
                {
                    PlaceOrderBySignals(tradingSystem);
                }
            }
         }
      }

      private void PlaceOrderByStatistcs(int quantity)
      {
         if (quantity == 0)
         {
            return;
         }

         eOrderSide orderSide = quantity > 0 ? eOrderSide.osdBuy : eOrderSide.osdSell;
         CQGOrder order_ = m_CEL.CreateOrder(eOrderType.otMarket, m_Instrument, m_Account, Math.Abs(quantity), orderSide, 0, 0, "AUTOTRADE");
         order_.Place();

         ExecutedOrders(order_);
      }

      private void PlaceOrderBySignals(CQGTradingSystem tradingSystem)
      {
         CQGTradingSystemTradesRow tradeRow = tradingSystem.TradesRows[tradingSystem.TradesRows.Count - 1];

         CQGTradeEntry tradeEntry;
         foreach (CQGTradingSystemTrade trade in tradeRow)
         {
            if (trade.TradeEntry.Signal)
            {
               tradeEntry = trade.TradeEntry;
               PlaceOrderBySignal(tradeEntry.Definition.OrderType, tradeEntry.Quantity, tradeEntry.Price,
                   tradeEntry.StopLimitPrice, modCommon.TradeSideToOrderSide(trade.Definition.Side));
            }
            foreach (CQGTradeExit tradeExit in trade.TradeExits)
            {
               if (tradeExit.Signal)
               {
                  PlaceOrderBySignal(tradeExit.Definition.OrderType, tradeExit.Quantity, tradeExit.Price,
                      tradeExit.StopLimitPrice, modCommon.TradeSideToOrderSide(trade.Definition.Side));
               }
            }
         }
      }

      private void PlaceOrderBySignal(eOrderType orderType, int quantity, double limitPrice,
                                      double stopPrice, eOrderSide orderSide)
      {
         if (orderType == eOrderType.otUndefined)
         {
            return;
         }

         double lmtPrice = 0;
         double stpPrice = 0;

         if (orderType == eOrderType.otLimit || orderType == eOrderType.otStopLimit)
         {
            lmtPrice = limitPrice;
         }

         if (orderType == eOrderType.otStop || orderType == eOrderType.otStopLimit)
         {
            stpPrice = stopPrice;
         }

         CQGOrder order = m_CEL.CreateOrder(orderType, m_Instrument, m_Account, quantity,
                                            orderSide, lmtPrice, stopPrice, "AUTOTRADE");
         order.Place();

         ExecutedOrders(order);
      }


      private void ExecutedOrders(CQGOrder order)
      {
         ListViewItem item = m_lvOrders.Items.Add(new ListViewItem());
         for (int i = System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocAutoTradeName); i <= System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocGUID); i++)
         {
            item.SubItems.Add("");
         }

         int iRow = m_lvOrders.Items.Count - 1;
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocAutoTradeName)].Text = m_Name;
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocAccount)].Text = order.Account.GWAccountName;
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocInstrument)].Text = order.InstrumentName;
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocSide)].Text = (order.Side == eOrderSide.osdSell ? "Sell" : "Buy").ToString();
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocPlaceTime)].Text = modCommon.N_A;
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocQuantity)].Text = modCommon.GetValueAsString(Math.Abs(order.Quantity), m_CEL);
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocFilledQuantity)].Text = modCommon.GetValueAsString(Math.Abs(order.FilledQuantity), m_CEL);
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocOrderType)].Text = order.Type.ToString();
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocOrderID)].Text = modCommon.GetValueAsString(order.OriginalOrderID, m_CEL);
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocGWStatus)].Text = modCommon.GetValueAsString(order.GWStatus, m_CEL);
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocState)].Text = modCommon.GetValueAsString(order.State, m_CEL);
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocUEName)].Text = order.UEName;
         m_lvOrders.Items[iRow].SubItems[System.Convert.ToInt32(modCommon.eExecutedOrdersColumns.eocGUID)].Text = order.GUID;
      }

      private string HistoricalPeriodToString(eHistoricalPeriod historicalPeriod)
      {
         string sHistoricalPeriod = "";

         switch (historicalPeriod)
         {
            case eHistoricalPeriod.hpUndefined:

               sHistoricalPeriod = "Undefined";
               break;
            case eHistoricalPeriod.hpDaily:

               sHistoricalPeriod = "Daily";
               break;
            case eHistoricalPeriod.hpWeekly:

               sHistoricalPeriod = "Weekly";
               break;
            case eHistoricalPeriod.hpMonthly:

               sHistoricalPeriod = "Monthly";
               break;
            case eHistoricalPeriod.hpQuarterly:

               sHistoricalPeriod = "Quarterly";
               break;
            case eHistoricalPeriod.hpSemiannual:

               sHistoricalPeriod = "Semiannual";
               break;
            case eHistoricalPeriod.hpYearly:

               sHistoricalPeriod = "Yearly";
               break;
         }

         return sHistoricalPeriod;
      }

      private string ContinuationTypeToString(eTimeSeriesContinuationType continuationType)
      {
         string sContinuationType = "";

         switch (continuationType)
         {
            case eTimeSeriesContinuationType.tsctNoContinuation:

               sContinuationType = "NC";
               break;
            case eTimeSeriesContinuationType.tsctStandard:

               sContinuationType = "St";
               break;
            case eTimeSeriesContinuationType.tsctStandardByMonth:

               sContinuationType = "StBM";
               break;
            case eTimeSeriesContinuationType.tsctActive:

               sContinuationType = "Ac";
               break;
            case eTimeSeriesContinuationType.tsctActiveByMonth:

               sContinuationType = "AcBM";
               break;
            case eTimeSeriesContinuationType.tsctAdjusted:

               sContinuationType = "Ad";
               break;
            case eTimeSeriesContinuationType.tsctAdjustedByMonth:

               sContinuationType = "AdBM";
               break;
         }

         return sContinuationType;
      }

   }

}
