using System.Windows.Forms;
using System.Collections;
using CQG;

namespace AutoTrader
{
	public class AutoTraderCollection
	{
		
		
		private CQGCEL m_CEL;
		private ListView m_lvAutoTrades;
		private ListView m_lvOrders;
		private Hashtable m_AutoTraderCollection;
		
		public AutoTraderCollection(CQGCEL CEL, ListView lvAT, ListView lvO)
		{
			m_CEL = CEL;
			m_lvAutoTrades = lvAT;
			m_lvOrders = lvO;
			m_AutoTraderCollection = new Hashtable();
		}
		
		public void DeleteItem(int row, int lastRows)
		{
			CQGTradingSystem ts;
			m_AutoTraderCollection.Remove(row);
			for (int i = row + 1; i <= lastRows; i++)
			{
				AutoTrader at = (AutoTrader) m_AutoTraderCollection[i];
				at.Row = i - 1;
				m_AutoTraderCollection.Add(i - 1, at);
				m_AutoTraderCollection.Remove(i);
			}
			int tsToBeRemoved = 0;
			for (int i = 0; i <= m_CEL.AllTradingSystems.Count - 1; i++)
			{
				ts = m_CEL.AllTradingSystems[i];
				if (System.Convert.ToInt32(ts.get_Tag(modCommon.ROW_TAG)) == row)
				{
					tsToBeRemoved = i;
				}
				else if (System.Convert.ToInt32(ts.get_Tag(modCommon.ROW_TAG)) > row)
				{
					ts.set_Tag(modCommon.ROW_TAG, System.Convert.ToInt32(ts.get_Tag(modCommon.ROW_TAG)) - 1);
				}
			}
			m_CEL.RemoveTradingSystem(m_CEL.AllTradingSystems[tsToBeRemoved]);
		}
		
		public void AddItem(int row)
		{
			if (! m_AutoTraderCollection.ContainsKey(row))
			{
				AutoTrader at = new AutoTrader(m_CEL, m_lvAutoTrades, m_lvOrders);
				at.Row = row;
				m_AutoTraderCollection.Add(row, at);
			}
		}
		
		public void TradingSystemResolved(CQGTradingSystem tradingSystem)
		{
			int row = System.Convert.ToInt32(tradingSystem.get_Tag(modCommon.ROW_TAG));
			
			if (m_AutoTraderCollection.ContainsKey(row))
			{
				AutoTrader at = (AutoTrader) m_AutoTraderCollection[row];
				at.TradingSystem = tradingSystem;
				at.PositionsChanged(tradingSystem);
			}
		}
		
		public void TradingSystemAddNotification(CQGTradingSystem cqg_trading_system)
		{
			TradingSystemNotification(cqg_trading_system);
		}
		
		public void TradingSystemUpdateNotification(CQGTradingSystem cqg_trading_system)
		{
			TradingSystemNotification(cqg_trading_system);
		}
		
		public void InstrumentResolved(string symbol_, CQGInstrument instrument)
		{
			AutoTrader at;
			
			foreach (object atKey in m_AutoTraderCollection.Keys)
			{
				at = (AutoTrader) m_AutoTraderCollection[atKey];
				if ((! at.IsSymbolResolved) &&(at.Symbol.ToUpper() == symbol_))
				{
					at.Instrument = instrument;
				}
			}
		}
		
		public void SetSymbol(string newSymbol, int row)
		{
			if (m_AutoTraderCollection.ContainsKey(row))
			{
				AutoTrader at = (AutoTrader) m_AutoTraderCollection[row];
				at.Symbol = newSymbol; 
			}
		}

		public void SetWorkingMode(bool signalMode, int row)
		{
			if (m_AutoTraderCollection.ContainsKey(row))
			{
				AutoTrader at = (AutoTrader) m_AutoTraderCollection[row];
				at.IsSignalModeSelected = signalMode;
			}
		}
			
		public void TradingSystemDefName(string newTsdName, int row)
		{
			if (m_AutoTraderCollection.ContainsKey(row))
			{
				AutoTrader at = (AutoTrader) m_AutoTraderCollection[row];
				at.TradingSystemDefName = newTsdName;
			}
		}
				
		public void SetName(string newName, int row)
		{
			if (m_AutoTraderCollection.ContainsKey(row))
			{
				AutoTrader at = (AutoTrader) m_AutoTraderCollection[row];
				at.Name = newName;
			}
		}
					
		public void AccountChanged(int newAccID_, int row)
		{
			if (m_AutoTraderCollection.ContainsKey(row))
			{
				AutoTrader at = (AutoTrader) m_AutoTraderCollection[row];
				CQGAccount acc = (CQGAccount) m_CEL.Accounts[newAccID_];
				at.Account = acc;
			}
		}
			
		public void CheckBoxClicked(int row, ref CheckState value_)
		{
			if (m_AutoTraderCollection.ContainsKey(row))
			{
				AutoTrader at = (AutoTrader) m_AutoTraderCollection[row];
				at.Activate(ref value_);
			}
		}
			
		public void SetTradingSystem(CQGTradingSystem newTrSys, int row)
		{
			CQGTradingSystem ts;
			
			for (int i = 0; i <= m_CEL.AllTradingSystems.Count - 1; i++)
			{
				ts = m_CEL.AllTradingSystems[i];
				if (System.Convert.ToInt32(ts.get_Tag(modCommon.ROW_TAG)) == row)
				{
					m_CEL.RemoveTradingSystem(ts);
				}
			}
			
			if (newTrSys != null)
			{
				newTrSys.set_Tag(modCommon.ROW_TAG, row);

				if (m_AutoTraderCollection.ContainsKey(row))
				{
					AutoTrader at = (AutoTrader) m_AutoTraderCollection[row];
					at.TradingSystem = newTrSys;
					at.PositionsChanged(newTrSys);
				}
			}
		}

      private void TradingSystemNotification(CQGTradingSystem tradingSystem)
      {
         int row = System.Convert.ToInt32(tradingSystem.get_Tag(modCommon.ROW_TAG));
			
         if (m_AutoTraderCollection.ContainsKey(row))
         {
             AutoTrader at = (AutoTrader) m_AutoTraderCollection[row];
             CQGTradingSystemTradesRow tradeRow = tradingSystem.TradesRows[tradingSystem.TradesRows.Count - 1];
             TradesRowData savedTradeRowData = at.TradeRowData;
             TradesRowData tradeRowData = new TradesRowData();
             tradeRowData.SetData(tradeRow);

             int compareRes = modCommon.Compare(savedTradeRowData, tradeRowData);
             bool doPositionChange = false;
             if (compareRes == -1)
             {
                 doPositionChange = true;
             }
             else if(compareRes == 0)
             {
                 if (at.IsSignalModeSelected)
                 {
                     if(!savedTradeRowData.Signaled)
                     {
                        doPositionChange = true;
                     }
                 }
                 else
                 {
                    doPositionChange = true;
                 }
             }
             if (doPositionChange)
             {
                 at.TradeRowData.SetData(tradeRow);
                 at.PositionsChanged(tradingSystem);
             }
          }
       }
	}
}
