Imports CQG
Public Class AutoTraderCollection

   Private m_CEL As CQGCEL
   Private m_lvAutoTrades As ListView
   Private m_lvOrders As ListView
   Private m_AutoTraderCollection As Hashtable

   Public Sub New(ByVal CEL As CQGCEL, _
                  ByVal lvAT As ListView, _
                  ByVal lvO As ListView)

      m_CEL = CEL
      m_lvAutoTrades = lvAT
      m_lvOrders = lvO
      m_AutoTraderCollection = New Hashtable

   End Sub

   Public Sub DeleteItem(ByVal row_ As Integer, _
                         ByVal lastRows As Integer)

      Dim ts As CQGTradingSystem
      m_AutoTraderCollection.Remove(row_)

      For i As Integer = row_ + 1 To lastRows
         Dim at As AutoTrader = CType(m_AutoTraderCollection(i), AutoTrader)
         at.Row = i - 1
         m_AutoTraderCollection.Add(i - 1, at)
         m_AutoTraderCollection.Remove(i)
      Next
      Dim tsToBeRemoved As Integer
      For i As Integer = 0 To m_CEL.AllTradingSystems.Count - 1
         ts = m_CEL.AllTradingSystems(i)
         If CInt(ts.Tag(ROW_TAG)) = row_ Then
            tsToBeRemoved = i
         ElseIf CInt(ts.Tag(ROW_TAG)) > row_ Then
            ts.Tag(ROW_TAG) = CInt(ts.Tag(ROW_TAG)) - 1
         End If
      Next
      m_CEL.RemoveTradingSystem(m_CEL.AllTradingSystems(tsToBeRemoved))

   End Sub

   Public Sub AddItem(ByVal row_ As Integer)
      If Not m_AutoTraderCollection.ContainsKey(row_) Then
         Dim at As AutoTrader = New AutoTrader(m_CEL, m_lvAutoTrades, m_lvOrders)
         at.Row = row_
         m_AutoTraderCollection.Add(row_, at)
      End If
   End Sub

   Public Sub TradingSystemResolved(ByVal tradingSystem As CQGTradingSystem)
      Dim row_ As Integer = CInt(tradingSystem.Tag(ROW_TAG))

      If m_AutoTraderCollection.ContainsKey(row_) Then
         Dim at As AutoTrader = CType(m_AutoTraderCollection.Item(row_), AutoTrader)
         at.TradingSystem = tradingSystem
         at.PositionsChanged(tradingSystem)
      End If
   End Sub

   Public Sub TradingSystemAddNotification(ByVal cqg_trading_system As CQGTradingSystem)

      TradingSystemNotification(cqg_trading_system)

   End Sub

   Public Sub TradingSystemUpdateNotification(ByVal cqg_trading_system As CQGTradingSystem)

      TradingSystemNotification(cqg_trading_system)

   End Sub

   Public Sub InstrumentResolved(ByVal symbol_ As String, _
                                 ByVal instrument As CQGInstrument)
      Dim at As AutoTrader

      For Each atKey As Object In m_AutoTraderCollection.Keys
         at = CType(m_AutoTraderCollection(atKey), AutoTrader)
         If (Not at.IsSymbolResolved) And (at.Symbol.ToUpper = symbol_) Then
            at.Instrument = instrument
         End If
      Next
   End Sub

   Public Sub SetSymbol(ByVal newSymbol_ As String, _
                        ByVal row_ As Integer)

      If m_AutoTraderCollection.ContainsKey(row_) Then
         Dim at As AutoTrader = CType(m_AutoTraderCollection.Item(row_), AutoTrader)
         at.Symbol = newSymbol_
      End If

   End Sub

   Public Sub TradingSystemDefName(ByVal newTsdName_ As String, _
                                   ByVal row_ As Integer)

      If m_AutoTraderCollection.ContainsKey(row_) Then
         Dim at As AutoTrader = CType(m_AutoTraderCollection.Item(row_), AutoTrader)
         at.TradingSystemDefName = newTsdName_
      End If

   End Sub

   Public Sub SetName(ByVal newName_ As String, _
                      ByVal row_ As Integer)

      If m_AutoTraderCollection.ContainsKey(row_) Then
         Dim at As AutoTrader = CType(m_AutoTraderCollection.Item(row_), AutoTrader)
         at.Name = newName_
      End If

   End Sub

   Public Sub AccountChanged(ByVal newAccID_ As Integer, _
                             ByVal row_ As Integer)

      If m_AutoTraderCollection.ContainsKey(row_) Then
         Dim at As AutoTrader = CType(m_AutoTraderCollection(row_), AutoTrader)
         Dim acc As CQGAccount = CType(m_CEL.Accounts.Item(newAccID_), CQGAccount)
         at.Account = acc
      End If

   End Sub

   Public Sub CheckBoxClicked(ByVal row_ As Integer, _
                              ByRef value_ As CheckState)

      If m_AutoTraderCollection.ContainsKey(row_) Then
         Dim at As AutoTrader = CType(m_AutoTraderCollection(row_), AutoTrader)
         at.Activate(value_)
      End If

   End Sub

   Public Sub SetTradingSystem(ByVal newTrSys_ As CQGTradingSystem, _
                               ByVal row_ As Integer)

      Dim ts As CQGTradingSystem

      For i As Integer = 0 To m_CEL.AllTradingSystems.Count - 1
         ts = m_CEL.AllTradingSystems.Item(i)
         If CInt(ts.Tag(ROW_TAG)) = row_ Then
            m_CEL.RemoveTradingSystem(ts)
         End If
      Next

      If Not newTrSys_ Is Nothing Then
         newTrSys_.Tag(ROW_TAG) = row_
      End If

      If m_AutoTraderCollection.ContainsKey(row_) Then
         Dim at As AutoTrader = CType(m_AutoTraderCollection.Item(row_), AutoTrader)
         at.TradingSystem = newTrSys_
         at.PositionsChanged(newTrSys_)
      End If

   End Sub

   Private Sub TradingSystemNotification(ByVal tradingSystem As CQGTradingSystem)

      Dim row_ As Integer = CInt(tradingSystem.Tag(ROW_TAG))

      If m_AutoTraderCollection.ContainsKey(row_) Then

         Dim at As AutoTrader = CType(m_AutoTraderCollection.Item(row_), AutoTrader)
         Dim tradeRow As CQGTradingSystemTradesRow = tradingSystem.TradesRows(tradingSystem.TradesRows.Count - 1)
         Dim savedTradeRowData As TradesRowData = at.TradeRowData
         Dim tradeRowData As TradesRowData = New TradesRowData
         tradeRowData.SetData(tradeRow)

         If modCommon.Compare(savedTradeRowData, tradeRowData) <= 0 Then
            at.TradeRowData.SetData(tradeRow)
            at.PositionsChanged(tradingSystem)
         End If

      End If

   End Sub

End Class