// TradingSystemDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "AutoTrader.h"
#include "AutoTraderDlg.h"
#include "AutoTraderReqDlg.h"

const long CAutoTraderDlg::AUTO_TRADERS_COUNT = 20;

CAutoTraderDlg::CAutoTraderDlg(CWnd* pParent /*=NULL*/)
                           : CDialog(CAutoTraderDlg::IDD, pParent)
{
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CAutoTraderDlg::DoDataExchange(
                           CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_LBL_DATA_CONNECTION, m_lblDataConnection);
   DDX_Control(pDX, IDC_LBL_GW_CONNECTION, m_lblGwConnection);
   DDX_Control(pDX, IDC_LBL_DEFINITIONS_STATUS, m_lblDefinitionsStatus);
   DDX_Control(pDX, IDC_LBL_ACCOUNTS_STATUS, m_lblAccountsStatus);
   DDX_Control(pDX, IDC_CMB_DEFINITIONS, m_cmbDefinitions);
   DDX_Control(pDX, IDC_LSV_AUTO_TRADERS, m_lvAutoTraders);
   DDX_Control(pDX, IDC_LSV_EXECUTED_ORDERS, m_lvExecutedOrders);
   DDX_Control(pDX, IDC_BTN_ADD, m_btnAdd);
   DDX_Control(pDX, IDC_BTN_REMOVE, m_btnRemove);
}

BEGIN_MESSAGE_MAP(CAutoTraderDlg, CDialog)
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_WM_DESTROY()
   ON_BN_CLICKED(IDC_BTN_ADD, OnBnClickedBtnAdd)
   ON_NOTIFY(LVN_ITEMCHANGED, IDC_LSV_AUTO_TRADERS, OnLvnItemchangedLsvAutoTraders)
   ON_BN_CLICKED(IDC_BTN_REMOVE, OnBnClickedBtnRemove)
END_MESSAGE_MAP()

BOOL CAutoTraderDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   // Set the icon for this dialog.  The framework does this automatically
   //  when the application's main window is not a dialog
   SetIcon(m_hIcon, TRUE);       // Set big icon
   SetIcon(m_hIcon, FALSE);      // Set small icon

   try
   {
      InitializeControls();
      InitializeCQGCEL();
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
}

void CAutoTraderDlg::OnPaint() 
{
   if (IsIconic())
   {
      CPaintDC dc(this); // device context for painting

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      // Center icon in client rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      // Draw the icon
      dc.DrawIcon(x, y, m_hIcon);
   }
   else
   {
      CDialog::OnPaint();
   }
}

HCURSOR CAutoTraderDlg::OnQueryDragIcon()
{
   return static_cast<HCURSOR>(m_hIcon);
}

void CAutoTraderDlg::OnOK()
{
   // Prevents closing dialog on Enter
}

void CAutoTraderDlg::OnDestroy()
{
   try
   {
      if (m_spCQGCEL)
      {
         HRESULT hr = ICQGCELDispEventImpl::DispEventUnadvise(m_spCQGCEL);
         ATLASSERT(SUCCEEDED(hr));

         m_spCQGCEL->Shutdown();
      }
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}

void CAutoTraderDlg::OnBnClickedBtnAdd()
{
   try
   {
      if (AUTO_TRADERS_COUNT <= m_lvAutoTraders.GetItemCount())
      {
         AfxMessageBox(_T("Maximum allowed auto traders number is reached."), MB_ICONEXCLAMATION);
         return;
      }

      // Get trading system definition name
      CString tradingSystemName;
      m_cmbDefinitions.GetWindowText(tradingSystemName);

      ICQGTradingSystemRequestPtr spRequest = m_spCQGCEL->CreateTradingSystemRequest(_bstr_t(tradingSystemName));
      spRequest->PutSessionsFilter(31); // In other words "all sessions". 31 is represented by "11111" bits, where each bit designates a session in "Chart Preference"/"Sessions".
      ICQGAccountsPtr spAccounts = m_spCQGCEL->GetAccounts();

      CAutoTraderReqDlg dlg(spRequest, spAccounts);
      if (IDOK == dlg.DoModal())
      {
         // Send request to CQGCEL
         ICQGTradingSystemPtr spTradingSystem = m_spCQGCEL->RequestTradingSystem(spRequest);

         _bstr_t tradingSystemId = spTradingSystem->GetId();
         Trader trader(m_spCQGCEL, dlg.GetAutoTraderName(), dlg.GetAccountID(), spTradingSystem);

         m_TradersMap.insert(std::pair<_bstr_t, Trader>(tradingSystemId, trader));

         const int row = m_lvAutoTraders.InsertItem(m_lvAutoTraders.GetItemCount(), _T(""));
         m_lvAutoTraders.SetItemText(row, atcTurnedOnAt, trader.GetActivationTime());
         m_lvAutoTraders.SetItemText(row, atcOpenPositions, CHelper::N_A);
         m_lvAutoTraders.SetItemText(row, atcName, dlg.GetAutoTraderName());
         m_lvAutoTraders.SetItemText(row, atcTradingSystem, spTradingSystem->GetDefinition()->GetName());
         m_lvAutoTraders.SetItemText(row, atcTradingSystemStatus,
                                             CHelper::RequestStatusToString(spTradingSystem->GetStatus()));
         m_lvAutoTraders.SetItemText(row, atcSymbol, trader.GetSymbol());
         m_lvAutoTraders.SetItemText(row, atcSymbolStatus, CHelper::NOT_RESOLVED);
         m_lvAutoTraders.SetItemText(row, atcAccountID, GetValueAsString(dlg.GetAccountID()));
         m_lvAutoTraders.SetItemText(row, atcAccountName, trader.GetAccountName());
         m_lvAutoTraders.SetItemText(row, atcFullName, CHelper::N_A);
         m_lvAutoTraders.SetItemText(row, atcPreferences, trader.GetPreferences());
         m_lvAutoTraders.SetItemText(row, atcParameters, trader.GetParameters());
         m_lvAutoTraders.SetItemText(row, atcTradingSystemID, static_cast<LPCTSTR>(ATL::CW2T(spTradingSystem->GetId())));
      }
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }
}

void CAutoTraderDlg::OnBnClickedBtnRemove()
{
   try
   {
      int row = m_lvAutoTraders.GetNextItem(-1, LVNI_SELECTED);
      if (-1 == row)
      {
         AfxMessageBox(_T("There is no item selected."), MB_ICONEXCLAMATION);
         return;
      }

      _bstr_t tradingSystemID = m_lvAutoTraders.GetItemText(row, atcTradingSystemID);
      ICQGTradingSystemPtr tradingSystem = m_spCQGCEL->GetAllTradingSystems()->GetItemById(tradingSystemID);
      
      m_spCQGCEL->RemoveTradingSystem(tradingSystem);

      m_lvAutoTraders.DeleteItem(row);
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }
}

void CAutoTraderDlg::OnLvnItemchangedLsvAutoTraders(
                           NMHDR *pNMHDR,
                           LRESULT *pResult)
{
   try
   {
      LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);

      const int selectState = (((pNMLV->uNewState & LVIS_STATEIMAGEMASK) >> 12) - 1);
      if (selectState < 0)
      {
         // If there is no item checked then do nothing
         return;
      }

      const int row = pNMLV->iItem;

      bool isSelected = (m_lvAutoTraders.GetCheck(row) != FALSE);
      CString tradingSystemID = m_lvAutoTraders.GetItemText(row, atcTradingSystemID);
      if (!tradingSystemID.IsEmpty())
      {
         Trader& trader = GetAutoTraderByID(_bstr_t(tradingSystemID));
         if (trader.GetReadyStatus())
         {
            trader.Activate(isSelected);
            m_lvAutoTraders.SetItemText(row, atcTurnedOnAt, trader.GetActivationTime());
         }
         else if (isSelected)
         {
            m_lvAutoTraders.SetCheck(row, FALSE);
         }
      }
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   *pResult = 0;
}
