// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include <cassert>
#include "Common.h"
#include "Helper.h"
#include "AutoTrader.h"
#include "AutoTraderDlg.h"

void CAutoTraderDlg::InitializeCQGCEL()
{
   // Create an instance of CQG API
   m_spCQGCEL.CreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);

   // Configure CQGCEL behavior
   ICQGAPIConfigPtr spConf = m_spCQGCEL->GetAPIConfiguration();

   spConf->PutReadyStatusCheck(rscOff);
   spConf->PutUsedFromATLClient(VARIANT_TRUE);
   spConf->PutCollectionsThrowException(VARIANT_FALSE);
   spConf->PutTimeZoneCode(tzCentral);
   spConf->PutUseOrderSide(VARIANT_TRUE);
   spConf->put_DefaultInstrumentSubscriptionLevel(dsNone);

   // Now advise the connection, to get events
   HRESULT hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   m_spCQGCEL->Startup();
}

void CAutoTraderDlg::InitializeControls()
{
   m_DefinitionsResolved = false;
   m_AccountsResolved = false;

   ClearAllData();
   InitializeAutoTradersList();
   InitializeExecutedOrdersList();

   OnDataConnectionStatusChanged(csConnectionDown);
}

void CAutoTraderDlg::InitializeAutoTradersList()
{
   m_lvAutoTraders.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT);
   m_lvAutoTraders.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   m_lvAutoTraders.InsertColumn(atcOn, _T("On"), LVCFMT_LEFT, 30);
   m_lvAutoTraders.InsertColumn(atcTurnedOnAt, _T("TurnedOnAt"), LVCFMT_LEFT, 100);
   m_lvAutoTraders.InsertColumn(atcOpenPositions, _T("OpenPositions"), LVCFMT_LEFT, 100);
   m_lvAutoTraders.InsertColumn(atcName, _T("Name"), LVCFMT_LEFT, 100);
   m_lvAutoTraders.InsertColumn(atcTradingSystem, _T("TradingSystem"), LVCFMT_LEFT, 100);
   m_lvAutoTraders.InsertColumn(atcTradingSystemStatus, _T("TradingSystemStatus"), LVCFMT_LEFT, 100);
   m_lvAutoTraders.InsertColumn(atcSymbol, _T("Symbol"), LVCFMT_LEFT, 100);
   m_lvAutoTraders.InsertColumn(atcSymbolStatus, _T("SymbolStatus"), LVCFMT_LEFT, 100);
   m_lvAutoTraders.InsertColumn(atcAccountID, _T("AccountID"), LVCFMT_LEFT, 100);
   m_lvAutoTraders.InsertColumn(atcAccountName, _T("AccountName"), LVCFMT_LEFT, 100);
   m_lvAutoTraders.InsertColumn(atcFullName, _T("FullName"), LVCFMT_LEFT, 100);
   m_lvAutoTraders.InsertColumn(atcPreferences, _T("Preferences"), LVCFMT_LEFT, 300);
   m_lvAutoTraders.InsertColumn(atcParameters, _T("Parameters"), LVCFMT_LEFT, 300);
   m_lvAutoTraders.InsertColumn(atcTradingSystemID, _T("TradingSystemID"), LVCFMT_LEFT, 0);
}

void CAutoTraderDlg::InitializeExecutedOrdersList()
{
   m_lvExecutedOrders.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvExecutedOrders.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   m_lvExecutedOrders.InsertColumn(eocAutoTradeName, _T("AutoTradeName"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocAccount, _T("Account"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocInstrument, _T("Instrument"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocSide, _T("Side"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocQuantity, _T("Quantity"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocFilledQuantity, _T("FilledQuantity"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocGWStatus, _T("GWStatus"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocState, _T("State"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocPlaceTime, _T("PlaceTime"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocOrderType, _T("OrderType"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocOrderID, _T("OrderID"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocUEName, _T("UEName"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocError, _T("Error"), LVCFMT_LEFT, 100);
   m_lvExecutedOrders.InsertColumn(eocGUID, _T("GUID"), LVCFMT_LEFT, 0);
}

void CAutoTraderDlg::ShowOrder(
                        const CString& autoTraderName,
                        const ICQGOrderPtr& order)
{
   const int row = m_lvExecutedOrders.InsertItem(m_lvExecutedOrders.GetItemCount(), _T(""));
   m_lvExecutedOrders.SetItemText(row, eocAutoTradeName, autoTraderName);
   m_lvExecutedOrders.SetItemText(row, eocAccount, ATL::CW2T(order->GetAccount()->GetGWAccountName()));
   m_lvExecutedOrders.SetItemText(row, eocInstrument, ATL::CW2T(order->GetInstrumentName()));
   m_lvExecutedOrders.SetItemText(row, eocSide, CHelper::OrderSideToString(order->GetSide()));
   m_lvExecutedOrders.SetItemText(row, eocQuantity, GetValueAsString(order->GetQuantity()));
   m_lvExecutedOrders.SetItemText(row, eocFilledQuantity, GetValueAsString(order->GetFilledQuantity()));
   m_lvExecutedOrders.SetItemText(row, eocGWStatus, CHelper::OrderStatusToString(order->GetGWStatus()));
   m_lvExecutedOrders.SetItemText(row, eocState, CHelper::OrderStateToString(order->GetState()));
   m_lvExecutedOrders.SetItemText(row, eocPlaceTime, GetValueAsString(order->GetPlaceTime(), VT_DATE));
   m_lvExecutedOrders.SetItemText(row, eocOrderType, CHelper::OrderTypeToString(order->GetType()));
   m_lvExecutedOrders.SetItemText(row, eocOrderID, order->GetOriginalOrderID());
   m_lvExecutedOrders.SetItemText(row, eocUEName, ATL::CW2T(order->GetUEName()));
   m_lvExecutedOrders.SetItemText(row, eocGUID, ATL::CW2T(order->GetGUID()));

   ICQGErrorPtr lastError = order->GetLastError();
   if (lastError)
   {
      m_lvExecutedOrders.SetItemText(row, eocError, ATL::CW2T(lastError->GetDescription()));
   }
}

void CAutoTraderDlg::UpdateOrder(
                        int row,
                        const ICQGOrderPtr& order)
{
   m_lvExecutedOrders.SetItemText(row, eocPlaceTime, GetValueAsString(order->GetPlaceTime(), VT_DATE));
   m_lvExecutedOrders.SetItemText(row, eocOrderID, order->GetOriginalOrderID());
   m_lvExecutedOrders.SetItemText(row, eocFilledQuantity, GetValueAsString(order->GetFilledQuantity()));
   m_lvExecutedOrders.SetItemText(row, eocGWStatus, CHelper::OrderStatusToString(order->GetGWStatus()));
   m_lvExecutedOrders.SetItemText(row, eocState, CHelper::OrderStateToString(order->GetState()));

   ICQGErrorPtr lastError = order->GetLastError();
   if (lastError)
   {
      m_lvExecutedOrders.SetItemText(row, eocError, ATL::CW2T(lastError->GetDescription()));
   }
}

int CAutoTraderDlg::FindOrderRow(
                        const _bstr_t& orderID)
{
   for (int i = 0; i < m_lvExecutedOrders.GetItemCount(); ++i)
   {
      CString orderID1 = m_lvExecutedOrders.GetItemText(i, eocGUID);
      if (0 == orderID1.Compare(ATL::CW2T(orderID)))
      {
         return i;
      }
   }

   return -1;
}

int CAutoTraderDlg::FindAutoTraderRow(
                        const _bstr_t& tradingSystemID)
{
   for (int i = 0; i < m_lvAutoTraders.GetItemCount(); ++i)
   {
      CString tradingSystemID1 = m_lvAutoTraders.GetItemText(i, atcTradingSystemID);
      if (0 == tradingSystemID1.Compare(ATL::CW2T(tradingSystemID)))
      {
         return i;
      }
   }

   return -1;
}

Trader& CAutoTraderDlg::GetAutoTraderByID(
                        const _bstr_t& tradingSystemID)
{
   const Traders::iterator it = m_TradersMap.find(tradingSystemID);

   if (it == m_TradersMap.end())
   {
      assert(!"Trading system not exist");
   }

   return (it->second);
}

void CAutoTraderDlg::ClearAllData()
{
   m_lvAutoTraders.DeleteAllItems();
   m_lvExecutedOrders.DeleteAllItems();
}

void CAutoTraderDlg::EnableControls()
{
   bool isDataConnectionUp = false;
   bool isGwConnectionUp = false;

   if ((m_spCQGCEL != NULL) && (m_spCQGCEL->GetIsStarted() != VARIANT_FALSE))
   {
      isDataConnectionUp = (csConnectionUp == m_spCQGCEL->GetEnvironment()->GetDataConnectionStatus());
      isGwConnectionUp = (csConnectionUp == m_spCQGCEL->GetEnvironment()->GetGWConnectionStatus());
   }

   m_btnAdd.EnableWindow(isDataConnectionUp && isGwConnectionUp && m_AccountsResolved && m_DefinitionsResolved);
}

CString CAutoTraderDlg::GetValueAsString(
                        ATL::CComVariant value,
                        VARTYPE valueType/*= VT_EMPTY*/) const
{
   if (VT_EMPTY != valueType)
   {
      value.vt = valueType;
   }

   // Check validness
   if (m_spCQGCEL->IsValid(value))
   {
      return Common::VariantToString(value);
   }
   return CHelper::N_A;
}
