/*
* =====================================================================
*  Filename: Common.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#include <stdexcept>

#pragma once

namespace Common
{
   /// \brief struct TradesRowData
   struct TradesRowData
   {
      /// \brief timestamps of trades row
      DATE m_timestamp;

      /// \brief timestamp offset of trades row 
      long m_timestampOffset;
   };

   /// \brief Converts raw date format to string.
   /// \param dateTime - Raw date
   /// \param withTime - Add time to string
   /// \param withDate - Add date to string
   /// \return Formatted string of date and time
   CString DateTimeToString(
                     DATE dateTime,
                     bool withTime = true,
                     bool withDate = true);

   /// \brief Converts variant value to string.
   /// \param value - Variant value to be converted
   /// \return String representation of the value
   CString VariantToString(
                     const ATL::CComVariant& value);

   /// \brief Converts variant value to string.
   /// \param value - Variant value to be converted
   /// \param fracPart - Fractional part length for the double value
   /// \return String representation of the value
   CString VariantToString(
                     const ATL::CComVariant& value,
                     int fracPart);

   /// \brief Converts variant value to string.
   /// \param value - Variant value to be converted
   /// \param withTime - Add time to string for the date value
   /// \return String representation of the value
   CString VariantToString(
                     const ATL::CComVariant& value,
                     bool withTime);

   /// \brief Converts variant value to string.
   /// \param value - Variant value to be converted
   /// \param withTime - Add time to string for the date value
   /// \param withDate - Add date to string for the date value
   /// \return String representation of the value
   CString VariantToString(
                     const ATL::CComVariant& value,
                     bool withTime,
                     bool withDate);

   /// \brief Compares timestamps of first trade row data with second
   /// \param first - first trades row data to compare
   /// \param second - second trades row data to compare
   /// \return 0 if fisrt = second, -1 if first < second, 1 if first > second
   int Compare(const TradesRowData& first, const TradesRowData& second);
}