// TradingSystem.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include <cassert>
#include "Helper.h"

const CString CHelper::N_A = _T("N/A");
const CString CHelper::NOT_RESOLVED = _T("Not Resolved");
const CString CHelper::DATE_FORMAT = _T("dd/MM/yyyy HH:mm");
const CString CHelper::UE_NAME = _T("AUTOTRADE");
const int CHelper::PARAM_FRAC_PART_LENGTH = 3;

void CHelper::SetComboBoxItem(
                        CComboBox& comboBoxCtrl,
                        DWORD_PTR itemData)
{
   for (int i = 0; i < comboBoxCtrl.GetCount(); ++i)
   {
      if (comboBoxCtrl.GetItemData(i) == itemData)
      {
         comboBoxCtrl.SetCurSel(i);
         return;
      }
   }

   assert(!"Invalid item data");
}

CString CHelper::ConnectionStatusToString(
                        eConnectionStatus connectionStatus)
{
   CString connectionStatusStr;

   switch (connectionStatus)
   {
   case csConnectionDelayed:
      connectionStatusStr = _T("Connection Delayed");
      break;
   case csConnectionDown:
      connectionStatusStr = _T("Connection Down");
      break;
   case csConnectionUp:
      connectionStatusStr = _T("Connection Up");
      break;
   default:
      connectionStatusStr = _T("Unknown");
   }

   return connectionStatusStr;
}

CString CHelper::ParameterTypeToString(
                        eUserFormulaParameterType parameterType)
{
   CString parameterTypeStr;

   switch (parameterType)
   {
   case ufptInt:
      parameterTypeStr = _T("Int");
      break;
   case ufptDouble:
      parameterTypeStr = _T("Double");
      break;
   default:
      assert(!"Unknown parameter type");
   }

   return parameterTypeStr;
}

CString CHelper::RequestStatusToString(
                        eRequestStatus requestStatus)
{
   CString requestStatusStr;

   switch (requestStatus)
   {
   case rsInProgress:
      requestStatusStr = _T("InProgress");
      break;
   case rsSuccess:
      requestStatusStr = _T("Success");
      break;
   case rsCanceled:
      requestStatusStr = _T("Canceled");
      break;
   case rsFailed:
      requestStatusStr = _T("Failed");
      break;
   default:
      assert(!"Unknown request status");
   }

   return requestStatusStr;
}

CString CHelper::HistoricalPeriodToString(
                        eHistoricalPeriod historicalPeriod)
{
   CString historicalPeriodStr;

   switch (historicalPeriod)
   {
   case hpUndefined:
      historicalPeriodStr = _T("Undefined");
      break;
   case hpDaily:
      historicalPeriodStr = _T("Daily");
      break;
   case hpWeekly:
      historicalPeriodStr = _T("Weekly");
      break;
   case hpMonthly:
      historicalPeriodStr = _T("Monthly");
      break;
   case hpQuarterly:
      historicalPeriodStr = _T("Quarterly");
      break;
   case hpSemiannual:
      historicalPeriodStr = _T("Semiannual");
      break;
   case hpYearly:
      historicalPeriodStr = _T("Yearly");
      break;
   default:
      assert(!"Unknown historical period");
   }

   return historicalPeriodStr;
}

CString CHelper::ContinuationTypeToString(
                        eTimeSeriesContinuationType continuationType)
{
   CString continuationTypeStr;

   switch (continuationType)
   {
   case tsctNoContinuation:
      continuationTypeStr = _T("NoContinuation");
      break;
   case tsctStandard:
      continuationTypeStr = _T("Standard");
      break;
   case tsctStandardByMonth:
      continuationTypeStr = _T("StandardByMonth");
      break;
   case tsctActive:
      continuationTypeStr = _T("Active");
      break;
   case tsctActiveByMonth:
      continuationTypeStr = _T("ActiveByMonth");
      break;
   case tsctAdjusted:
      continuationTypeStr = _T("Adjusted");
      break;
   case tsctAdjustedByMonth:
      continuationTypeStr = _T("AdjustedByMonth");
      break;
   default:
      assert(!"Unknown continuation type");
   }

   return continuationTypeStr;
}

CString CHelper::ContinuationTypeToShortString(
                        eTimeSeriesContinuationType continuationType)
{
   CString continuationTypeStr;

   switch (continuationType)
   {
   case tsctNoContinuation:
      continuationTypeStr = _T("NC");
      break;
   case tsctStandard:
      continuationTypeStr = _T("Std");
      break;
   case tsctStandardByMonth:
      continuationTypeStr = _T("StdBM");
      break;
   case tsctActive:
      continuationTypeStr = _T("Act");
      break;
   case tsctActiveByMonth:
      continuationTypeStr = _T("ActBM");
      break;
   case tsctAdjusted:
      continuationTypeStr = _T("Adj");
      break;
   case tsctAdjustedByMonth:
      continuationTypeStr = _T("AdjBM");
      break;
   default:
      assert(!"Unknown continuation type");
   }

   return continuationTypeStr;
}

CString CHelper::SessionFlagToString(
                        eSessionFlag sessionFlag)
{
   CString sessionFlagStr = _T("Undefined");

   if (sfDailyFromIntraday == (sessionFlag & sfDailyFromIntraday))
   {
      sessionFlagStr = _T("DailyFromIntraday");
   }

   return sessionFlagStr;
}

CString CHelper::OrderTypeToString(
                        eOrderType orderType)
{
   CString orderTypeStr;

   switch (orderType)
   {
   case otUndefined:
      orderTypeStr = _T("Undefined");
      break;
   case otMarket:
      orderTypeStr = _T("Market");
      break;
   case otLimit:
      orderTypeStr = _T("Limit");
      break;
   case otStop:
      orderTypeStr = _T("Stop");
      break;
   case otStopLimit:
      orderTypeStr = _T("Stop Limit");
      break;
   default:
      assert(!"Unknown order type");
   }

   return orderTypeStr;
}

CString CHelper::OrderSideToString(
                        eOrderSide side)
{
   CString sideString;

   switch (side)
   {
   case osdBuy:
      sideString = _T("Buy");
      break;
   case osdSell:
      sideString = _T("Sell");
      break;
   case osdUndefined:
      sideString = _T("Undefined");
      break;
   default:
      assert(!"Unknown order side");
   }

   return sideString;
}

CString CHelper::OrderStatusToString(
                        eOrderStatus status)
{
   CString statusString;

   switch (status)
   {
   case osActiveAt:
      statusString = _T("ActiveAt");
      break;
   case osBusted:
      statusString = _T("Busted");
      break;
   case osCanceled:
      statusString = _T("Canceled");
      break;
   case osContingent:
      statusString = _T("Contingent");
      break;
   case osDisconnected:
      statusString = _T("Disconnected");
      break;
   case osExpired:
      statusString = _T("Expired");
      break;
   case osFilled:
      statusString = _T("Filled");
      break;
   case osInCancel:
      statusString = _T("InCancel");
      break;
   case osInClient:
      statusString = _T("InClient");
      break;
   case osInModify:
      statusString = _T("InModify");
      break;
   case osInOrderBook:
      statusString = _T("InOrderBook");
      break;
   case osInTransit:
      statusString = _T("InTransit");
      break;
   case osInTransitTimeout:
      statusString = _T("InTransitTimeout");
      break;
   case osNotSent:
      statusString = _T("NotSent");
      break;
   case osParked:
      statusString = _T("Parked");
      break;
   case osRejectFCM:
      statusString = _T("RejectFCM");
      break;
   case osRejectGW:
      statusString = _T("RejectGW");
      break;
   default:
      assert(!"Unknown order status");
   }

   return statusString;
}

CString CHelper::OrderStateToString(
                        eOrderLocalState state)
{
   CString stateString;

   switch (state)
   {
   case olsNotSent:
      stateString = _T("NotSent");
      break;
   case olsInQueue:
      stateString = _T("InQueue");
      break;
   case olsQueueTimeout:
      stateString = _T("QueueTimeout");
      break;
   case olsModifySent:
      stateString = _T("ModifySent");
      break;
   case olsCancelSent:
      stateString = _T("CancelSent");
      break;
   case olsActivateSent:
      stateString = _T("ActivateSent");
      break;
   case olsInClient:
      stateString = _T("InClient");
      break;
   case olsAtGW:
      stateString = _T("AtGW");
      break;
   case olsRemoved:
      stateString = _T("Removed");
      break;
   default:
      assert(!"Unknown order state");
   }

   return stateString;
}