using System;
using CQG;

namespace AutoTrader
{
	/// <summary>
	/// Summary description for TradesRowData.
	/// </summary>
	public class TradesRowData
	{
      /// <summary>
      /// Trade row timestamp.
      /// </summary>
      private DateTime m_TradeRowTimestamp;

      /// <summary>
      /// Trade row timestamp offset.
      /// </summary>
      private long m_TradeRowTimestampOffset;

      /// <summary>
      /// Flag indicating whether trade row is in signaled state.
      /// </summary>
      private bool m_IsSignaled;

      /// <summary>
      /// TradesRowData constructor.
      /// </summary>
      public TradesRowData()
      {
         m_TradeRowTimestampOffset = long.MinValue;
         m_IsSignaled = false;
      }

      /// <summary>
      /// Gets timestamp of the trades row.
      /// </summary>
      public DateTime Timestamp
      {
         get { return m_TradeRowTimestamp; }
      }

      /// <summary>
      /// Gets timestamp offset of trades row.
      /// </summary>
      public long TimestampOffset
      {
         get { return m_TradeRowTimestampOffset; }
      }

      /// <summary>
      /// Gets flag indicating whether trades row is signaled or not.
      /// </summary>
      public bool Signaled
      {
         get { return m_IsSignaled; }
      }

      /// <summary>
      /// Sets trading system trades row data.
      /// </summary>
      /// <param name="tradesRow">CQGTradingSystemTradesRow object.</param>
      public void SetData(CQGTradingSystemTradesRow tradesRow)
      {
         if (tradesRow == null)
         {
            return;
         }
         
         m_TradeRowTimestamp = tradesRow.Timestamp;
         m_TradeRowTimestampOffset = tradesRow.TimestampOffset;
         m_IsSignaled = false;

         foreach (CQGTradingSystemTrade trade in tradesRow)
         {
            if (trade.TradeEntry.Signal)
            {
               m_IsSignaled = true;
               return;
            }

            foreach (CQGTradeExit tradeExit in trade.TradeExits)
            {
               if (tradeExit.Signal)
               {
                  m_IsSignaled = true;
                  return;
               }
            }
         }
      }
	}
}
