using System;
using CQG;

namespace AutoTrader
{
    sealed class modCommon
    {
		
        public enum eExecutedOrdersColumns
        {
            eocAutoTradeName = 0,
            eocAccount,
            eocInstrument,
            eocSide,
            eocQuantity,
            eocFilledQuantity,
            eocPlaceTime,
            eocOrderType,
            eocOrderID,
            eocGWStatus,
            eocState,
            eocUEName,
            eocError,
            eocGUID
        }
		
        // Specifies the not available string
        public const string N_A = "N/A";

        public const string ROW_TAG = "ROW";
		
        public static string GetValueAsString(object val, CQGCEL cel)
        {
            string sResult = "";
			
            try
            {
                if (cel.IsValid(val))
                {
                    if (val.GetType().FullName == "System.DateTime")
                    {
                        sResult = System.Convert.ToDateTime(val).ToString("dd/MM/yyyy HH:mm");
                    }
                    else if (val.GetType().FullName == "System.Double")
                    {
                        sResult = Math.Round(System.Convert.ToDouble(val), 2).ToString();
                    }
                    else
                    {
                        sResult = val.ToString();
                    }
                }
                else
                {
                    sResult = N_A;
                }
            }
            catch
            {
                sResult = N_A;
            }
			
            return sResult;
        }

        /// <summary>
        /// Compares Timestamps of first and second trade row datas
        /// </summary>
        /// <param name="first">first trade row data to compare</param>
        /// <param name="second">second trade row data to compare</param>
        /// <returns>
        /// 0 if first = second, -1 if first < second, 1 if first > second
        /// </returns>
        public static int Compare(TradesRowData first, TradesRowData second)
        {
            if (first.Timestamp < second.Timestamp)
            {
                return -1;
            }
            else if (first.Timestamp > second.Timestamp)
            {
                return 1;
            }
            else
            {
                if (first.TimestampOffset < second.TimestampOffset)
                {
                    return -1;
                }
                else if (first.TimestampOffset > second.TimestampOffset)
                {
                    return 1;
                }
            }

            return 0;
        }

        public static eOrderSide TradeSideToOrderSide(eTradeSide tradeSide)
        {
            switch (tradeSide)
            {
                case eTradeSide.tsBuy:
                    return eOrderSide.osdBuy;
                case eTradeSide.tsSell:
                    return eOrderSide.osdSell;
                default:
                    return eOrderSide.osdUndefined;
            }
        }	
    }
        
}
