// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "Condition.h"
#include "ConditionDlg.h"

STDMETHODIMP CConditionDlg::OnDataError(
                              LPDISPATCH /*obj*/,
                              BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CConditionDlg::OnDataConnectionStatusChanged(
                              eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      EnableControls();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}


STDMETHODIMP CConditionDlg::OnConditionDefinitionsResolved(
                              ICQGConditionDefinitions* cqg_condition_definitions, 
                              ICQGError* cqg_error)
{
   try
   {
      // Raise error, when any error occurred during request
      VARIANT_BOOL isValid;
      HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &isValid);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      if (isValid)
      {
         ATL::CComBSTR errDescription;
         hr = cqg_error->get_Description(&errDescription);
         Common::ThrowOnCOMError<ICQGError>(cqg_error, hr);

         AfxMessageBox(ATL::CW2T(errDescription), MB_ICONEXCLAMATION);

         return S_OK;
      }

      m_spConditionDefinitions.Release();
      m_spConditionDefinitions = cqg_condition_definitions;

      // Fill up new data
      m_cmbDefinition.ResetContent();

      long count;
      hr = cqg_condition_definitions->get_Count(&count);
      Common::ThrowOnCOMError<ICQGConditionDefinitions>(cqg_condition_definitions, hr);

      if (count <= 0)
      {
         return S_OK;
      }

      for (long i = 0; i < count; ++i)
      {
         ATL::CComPtr<ICQGConditionDefinition> spConditionDefinition;
         hr = cqg_condition_definitions->get_Item(i, &spConditionDefinition);
         Common::ThrowOnCOMError<ICQGConditionDefinitions>(cqg_condition_definitions, hr);

         ATL::CComBSTR name;
         hr = spConditionDefinition->get_Name(&name);
         Common::ThrowOnCOMError(spConditionDefinition, hr);

         const int row = m_cmbDefinition.AddString(ATL::CW2T(name));
         m_cmbDefinition.SetItemData(row, i);
      }

      // Select first definition
      m_cmbDefinition.SetCurSel(0);
      OnCbnSelchangeCmbDefinition();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CConditionDlg::OnConditionResolved(
                              ICQGCondition* cqg_condition,
                              ICQGError* /*cqg_error*/)
{
   try
   {
      if (!m_spCondition)
      {
         return S_OK;
      }

      // Get selected condition ID
      ATL::CComBSTR selConditionID;
      HRESULT hr = m_spCondition->get_Id(&selConditionID);
      Common::ThrowOnCOMError(m_spCondition, hr);

      // Get changed condition ID
      ATL::CComBSTR conditionID;
      hr = cqg_condition->get_Id(&conditionID);
      Common::ThrowOnCOMError<ICQGCondition>(cqg_condition, hr);

      // Check if condition resolved in the selected request
      if (selConditionID == conditionID)
      {
         ClearAllData();
         ShowAllData();

         EnableControls();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CConditionDlg::OnConditionAdded(
                              ICQGCondition* cqg_condition)
{
   try
   {
      if (!m_spCondition)
      {
         return S_OK;
      }

      // Get selected condition ID
      ATL::CComBSTR selConditionID;
      HRESULT hr = m_spCondition->get_Id(&selConditionID);
      Common::ThrowOnCOMError(m_spCondition, hr);

      // Get changed condition ID
      ATL::CComBSTR conditionID;
      hr = cqg_condition->get_Id(&conditionID);
      Common::ThrowOnCOMError<ICQGCondition>(cqg_condition, hr);

      // Check if record added in the selected request
      if (selConditionID == conditionID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CConditionDlg::OnConditionInserted(
                              ICQGCondition* cqg_condition,
                              long /*index_*/)
{
   try
   {
      if (!m_spCondition)
      {
         return S_OK;
      }

      // Get selected condition ID
      ATL::CComBSTR selConditionID;
      HRESULT hr = m_spCondition->get_Id(&selConditionID);
      Common::ThrowOnCOMError(m_spCondition, hr);

      // Get changed condition ID
      ATL::CComBSTR conditionID;
      hr = cqg_condition->get_Id(&conditionID);
      Common::ThrowOnCOMError<ICQGCondition>(cqg_condition, hr);

      // Check if record inserted in the selected request
      if (selConditionID == conditionID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CConditionDlg::OnConditionRemoved(
                              ICQGCondition* cqg_condition,
                              long /*index_*/)
{
   try
   {
      if (!m_spCondition)
      {
         return S_OK;
      }

      // Get selected condition ID
      ATL::CComBSTR selConditionID;
      HRESULT hr = m_spCondition->get_Id(&selConditionID);
      Common::ThrowOnCOMError(m_spCondition, hr);

      // Get changed condition ID
      ATL::CComBSTR conditionID;
      hr = cqg_condition->get_Id(&conditionID);
      Common::ThrowOnCOMError<ICQGCondition>(cqg_condition, hr);

      // Check if record removed form the selected request
      if (selConditionID == conditionID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CConditionDlg::OnConditionUpdated(
                              ICQGCondition* cqg_condition,
                              long index_)
{
   try
   {
      if (!m_spCondition)
      {
         return S_OK;
      }

      // Get selected condition ID
      ATL::CComBSTR selConditionID;
      HRESULT hr = m_spCondition->get_Id(&selConditionID);
      Common::ThrowOnCOMError(m_spCondition, hr);

      // Get changed condition ID
      ATL::CComBSTR conditionID;
      hr = cqg_condition->get_Id(&conditionID);
      Common::ThrowOnCOMError<ICQGCondition>(cqg_condition, hr);

      // Check if record updated in the selected request
      if (selConditionID == conditionID)
      {
         // Find row for specified condition result
         CString indexStr;
         indexStr.Format(_T("%d"), index_ + 1);
         LVFINDINFO findInfo;
         findInfo.flags = LVFI_STRING;
         findInfo.psz = indexStr;
         const int row = m_lvLastCondition.FindItem(&findInfo);

         // If found change it's properties
         if (row > -1)
         {
            ShowRecord(index_ + 1, m_lvLastCondition, row);
         }
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}