// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "ConstantVolumeBars.h"
#include "ConstantVolumeBarsDlg.h"

void CConstantVolumeBarsDlg::InitializeCQGCEL()
{
   // Create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   Common::ThrowOnCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   Common::ThrowOnCOMError(m_spCQGCEL, hr);
}

void CConstantVolumeBarsDlg::InitializeControls()
{
   InitializeList(m_lvLastConstantVolumeBars);
   InitializeList(m_lvConstantVolumeBars);

   ClearAllData();

   OnDataConnectionStatusChanged(csConnectionDown);

   FillIndices();
}

void CConstantVolumeBarsDlg::InitializeList(
                        CListCtrl& listCtrl)
{
   listCtrl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   listCtrl.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   listCtrl.InsertColumn(rcIndex, _T("#"), LVCFMT_LEFT, 40);
   listCtrl.InsertColumn(rcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 150);
   listCtrl.InsertColumn(rcOpen, _T("Open"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcHigh, _T("High"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcLow, _T("Low"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcClose, _T("Close"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcMid, _T("Mid"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcHLC3, _T("HLC3"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcAvg, _T("Avg"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTrueHigh, _T("True High"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTrueLow, _T("True Low"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcRange, _T("Range"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTrueRange, _T("True Range"), LVCFMT_LEFT, 80);
   listCtrl.InsertColumn(rcActualVolume, _T("Actual Vol"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTickVolume, _T("Tick Vol"), LVCFMT_LEFT, 50);
   listCtrl.InsertColumn(rcAskVolume, _T("Ask Vol"), LVCFMT_LEFT, 50);
   listCtrl.InsertColumn(rcBidVolume, _T("Bid Vol"), LVCFMT_LEFT, 50);
}

void CConstantVolumeBarsDlg::ClearAllData()
{
   m_lblStatus.SetWindowText(_T(""));

   ClearRequestParams();

   ClearTags();

   ClearOutputParams();
   ClearIndices();

   ClearRecords(m_lvLastConstantVolumeBars);
   ClearRecords(m_lvConstantVolumeBars);

   m_lblAllRecsCount.SetWindowText(_T(""));
   m_lblFltrRecsCount.SetWindowText(_T(""));
}

void CConstantVolumeBarsDlg::ClearRequestParams()
{
   UpdateData();

   m_Symbol = _T("");
   m_RangeStart = _T("");
   m_RangeEnd = _T("");
   m_UpdatesEnabled = _T("");
   m_IncludeEnd = _T("");
   m_VolumeType = _T("");
   m_VolumeLevel = _T("");
   m_IncludeFlatTicks = _T("");
   m_SessionsFilter = _T("");
   m_Continuation = _T("");
   m_EqualizeCloses = _T("");
   m_DaysBeforeExpiration = _T("");
   m_SessionFlags = _T("");
   m_SubscriptionLevel = _T("");
   m_RecalcPeriod = _T("");

   UpdateData(FALSE);

   m_lblError.SetWindowText(_T(""));
}

void CConstantVolumeBarsDlg::ClearOutputParams()
{
   UpdateData();

   m_OutputStart = _T("");
   m_OutputEnd = _T("");

   UpdateData(FALSE);
}

void CConstantVolumeBarsDlg::ClearTags()
{
   m_edTagName.SetWindowText(_T(""));
   m_edTagValue.SetWindowText(_T(""));
}

void CConstantVolumeBarsDlg::ClearRecords(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();
}

void CConstantVolumeBarsDlg::ShowAllData()
{
   ShowRequestParams();

   eRequestStatus status;
   HRESULT hr = m_spConstantVolumeBars->get_Status(&status);
   Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

   m_lblStatus.SetWindowText(CHelper::RequestStatusToString(status));

   if (status == rsSuccess)
   {
      ShowOutputParams();

      FillIndices();

      long count;
      hr = m_spConstantVolumeBars->get_Count(&count);
      Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

      // Show records
      const long startIndex = max(count - m_ShowCount + 1, 1);
      const long endIndex = count;
      ShowRecords(startIndex, endIndex, m_lvLastConstantVolumeBars);

      // Show all records count
      m_lblAllRecsCount.SetWindowText(GetValueAsString(count));
   }
}

void CConstantVolumeBarsDlg::ShowChangedData()
{
   // Clear the list box
   ClearRecords(m_lvLastConstantVolumeBars);

   // Get records count
   long count;
   HRESULT hr = m_spConstantVolumeBars->get_Count(&count);
   Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

   // Show records
   const long startIndex = max(count - m_ShowCount + 1, 1);
   const long endIndex = count;
   ShowRecords(startIndex, endIndex, m_lvLastConstantVolumeBars);

   // Show all records count
   m_lblAllRecsCount.SetWindowText(GetValueAsString(count));

   // Show output parameters
   ShowOutputParams();
}

void CConstantVolumeBarsDlg::ShowRequestParams()
{
   UpdateData();

   // Get request from constant volume bars collection
   ATL::CComPtr<ICQGConstantVolumeBarsRequest> spRequest;
   HRESULT hr = m_spConstantVolumeBars->get_Request(&spRequest);
   Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

   // Show request parameters
   ATL::CComBSTR symbol;
   hr = spRequest->get_Symbol(&symbol);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeStart;
   hr = spRequest->get_RangeStart(&rangeStart);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeEnd;
   hr = spRequest->get_RangeEnd(&rangeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL updatesEnabled;
   hr = spRequest->get_UpdatesEnabled(&updatesEnabled);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL includeEnd;
   hr = spRequest->get_IncludeEnd(&includeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   eCvbVolumeType volumType;
   hr = spRequest->get_VolumeType(&volumType);
   Common::ThrowOnCOMError(spRequest, hr);

   double volumeLevel = 0.;
   hr = spRequest->get_VolumeLevelFractional(&volumeLevel);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL includeFlatTicks;
   hr = spRequest->get_IncludeFlatTicks(&includeFlatTicks);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant sessionsFilter;
   hr = spRequest->get_SessionsFilter(&sessionsFilter);
   Common::ThrowOnCOMError(spRequest, hr);

   eTimeSeriesContinuationType continuation;
   hr = spRequest->get_Continuation(&continuation);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL equalizeCloses;
   hr = spRequest->get_EqualizeCloses(&equalizeCloses);
   Common::ThrowOnCOMError(spRequest, hr);

   long daysBeforeExpiration;
   hr = spRequest->get_DaysBeforeExpiration(&daysBeforeExpiration);
   Common::ThrowOnCOMError(spRequest, hr);

   eSessionFlag sessionFlags;
   hr = spRequest->get_SessionFlags(&sessionFlags);
   Common::ThrowOnCOMError(spRequest, hr);
   
   eTimeSeriesSubscriptionLevel subscriptionLevel;
   hr = spRequest->get_SubscriptionLevel(&subscriptionLevel);
   Common::ThrowOnCOMError(spRequest, hr);
   
   long recalcPeriod;
   hr = spRequest->get_RecalcPeriod(&recalcPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   m_Symbol = ATL::CW2T(symbol);
   m_RangeStart = GetValueAsString(rangeStart);
   m_RangeEnd = GetValueAsString(rangeEnd);
   m_UpdatesEnabled = GetValueAsString(updatesEnabled, VT_BOOL);
   m_IncludeEnd = GetValueAsString(includeEnd, VT_BOOL);
   m_VolumeType = CHelper::VolumeTypeToString(volumType);
   m_VolumeLevel = GetValueAsString(volumeLevel);
   m_IncludeFlatTicks = GetValueAsString(includeFlatTicks, VT_BOOL);

   CString filter = GetValueAsString(sessionsFilter);
   if (filter == "31")
   {
      filter = "All sessions (31)";
   }
   else if (filter == "0")
   {
      filter = "Primary session only (0)";
   }
   m_SessionsFilter = filter;
   m_Continuation = CHelper::ContinuationTypeToString(continuation);
   m_EqualizeCloses = GetValueAsString(equalizeCloses, VT_BOOL);
   m_DaysBeforeExpiration = GetValueAsString(daysBeforeExpiration);
   m_SessionFlags = CHelper::SessionFlagToString(sessionFlags);
   m_SubscriptionLevel = CHelper::SubscriptionLevelToString(subscriptionLevel);
   if (subscriptionLevel == tslEndOfBarAndPeriod)
   {
      m_RecalcPeriod = GetValueAsString(recalcPeriod);
   }
   
   UpdateData(FALSE);

   // Get error from constant volume bars collection and show it if exist
   ATL::CComPtr<ICQGError> spError;
   hr = m_spConstantVolumeBars->get_LastError(&spError);
   Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

   if (spError)
   {
      ATL::CComBSTR errDescription;
      hr = spError->get_Description(&errDescription);
      Common::ThrowOnCOMError(spError, hr);

      m_lblError.SetWindowText(ATL::CW2T(errDescription));
   }
}

void CConstantVolumeBarsDlg::ShowOutputParams()
{
   UpdateData();

   // Get output parameters from constant volume bars collection and show them
   DATE startTimestamp;
   HRESULT hr = m_spConstantVolumeBars->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

   DATE endTimestamp;
   hr = m_spConstantVolumeBars->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

   m_OutputStart = GetValueAsString(startTimestamp, VT_DATE);
   m_OutputEnd = GetValueAsString(endTimestamp, VT_DATE);

   UpdateData(FALSE);
}

void CConstantVolumeBarsDlg::ShowRecords(
                        long startIndex,
                        long endIndex,
                        CListCtrl& listCtrl)
{
   for (long i = startIndex; i <= endIndex; ++i)
   {
      AddRecord(i, listCtrl);
   }
}

void CConstantVolumeBarsDlg::AddRecord(
                        long recordIndex,
                        CListCtrl& listCtrl)
{
   // Add empty row
   int row = listCtrl.InsertItem(listCtrl.GetItemCount(), _T(""));

   ShowRecord(recordIndex, listCtrl, row);
}

void CConstantVolumeBarsDlg::ShowRecord(
                        long recordIndex,
                        CListCtrl& listCtrl,
                        long row)
{
   // Get specified ConstantVolumeBar from constant volume bars collection by its index
   ATL::CComPtr<ICQGConstantVolumeBar> spConstantVolumeBar;
   HRESULT hr = m_spConstantVolumeBars->get_Item(recordIndex - 1, &spConstantVolumeBar);
   Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

   // Get ConstantVolumeBar timestamp to show it
   DATE timestamp;
   hr = spConstantVolumeBar->get_Timestamp(&timestamp);
   Common::ThrowOnCOMError(spConstantVolumeBar, hr);

   listCtrl.SetItemText(row, rcIndex, GetValueAsString(recordIndex));
   listCtrl.SetItemText(row, rcTimestamp, GetValueAsString(timestamp, VT_DATE, true));

   // Show parameters into given listbox
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_Open, listCtrl, row, rcOpen);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_High, listCtrl, row, rcHigh);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_Low, listCtrl, row, rcLow);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_Close, listCtrl, row, rcClose);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_Mid, listCtrl, row, rcMid);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_HLC3, listCtrl, row, rcHLC3);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_Avg, listCtrl, row, rcAvg);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_TrueHigh, listCtrl, row, rcTrueHigh);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_TrueLow, listCtrl, row, rcTrueLow);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_Range, listCtrl, row, rcRange);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_TrueRange, listCtrl, row, rcTrueRange);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_ActualVolumeFractional, listCtrl, row, rcActualVolume);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_TickVolume, listCtrl, row, rcTickVolume);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_AskVolume, listCtrl, row, rcAskVolume);
   ShowProperty(spConstantVolumeBar, &ICQGConstantVolumeBar::get_BidVolume, listCtrl, row, rcBidVolume);
}

template <typename T>
void CConstantVolumeBarsDlg::ShowProperty(
                        const ATL::CComPtr<ICQGConstantVolumeBar>& spConstantVolumeBar,
                        HRESULT (__stdcall ICQGConstantVolumeBar::*get_Property)(T*),
                        CListCtrl& listCtrl,
                        int row,
                        eRecordsColumns col)
{
   T value;
   HRESULT hr = (spConstantVolumeBar->*get_Property)(&value);
   Common::ThrowOnCOMError(spConstantVolumeBar, hr);

   listCtrl.SetItemText(row, col, GetValueAsString(value));
}

void CConstantVolumeBarsDlg::ClearIndices()
{
   m_edStart.SetValue(0);
   m_edEnd.SetValue(0);
}

void CConstantVolumeBarsDlg::FillIndices()
{
   if (m_spConstantVolumeBars == NULL)
   {
      return;
   }

   eRequestStatus status;
   HRESULT hr = m_spConstantVolumeBars->get_Status(&status);
   Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

   if (status != rsSuccess)
   {
      return;
   }

   // Fill start and end indices controls.
   m_edStart.SetValue(1);

   long count;
   hr = m_spConstantVolumeBars->get_Count(&count);
   Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

   m_edEnd.SetValue(count);
}

void CConstantVolumeBarsDlg::EnableControls()
{
   bool isDataConnectionUp = false;
   HRESULT hr;
   VARIANT_BOOL isStarted = VARIANT_FALSE;

   if (m_spCQGCEL)
   {
      hr = m_spCQGCEL->get_IsStarted(&isStarted);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }

   if (isStarted)
   {
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      hr = m_spCQGCEL->get_Environment(&spEnvironment);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      eConnectionStatus dataConnectionStatus;
      hr = spEnvironment->get_DataConnectionStatus(&dataConnectionStatus);
      Common::ThrowOnCOMError(spEnvironment, hr);

      isDataConnectionUp = (dataConnectionStatus == csConnectionUp);
   }

   m_btnRequest.EnableWindow(isDataConnectionUp);

   bool isConstantVolumeBarsSelected = (m_spConstantVolumeBars != NULL);

   m_btnRemove.EnableWindow(isConstantVolumeBarsSelected && isDataConnectionUp);
   m_btnRemoveAll.EnableWindow(isConstantVolumeBarsSelected && isDataConnectionUp);

   m_edTagName.EnableWindow(isConstantVolumeBarsSelected);
   m_edTagValue.EnableWindow(isConstantVolumeBarsSelected);

   bool isConstantVolumeBarsSucceeded = false;
   if (isConstantVolumeBarsSelected)
   {
      eRequestStatus status;
      hr = m_spConstantVolumeBars->get_Status(&status);
      Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

      isConstantVolumeBarsSucceeded = (status == rsSuccess);
   }

   m_edStart.EnableWindow(isConstantVolumeBarsSucceeded);
   m_spnStart.EnableWindow(isConstantVolumeBarsSucceeded);
   m_edEnd.EnableWindow(isConstantVolumeBarsSucceeded);
   m_spnEnd.EnableWindow(isConstantVolumeBarsSucceeded);

   m_btnShow.EnableWindow(isConstantVolumeBarsSucceeded);
}

CString CConstantVolumeBarsDlg::GetValueAsString(
                        ATL::CComVariant value,
                        VARENUM valueType/*= VT_EMPTY*/,
                        bool withMilliseconds /*= false*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return Common::VariantToString(value, withMilliseconds);
   }
   return CHelper::m_N_A;
}