// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "CustomSessions.h"
#include "CustomSessionsDlg.h"

void CCustomSessionsDlg::InitializeCQGCEL()
{
   // Create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   Common::ThrowOnCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   Common::ThrowOnCOMError(m_spCQGCEL, hr);
}

void CCustomSessionsDlg::InitializeControls()
{
   InitWorkingDays();
   ClearCustomSessions();

   EnableControls();
}

void CCustomSessionsDlg::ClearCustomSessions()
{
   m_spSessionsCollection = NULL;

   m_cmbCSCollection.ResetContent();

   m_lblType.SetWindowText(_T(""));

   ClearCustomSession();
}

void CCustomSessionsDlg::ClearCustomSession()
{
   m_cmbName.ResetContent();

   ClearCustomSessionData();
}

void CCustomSessionsDlg::ClearCustomSessionData()
{
   m_lblNumber.SetWindowText(_T(""));
   m_lblActivationDate.SetWindowText(_T(""));
   m_lblPrimaryFlag.SetWindowText(_T(""));
   m_lblStart.SetWindowText(_T(""));
   m_lblEnd.SetWindowText(_T(""));

   ClearWorkingDays();
}

void CCustomSessionsDlg::InitWorkingDays()
{
   m_lsvWorkingDays.SetExtendedStyle(LVS_EX_CHECKBOXES);
   m_lsvWorkingDays.InsertColumn(0, _T("Check"), LVCFMT_LEFT, 20);
   m_lsvWorkingDays.InsertColumn(1, _T("WeekDay"), LVCFMT_LEFT, 130);

   m_lsvWorkingDays.InsertItem(wdSunday, _T(""));
   m_lsvWorkingDays.SetItemText(wdSunday, 1, _T("Sunday"));

   m_lsvWorkingDays.InsertItem(wdMonday, _T(""));
   m_lsvWorkingDays.SetItemText(wdMonday, 1,  _T("Monday"));

   m_lsvWorkingDays.InsertItem(wdTuesday, _T(""));
   m_lsvWorkingDays.SetItemText(wdTuesday, 1, _T("Tuesday"));

   m_lsvWorkingDays.InsertItem(wdWednesday, _T(""));
   m_lsvWorkingDays.SetItemText(wdWednesday, 1, _T("Wednesday"));

   m_lsvWorkingDays.InsertItem(wdThursday, _T(""));
   m_lsvWorkingDays.SetItemText(wdThursday, 1, _T("Thursday"));

   m_lsvWorkingDays.InsertItem(wdFriday, _T(""));
   m_lsvWorkingDays.SetItemText(wdFriday, 1, _T("Friday"));

   m_lsvWorkingDays.InsertItem(wdSaturday, _T(""));
   m_lsvWorkingDays.SetItemText(wdSaturday, 1, _T("Saturday"));
}

void CCustomSessionsDlg::ClearWorkingDays()
{
   int count = m_lsvWorkingDays.GetItemCount();
   for (int i = 0; i < count; ++i)
   {
      m_lsvWorkingDays.SetCheck(i, false);
   }
}

void CCustomSessionsDlg::ShowWorkingDays(
                        eSessionWeekDays weekDay)
{
   m_lsvWorkingDays.SetCheck(wdSunday, (weekDay & swdSunday) == swdSunday);
   m_lsvWorkingDays.SetCheck(wdMonday, (weekDay & swdMonday) == swdMonday);
   m_lsvWorkingDays.SetCheck(wdTuesday, (weekDay & swdTuesday) == swdTuesday);
   m_lsvWorkingDays.SetCheck(wdWednesday, (weekDay & swdWednesday) == swdWednesday);
   m_lsvWorkingDays.SetCheck(wdThursday, (weekDay & swdThursday) == swdThursday);
   m_lsvWorkingDays.SetCheck(wdFriday, (weekDay & swdFriday) == swdFriday);
   m_lsvWorkingDays.SetCheck(wdSaturday, (weekDay & swdSaturday) == swdSaturday);
}

void CCustomSessionsDlg::EnableControls()
{
   bool isDataConnectionUp = false;

   VARIANT_BOOL isStarted;
   HRESULT hr = m_spCQGCEL->get_IsStarted(&isStarted);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isStarted)
   {
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      hr = m_spCQGCEL->get_Environment(&spEnvironment);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      eConnectionStatus dataConnectionStatus;
      hr = spEnvironment->get_DataConnectionStatus(&dataConnectionStatus);
      Common::ThrowOnCOMError(spEnvironment, hr);

      isDataConnectionUp = (dataConnectionStatus == csConnectionUp);
   }

   m_btnRequestCS.EnableWindow(isDataConnectionUp);
}

CString CCustomSessionsDlg::GetValueAsString(
                        ATL::CComVariant value,
                        VARENUM valueType/*= VT_EMPTY*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return Common::VariantToString(value);
   }
   return CHelper::m_N_A;
}

CString CCustomSessionsDlg::GetValueAsTimeOfDay(ATL::CComVariant value) const
{
   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      COleDateTime oleDateTime(value);
      if (oleDateTime.GetStatus() == COleDateTime::valid)
      {
         return oleDateTime.Format(_T("%H:%M"));
      }
   }
   return CHelper::m_N_A;
}
