// CustomSessionsDlg.cpp : implementation file
//

#include "stdafx.h"
#include <cassert>
#include <string>
#include "Common.h"
#include "Helper.h"
#include "CustomSessions.h"
#include "CustomSessionsDlg.h"

CCustomSessionsDlg::CCustomSessionsDlg(CWnd* pParent /*=NULL*/)
                           : CDialog(CCustomSessionsDlg::IDD, pParent)
{
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CCustomSessionsDlg::DoDataExchange(
                           CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_LBL_DATA_CONNECTION, m_lblDataConnection);
   DDX_Control(pDX, IDC_BTN_REQUEST_CS, m_btnRequestCS);
   DDX_Control(pDX, IDC_CMB_CS_COLLECTION, m_cmbCSCollection);
   DDX_Control(pDX, IDC_CMB_NAME, m_cmbName);
   DDX_Control(pDX, IDC_LBL_TYPE, m_lblType);
   DDX_Control(pDX, IDC_LBL_NUMBER, m_lblNumber);
   DDX_Control(pDX, IDC_LBL_PRIMARY_FLAG, m_lblPrimaryFlag);
   DDX_Control(pDX, IDC_LBL_START, m_lblStart);
   DDX_Control(pDX, IDC_LBL_END, m_lblEnd);
   DDX_Control(pDX, IDC_LBL_ACTIVATION_DATE, m_lblActivationDate);
   DDX_Control(pDX, IDC_LSV_WORKING_DAYS, m_lsvWorkingDays);
}

BEGIN_MESSAGE_MAP(CCustomSessionsDlg, CDialog)
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_WM_DESTROY()
   ON_CBN_SELCHANGE(IDC_CMB_CS_COLLECTION, OnCbnSelchangeCmbCsCollection)
   ON_CBN_SELCHANGE(IDC_CMB_NAME, OnCbnSelchangeCmbName)
   ON_BN_CLICKED(IDC_BTN_REQUEST_CS, OnBnClickedBtnRequestCs)
END_MESSAGE_MAP()

BOOL CCustomSessionsDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      // Set the icon for this dialog.  The framework does this automatically
      //  when the application's main window is not a dialog
      SetIcon(m_hIcon, TRUE);       // Set big icon
      SetIcon(m_hIcon, FALSE);      // Set small icon

      InitializeCQGCEL();
      InitializeControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
}

void CCustomSessionsDlg::OnPaint() 
{
   if (IsIconic())
   {
      CPaintDC dc(this); // device context for painting

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      // Center icon in client rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      // Draw the icon
      dc.DrawIcon(x, y, m_hIcon);
   }
   else
   {
      CDialog::OnPaint();
   }
}

HCURSOR CCustomSessionsDlg::OnQueryDragIcon()
{
   return static_cast<HCURSOR>(m_hIcon);
}

void CCustomSessionsDlg::OnOK()
{
   // Override to prevent dialog from closing on Enter
}

void CCustomSessionsDlg::OnDestroy()
{
   try
   {
      HRESULT hr = ICQGCELDispEventImpl::DispEventUnadvise(m_spCQGCEL);
      assert(SUCCEEDED(hr));

      hr = m_spCQGCEL->Shutdown();
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}

void CCustomSessionsDlg::OnBnClickedBtnRequestCs()
{
   try
   {
      // Clearing custom sessions
      ClearCustomSessions();

      // Requesting new custom sessions
      HRESULT hr = m_spCQGCEL->RequestCustomSessions();
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomSessionsDlg::OnCbnSelchangeCmbCsCollection()
{
   try
   {
      ClearCustomSession();

      long idx = static_cast<long>(m_cmbCSCollection.GetItemData(m_cmbCSCollection.GetCurSel()));

      // Get selected custom sessions from collection
      ATL::CComPtr<ICQGSessions> spCustomSessions;
      HRESULT hr = m_spSessionsCollection->get_Item(ATL::CComVariant(idx), &spCustomSessions);
      Common::ThrowOnCOMError(m_spSessionsCollection, hr);

      // Get custom sessions' properties and show them
      eSymbolSessionType symbolSessionType;
      hr = spCustomSessions->get_Type(&symbolSessionType);
      Common::ThrowOnCOMError(spCustomSessions, hr);

      m_lblType.SetWindowText(CHelper::SymbolSessionTypeToString(symbolSessionType));

      long count;
      hr = spCustomSessions->get_Count(&count);
      Common::ThrowOnCOMError(spCustomSessions, hr);

      if (count > 0)
      {
         for (long i = 0; i < count; ++i)
         {
            ATL::CComPtr<ICQGSession> spCustomSession;
            hr = spCustomSessions->get_Item(ATL::CComVariant(i), &spCustomSession);
            Common::ThrowOnCOMError(spCustomSessions, hr);

            ATL::CComBSTR name;
            hr = spCustomSession->get_Name(&name);
            Common::ThrowOnCOMError(spCustomSession, hr);

            int row = m_cmbName.AddString(ATL::CW2T(name));
            m_cmbName.SetItemData(row, i);
         }

         // Select first item
         m_cmbName.SetCurSel(0);
         OnCbnSelchangeCmbName();
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomSessionsDlg::OnCbnSelchangeCmbName()
{
   try
   {
      ClearCustomSessionData();

      // Get selected custom sessions from collection
      long idx = static_cast<long>(m_cmbCSCollection.GetItemData(m_cmbCSCollection.GetCurSel()));
      ATL::CComPtr<ICQGSessions> spCustomSessions;
      HRESULT hr = m_spSessionsCollection->get_Item(ATL::CComVariant(idx), &spCustomSessions);
      Common::ThrowOnCOMError(m_spSessionsCollection, hr);

      // Get selected custom session from custom sessions
      idx = static_cast<long>(m_cmbName.GetItemData(m_cmbName.GetCurSel()));
      ATL::CComPtr<ICQGSession> spCustomSession;
      hr = spCustomSessions->get_Item(ATL::CComVariant(idx), &spCustomSession);
      Common::ThrowOnCOMError(spCustomSessions, hr);

      // Get custom session's properties and show them
      ATL::CComBSTR name;
      hr = spCustomSession->get_Name(&name);
      Common::ThrowOnCOMError(spCustomSession, hr);

      short int number = 0;
      hr = spCustomSession->get_Number(&number);
      Common::ThrowOnCOMError(spCustomSession, hr);

      VARIANT_BOOL primaryFlag;
      hr = spCustomSession->get_PrimaryFlag(&primaryFlag);
      Common::ThrowOnCOMError(spCustomSession, hr);

      DATE activationDate;
      hr = spCustomSession->get_ActivationDate(&activationDate);
      Common::ThrowOnCOMError(spCustomSession, hr);

      DATE startTime;
      hr = spCustomSession->get_StartTime(&startTime);
      Common::ThrowOnCOMError(spCustomSession, hr);

      DATE endTime;
      hr = spCustomSession->get_EndTime(&endTime);
      Common::ThrowOnCOMError(spCustomSession, hr);

      eSessionWeekDays sessionWeekDays;
      hr = spCustomSession->get_WorkingWeekDays(&sessionWeekDays);
      Common::ThrowOnCOMError(spCustomSession, hr);

      CString strNumber;
      strNumber.Format(_T("%d"), static_cast<int>(number));

      m_lblNumber.SetWindowText(strNumber);
      m_lblPrimaryFlag.SetWindowText(GetValueAsString(primaryFlag, VT_BOOL));
      m_lblActivationDate.SetWindowText(GetValueAsString(activationDate, VT_DATE));
      m_lblStart.SetWindowText(GetValueAsTimeOfDay(startTime));
      m_lblEnd.SetWindowText(GetValueAsTimeOfDay(endTime));

      ShowWorkingDays(sessionWeekDays);
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}
