// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include <cassert>
#include <algorithm>
#include "Common.h"
#include "Helper.h"
#include "CustomStudy.h"
#include "CustomStudyDlg.h"

void CCustomStudyDlg::InitializeCQGCEL()
{
   // Create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   Common::ThrowOnCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   Common::ThrowOnCOMError(m_spCQGCEL, hr);
}

void CCustomStudyDlg::InitializeControls()
{
   InitializeList(m_lvLastCustomStudy);
   InitializeList(m_lvCustomStudy);

   ClearAllData();
   ClearAllDefinitionsData();

   InitializeDefParametersList();
   InitializeDefCurvesList();

   InitializeParametersList();

   OnDataConnectionStatusChanged(csConnectionDown);

   m_dtpStart.SetFormat(CHelper::DATE_FORMAT);
   m_dtpEnd.SetFormat(CHelper::DATE_FORMAT);

   UpdateData();
   m_FilterType = ftByIndices;
   UpdateData(FALSE);
   OnBnClickedRdIndices();
}

void CCustomStudyDlg::InitializeDefParametersList()
{
   m_lvDefParameters.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvDefParameters.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   m_lvDefParameters.InsertColumn(dpcName, _T("Name"), LVCFMT_LEFT, 100);
   m_lvDefParameters.InsertColumn(dpcComment, _T("Comment"), LVCFMT_LEFT, 160);
   m_lvDefParameters.InsertColumn(dpcType, _T("Type"), LVCFMT_LEFT, 50);
   m_lvDefParameters.InsertColumn(dpcDefaultValue, _T("Def. Value"), LVCFMT_LEFT, 70);

   COLORREF crBkColor = ::GetSysColor(COLOR_3DFACE);
   m_lvDefParameters.SetBkColor(crBkColor);
   m_lvDefParameters.SetTextBkColor(crBkColor);
}

void CCustomStudyDlg::InitializeDefCurvesList()
{
   m_lvDefCurves.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvDefCurves.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   m_lvDefCurves.InsertColumn(0, _T("Name"), LVCFMT_LEFT, 200);

   COLORREF crBkColor = ::GetSysColor(COLOR_3DFACE);
   m_lvDefCurves.SetBkColor(crBkColor);
   m_lvDefCurves.SetTextBkColor(crBkColor);
}

void CCustomStudyDlg::InitializeParametersList()
{
   m_lvParameters.InsertColumn(pcName, _T("Name"), LVCFMT_LEFT, 200);
   m_lvParameters.InsertColumn(pcValue, _T("Value"), LVCFMT_LEFT, 100);

   COLORREF crBkColor = ::GetSysColor(COLOR_3DFACE);
   m_lvParameters.SetBkColor(crBkColor);
   m_lvParameters.SetTextBkColor(crBkColor);
}

void CCustomStudyDlg::InitializeList(
                        CListCtrl& listCtrl)
{
   listCtrl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   listCtrl.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   listCtrl.InsertColumn(rcIndex, _T("#"), LVCFMT_LEFT, 40);
   listCtrl.InsertColumn(rcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 150);
}

void CCustomStudyDlg::ClearAllData()
{
   m_lblStatus.SetWindowText(_T(""));

   ClearRequestParams();

   ClearTags();

   ClearOutputParams();
   ClearDatesAndIndices();

   ClearRecords(m_lvLastCustomStudy);
   ClearRecords(m_lvCustomStudy);

   ClearHeaders(m_lvLastCustomStudy);
   ClearHeaders(m_lvCustomStudy);

   m_lblAllRecsCount.SetWindowText(_T(""));
   m_lblFltrRecsCount.SetWindowText(_T(""));
}

void CCustomStudyDlg::ClearAllDefinitionsData()
{
   m_cmbDefinition.ResetContent();
   ClearDefinitionData();

   m_edDefinitionName.SetWindowText(_T(""));
}

void CCustomStudyDlg::ClearDefinitionData()
{
   m_lvDefParameters.DeleteAllItems();
   m_lvDefCurves.DeleteAllItems();
   m_lblDefAbbreviation.SetWindowText(_T(""));
}

void CCustomStudyDlg::ClearRequestParams()
{
   m_lvParameters.DeleteAllItems();

   UpdateData();

   m_DefinitionName = _T("");
   m_BaseExpression = _T("");
   m_RangeStart = _T("");
   m_RangeEnd = _T("");
   m_IntradayPeriod = _T("");
   m_HistoricalPeriod = _T("");
   m_UpdatesEnabled = _T("");
   m_IncludeEnd = _T("");
   m_SessionsFilter = _T("");
   m_Continuation = _T("");
   m_EqualizeCloses = _T("");
   m_DaysBeforeExpiration = _T("");
   m_SessionFlags = _T("");
   m_SubscriptionLevel = _T("");
   m_RecalcPeriod = _T("");

   UpdateData(FALSE);

   m_lblError.SetWindowText(_T(""));
}

void CCustomStudyDlg::ClearOutputParams()
{
   UpdateData();

   m_OutputStart = _T("");
   m_OutputEnd = _T("");

   UpdateData(FALSE);
}

void CCustomStudyDlg::ClearTags()
{
   m_edTagName.SetWindowText(_T(""));
   m_edTagValue.SetWindowText(_T(""));
}

void CCustomStudyDlg::ClearHeaders(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();

   const int columnCount = listCtrl.GetHeaderCtrl()->GetItemCount() - rcCurvesStart;

   // Delete headers
   for (int i = 0; i < columnCount; ++i)
   {
      listCtrl.DeleteColumn(rcCurvesStart);
   }
}

void CCustomStudyDlg::ClearRecords(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();
}

void CCustomStudyDlg::ShowAllData()
{
   ShowRequestParams();

   eRequestStatus status;
   HRESULT hr = m_spCustomStudy->get_Status(&status);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);

   m_lblStatus.SetWindowText(CHelper::RequestStatusToString(status));

   if (status == rsSuccess)
   {
      ShowOutputParams();

      FillDatesAndIndices();

      // Show headers
      ShowHeaders(m_lvLastCustomStudy);
      ShowHeaders(m_lvCustomStudy);

      long count;
      hr = m_spCustomStudy->get_Count(&count);
      Common::ThrowOnCOMError(m_spCustomStudy, hr);

      // Show records
      const long startIndex = (std::max<long>)(count - SHOW_COUNT + 1, 1);
      ShowRecords(startIndex, count, m_lvLastCustomStudy);

      // Show all records count
      m_lblAllRecsCount.SetWindowText(GetValueAsString(count));
   }
}

void CCustomStudyDlg::ShowChangedData()
{
   // Clear the list box
   ClearRecords(m_lvLastCustomStudy);

   // Get records count
   long count;
   HRESULT hr = m_spCustomStudy->get_Count(&count);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);

   // Show records
   const long startIndex = (std::max<long>)(count - SHOW_COUNT + 1, 1);
   ShowRecords(startIndex, count, m_lvLastCustomStudy);

   // Show all records count
   m_lblAllRecsCount.SetWindowText(GetValueAsString(count));

   // Show output parameters
   ShowOutputParams();
}

void CCustomStudyDlg::ShowRequestParams()
{
   UpdateData();

   // Get request from custom study collection
   ATL::CComPtr<ICQGCustomStudyRequest> spRequest;
   HRESULT hr = m_spCustomStudy->get_Request(&spRequest);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);

   ShowReqDefinitionParameters(spRequest);

   // Show request parameters
   ATL::CComBSTR baseExpression;
   hr = spRequest->get_BaseExpression(&baseExpression);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeStart;
   hr = spRequest->get_RangeStart(&rangeStart);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeEnd;
   hr = spRequest->get_RangeEnd(&rangeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   long intradayPeriod;
   hr = spRequest->get_IntradayPeriod(&intradayPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   eHistoricalPeriod historicalPeriod;
   hr = spRequest->get_HistoricalPeriod(&historicalPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL updatesEnabled;
   hr = spRequest->get_UpdatesEnabled(&updatesEnabled);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL includeEnd;
   hr = spRequest->get_IncludeEnd(&includeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant sessionsFilter;
   hr = spRequest->get_SessionsFilter(&sessionsFilter);
   Common::ThrowOnCOMError(spRequest, hr);

   eTimeSeriesContinuationType continuation;
   hr = spRequest->get_Continuation(&continuation);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL equalizeCloses;
   hr = spRequest->get_EqualizeCloses(&equalizeCloses);
   Common::ThrowOnCOMError(spRequest, hr);

   long daysBeforeExpiration;
   hr = spRequest->get_DaysBeforeExpiration(&daysBeforeExpiration);
   Common::ThrowOnCOMError(spRequest, hr);

   eSessionFlag sessionFlags;
   hr = spRequest->get_SessionFlags(&sessionFlags);
   Common::ThrowOnCOMError(spRequest, hr);
   
   eTimeSeriesSubscriptionLevel subscriptionLevel;
   hr = spRequest->get_SubscriptionLevel(&subscriptionLevel);
   Common::ThrowOnCOMError(spRequest, hr);
   
   long recalcPeriod;
   hr = spRequest->get_RecalcPeriod(&recalcPeriod);
   Common::ThrowOnCOMError(spRequest, hr);
   
   m_BaseExpression = ATL::CW2T(baseExpression);
   m_RangeStart = GetValueAsString(rangeStart);
   m_RangeEnd = GetValueAsString(rangeEnd);
   m_IntradayPeriod = GetValueAsString(intradayPeriod);
   m_HistoricalPeriod = CHelper::HistoricalPeriodToString(historicalPeriod);
   m_UpdatesEnabled = GetValueAsString(updatesEnabled, VT_BOOL);
   m_IncludeEnd = GetValueAsString(includeEnd, VT_BOOL);

   CString filter = GetValueAsString(sessionsFilter);
   if (filter == "31")
   {
      filter = "All sessions (31)";
   }
   else if (filter == "0")
   {
      filter = "Primary session only (0)";
   }
   m_SessionsFilter = filter;

   m_Continuation = CHelper::ContinuationTypeToString(continuation);
   m_EqualizeCloses = GetValueAsString(equalizeCloses, VT_BOOL);
   m_DaysBeforeExpiration = GetValueAsString(daysBeforeExpiration);
   m_SessionFlags = CHelper::SessionFlagToString(sessionFlags);
   m_SubscriptionLevel = CHelper::SubscriptionLevelToString(subscriptionLevel);
   if (subscriptionLevel == tslEndOfBarAndPeriod)
   {
      m_RecalcPeriod = GetValueAsString(recalcPeriod);
   }
   
   UpdateData(FALSE);

   // Get error from custom study collection and show it if exist
   ATL::CComPtr<ICQGError> spError;
   hr = m_spCustomStudy->get_LastError(&spError);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);

   if (spError)
   {
      ATL::CComBSTR errDescription;
      hr = spError->get_Description(&errDescription);
      Common::ThrowOnCOMError(spError, hr);

      m_lblError.SetWindowText(ATL::CW2T(errDescription));
   }
}

void CCustomStudyDlg::ShowReqDefinitionParameters(
                        const ATL::CComPtr<ICQGCustomStudyRequest>& spCustomStudyRequest)
{
   // Get definition from request
   ATL::CComPtr<ICQGCustomStudyDefinition> spCustomStudyDefinition;
   HRESULT hr = spCustomStudyRequest->get_Definition(&spCustomStudyDefinition);
   Common::ThrowOnCOMError(spCustomStudyRequest, hr);

   ATL::CComBSTR definitionName;
   hr = spCustomStudyDefinition->get_Name(&definitionName);
   Common::ThrowOnCOMError(spCustomStudyDefinition, hr);

   m_DefinitionName = ATL::CW2T(definitionName);

   // Fill up parameters in the list control
   m_lvParameters.DeleteAllItems();

   ATL::CComPtr<ICQGParameterDefinitions> spParameterDefinitions;
   hr = spCustomStudyDefinition->get_ParameterDefinitions(&spParameterDefinitions);
   Common::ThrowOnCOMError(spCustomStudyDefinition, hr);

   long count;
   hr = spParameterDefinitions->get_Count(&count);
   Common::ThrowOnCOMError(spParameterDefinitions, hr);

   for (long i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGParameterDefinition> spParameterDefinition;
      hr = spParameterDefinitions->get_Item(i, &spParameterDefinition);
      Common::ThrowOnCOMError(spParameterDefinitions, hr);

      ATL::CComBSTR name;
      hr = spParameterDefinition->get_Name(&name);
      Common::ThrowOnCOMError(spCustomStudyDefinition, hr);

      ATL::CComVariant value;
      hr = spCustomStudyRequest->get_Parameter(ATL::CComVariant(name), &value);
      Common::ThrowOnCOMError(spCustomStudyRequest, hr);

      const int row = m_lvParameters.InsertItem(m_lvParameters.GetItemCount(), _T(""));
      m_lvParameters.SetItemText(row, pcName, ATL::CW2T(name));
      m_lvParameters.SetItemText(row, pcValue,
                  Common::VariantToString(value, CHelper::PARAM_FRAC_PART_LENGTH));
   }
}

void CCustomStudyDlg::ShowOutputParams()
{
   UpdateData();

   // Get output parameters from custom study collection and show them
   DATE startTimestamp;
   HRESULT hr = m_spCustomStudy->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);

   DATE endTimestamp;
   hr = m_spCustomStudy->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);

   m_OutputStart = GetValueAsString(startTimestamp, VT_DATE);
   m_OutputEnd = GetValueAsString(endTimestamp, VT_DATE);

   UpdateData(FALSE);
}

void CCustomStudyDlg::ShowHeaders(
                        CListCtrl& listCtrl)
{
   ATL::CComPtr<ICQGCurveHeaders> spCurveHeaders;
   HRESULT hr = m_spCustomStudy->get_CurveHeaders(&spCurveHeaders);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);

   long count;
   hr = spCurveHeaders->get_Count(&count);
   Common::ThrowOnCOMError(spCurveHeaders, hr);

   for(int i = 0; i < count; ++i)
   {
      ATL::CComBSTR header;
      hr = spCurveHeaders->get_Item(i, &header);
      Common::ThrowOnCOMError(spCurveHeaders, hr);

      listCtrl.InsertColumn(rcCurvesStart + i, header, LVCFMT_LEFT, 60);
   }
}

void CCustomStudyDlg::ShowRecords(
                        long startIndex,
                        long endIndex,
                        CListCtrl& listCtrl)
{
   for (long i = startIndex; i <= endIndex; ++i)
   {
      AddRecord(i, listCtrl);
   }
}

void CCustomStudyDlg::AddRecord(
                        long recordIndex,
                        CListCtrl& listCtrl)
{
   // Add empty row
   const int row = listCtrl.InsertItem(listCtrl.GetItemCount(), _T(""));

   ShowRecord(recordIndex, listCtrl, row);
}

void CCustomStudyDlg::ShowRecord(
                        long recordIndex,
                        CListCtrl& listCtrl,
                        long row)
{
   // Get headers count
   ATL::CComPtr<ICQGCurves> spCurves;
   HRESULT hr = m_spCustomStudy->get_Item(recordIndex - 1, &spCurves);
   Common::ThrowOnCOMError(m_spCustomStudy, hr); 

   long count;
   hr = spCurves->get_Count(&count);
   Common::ThrowOnCOMError(spCurves, hr); 

   // Get CustomStudyResult timestamp to show it
   DATE timestamp;
   hr = spCurves->get_Timestamp(&timestamp);
   Common::ThrowOnCOMError(spCurves, hr);

   listCtrl.SetItemText(row, rcIndex, GetValueAsString(recordIndex));
   listCtrl.SetItemText(row, rcTimestamp, GetValueAsString(timestamp, VT_DATE, true));

   for (int i = 0; i < count; ++i)
   {
      double value;
      hr = spCurves->get_Item(ATL::CComVariant(i), &value);
      Common::ThrowOnCOMError(spCurves, hr); 

      listCtrl.SetItemText(row, rcCurvesStart + i, GetValueAsString(value));
   }
}

void CCustomStudyDlg::ClearDatesAndIndices()
{
   m_edStart.SetValue(0);
   m_edEnd.SetValue(0);

   static const COleDateTime nullDate;

   m_dtpStart.SetTime(nullDate);
   m_dtpEnd.SetTime(nullDate);
}

void CCustomStudyDlg::FilterTypeChanged()
{
   UpdateData();

   FillDatesAndIndices();

   const int indicesShowStatus = (m_FilterType == ftByIndices ? SW_SHOW : SW_HIDE);
   m_edStart.ShowWindow(indicesShowStatus);
   m_spnStart.ShowWindow(indicesShowStatus);
   m_spnEnd.ShowWindow(indicesShowStatus);
   m_edEnd.ShowWindow(indicesShowStatus);

   const int datesShowStatus = (m_FilterType == ftByIndices ? SW_HIDE : SW_SHOW);
   m_dtpStart.ShowWindow(datesShowStatus);
   m_dtpEnd.ShowWindow(datesShowStatus);
}

void CCustomStudyDlg::FillDatesAndIndices()
{
   if (m_spCustomStudy == NULL)
   {
      return;
   }

   eRequestStatus status;
   HRESULT hr = m_spCustomStudy->get_Status(&status);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);

   if (status != rsSuccess)
   {
      return;
   }

   // Get start\end timestamps from custom study collection and fill them into start\end time controls
   DATE startTimestamp;
   hr = m_spCustomStudy->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);
   m_dtpStart.SetTime(startTimestamp);

   DATE endTimestamp;
   hr = m_spCustomStudy->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);
   m_dtpEnd.SetTime(endTimestamp);

   // Fill start and end indices controls.
   m_edStart.SetValue(1);

   long count;
   hr = m_spCustomStudy->get_Count(&count);
   Common::ThrowOnCOMError(m_spCustomStudy, hr);

   m_edEnd.SetValue(count);
}

void CCustomStudyDlg::EnableControls()
{
   bool isDataConnectionUp = false;
   HRESULT hr;
   VARIANT_BOOL isStarted = VARIANT_FALSE;

   if (m_spCQGCEL)
   {
      hr = m_spCQGCEL->get_IsStarted(&isStarted);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }
   
   if (isStarted)
   {
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      hr = m_spCQGCEL->get_Environment(&spEnvironment);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      eConnectionStatus dataConnectionStatus;
      hr = spEnvironment->get_DataConnectionStatus(&dataConnectionStatus);
      Common::ThrowOnCOMError(spEnvironment, hr);

      isDataConnectionUp = (dataConnectionStatus == csConnectionUp);
   }

   m_btnRequestDefinition.EnableWindow(isDataConnectionUp);
   m_btnRequest.EnableWindow(isDataConnectionUp);

   const bool isCustomStudySelected = (m_spCustomStudy != NULL);

   m_btnRemove.EnableWindow(isCustomStudySelected && isDataConnectionUp);
   m_btnRemoveAll.EnableWindow(isCustomStudySelected && isDataConnectionUp);

   m_edTagName.EnableWindow(isCustomStudySelected);
   m_edTagValue.EnableWindow(isCustomStudySelected);

   bool isCustomStudySucceeded = false;
   if (isCustomStudySelected)
   {
      eRequestStatus status;
      hr = m_spCustomStudy->get_Status(&status);
      Common::ThrowOnCOMError(m_spCustomStudy, hr);

      isCustomStudySucceeded = (status == rsSuccess);
   }

   m_rdIndices.EnableWindow(isCustomStudySucceeded);
   m_rdDates.EnableWindow(isCustomStudySucceeded);

   m_edStart.EnableWindow(isCustomStudySucceeded);
   m_edEnd.EnableWindow(isCustomStudySucceeded);
   m_spnStart.EnableWindow(isCustomStudySucceeded);
   m_spnEnd.EnableWindow(isCustomStudySucceeded);
   m_dtpStart.EnableWindow(isCustomStudySucceeded);
   m_dtpEnd.EnableWindow(isCustomStudySucceeded);

   m_btnShow.EnableWindow(isCustomStudySucceeded);
}

CString CCustomStudyDlg::GetValueAsString(
                        ATL::CComVariant value,
                        VARENUM valueType/*= VT_EMPTY*/,
                        bool withMilliseconds /*= false*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return Common::VariantToString(value, withMilliseconds);
   }
   return CHelper::N_A;
}