// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "CustomStudy.h"
#include "CustomStudyDlg.h"

STDMETHODIMP CCustomStudyDlg::OnDataError(
                              LPDISPATCH /*obj*/,
                              BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CCustomStudyDlg::OnDataConnectionStatusChanged(
                              eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      EnableControls();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}


STDMETHODIMP CCustomStudyDlg::OnCustomStudyDefinitionsResolved(
                              ICQGCustomStudyDefinitions* cqg_customStudy_definitions, 
                              ICQGError* cqg_error)
{
   try
   {
      // Raise error, when any error occurred during request
      VARIANT_BOOL isValid;
      HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &isValid);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      if (isValid)
      {
         ATL::CComBSTR errDescription;
         hr = cqg_error->get_Description(&errDescription);
         Common::ThrowOnCOMError<ICQGError>(cqg_error, hr);

         AfxMessageBox(ATL::CW2T(errDescription), MB_ICONEXCLAMATION);

         return S_OK;
      }

      m_spCustomStudyDefinitions.Release();
      m_spCustomStudyDefinitions = cqg_customStudy_definitions;

      // Fill up new data
      m_cmbDefinition.ResetContent();

      long count;
      hr = cqg_customStudy_definitions->get_Count(&count);
      Common::ThrowOnCOMError<ICQGCustomStudyDefinitions>(cqg_customStudy_definitions, hr);

      if (count <= 0)
      {
         return S_OK;
      }

      for (long i = 0; i < count; ++i)
      {
         ATL::CComPtr<ICQGCustomStudyDefinition> spCustomStudyDefinition;
         hr = cqg_customStudy_definitions->get_Item(i, &spCustomStudyDefinition);
         Common::ThrowOnCOMError<ICQGCustomStudyDefinitions>(cqg_customStudy_definitions, hr);

         ATL::CComBSTR name;
         hr = spCustomStudyDefinition->get_Name(&name);
         Common::ThrowOnCOMError(spCustomStudyDefinition, hr);

         const int row = m_cmbDefinition.AddString(ATL::CW2T(name));
         m_cmbDefinition.SetItemData(row, i);
      }

      // Select first definition
      m_cmbDefinition.SetCurSel(0);
      OnCbnSelchangeCmbDefinition();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CCustomStudyDlg::OnCustomStudyResolved(
                              ICQGCustomStudy* cqg_customStudy,
                              ICQGError* /*cqg_error*/)
{
   try
   {
      if (!m_spCustomStudy)
      {
         return S_OK;
      }

      // Get selected custom study ID
      ATL::CComBSTR selCustomStudyID;
      HRESULT hr = m_spCustomStudy->get_Id(&selCustomStudyID);
      Common::ThrowOnCOMError(m_spCustomStudy, hr);

      // Get changed custom study ID
      ATL::CComBSTR customStudyID;
      hr = cqg_customStudy->get_Id(&customStudyID);
      Common::ThrowOnCOMError<ICQGCustomStudy>(cqg_customStudy, hr);

      // Check if custom study resolved in the selected request
      if (selCustomStudyID == customStudyID)
      {
         ClearAllData();
         ShowAllData();

         EnableControls();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CCustomStudyDlg::OnCustomStudyAdded(
                              ICQGCustomStudy* cqg_customStudy)
{
   try
   {
      if (!m_spCustomStudy)
      {
         return S_OK;
      }

      // Get selected custom study ID
      ATL::CComBSTR selCustomStudyID;
      HRESULT hr = m_spCustomStudy->get_Id(&selCustomStudyID);
      Common::ThrowOnCOMError(m_spCustomStudy, hr);

      // Get changed custom study ID
      ATL::CComBSTR customStudyID;
      hr = cqg_customStudy->get_Id(&customStudyID);
      Common::ThrowOnCOMError<ICQGCustomStudy>(cqg_customStudy, hr);

      // Check if record added in the selected request
      if (selCustomStudyID == customStudyID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CCustomStudyDlg::OnCustomStudyInserted(
                              ICQGCustomStudy* cqg_customStudy,
                              long /*index_*/)
{
   try
   {
      if (!m_spCustomStudy)
      {
         return S_OK;
      }

      // Get selected custom study ID
      ATL::CComBSTR selCustomStudyID;
      HRESULT hr = m_spCustomStudy->get_Id(&selCustomStudyID);
      Common::ThrowOnCOMError(m_spCustomStudy, hr);

      // Get changed custom study ID
      ATL::CComBSTR customStudyID;
      hr = cqg_customStudy->get_Id(&customStudyID);
      Common::ThrowOnCOMError<ICQGCustomStudy>(cqg_customStudy, hr);

      // Check if record inserted in the selected request
      if (selCustomStudyID == customStudyID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CCustomStudyDlg::OnCustomStudyRemoved(
                              ICQGCustomStudy* cqg_customStudy,
                              long /*index_*/)
{
   try
   {
      if (!m_spCustomStudy)
      {
         return S_OK;
      }

      // Get selected custom study ID
      ATL::CComBSTR selCustomStudyID;
      HRESULT hr = m_spCustomStudy->get_Id(&selCustomStudyID);
      Common::ThrowOnCOMError(m_spCustomStudy, hr);

      // Get changed custom study ID
      ATL::CComBSTR customStudyID;
      hr = cqg_customStudy->get_Id(&customStudyID);
      Common::ThrowOnCOMError<ICQGCustomStudy>(cqg_customStudy, hr);

      // Check if record removed from the selected request
      if (selCustomStudyID == customStudyID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CCustomStudyDlg::OnCustomStudyUpdated(
                              ICQGCustomStudy* cqg_customStudy,
                              long index_)
{
   try
   {
      if (!m_spCustomStudy)
      {
         return S_OK;
      }

      // Get selected custom study ID
      ATL::CComBSTR selCustomStudyID;
      HRESULT hr = m_spCustomStudy->get_Id(&selCustomStudyID);
      Common::ThrowOnCOMError(m_spCustomStudy, hr);

      // Get changed custom study ID
      ATL::CComBSTR customStudyID;
      hr = cqg_customStudy->get_Id(&customStudyID);
      Common::ThrowOnCOMError<ICQGCustomStudy>(cqg_customStudy, hr);

      // Check if record updated in the selected request
      if (selCustomStudyID == customStudyID)
      {
         // Find row for specified custom study result
         CString indexStr;
         indexStr.Format(_T("%d"), index_ + 1);
         LVFINDINFO findInfo;
         findInfo.flags = LVFI_STRING;
         findInfo.psz = indexStr;
         const int row = m_lvLastCustomStudy.FindItem(&findInfo);

         // If found change it's properties
         if (row > -1)
         {
            ShowRecord(index_ + 1, m_lvLastCustomStudy, row);
         }
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}