// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "DataSources.h"
#include "DataSourcesDlg.h"

STDMETHODIMP CDataSourcesDlg::OnDataError(
                           LPDISPATCH /*obj*/,
                           BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CDataSourcesDlg::OnDataConnectionStatusChanged(
                           eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      EnableControls();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CDataSourcesDlg::OnDataSourcesResolved(
                           ICQGDataSources* cqg_data_sources,
                           ICQGError* cqg_error)
{
   try
   {
      // Clearing data sources and data source symbols
      ClearDataSources();

      // Raise error, when any error occurred during request
      VARIANT_BOOL isValid;
      HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &isValid);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
      
      if (isValid)
      {
         ATL::CComBSTR errDescription;
         hr = cqg_error->get_Description(&errDescription);
         Common::ThrowOnCOMError<ICQGError>(cqg_error, hr);

         AfxMessageBox(ATL::CW2T(errDescription), MB_ICONEXCLAMATION);

         return S_OK;
      }

      // Add resolved data sources' abbreviations to the combo box
      m_spDataSources = cqg_data_sources;

      long count;
      hr = m_spDataSources->get_Count(&count);
      Common::ThrowOnCOMError(m_spDataSources, hr);

      if (count > 0)
      {
         for (long i = 0; i < count; ++i)
         {
            ATL::CComPtr<ICQGDataSource> spDataSource;
            hr = m_spDataSources->get_Item(i, &spDataSource);
            Common::ThrowOnCOMError(m_spDataSources, hr);

            ATL::CComBSTR abbreviation;
            hr = spDataSource->get_Abbreviation(&abbreviation);
            Common::ThrowOnCOMError(spDataSource, hr);

            int row = m_cmbDSAbbreviation.AddString(ATL::CW2T(abbreviation));
            m_cmbDSAbbreviation.SetItemData(row, i);
         }

         // Select first item
         m_cmbDSAbbreviation.SetCurSel(0);
         OnCbnSelchangeCmbDsAbbreviation();

         // Changing buttons enablements
         EnableControls();
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}


STDMETHODIMP CDataSourcesDlg::OnDataSourceSymbolsResolved(
                           BSTR data_source_abbreviation,
                           ICQGDataSourceSymbols* cqg_data_source_symbols,
                           ICQGError* cqg_error)
{
   try
   {
      // Clearing data source symbols
      ClearDataSourceSymbols();

      // Raise error, when any error occurred during request
      VARIANT_BOOL isValid;
      HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &isValid);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      if (isValid)
      {
         ATL::CComBSTR errDescription;
         hr = cqg_error->get_Description(&errDescription);
         Common::ThrowOnCOMError<ICQGError>(cqg_error, hr);

         AfxMessageBox(ATL::CW2T(errDescription), MB_ICONEXCLAMATION);

         return S_OK;
      }

      // Add resolved data source symbols' abbreviations to the combo box
      m_spDataSourceSymbols = cqg_data_source_symbols;

      long count;
      hr = m_spDataSourceSymbols->get_Count(&count);
      Common::ThrowOnCOMError(m_spDataSourceSymbols, hr);

      if (count > 0)
      {
         for (long i = 0; i < count; ++i)
         {
            ATL::CComPtr<ICQGDataSourceSymbol> spDataSourceSymbol;
            hr = m_spDataSourceSymbols->get_Item(i, &spDataSourceSymbol);
            Common::ThrowOnCOMError(m_spDataSourceSymbols, hr);

            ATL::CComBSTR abbreviation;
            hr = spDataSourceSymbol->get_Abbreviation(&abbreviation);
            Common::ThrowOnCOMError(spDataSourceSymbol, hr);

            int row = m_cmbDSSAbbreviation.AddString(ATL::CW2T(abbreviation));
            m_cmbDSSAbbreviation.SetItemData(row, i);
         }

         // Select first item
         m_cmbDSSAbbreviation.SetCurSel(0);
         OnCbnSelchangeCmbDssAbbreviation();

         m_lblDSSDSAbbreviation.SetWindowText(ATL::CW2T(data_source_abbreviation));
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}
