// DataSourcesDlg.cpp : implementation file
//

#include "stdafx.h"
#include <cassert>
#include "Common.h"
#include "Helper.h"
#include "DataSources.h"
#include "DataSourcesDlg.h"

CDataSourcesDlg::CDataSourcesDlg(CWnd* pParent /*=NULL*/)
                           : CDialog(CDataSourcesDlg::IDD, pParent)
{
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CDataSourcesDlg::DoDataExchange(
                           CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_LBL_DATA_CONNECTION, m_lblDataConnection);
   DDX_Control(pDX, IDC_BTN_REQUEST_DS, m_btnRequestDS);
   DDX_Control(pDX, IDC_BTN_REQUEST_DSS, m_btnRequestDSS);
   DDX_Control(pDX, IDC_ED_DS_ABBREVIATION, m_edDSAbbreviation);
   DDX_Control(pDX, IDC_CMB_DS_ABBREVIATION, m_cmbDSAbbreviation);
   DDX_Control(pDX, IDC_LBL_DS_STATUS, m_lblDSStatus);
   DDX_Control(pDX, IDC_LBL_DS_NAME, m_lblDSName);
   DDX_Control(pDX, IDC_LBL_DS_SYMBOL_TYPES, m_lblDSSymbolTypes);
   DDX_Control(pDX, IDC_CMB_DSS_ABBREVIATION, m_cmbDSSAbbreviation);
   DDX_Control(pDX, IDC_LBL_DSS_TYPE, m_lblDSSType);
   DDX_Control(pDX, IDC_LBL_DSS_DESCRIPTION, m_lblDSSDescription);
   DDX_Control(pDX, IDC_LBL_DSS_DS_ABBREVIATION, m_lblDSSDSAbbreviation);
}

BEGIN_MESSAGE_MAP(CDataSourcesDlg, CDialog)
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_WM_DESTROY()
   ON_EN_CHANGE(IDC_ED_DS_ABBREVIATION, OnEnChangeEdDsAbbreviation)
   ON_BN_CLICKED(IDC_BTN_REQUEST_DS, OnBnClickedBtnRequestDs)
   ON_BN_CLICKED(IDC_BTN_REQUEST_DSS, OnBnClickedBtnRequestDss)
   ON_CBN_SELCHANGE(IDC_CMB_DS_ABBREVIATION, OnCbnSelchangeCmbDsAbbreviation)
   ON_CBN_SELCHANGE(IDC_CMB_DSS_ABBREVIATION, OnCbnSelchangeCmbDssAbbreviation)
END_MESSAGE_MAP()

BOOL CDataSourcesDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      // Set the icon for this dialog.  The framework does this automatically
      //  when the application's main window is not a dialog
      SetIcon(m_hIcon, TRUE);       // Set big icon
      SetIcon(m_hIcon, FALSE);      // Set small icon

      InitializeCQGCEL();
      InitializeControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
}

void CDataSourcesDlg::OnPaint() 
{
   if (IsIconic())
   {
      CPaintDC dc(this); // device context for painting

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      // Center icon in client rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      // Draw the icon
      dc.DrawIcon(x, y, m_hIcon);
   }
   else
   {
      CDialog::OnPaint();
   }
}

HCURSOR CDataSourcesDlg::OnQueryDragIcon()
{
   return static_cast<HCURSOR>(m_hIcon);
}

void CDataSourcesDlg::OnOK()
{
   // Override to prevent dialog from closing on Enter
}

void CDataSourcesDlg::OnDestroy()
{
   try
   {
      HRESULT hr = ICQGCELDispEventImpl::DispEventUnadvise(m_spCQGCEL);
      assert(SUCCEEDED(hr));

      hr = m_spCQGCEL->Shutdown();
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}

void CDataSourcesDlg::OnEnChangeEdDsAbbreviation()
{
   try
   {
      EnableControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CDataSourcesDlg::OnBnClickedBtnRequestDs()
{
   try
   {
      // Clearing data sources and data source symbols
      ClearDataSources();

      // Requesting new data sources
      HRESULT hr = m_spCQGCEL->RequestDataSources();
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      // Changing buttons enablements
      EnableControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CDataSourcesDlg::OnBnClickedBtnRequestDss()
{
   try
   {
      // Clearing data source symbols
      ClearDataSourceSymbols();
      
      // Get data source abbreviation
      CString dsAbbreviation;
      m_edDSAbbreviation.GetWindowText(dsAbbreviation);

      // Requesting specified data source symbols
      HRESULT hr = m_spCQGCEL->RequestDataSourceSymbols(ATL::CT2W(dsAbbreviation));
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CDataSourcesDlg::OnCbnSelchangeCmbDsAbbreviation()
{
   try
   {
      ClearDataSource();
      
      long idx = static_cast<long>(m_cmbDSAbbreviation.GetItemData(m_cmbDSAbbreviation.GetCurSel()));

      // Get selected data source
      ATL::CComPtr<ICQGDataSource> spDataSource;
      HRESULT hr = m_spDataSources->get_Item(idx, &spDataSource);
      Common::ThrowOnCOMError(m_spDataSources, hr);

      // Get data source's properties and show them
      ATL::CComBSTR name;
      hr = spDataSource->get_Name(&name);
      Common::ThrowOnCOMError(spDataSource, hr);

      eDataSourceStatus dataSourceStatus;
      hr = spDataSource->get_Status(&dataSourceStatus);
      Common::ThrowOnCOMError(spDataSource, hr);

      eSymbolType symbolTypes;
      hr = spDataSource->get_SymbolTypes(&symbolTypes);
      Common::ThrowOnCOMError(spDataSource, hr);

      m_lblDSName.SetWindowText(ATL::CW2T(name));
      m_lblDSStatus.SetWindowText(CHelper::DataSourceStatusToString(dataSourceStatus));
      m_lblDSSymbolTypes.SetWindowText(CHelper::SymbolTypeToString(symbolTypes));

      // Changing text in the text box
      ATL::CComBSTR abbreviation;
      hr = spDataSource->get_Abbreviation(&abbreviation);
      Common::ThrowOnCOMError(spDataSource, hr);

      m_edDSAbbreviation.SetWindowText(ATL::CW2T(abbreviation));

      ClearDataSourceSymbols();

      // Changing buttons enablements
      EnableControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CDataSourcesDlg::OnCbnSelchangeCmbDssAbbreviation()
{
   try
   {
      ClearDataSourceSymbol();
      
      long idx = static_cast<long>(m_cmbDSSAbbreviation.GetItemData(m_cmbDSSAbbreviation.GetCurSel()));

      // Get selected data source symbol
      ATL::CComPtr<ICQGDataSourceSymbol> spDataSourceSymbol;
      HRESULT hr = m_spDataSourceSymbols->get_Item(idx, &spDataSourceSymbol);
      Common::ThrowOnCOMError(m_spDataSourceSymbols, hr);

      // Get data source symbol's properties and show them
      ATL::CComBSTR description;
      hr = spDataSourceSymbol->get_Description(&description);
      Common::ThrowOnCOMError(spDataSourceSymbol, hr);

      eSymbolType symbolType;
      hr = spDataSourceSymbol->get_Type(&symbolType);
      Common::ThrowOnCOMError(spDataSourceSymbol, hr);

      m_lblDSSDescription.SetWindowText(ATL::CW2T(description));
      m_lblDSSType.SetWindowText(CHelper::SymbolTypeToString(symbolType));
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}
