/*
* =====================================================================
*  Filename: DataSourcesDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved.
* =====================================================================
*/

#pragma once

#include "afxwin.h"
#include "afxcmn.h"
#include "afxdtctl.h"

class CDataSourcesDlg;

/**
\typedef ATL::IDispEventImpl<1, CDataSourcesDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CDataSourcesDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

// CDataSourcesDlg dialog
class CDataSourcesDlg : public CDialog, public ICQGCELDispEventImpl
{

public:

   /// \brief CDataSourcesDlg constructor.
   /// \param pParent - parent window
   explicit CDataSourcesDlg(
                  CWnd* pParent = NULL); 

   /// \enum Dialog Data
   enum { IDD = IDD_DATA_SOURCES_DIALOG };

private:

   /// \brief Dialog icon.
   HICON m_hIcon;

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
                  CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter
   virtual void OnOK();

private:

   /// \brief Prepares the container for painting.
   afx_msg void OnPaint();

   /// \brief The system calls this function to obtain the cursor to
   /// \brief display while the user drags the minimized window.
   afx_msg HCURSOR OnQueryDragIcon();

   /// \brief Used to unadvise from CQGCEL events and shutdown it.
   afx_msg void OnDestroy();

   /// \brief Changes data source symbols query button enablement.
   afx_msg void OnEnChangeEdDsAbbreviation();

   /// \brief Clears data sources and data source symbols, and requests the new ones.
   afx_msg void OnBnClickedBtnRequestDs();

   /// \brief Clears data source symbols and requests the new ones.
   afx_msg void OnBnClickedBtnRequestDss();

   /// \brief Shows all necessary data sources information.
   afx_msg void OnCbnSelchangeCmbDsAbbreviation();

   /// \brief Shows all necessary data source symbols information.
   afx_msg void OnCbnSelchangeCmbDssAbbreviation();

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CDataSourcesDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000000a, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000003, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000012, OnDataSourcesResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000013, OnDataSourceSymbolsResolved)
   END_SINK_MAP()

private:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - string describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                  LPDISPATCH obj,
                  BSTR errorDescription);

   /// \brief Fired when some changes occurred in the connection with the CQG data server.
   /// \param newStatus - new connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when list of data sources is resolved or when some
   /// \brief error occurred during data sources request processing
   /// \param cqg_data_sources - Collection of resolved data sources
   /// \param cqg_error - CQGError object describing last error occurred during processing the request
   /// \return S_OK
   STDMETHOD(OnDataSourcesResolved)(
                  ICQGDataSources* cqg_data_sources,
                  ICQGError* cqg_error);

   /// \brief Fired when list of symbols in given data source is resolved or when some
   /// \brief error occurred during data source symbols request processing
   /// \param data_source_abbreviation - Abbreviation of the data source symbols of which are resolved
   /// \param cqg_data_source_symbols - Collection of resolved data source symbols
   /// \param cqg_error - CQGError object describing last error occurred during processing the request
   /// \return S_OK
   STDMETHOD(OnDataSourceSymbolsResolved)(
                  BSTR data_source_abbreviation,
                  ICQGDataSourceSymbols* cqg_data_source_symbols,
                  ICQGError* cqg_error);

private:

   /// \brief Initializes CQGCEL object, and starts the CQGCEL.
   void InitializeCQGCEL();

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Enables/disables controls on the form based on connection status and other parameters.
   void EnableControls();

   /// \brief Clears all data sources.
   void ClearDataSources();

   /// \brief Clears data source information.
   void ClearDataSource();

   /// \brief Clears data source all symbols.
   void ClearDataSourceSymbols();

   /// \brief Clears data source symbol information.
   void ClearDataSourceSymbol();

private:

   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Selected data sources object.
   ATL::CComPtr<ICQGDataSources> m_spDataSources;

   /// \brief Selected data source symbols object.
   ATL::CComPtr<ICQGDataSourceSymbols> m_spDataSourceSymbols;

private:

   /// \brief Data connection indicator Label control.
   CStatic m_lblDataConnection;

   /// \brief Request data sources Button control.
   CButton m_btnRequestDS;

   /// \brief Request data source symbols Button control.
   CButton m_btnRequestDSS;

   /// \brief Data source abbreviation Edit control.
   CEdit m_edDSAbbreviation;

   /// \brief Data source abbreviations ComboBox control.
   CComboBox m_cmbDSAbbreviation;

   /// \brief Data source status Label control.
   CStatic m_lblDSStatus;

   /// \brief Data source name Label control.
   CStatic m_lblDSName;

   /// \brief Data source symbol types Label control.
   CStatic m_lblDSSymbolTypes;

   /// \brief Data source symbols abbreviation ComboBox control.
   CComboBox m_cmbDSSAbbreviation;

   /// \brief Data source symbols type Label control.
   CStatic m_lblDSSType;
   
   /// \brief Data source symbols description Label control.
   CStatic m_lblDSSDescription;

   /// \brief Data source symbols data source abbreviation Label control.
   CStatic m_lblDSSDSAbbreviation;
};
