// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "Expression.h"
#include "ExpressionDlg.h"

void CExpressionDlg::InitializeCQGCEL()
{
   // Create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   Common::ThrowOnCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   Common::ThrowOnCOMError(m_spCQGCEL, hr);
}

void CExpressionDlg::InitializeControls()
{
   InitializeList(m_lvLastExpression);
   InitializeList(m_lvExpression);

   InitializeSubExpressionsList();

   ClearAllData();

   OnDataConnectionStatusChanged(csConnectionDown);

   m_dtpStart.SetFormat(CHelper::DateFormat);
   m_dtpEnd.SetFormat(CHelper::DateFormat);

   UpdateData();
   m_FilterType = ftByIndices;
   UpdateData(FALSE);
   OnBnClickedRdIndices();
}

void CExpressionDlg::InitializeList(
                        CListCtrl& listCtrl)
{
   listCtrl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   listCtrl.ModifyStyle(0, LVS_SINGLESEL);

   listCtrl.InsertColumn(rcIndex, _T("#"), LVCFMT_LEFT, 40);
   listCtrl.InsertColumn(rcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 150);
}

void CExpressionDlg::InitializeSubExpressionsList()
{
   m_lvSubExpressions.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);

   m_lvSubExpressions.InsertColumn(secSubExpression, _T("Sub Expression"), LVCFMT_LEFT, 170);
   m_lvSubExpressions.InsertColumn(secAlias, _T("Alias"), LVCFMT_LEFT, 170);
}

void CExpressionDlg::ClearAllData()
{
   m_lblStatus.SetWindowText(_T(""));

   ClearRequestParams();

   ClearTags();

   ClearOutputParams();
   ClearDatesAndIndices();

   ClearRecords(m_lvLastExpression);
   ClearRecords(m_lvExpression);

   ClearHeaders(m_lvLastExpression);
   ClearHeaders(m_lvExpression);

   m_lblAllRecsCount.SetWindowText(_T(""));
   m_lblFltrRecsCount.SetWindowText(_T(""));
}

void CExpressionDlg::ClearRequestParams()
{
   m_lvSubExpressions.DeleteAllItems();

   UpdateData();

   m_RangeStart = _T("");
   m_RangeEnd = _T("");
   m_IntradayPeriod = _T("");
   m_HistoricalPeriod = _T("");
   m_UpdatesEnabled = _T("");
   m_IncludeEnd = _T("");
   m_SessionsFilter = _T("");
   m_Continuation = _T("");
   m_EqualizeCloses = _T("");
   m_DaysBeforeExpiration = _T("");
   m_SessionFlags = _T("");
   m_SubscriptionLevel = _T("");
   m_RecalcPeriod = _T("");

   UpdateData(FALSE);

   m_lblError.SetWindowText(_T(""));
}

void CExpressionDlg::ClearOutputParams()
{
   UpdateData();

   m_OutputStart = _T("");
   m_OutputEnd = _T("");

   UpdateData(FALSE);
}

void CExpressionDlg::ClearTags()
{
   m_edTagName.SetWindowText(_T(""));
   m_edTagValue.SetWindowText(_T(""));
}

void CExpressionDlg::ClearHeaders(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();

   const int columnCount = listCtrl.GetHeaderCtrl()->GetItemCount() - rcOutputsStart;

   // Delete headers
   for (int i = 0; i < columnCount; ++i)
   {
      listCtrl.DeleteColumn(rcOutputsStart);
   }
}


void CExpressionDlg::ClearRecords(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();
}

void CExpressionDlg::ShowAllData()
{
   ShowRequestParams();

   eRequestStatus status;
   HRESULT hr = m_spExpression->get_Status(&status);
   Common::ThrowOnCOMError(m_spExpression, hr);

   m_lblStatus.SetWindowText(CHelper::RequestStatusToString(status));

   if (status == rsSuccess)
   {
      ShowOutputParams();

      FillDatesAndIndices();

      // Show headers
      ShowHeaders(m_lvLastExpression);
      ShowHeaders(m_lvExpression);

      long count;
      hr = m_spExpression->get_Count(&count);
      Common::ThrowOnCOMError(m_spExpression, hr);

      // Show records
      const long startIndex = max(count - m_ShowCount + 1, 1);
      const long endndex = count;
      ShowRecords(startIndex, endndex, m_lvLastExpression);

      // Show all records count
      m_lblAllRecsCount.SetWindowText(GetValueAsString(count));
   }
}

void CExpressionDlg::ShowChangedData()
{
   // Clear the list box
   ClearRecords(m_lvLastExpression);

   // Get records count
   long count;
   HRESULT hr = m_spExpression->get_Count(&count);
   Common::ThrowOnCOMError(m_spExpression, hr);

   // Show records
   const long startIndex = max(count - m_ShowCount + 1, 1);
   const long endIndex = count;
   ShowRecords(startIndex, endIndex, m_lvLastExpression);

   // Show all records count
   m_lblAllRecsCount.SetWindowText(GetValueAsString(count));

   // Show output parameters
   ShowOutputParams();
}

void CExpressionDlg::ShowRequestParams()
{
   // Get request from expression collection
   ATL::CComPtr<ICQGExpressionRequest> spRequest;
   HRESULT hr = m_spExpression->get_Request(&spRequest);
   Common::ThrowOnCOMError(m_spExpression, hr);

   // Get request parameters
   ATL::CComVariant rangeStart;
   hr = spRequest->get_RangeStart(&rangeStart);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeEnd;
   hr = spRequest->get_RangeEnd(&rangeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   long intradayPeriod;
   hr = spRequest->get_IntradayPeriod(&intradayPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   eHistoricalPeriod historicalPeriod;
   hr = spRequest->get_HistoricalPeriod(&historicalPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL updatesEnabled;
   hr = spRequest->get_UpdatesEnabled(&updatesEnabled);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL includeEnd;
   hr = spRequest->get_IncludeEnd(&includeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant sessionsFilter;
   hr = spRequest->get_SessionsFilter(&sessionsFilter);
   Common::ThrowOnCOMError(spRequest, hr);

   eTimeSeriesContinuationType continuation;
   hr = spRequest->get_Continuation(&continuation);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL equalizeCloses;
   hr = spRequest->get_EqualizeCloses(&equalizeCloses);
   Common::ThrowOnCOMError(spRequest, hr);

   long daysBeforeExpiration;
   hr = spRequest->get_DaysBeforeExpiration(&daysBeforeExpiration);
   Common::ThrowOnCOMError(spRequest, hr);

   eSessionFlag sessionFlags;
   hr = spRequest->get_SessionFlags(&sessionFlags);
   Common::ThrowOnCOMError(spRequest, hr);

   // Show sub expressions
   long count;
   hr = spRequest->get_SubExpressionCount(&count);
   Common::ThrowOnCOMError(spRequest, hr);

   for(int i = 0; i < count; ++i)
   {
      ATL::CComBSTR subExpression;
      hr = spRequest->get_SubExpression(i, &subExpression);
      Common::ThrowOnCOMError(spRequest, hr);

      ATL::CComBSTR alias;
      hr = spRequest->get_SubExpressionAlias(i, &alias);
      Common::ThrowOnCOMError(spRequest, hr);

      CString subExpressionStr;
      subExpressionStr.Format(_T("%s;  %s"), subExpression, alias);

      // Add sub expression to the list
      int row = m_lvSubExpressions.InsertItem(m_lvSubExpressions.GetItemCount(), _T(""));
      m_lvSubExpressions.SetItemText(row, secSubExpression, subExpression);
      m_lvSubExpressions.SetItemText(row, secAlias, alias);
   }
   
   eTimeSeriesSubscriptionLevel subscriptionLevel;
   hr = spRequest->get_SubscriptionLevel(&subscriptionLevel);
   Common::ThrowOnCOMError(spRequest, hr);
   
   long recalcPeriod;
   hr = spRequest->get_RecalcPeriod(&recalcPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   // Show request parameters
   UpdateData();

   m_RangeStart = GetValueAsString(rangeStart);
   m_RangeEnd = GetValueAsString(rangeEnd);
   m_IntradayPeriod = GetValueAsString(intradayPeriod);
   m_HistoricalPeriod = CHelper::HistoricalPeriodToString(historicalPeriod);
   m_UpdatesEnabled = GetValueAsString(updatesEnabled, VT_BOOL);
   m_IncludeEnd = GetValueAsString(includeEnd, VT_BOOL);
   
   CString filter = GetValueAsString(sessionsFilter);
   if (filter == "31")
   {
      filter = "All sessions (31)";
   }
   else if (filter == "0")
   {
      filter = "Primary session only (0)";
   }
   m_SessionsFilter = filter;

   m_Continuation = CHelper::ContinuationTypeToString(continuation);
   m_EqualizeCloses = GetValueAsString(equalizeCloses, VT_BOOL);
   m_DaysBeforeExpiration = GetValueAsString(daysBeforeExpiration);
   m_SessionFlags = CHelper::SessionFlagToString(sessionFlags);
   m_SubscriptionLevel = CHelper::SubscriptionLevelToString(subscriptionLevel);
   if (subscriptionLevel == tslEndOfBarAndPeriod)
   {
      m_RecalcPeriod = GetValueAsString(recalcPeriod);
   }
   
   UpdateData(FALSE);

   // Get error from expression collection and show it if exist
   ATL::CComPtr<ICQGError> spError;
   hr = m_spExpression->get_LastError(&spError);
   Common::ThrowOnCOMError(m_spExpression, hr);

   if (spError)
   {
      ATL::CComBSTR errDescription;
      hr = spError->get_Description(&errDescription);
      Common::ThrowOnCOMError(spError, hr);

      m_lblError.SetWindowText(ATL::CW2T(errDescription));
   }
}

void CExpressionDlg::ShowOutputParams()
{
   UpdateData();

   // Get output parameters from expression collection and show them
   DATE startTimestamp;
   HRESULT hr = m_spExpression->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spExpression, hr);

   DATE endTimestamp;
   hr = m_spExpression->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spExpression, hr);

   m_OutputStart = GetValueAsString(startTimestamp, VT_DATE);
   m_OutputEnd = GetValueAsString(endTimestamp, VT_DATE);

   UpdateData(FALSE);
}

void CExpressionDlg::ShowHeaders(
                        CListCtrl& listCtrl)
{
   ATL::CComPtr<ICQGExpressionOutputHeaders> spExpressionOutputHeaders;
   HRESULT hr = m_spExpression->get_OutputHeaders(&spExpressionOutputHeaders);
   Common::ThrowOnCOMError(m_spExpression, hr);

   long count;
   hr = spExpressionOutputHeaders->get_Count(&count);
   Common::ThrowOnCOMError(spExpressionOutputHeaders, hr);

   for(int i = 0; i < count; ++i)
   {
      ATL::CComBSTR header;
      hr = spExpressionOutputHeaders->get_Item(i, &header);
      Common::ThrowOnCOMError(spExpressionOutputHeaders, hr);

      listCtrl.InsertColumn(rcOutputsStart + i, header, LVCFMT_LEFT, 60);
   }
}

void CExpressionDlg::ShowRecords(
                        long startIndex,
                        long endIndex,
                        CListCtrl& listCtrl)
{
   for (long i = startIndex; i <= endIndex; ++i)
   {
      AddRecord(i, listCtrl);
   }
}

void CExpressionDlg::AddRecord(
                        long recordIndex,
                        CListCtrl& listCtrl)
{
   // Add empty row
   int row = listCtrl.InsertItem(listCtrl.GetItemCount(), _T(""));

   ShowRecord(recordIndex, listCtrl, row);
}

void CExpressionDlg::ShowRecord(
                        long recordIndex,
                        CListCtrl& listCtrl,
                        long row)
{
   // Get specified ExpressionOutputs from expression collection by its index
   ATL::CComPtr<ICQGExpressionOutputs> spExpressionOutputs;
   HRESULT hr = m_spExpression->get_Item(recordIndex - 1, &spExpressionOutputs);
   Common::ThrowOnCOMError(m_spExpression, hr);

   // Get ExpressionOutputs timestamp to show it
   DATE timestamp;
   hr = spExpressionOutputs->get_Timestamp(&timestamp);
   Common::ThrowOnCOMError(spExpressionOutputs, hr);

   listCtrl.SetItemText(row, rcIndex, GetValueAsString(recordIndex));
   listCtrl.SetItemText(row, rcTimestamp, GetValueAsString(timestamp, VT_DATE, true));

   // Show parameters into given listbox
   long count;
   hr = spExpressionOutputs->get_Count(&count);
   Common::ThrowOnCOMError(spExpressionOutputs, hr);
   
   for(int i = 0; i < count; ++i)
   {
      double value;
      hr = spExpressionOutputs->get_Item(ATL::CComVariant(i), &value);
      Common::ThrowOnCOMError(spExpressionOutputs, hr);

      listCtrl.SetItemText(row, rcOutputsStart + i, GetValueAsString(value));
   }
}

void CExpressionDlg::ClearDatesAndIndices()
{
   m_edStart.SetValue(0);
   m_edEnd.SetValue(0);

   static const COleDateTime nullDate;

   m_dtpStart.SetTime(nullDate);
   m_dtpEnd.SetTime(nullDate);
}

void CExpressionDlg::FilterTypeChanged()
{
   UpdateData();

   FillDatesAndIndices();

   int indicesShowStatus = (m_FilterType == ftByIndices ? SW_SHOW : SW_HIDE);
   m_edStart.ShowWindow(indicesShowStatus);
   m_spnStart.ShowWindow(indicesShowStatus);
   m_spnEnd.ShowWindow(indicesShowStatus);
   m_edEnd.ShowWindow(indicesShowStatus);

   int datesShowStatus = (m_FilterType == ftByIndices ? SW_HIDE : SW_SHOW);
   m_dtpStart.ShowWindow(datesShowStatus);
   m_dtpEnd.ShowWindow(datesShowStatus);
}

void CExpressionDlg::FillDatesAndIndices()
{
   if (m_spExpression == NULL)
   {
      return;
   }

   eRequestStatus status;
   HRESULT hr = m_spExpression->get_Status(&status);
   Common::ThrowOnCOMError(m_spExpression, hr);

   if (status != rsSuccess)
   {
      return;
   }

   // Get start\end timestamps from expression collection and fill them into start\end time controls
   DATE startTimestamp;
   hr = m_spExpression->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spExpression, hr);
   m_dtpStart.SetTime(startTimestamp);

   DATE endTimestamp;
   hr = m_spExpression->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spExpression, hr);
   m_dtpEnd.SetTime(endTimestamp);

   // Fill start and end indices controls.
   m_edStart.SetValue(1);

   long count;
   hr = m_spExpression->get_Count(&count);
   Common::ThrowOnCOMError(m_spExpression, hr);

   m_edEnd.SetValue(count);
}

void CExpressionDlg::EnableControls()
{
   bool isDataConnectionUp = false;
   HRESULT hr;
   VARIANT_BOOL isStarted = VARIANT_FALSE;

   if (m_spCQGCEL)
   {
      hr = m_spCQGCEL->get_IsStarted(&isStarted);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }
   

   if (isStarted)
   {
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      hr = m_spCQGCEL->get_Environment(&spEnvironment);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      eConnectionStatus dataConnectionStatus;
      hr = spEnvironment->get_DataConnectionStatus(&dataConnectionStatus);
      Common::ThrowOnCOMError(spEnvironment, hr);

      isDataConnectionUp = (dataConnectionStatus == csConnectionUp);
   }

   m_btnRequest.EnableWindow(isDataConnectionUp);

   bool isExpressionSelected = (m_spExpression != NULL);

   m_btnRemove.EnableWindow(isExpressionSelected && isDataConnectionUp);
   m_btnRemoveAll.EnableWindow(isExpressionSelected && isDataConnectionUp);

   m_edTagName.EnableWindow(isExpressionSelected);
   m_edTagValue.EnableWindow(isExpressionSelected);

   bool isExpressionSucceeded = false;
   if (isExpressionSelected)
   {
      eRequestStatus status;
      hr = m_spExpression->get_Status(&status);
      Common::ThrowOnCOMError(m_spExpression, hr);

      isExpressionSucceeded = (status == rsSuccess);
   }

   m_rdIndices.EnableWindow(isExpressionSucceeded);
   m_rdDates.EnableWindow(isExpressionSucceeded);

   m_edStart.EnableWindow(isExpressionSucceeded);
   m_edEnd.EnableWindow(isExpressionSucceeded);
   m_spnStart.EnableWindow(isExpressionSucceeded);
   m_spnEnd.EnableWindow(isExpressionSucceeded);
   m_dtpStart.EnableWindow(isExpressionSucceeded);
   m_dtpEnd.EnableWindow(isExpressionSucceeded);

   m_btnShow.EnableWindow(isExpressionSucceeded);
}

CString CExpressionDlg::GetValueAsString(
                        ATL::CComVariant value,
                        VARENUM valueType/*= VT_EMPTY*/,
                        bool withMilliseconds /*= false*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return Common::VariantToString(value, withMilliseconds);
   }
   return CHelper::m_N_A;
}