// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "Expression.h"
#include "ExpressionDlg.h"

STDMETHODIMP CExpressionDlg::OnDataError(
                           LPDISPATCH /*obj*/,
                           BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CExpressionDlg::OnDataConnectionStatusChanged(
                           eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      EnableControls();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CExpressionDlg::OnExpressionResolved(
                           ICQGExpression* cqg_expression,
                           ICQGError* /*cqg_error*/)
{
   try
   {
      if (!m_spExpression)
      {
         return S_OK;
      }

      // Get selected expression ID
      ATL::CComBSTR selExpressionID;
      HRESULT hr = m_spExpression->get_Id(&selExpressionID);
      Common::ThrowOnCOMError(m_spExpression, hr);

      // Get changed expression ID
      ATL::CComBSTR expressionID;
      hr = cqg_expression->get_Id(&expressionID);
      Common::ThrowOnCOMError<ICQGExpression>(cqg_expression, hr);

      // Check if expression resolved in the selected request
      if (selExpressionID == expressionID)
      {
         ClearAllData();
         ShowAllData();

         EnableControls();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CExpressionDlg::OnExpressionAdded(
                           ICQGExpression* cqg_expression)
{
   try
   {
      if (!m_spExpression)
      {
         return S_OK;
      }

      // Get selected expression ID
      ATL::CComBSTR selExpressionID;
      HRESULT hr = m_spExpression->get_Id(&selExpressionID);
      Common::ThrowOnCOMError(m_spExpression, hr);

      // Get changed expression ID
      ATL::CComBSTR expressionID;
      hr = cqg_expression->get_Id(&expressionID);
      Common::ThrowOnCOMError<ICQGExpression>(cqg_expression, hr);

      // Check if record added in the selected request
      if (selExpressionID == expressionID)
      {
         // Show changed records
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CExpressionDlg::OnExpressionInserted(
                           ICQGExpression* cqg_expression,
                           long /*index_*/)
{
   try
   {
      if (!m_spExpression)
      {
         return S_OK;
      }

      // Get selected expression ID
      ATL::CComBSTR selExpressionID;
      HRESULT hr = m_spExpression->get_Id(&selExpressionID);
      Common::ThrowOnCOMError(m_spExpression, hr);

      // Get changed expression ID
      ATL::CComBSTR expressionID;
      hr = cqg_expression->get_Id(&expressionID);
      Common::ThrowOnCOMError<ICQGExpression>(cqg_expression, hr);

      // Check if record inserted in the selected request
      if (selExpressionID == expressionID)
      {
         // Show changed records
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CExpressionDlg::OnExpressionRemoved(
                          ICQGExpression* cqg_expression,
                          long /*index_*/)
{
   try
   {
      if (!m_spExpression)
      {
         return S_OK;
      }

      // Get selected expression ID
      ATL::CComBSTR selExpressionID;
      HRESULT hr = m_spExpression->get_Id(&selExpressionID);
      Common::ThrowOnCOMError(m_spExpression, hr);

      // Get changed expression ID
      ATL::CComBSTR expressionID;
      hr = cqg_expression->get_Id(&expressionID);
      Common::ThrowOnCOMError<ICQGExpression>(cqg_expression, hr);

      // Check if record removed from the selected request
      if (selExpressionID == expressionID)
      {
         // Show changed records
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CExpressionDlg::OnExpressionUpdated(
                           ICQGExpression* cqg_expression,
                           long index_)
{
   try
   {
      if (!m_spExpression)
      {
         return S_OK;
      }

      // Get selected expression ID
      ATL::CComBSTR selExpressionID;
      HRESULT hr = m_spExpression->get_Id(&selExpressionID);
      Common::ThrowOnCOMError(m_spExpression, hr);

      // Get changed expression ID
      ATL::CComBSTR expressionID;
      hr = cqg_expression->get_Id(&expressionID);
      Common::ThrowOnCOMError<ICQGExpression>(cqg_expression, hr);

      // Check if record updated in the selected request
      if (selExpressionID == expressionID)
      {
         // Find row for specified expression output
         CString indexStr;
         indexStr.Format(_T("%d"), index_ + 1);
         LVFINDINFO findInfo;
         findInfo.flags = LVFI_STRING;
         findInfo.psz = indexStr;
         int row = m_lvLastExpression.FindItem(&findInfo);

         // If found change it's properties
         if (row > -1)
         {
            ShowRecord(index_ + 1, m_lvLastExpression, row);
         }
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}