// ExpressionReqDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "Expression.h"
#include "ExpressionReqDlg.h"

namespace
{
   VARIANT_BOOL boolToVarBool(BOOL value)
   {
      return value ? VARIANT_TRUE : VARIANT_FALSE;
   }
}

CExpressionReqDlg::CExpressionReqDlg(
                              const ATL::CComPtr<ICQGExpressionRequest>& spRequest,
                              const DATE lineTime,
                              CWnd* pParent /*=NULL*/)
                              : CDialog(CExpressionReqDlg::IDD, pParent),
                                m_spRequest(spRequest),
                                m_LineTime(lineTime),
                                m_RangeStartDate(COleDateTime::GetCurrentTime()),
                                m_RangeEndDate(COleDateTime::GetCurrentTime()),
                                m_RequestType(rtIntInt),
                                m_EqualizeCloses(FALSE),
                                m_UpdatesEnabled(FALSE),
                                m_DailyFromIntraday(FALSE),
                                m_CustomSession(FALSE)
{
   // Initialize local variables
   m_edRangeStart.NegativeAllowed(true);
   m_edRangeEnd.NegativeAllowed(true);
}

void CExpressionReqDlg::DoDataExchange(
                              CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);

   DDX_Control(pDX, IDC_ED_SUB_EXPRESSION, m_edSubExpression);
   DDX_Control(pDX, IDC_ED_ALIAS, m_edAlias);

   DDX_Control(pDX, IDC_LSV_SUB_EXPRESSIONS, m_lvSubExpressions);

   DDX_Control(pDX, IDC_DTP_RANGE_START, m_dtpRangeStart);
   DDX_DateTimeCtrl(pDX, IDC_DTP_RANGE_START, m_RangeStartDate);
   DDX_Control(pDX, IDC_ED_RANGE_START, m_edRangeStart);
   DDX_Control(pDX, IDC_SPN_RANGE_START, m_spnRangeStart);
   
   DDX_Control(pDX, IDC_DTP_RANGE_END, m_dtpRangeEnd);
   DDX_DateTimeCtrl(pDX, IDC_DTP_RANGE_END, m_RangeEndDate);
   DDX_Control(pDX, IDC_ED_RANGE_END, m_edRangeEnd);
   DDX_Control(pDX, IDC_SPN_RANGE_END, m_spnRangeEnd);

   DDX_Radio(pDX, IDC_RD_INT_INT, m_RequestType);

   DDX_Control(pDX, IDC_CHK_INCLUDE_END, m_chkIncludeEnd);
   DDX_Check(pDX, IDC_CHK_INCLUDE_END, m_IncludeEnd);

   DDX_Control(pDX, IDC_CMB_HISTORICAL, m_cmbHistorical);

   DDX_Control(pDX, IDC_ED_INTRADAY, m_edIntraday);
   DDX_Control(pDX, IDC_SPN_INTRADAY, m_spnIntraday);

   DDX_Control(pDX, IDC_CMB_CONTINUATION, m_cmbContinuation);

   DDX_Control(pDX, IDC_CHK_EQUALIZE_CLOSES, m_chkEqualizeCloses);
   DDX_Check(pDX, IDC_CHK_EQUALIZE_CLOSES, m_EqualizeCloses);

   DDX_Control(pDX, IDC_ED_DAYS_BEFORE_EXP, m_edDaysBeforeExp);
   DDX_Control(pDX, IDC_SPN_DAYS_BEFORE_EXP, m_spnDaysBeforeExp);

   DDX_Check(pDX, IDC_CHK_UPDATES_ENABLED, m_UpdatesEnabled);

   DDX_Check(pDX, IDC_CHK_DAILY_FROM_INTR, m_DailyFromIntraday);

   DDX_Control(pDX, IDC_ED_SESSIONS_FILTER, m_edSessionsFilter);
   DDX_Control(pDX, IDC_SPN_SESSIONS_FILTER, m_spnSessionsFilter);
   
   DDX_Control(pDX, IDC_ED_CUSTOM_SESSIONS_FILTER, m_edCustomSessionsFilter);
   DDX_Text(pDX, IDC_ED_CUSTOM_SESSIONS_FILTER, m_CustomSessionsFilter);

   DDX_Check(pDX, IDC_CHK_CUSTOM_SESSION, m_CustomSession);
   
   DDX_Control(pDX, IDC_CMB_SUBSCRIPTION, m_cmbSubscription);
   
   DDX_Control(pDX, IDC_ST_RECALC_PERIOD, m_lblRecalcPeriod);
   DDX_Control(pDX, IDC_ED_RECALC_PERIOD, m_edRecalcPeriod);
   DDX_Control(pDX, IDC_SPN_RECALC_PERIOD, m_spnRecalcPeriod);
}

BOOL CExpressionReqDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      InitializeControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;
}

BEGIN_MESSAGE_MAP(CExpressionReqDlg, CDialog)
   ON_BN_CLICKED(IDOK, OnBnClickedOk)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_SESSIONS_FILTER, OnDeltaposSpnSessionsFilter)
   ON_BN_CLICKED(IDC_CHK_CUSTOM_SESSION, OnBnClickedChkCustomSession)
   ON_CBN_SELCHANGE(IDC_CMB_CONTINUATION, OnCbnSelChangeCmbContinuation)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_INTRADAY, OnDeltaposSpnIntraday)
   ON_CBN_SELCHANGE(IDC_CMB_HISTORICAL, OnCbnSelChangeCmbHistorical)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RANGE_END, OnDeltaposSpnRangeEnd)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RANGE_START, OnDeltaposSpnRangeStart)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_DAYS_BEFORE_EXP, OnDeltaposSpnDaysBeforeExp)
   ON_BN_CLICKED(IDC_RD_INT_INT, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_RD_DATE_INT, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_RD_DATE_DATE, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_BTN_ADD_SUB_EXPRESSION, OnBnClickedBtnAddSubExpression)
   ON_CBN_SELCHANGE(IDC_CMB_SUBSCRIPTION, OnCbnSelChangeCmbSubscription)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RECALC_PERIOD, OnDeltaposSpnRecalcPeriod)
END_MESSAGE_MAP()

void CExpressionReqDlg::InitializeControls()
{
   RequestTypeChanged();

   InitializeSubExpressionsList();

   InitializeHistoricalCombo();
   InitializeContinuationCombo();
   InitializeSubscriptionCombo();

   m_dtpRangeStart.SetFormat(CHelper::DateFormat);
   m_dtpRangeEnd.SetFormat(CHelper::DateFormat);

   // Set default parameters
   SetDefaultParameters();
}

void CExpressionReqDlg::SetDefaultParameters()
{
   UpdateData();

   // Get default values form request
   long defIntradayPeriod;
   HRESULT hr = m_spRequest->get_IntradayPeriod(&defIntradayPeriod);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eHistoricalPeriod defHistoricalPeriod;
   hr = m_spRequest->get_HistoricalPeriod(&defHistoricalPeriod);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defUpdatesEnabled;
   hr = m_spRequest->get_UpdatesEnabled(&defUpdatesEnabled);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defIncludeEnd;
   hr = m_spRequest->get_IncludeEnd(&defIncludeEnd);
   Common::ThrowOnCOMError(m_spRequest, hr);

   ATL::CComVariant defSessionsFilter;
   hr = m_spRequest->get_SessionsFilter(&defSessionsFilter);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eTimeSeriesContinuationType defContinuation;
   hr = m_spRequest->get_Continuation(&defContinuation);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defEqualizeCloses;
   hr = m_spRequest->get_EqualizeCloses(&defEqualizeCloses);
   Common::ThrowOnCOMError(m_spRequest, hr);

   long defDaysBeforeExp;
   hr = m_spRequest->get_DaysBeforeExpiration(&defDaysBeforeExp);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eSessionFlag defSessionFlags;
   hr = m_spRequest->get_SessionFlags(&defSessionFlags);
   Common::ThrowOnCOMError(m_spRequest, hr);
   
   eTimeSeriesSubscriptionLevel defSubscription;
   hr = m_spRequest->get_SubscriptionLevel(&defSubscription);
   Common::ThrowOnCOMError(m_spRequest, hr);
   
   long defRecalcPeriod;
   hr = m_spRequest->get_RecalcPeriod(&defRecalcPeriod);
   Common::ThrowOnCOMError(m_spRequest, hr);

   // Set controls default values
   m_edIntraday.SetValue(defIntradayPeriod);
   CHelper::SetComboBoxItem(m_cmbHistorical, (DWORD_PTR)defHistoricalPeriod);
   m_UpdatesEnabled = defUpdatesEnabled;
   m_IncludeEnd = defIncludeEnd;
   CHelper::SetComboBoxItem(m_cmbContinuation, (DWORD_PTR)defContinuation);
   m_EqualizeCloses = defEqualizeCloses;
   m_edDaysBeforeExp.SetValue(defDaysBeforeExp);
   CHelper::SetComboBoxItem(m_cmbSubscription, (DWORD_PTR)defSubscription);
   m_edRecalcPeriod.SetValue(defRecalcPeriod);
   SetSessionFlags(defSessionFlags);
   if (defSessionsFilter.vt == VT_UI4)
   {
      m_CustomSession = FALSE;
      m_edSessionsFilter.SetValue(defSessionsFilter.intVal);
      m_CustomSessionsFilter = _T("");
   }
   else if (defSessionsFilter.vt == VT_BSTR)
   {
      m_CustomSession = TRUE;
      m_edSessionsFilter.SetValue(0);
      m_CustomSessionsFilter = defSessionsFilter.bstrVal;
   }
   OnCbnSelChangeCmbHistorical();
   OnCbnSelChangeCmbContinuation();
   OnBnClickedChkCustomSession();

   m_edRangeStart.SetValue(0);
   m_RangeStartDate = m_LineTime - 1;
   m_edRangeEnd.SetValue(-10);
   m_RangeEndDate = m_LineTime;

   UpdateData(FALSE);
}

void CExpressionReqDlg::OnEnterEdSubExpression()
{
   CString subExpressionStr;
   m_edSubExpression.GetWindowText(subExpressionStr);
   subExpressionStr.Trim();

   if (subExpressionStr.GetLength() > 0)
   {
      m_edAlias.SetFocus();
   }   
}

void CExpressionReqDlg::OnEnterEdAlias()
{
   OnBnClickedBtnAddSubExpression();
}

void CExpressionReqDlg::OnBnClickedOk()
{
   try
   {
      // If enter clicked when one of the sub expression edit boxes is active
      // then call corresponding function and exit from current
      CWnd* pWnd = GetFocus();
   
      if (pWnd == &m_edSubExpression)
      {
         OnEnterEdSubExpression();
         return;
      }
      else if (pWnd == &m_edAlias)
      {
         OnEnterEdAlias();
         return;
      }

      UpdateData();

      // Set request parameters
      HRESULT hr;
      if (m_RequestType == rtIntInt)
      {
         hr = m_spRequest->put_RangeStart(ATL::CComVariant(m_edRangeStart.GetValue()));
      }
      else
      {
         hr = m_spRequest->put_RangeStart(ATL::CComVariant(m_RangeStartDate.m_dt, VT_DATE));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_RequestType == rtDateDate)
      {
         hr = m_spRequest->put_RangeEnd(ATL::CComVariant(m_RangeEndDate.m_dt, VT_DATE));
      }
      else
      {
         hr = m_spRequest->put_RangeEnd(ATL::CComVariant(m_edRangeEnd.GetValue()));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_IncludeEnd(boolToVarBool(m_IncludeEnd));
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get historical period
      eHistoricalPeriod historicalPeriod = 
         static_cast<eHistoricalPeriod>
            (m_cmbHistorical.GetItemData(m_cmbHistorical.GetCurSel()));

      if (historicalPeriod == hpUndefined)
      {
         hr = m_spRequest->put_IntradayPeriod(m_edIntraday.GetValue());
      }
      else
      {
         hr = m_spRequest->put_HistoricalPeriod(historicalPeriod);
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get continuation type
      eTimeSeriesContinuationType continuationType = 
         static_cast<eTimeSeriesContinuationType>
            (m_cmbContinuation.GetItemData(m_cmbContinuation.GetCurSel()));

      hr = m_spRequest->put_Continuation(continuationType);
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_EqualizeCloses(boolToVarBool(m_EqualizeCloses));
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_DaysBeforeExpiration(m_edDaysBeforeExp.GetValue());
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_UpdatesEnabled(boolToVarBool(m_UpdatesEnabled));
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_CustomSession)
      {
         hr = m_spRequest->put_SessionsFilter(ATL::CComVariant(m_CustomSessionsFilter));
      }
      else
      {
         hr = m_spRequest->put_SessionsFilter(ATL::CComVariant(m_edSessionsFilter.GetValue()));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_SessionFlags(GetSessionFlags());
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get subscription type
      eTimeSeriesSubscriptionLevel subscriptionLevel = 
         static_cast<eTimeSeriesSubscriptionLevel>(m_cmbSubscription.GetItemData(m_cmbSubscription.GetCurSel()));

      hr = m_spRequest->put_SubscriptionLevel(subscriptionLevel);
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_RecalcPeriod(m_edRecalcPeriod.GetValue());
      Common::ThrowOnCOMError(m_spRequest, hr);
      
      // Close request window
      OnOK();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExpressionReqDlg::InitializeSubExpressionsList()
{
   m_lvSubExpressions.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvSubExpressions.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   m_lvSubExpressions.InsertColumn(secSubExpression, _T("Sub Expression"), LVCFMT_LEFT, 200);
   m_lvSubExpressions.InsertColumn(secAlias, _T("Alias"), LVCFMT_LEFT, 200);
}

void CExpressionReqDlg::InitializeHistoricalCombo()
{
   int row = m_cmbHistorical.AddString(_T("Undefined"));
   m_cmbHistorical.SetItemData(row, hpUndefined);

   row = m_cmbHistorical.AddString(_T("Daily"));
   m_cmbHistorical.SetItemData(row, hpDaily);

   row = m_cmbHistorical.AddString(_T("Weekly"));
   m_cmbHistorical.SetItemData(row, hpWeekly);

   row = m_cmbHistorical.AddString(_T("Monthly"));
   m_cmbHistorical.SetItemData(row, hpMonthly);

   row = m_cmbHistorical.AddString(_T("Quarterly"));
   m_cmbHistorical.SetItemData(row, hpQuarterly);

   row = m_cmbHistorical.AddString(_T("Semiannual"));
   m_cmbHistorical.SetItemData(row, hpSemiannual);

   row = m_cmbHistorical.AddString(_T("Yearly"));
   m_cmbHistorical.SetItemData(row, hpYearly);
}

void CExpressionReqDlg::InitializeContinuationCombo()
{
   int row = m_cmbContinuation.AddString(_T("NoContinuation"));
   m_cmbContinuation.SetItemData(row, tsctNoContinuation);

   row = m_cmbContinuation.AddString(_T("Standard"));
   m_cmbContinuation.SetItemData(row, tsctStandard);

   row = m_cmbContinuation.AddString(_T("StandardByMonth"));
   m_cmbContinuation.SetItemData(row, tsctStandardByMonth);

   row = m_cmbContinuation.AddString(_T("Active"));
   m_cmbContinuation.SetItemData(row, tsctActive);

   row = m_cmbContinuation.AddString(_T("ActiveByMonth"));
   m_cmbContinuation.SetItemData(row, tsctActiveByMonth);

   row = m_cmbContinuation.AddString(_T("Adjusted"));
   m_cmbContinuation.SetItemData(row, tsctAdjusted);

   row = m_cmbContinuation.AddString(_T("AdjustedByMonth"));
   m_cmbContinuation.SetItemData(row, tsctAdjustedByMonth);
}

void CExpressionReqDlg::InitializeSubscriptionCombo()
{
   int row = m_cmbSubscription.AddString(_T("EachBar"));
   m_cmbSubscription.SetItemData(row, tslEachBar);

   row = m_cmbSubscription.AddString(_T("EachTick"));
   m_cmbSubscription.SetItemData(row, tslEachTick);

   row = m_cmbSubscription.AddString(_T("EndOfBarAndPeriod"));
   m_cmbSubscription.SetItemData(row, tslEndOfBarAndPeriod);
}

void CExpressionReqDlg::OnBnClickedBtnAddSubExpression()
{
   try
   {
      CString subExpressionStr;
      m_edSubExpression.GetWindowText(subExpressionStr);
      subExpressionStr.Trim();

      CString aliasStr;
      m_edAlias.GetWindowText(aliasStr);
      aliasStr.Trim();

      if (subExpressionStr.GetLength() > 0)
      {
         // Add expression and alias to request
         // Alias is generated by the CQGCEL if it is not specified
         HRESULT hr = m_spRequest->AddSubExpression(ATL::CW2T(subExpressionStr), ATL::CW2T(aliasStr));
         Common::ThrowOnCOMError(m_spRequest, hr);

         // Retrieve added values (they can be slightly changed)
         long count;
         hr = m_spRequest->get_SubExpressionCount(&count);
         Common::ThrowOnCOMError(m_spRequest, hr);

         ATL::CComBSTR subExpression;
         hr = m_spRequest->get_SubExpression(count - 1, &subExpression);
         Common::ThrowOnCOMError(m_spRequest, hr);

         ATL::CComBSTR alias;
         hr = m_spRequest->get_SubExpressionAlias(count - 1, &alias);
         Common::ThrowOnCOMError(m_spRequest, hr);

         // Add sub expression to the list
         int row = m_lvSubExpressions.InsertItem(m_lvSubExpressions.GetItemCount(), _T(""));
         m_lvSubExpressions.SetItemText(row, secSubExpression, subExpression);
         m_lvSubExpressions.SetItemText(row, secAlias, alias);

         // Clear added values from input fields
         m_edSubExpression.SetWindowText(_T(""));
         m_edAlias.SetWindowText(_T(""));
      }

      m_edSubExpression.SetFocus();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExpressionReqDlg::OnBnClickedRdRequestType()
{
   try
   {
      RequestTypeChanged();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExpressionReqDlg::OnCbnSelChangeCmbHistorical()
{
   try
   {
      // Get historical period
      eHistoricalPeriod historicalPeriod = 
         static_cast<eHistoricalPeriod>
            (m_cmbHistorical.GetItemData(m_cmbHistorical.GetCurSel()));

      m_edIntraday.EnableWindow(historicalPeriod == hpUndefined);
      m_spnIntraday.EnableWindow(historicalPeriod == hpUndefined);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExpressionReqDlg::OnDeltaposSpnIntraday(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edIntraday.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CExpressionReqDlg::OnDeltaposSpnRangeStart(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRangeStart.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CExpressionReqDlg::OnDeltaposSpnRangeEnd(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRangeEnd.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CExpressionReqDlg::OnDeltaposSpnDaysBeforeExp(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edDaysBeforeExp.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CExpressionReqDlg::OnDeltaposSpnSessionsFilter(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edSessionsFilter.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CExpressionReqDlg::OnBnClickedChkCustomSession()
{
   try
   {
      UpdateData();

      m_spnSessionsFilter.ShowWindow(m_CustomSession ? SW_HIDE : SW_SHOW);
      m_edSessionsFilter.ShowWindow(m_CustomSession ? SW_HIDE : SW_SHOW);

      m_edCustomSessionsFilter.ShowWindow(m_CustomSession ? SW_SHOW : SW_HIDE);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExpressionReqDlg::OnCbnSelChangeCmbContinuation()
{
   try
   {
      // Get continuation type
      eTimeSeriesContinuationType continuationType = 
         static_cast<eTimeSeriesContinuationType>
            (m_cmbContinuation.GetItemData(m_cmbContinuation.GetCurSel()));

      switch (continuationType)
      {
      case tsctNoContinuation:
      case tsctStandard:
      case tsctStandardByMonth:
         m_chkEqualizeCloses.EnableWindow(FALSE);
         m_edDaysBeforeExp.EnableWindow(FALSE);
         m_spnDaysBeforeExp.EnableWindow(FALSE);
         break;
      case tsctActive:
      case tsctActiveByMonth:
         m_chkEqualizeCloses.EnableWindow(TRUE);
         m_edDaysBeforeExp.EnableWindow(FALSE);
         m_spnDaysBeforeExp.EnableWindow(FALSE);
         break;
      default:
         m_chkEqualizeCloses.EnableWindow(TRUE);
         m_edDaysBeforeExp.EnableWindow(TRUE);
         m_spnDaysBeforeExp.EnableWindow(TRUE);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExpressionReqDlg::RequestTypeChanged()
{
   UpdateData();

   m_dtpRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_HIDE : SW_SHOW);
   m_edRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_SHOW : SW_HIDE);
   m_spnRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_SHOW : SW_HIDE);

   m_dtpRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_SHOW : SW_HIDE);
   m_edRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_HIDE : SW_SHOW);
   m_spnRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_HIDE : SW_SHOW);

   m_chkIncludeEnd.EnableWindow(m_RequestType == rtDateDate);
}

void CExpressionReqDlg::SetSessionFlags(
                              eSessionFlag sessionFlag)
{
   UpdateData();
   m_DailyFromIntraday = ((sessionFlag & sfDailyFromIntraday) == sfDailyFromIntraday);
   UpdateData(FALSE);
}

eSessionFlag CExpressionReqDlg::GetSessionFlags()
{
   UpdateData();

   int sessionFlag = sfUndefined;

   if (m_DailyFromIntraday)
   {
      sessionFlag = (sessionFlag | sfDailyFromIntraday);
   }

   return (eSessionFlag)sessionFlag;
}

void CExpressionReqDlg::OnCbnSelChangeCmbSubscription()
{
    try
   {
      // Get subscription level
      eTimeSeriesSubscriptionLevel subscriptionLevel = 
         static_cast<eTimeSeriesSubscriptionLevel>(m_cmbSubscription.GetItemData(m_cmbSubscription.GetCurSel()));

      if (subscriptionLevel == tslEndOfBarAndPeriod)
      {
         m_lblRecalcPeriod.EnableWindow(TRUE);
         m_edRecalcPeriod.EnableWindow(TRUE);
         m_spnRecalcPeriod.EnableWindow(TRUE);
      }
      else
      {
         m_lblRecalcPeriod.EnableWindow(FALSE);
         m_edRecalcPeriod.EnableWindow(FALSE);
         m_spnRecalcPeriod.EnableWindow(FALSE);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExpressionReqDlg::OnDeltaposSpnRecalcPeriod(
                                NMHDR *pNMHDR,
                                LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRecalcPeriod.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}