#pragma once


// CBracketPropertiesDlg dialog

class CBracketPropertiesDlg : public CDialog
{
   DECLARE_DYNAMIC(CBracketPropertiesDlg)

public:
   enum { IDD = IDD_BRACKET_PROPERTIES_DIALOG };

   CBracketPropertiesDlg(ATL::CComPtr<ICQGCEL> cqgcel,
       ATL::CComPtr<ICQGOrder> order,
       eOrderSide orderSide,
       CWnd* pParent = nullptr);   // standard constructor

protected:
   virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

    /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

    /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
    /// \return Specifies whether the framework shall set the input 
    /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

private:
   /// \brief Initializes all controls.
   void intializeControls();

   /// \brief Initializes speculation types combo-box.
   void initializeSpeculationTypes();

   /// \brief Initializes stop-loss order types combo-box.
   void initializeStopLossOrderTypes();

   /// \brief Initializes target offset types combo-box.
   void initializeTargetOffsetTypes();

   /// \brief Initializes stop-loss offset types combo-box.
   void initializeStopLossOffsetType();

private:
   /// \brief Gets price depending on order type.
   double getPrice();

   /// \brief Creates info line
   std::string createInfoLine();

   /// \brief Increments/Decrements value of edit box depending on combo-box state
   /// \param pNMHDR
   /// \param pResult
   /// \param cmbOffsetType - combo-box from which value spin will be decided
   /// \param edTickPrice - edit control to change
   void spinTickPrice(NMHDR* pNMHDR, LRESULT* pResult, CComboBox& cmbOffsetType, CEdit& edTickPrice);

   /// \brief Changes specified edit box's tick.
   /// \param editCtrl - Edit box control, which tick will be changed
   /// \param delta - Change type
   static void ChangeTick(CEdit& editCtrl, int delta);

private:
   /// \brief Info lable control.
   CStatic m_lblInfoLine;

   /// \brief Stop-loss offset type label control.
   CStatic m_lblStopLossOffsetType;

   /// \brief Hedge lable control.
   CStatic m_lblHedge;

   /// \brief Place button control.
   CButton m_btnPlace;

   /// \brief Cancel button control.
   CButton m_btnCancel;

   /// \brief Combo-box with speculation types control.
   CComboBox m_cmbSpeculationType;

   /// \brief Combo-box with target offset types control.
   CComboBox m_cmbTargetOffsetType;

   /// \brief Combo-box with stop-loss offset types control.
   CComboBox m_cmbStopLossOffsetType;

   /// \brief Combo-box with stop-loss order types controls.
   CComboBox m_cmbStopLossOrderType;

   /// \brief Include target check-box control.
   CButton m_chkIncludeTarget;

   /// \brief Include stop-loss check-box control.
   CButton m_chkIncludeStopLoss;

   /// \brief Limit price edit box control.
   CEdit m_edTargetLimitTickPrice;

   /// \brief Stop price edit box control.
   CEdit m_edStopLossStopTickPrice;

   /// \brief Limit price edit box control.
   CEdit m_edStopLossLimitTickPrice;

   /// \brief Quantity spin button control.
   CSpinButtonCtrl m_spnTargetTickPrice;

   /// \brief Stop price spin button control.
   CSpinButtonCtrl m_spnStopLossStopTickPrice;

   /// \brief Limit price spin button control.
   CSpinButtonCtrl m_spnStopLossLimitTickPrice;

private:
   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCEL;

   /// \brief CQGOrder object.
   ATL::CComPtr<ICQGOrder> m_spOrder;

   /// \brief Main order side.
   eOrderSide m_mainOrderSide;
public:
   afx_msg void OnBnClickedChkBrktIncludeStoploss();
   afx_msg void OnBnClickedChkBrktIncludeTarget();
   afx_msg void OnCbnSelchangeCmbBrktStoplossOrdertype();
   afx_msg void OnCbnSelchangeCmbBrktStoplossOffsettype();
   afx_msg void OnBnClickedBtnBrktCancel();
   afx_msg void OnDeltaposSpnBrktTargetTickprice(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnDeltaposSpnBrktStoplossStopTickprice(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnDeltaposSpnBrktStoplossLimitTickprice(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnCbnSelchangeCmbBrktTargetOffsettype();
   afx_msg void OnBnClickedBtnBrktPlace();
};
