using System;
using System.Windows.Forms;

namespace OrderPlacingSample
{
   static class ErrorHandler
   {
      /// <summary>
      /// Shows pop-up with specified error message.
      /// </summary>
      /// <param name="message">Error description.</param>
      /// <param name="caption">Pop-up window caption.</param>
      /// <param name="owner">Owner of this pop-up window.</param>
      public static void ShowErrorMessage(string message, string caption, Form owner)
      {
         if (caption == String.Empty)
         {
            caption = "Error";
         }

         if (owner == null)
         {
            MessageBox.Show(message,
                caption,
                MessageBoxButtons.OK,
                MessageBoxIcon.Error);
         }
         else
         {
            MessageBox.Show(owner,
                message,
                caption,
                MessageBoxButtons.OK,
                MessageBoxIcon.Error);
         }
      }

      /// <summary>
      /// Creates error message.
      /// </summary>
      /// <param name="modName">Module name, which handles an error or in which the error has occured.</param>
      /// <param name="funcName">Function name, which handles an error or in which the error has occured.</param>
      /// <param name="errorDescription">Error description.</param>
      private static string getErrorMessage(string modName, string funcName, string errorDescription)
      {
         return "Module: " + modName + Environment.NewLine +
                "Function: " + funcName + Environment.NewLine +
                "Error Description: " + errorDescription;
      }

      /// <summary>
      /// Shows pop-up window with specified exception information.
      /// </summary>
      /// <param name="modName">Module name, in which the exception has occurred.</param>
      /// <param name="funcName">Function name, in which the exception has occurred.</param>
      /// <param name="ex">The exception generated in the described function of the described module.</param>
      public static void HandleError(string modName, string funcName, Exception ex)
      {
         MessageBox.Show(getErrorMessage(modName, funcName, ex.Message),
                        "OrderPlacing", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }

      /// <summary>
      /// Handles CQGError and shows a message box.
      /// </summary>
      /// <param name="modName">Module name, which handles an error.</param>
      /// <param name="funcName">Function name, which handles an error.</param>
      /// <param name="error">CQGError object.</param>
      public static void HandleError(string modName, string funcName, CQG.CQGError error)
      {
         MessageBox.Show(getErrorMessage(modName, funcName, error.Description),
                         "OrderPlacing", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
      }
   }
}
