// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include <vector>
#include <cassert>
#include <stdexcept>
#include "Common.h"
#include "OrderProperties.h"
#include "OrderPropertiesDlg.h"

void COrderPropertiesDlg::InitializeCQGCEL()
{
   // create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   AssertCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   AssertCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   AssertCOMError(spConf, hr);

   hr = spConf->put_UseOrderSide(VARIANT_TRUE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_DefPositionSubscriptionLevel(pslSnapshotAndUpdates);
   AssertCOMError(spConf, hr);

   // Set NewInstrumentMode to request only instrument metadata with CreateInstrumentRequest:
   // http://partners.cqg.com/sites/default/files/docs/CQGAPI_4.0R/webframe.html#CQG~CQGAPIConfig~NewInstrumentMode.html
   hr = spConf->put_NewInstrumentMode(VARIANT_TRUE);
   AssertCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   AssertCOMError(m_spCQGCEL, hr);
}

void COrderPropertiesDlg::InitializeControls()
{
   InitializeOrdersList();
   InitializeOrderPropList();
   InitializeFillsList();
   InitializeTransactionsList();
   InitializeStrategyLegFillsList();
   InitializeStrategyLegOrdersList();
   InitializeHangOrderIdsList();

   ClearOrders();
   m_IsInQuery = false;

   OnDataConnectionStatusChanged(csConnectionDown);
   OnGWConnectionStatusChanged(csConnectionDown);
}

void COrderPropertiesDlg::InitializeOrdersList()
{
   m_lvOrders.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvOrders.ModifyStyle(LVS_SHOWSELALWAYS, LVS_SHOWSELALWAYS);

   m_lvOrders.InsertColumn(ocID, _T("ID"), LVCFMT_LEFT, 115);
   m_lvOrders.InsertColumn(ocInstrument, _T("Instrument"), LVCFMT_LEFT, 115);
   m_lvOrders.InsertColumn(ocSide, _T("Side"), LVCFMT_LEFT, 115);
   m_lvOrders.InsertColumn(ocType, _T("Type"), LVCFMT_LEFT, 115);
   m_lvOrders.InsertColumn(ocState, _T("State"), LVCFMT_LEFT, 115);
}

void COrderPropertiesDlg::InitializeOrderPropList()
{
   m_lvOrderProps.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvOrderProps.InsertColumn(opcName, _T("Name"), LVCFMT_LEFT, 110);
   m_lvOrderProps.InsertColumn(opcValue, _T("Value"), LVCFMT_LEFT, 160);
}

void COrderPropertiesDlg::InitializeFillsList()
{
   m_lvOrderFills.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvOrderFills.InsertColumn(ofcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 110);
   m_lvOrderFills.InsertColumn(ofcFillID, _T("Fill ID"), LVCFMT_LEFT, 60);
   m_lvOrderFills.InsertColumn(ofcStatus, _T("Status"), LVCFMT_LEFT, 50);
   m_lvOrderFills.InsertColumn(ofcBought, _T("Bght"), LVCFMT_LEFT, 35);
   m_lvOrderFills.InsertColumn(ofcSold, _T("Sold"), LVCFMT_LEFT, 35);
   m_lvOrderFills.InsertColumn(ofcInstrument, _T("Instrument"), LVCFMT_LEFT, 90);
   m_lvOrderFills.InsertColumn(ofcPrice, _T("Price"), LVCFMT_LEFT, 60);
   m_lvOrderFills.InsertColumn(ofcCurrency, _T("Curr."), LVCFMT_LEFT, 40);
   m_lvOrderFills.InsertColumn(ofcGWTradeID, _T("GWT ID"), LVCFMT_LEFT, 50);
   m_lvOrderFills.InsertColumn(ofcAggressive, _T("Aggr."), LVCFMT_LEFT, 40);
}

void COrderPropertiesDlg::InitializeTransactionsList()
{
   m_lvOrderTransactions.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvOrderTransactions.InsertColumn(otcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 110);
   m_lvOrderTransactions.InsertColumn(otcTransactionID, _T("Trans. ID"), LVCFMT_LEFT, 60);
   m_lvOrderTransactions.InsertColumn(otcStatus, _T("Status"), LVCFMT_LEFT, 55);
   m_lvOrderTransactions.InsertColumn(otcFillID, _T("Fill ID"), LVCFMT_LEFT, 60);
   m_lvOrderTransactions.InsertColumn(otcRejectReason, _T("Rej. Reason"), LVCFMT_LEFT, 80);

   m_lvOrderTransactions.InsertColumn(otcPropName, _T("Prop. Name"), LVCFMT_LEFT, 70);
   m_lvOrderTransactions.InsertColumn(otcPropPreviousValue, _T("Old Value"), LVCFMT_LEFT, 70);
   m_lvOrderTransactions.InsertColumn(otcPropCurrentValue, _T("New Value"), LVCFMT_LEFT, 70);
}

void COrderPropertiesDlg::InitializeStrategyLegFillsList()
{
   m_lvStrategyLegFills.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvStrategyLegFills.InsertColumn(slfcOriginalOrderId, _T("OroginalOrderId"), LVCFMT_LEFT, 95);
   m_lvStrategyLegFills.InsertColumn(slfcExecutionId, _T("ExecutionId"), LVCFMT_LEFT, 200);
   m_lvStrategyLegFills.InsertColumn(slfcQuantity, _T("Quantity"), LVCFMT_LEFT, 55);
}

void COrderPropertiesDlg::InitializeStrategyLegOrdersList()
{
   m_lvStrategyLegOrders.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvStrategyLegOrders.InsertColumn(ocID, _T("ID"), LVCFMT_LEFT, 95);
   m_lvStrategyLegOrders.InsertColumn(ocInstrument, _T("Instrument"), LVCFMT_LEFT, 85);
   m_lvStrategyLegOrders.InsertColumn(ocSide, _T("Side"), LVCFMT_LEFT, 35);
   m_lvStrategyLegOrders.InsertColumn(ocType, _T("Type"), LVCFMT_LEFT, 38);
   m_lvStrategyLegOrders.InsertColumn(ocState, _T("State"), LVCFMT_LEFT, 40);
}

void COrderPropertiesDlg::InitializeHangOrderIdsList()
{
   m_lvHangOrderIds.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvHangOrderIds.InsertColumn(oidcHangOrderId, _T("HangOrderIds"), LVCFMT_LEFT, 120);
}

void COrderPropertiesDlg::ClearOrders()
{
   m_spSelectedOrder = NULL;

   ClearProperties();
   ClearFills();
   ClearTransactions();
   ClearStrategyLegFills();
   ClearStrategyLegOrders();

   // Clear orders
   m_lvOrders.DeleteAllItems();

   // Disable filtering
   m_chkFilter.EnableWindow(FALSE);
   ClearFilterCheckBox();
}

void COrderPropertiesDlg::ClearProperties()
{
   m_lvOrderProps.DeleteAllItems();
}

void COrderPropertiesDlg::ClearFills()
{
   m_lvOrderFills.DeleteAllItems();
}

void COrderPropertiesDlg::ClearTransactions()
{
   m_lvOrderTransactions.DeleteAllItems();

   // Clear transaction's strategy leg fills and hang order ids
   ClearStrategyLegFills();
   ClearHangOrderIds();
}

void COrderPropertiesDlg::ClearStrategyLegFills()
{
   m_lvStrategyLegFills.DeleteAllItems();
}

void COrderPropertiesDlg::ClearStrategyLegOrders()
{
   m_lvStrategyLegOrders.DeleteAllItems();
}

void COrderPropertiesDlg::ClearHangOrderIds()
{
   m_lvHangOrderIds.DeleteAllItems();
}

void COrderPropertiesDlg::ClearFilterCheckBox()
{
   m_chkFilter.SetWindowText(m_FilterText);
   m_chkFilter.SetCheck(BST_UNCHECKED);
}

void COrderPropertiesDlg::EnableControls(
                                 bool enable)
{
   m_btnQueryOrders.EnableWindow(enable && !m_IsInQuery);
   m_btnCancelQuery.EnableWindow(enable && m_IsInQuery);
   m_cmbAccounts.EnableWindow(enable && !m_IsInQuery);
}

void COrderPropertiesDlg::SetGWLogonButtonStatus(
                                 eConnectionStatus status)
{
   if (!m_spCQGCEL)
   {
      m_btnGWLogOnOff.EnableWindow(false);
      return;
   }

   if (status == csConnectionUp)
   {
      m_btnGWLogOnOff.EnableWindow(true);
      m_btnGWLogOnOff.SetWindowText(_T("Log Off"));
      m_IsGWLoggedOn = true;

      // Set GW username
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      HRESULT hr = m_spCQGCEL->get_Environment(&spEnvironment);
      AssertCOMError(m_spCQGCEL, hr);

      ATL::CComBSTR username;
      hr = spEnvironment->get_GWLogonName(&username);
      AssertCOMError(spEnvironment, hr);

      m_lblGWUsername.SetWindowText(ATL::CW2T(username));
   } 
   else if (status == csConnectionNotLoggedOn || status == csConnectionDown)
   {
      m_btnGWLogOnOff.EnableWindow(true);
      m_btnGWLogOnOff.SetWindowText(_T("Log On"));
      m_IsGWLoggedOn = false;

      m_lblGWUsername.SetWindowText(_T(""));
   }
   else
   {
      m_btnGWLogOnOff.EnableWindow(false);
   }
}

void COrderPropertiesDlg::ShowQueryStatus(
                                 const ATL::CComPtr<ICQGOrdersQuery>& spOrdersQuery)
{
   eRequestStatus status;
   HRESULT hr = spOrdersQuery->get_Status(&status);
   AssertCOMError(spOrdersQuery, hr);

   long ID;
   hr = spOrdersQuery->get_QueryID(&ID);
   AssertCOMError(spOrdersQuery, hr);

   m_lblQueryStatus.SetWindowText(RequestStatusToString(status));
}

void COrderPropertiesDlg::ReloadAccounts()
{
   // Clear all previous data
   m_cmbAccounts.ResetContent();

   ATL::CComPtr<ICQGAccounts> spAccounts;
   HRESULT hr = m_spCQGCEL->get_Accounts(&spAccounts);
   AssertCOMError(m_spCQGCEL, hr);

   long count;
   hr = spAccounts->get_Count(&count);
   AssertCOMError(spAccounts, hr);

   if (count == 0)
   {
      // No accounts
      return;
   }

   // Iterate over accounts collection and add them to the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   hr = spAccounts->get__NewEnum(&spEnumVariant);
   AssertCOMError(spAccounts, hr);

   std::vector<ATL::CComVariant> pItems(count);

   ULONG celtFetched;
   hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      ATLASSERT(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGAccount> spAccount;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGAccount), (void** )&spAccount);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      // Requests subscription for each instrument on 
      // which the current account has a position
      spAccount->put_AutoSubscribeInstruments(VARIANT_TRUE);

      // Add current account to the list
      InsertAccount(spAccount);
   }

   // Select the first account in the list
   if (m_cmbAccounts.GetCount() > 0)
   {
      m_cmbAccounts.SetCurSel(0);
   }
}

void COrderPropertiesDlg::InsertAccount(
                                 const ATL::CComPtr<ICQGAccount>& spAccount)
{
   // Create string for specified account to add to the list
   ATL::CComBSTR accName;
   HRESULT hr = spAccount->get_GWAccountName(&accName);
   AssertCOMError(spAccount, hr);

   ATL::CComBSTR FcmAccID;
   hr = spAccount->get_FcmAccountID(&FcmAccID);
   AssertCOMError(spAccount, hr);

   CString accString = ATL::CW2T(accName);
   accString = accString + _T(" (") + ATL::CW2T(FcmAccID) + _T(")");

   // Get account ID to map with item
   long accID;
   hr = spAccount->get_GWAccountID(&accID);
   AssertCOMError(spAccount, hr);

   // Add new account and map ID with item
   int row = m_cmbAccounts.AddString(accString);
   m_cmbAccounts.SetItemData(row, accID);
}

ATL::CComPtr<ICQGAccount> COrderPropertiesDlg::GetSelectedAccount() const
{
   ATL::CComPtr<ICQGAccounts> spAccounts;
   HRESULT hr = m_spCQGCEL->get_Accounts(&spAccounts);
   AssertCOMError(m_spCQGCEL, hr);

   long accID = static_cast<long>(m_cmbAccounts.GetItemData(m_cmbAccounts.GetCurSel()));

   ATL::CComPtr<ICQGAccount> spAccount;
   hr = spAccounts->get_Item(accID, &spAccount);
   AssertCOMError(spAccounts, hr);

   return spAccount;
}

CString COrderPropertiesDlg::GetDisplayPrice(
                                 const ATL::CComPtr<ICQGOrder> spOrder,
                                 double price) const
{
   assert(spOrder != NULL);

   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(price), &isValid);
   AssertCOMError(m_spCQGCEL, hr);

   // Get instrument to get price display format
   ATL::CComPtr<ICQGInstrument> spInstrument;
   hr = spOrder->get_Instrument(&spInstrument);
   AssertCOMError(spOrder, hr);

   CString displayPrice;
   if (isValid && spInstrument != NULL)
   {
      // Get display price
      ATL::CComBSTR displayPriceBSTR;
      hr = spInstrument->ToDisplayPrice(price, &displayPriceBSTR);
      AssertCOMError(spInstrument, hr);

      displayPrice = ATL::CW2T(displayPriceBSTR);
   }
   else
   {
      displayPrice = m_N_A;
   }

   return displayPrice;
}

void COrderPropertiesDlg::ShowOrders(
                                 const ATL::CComPtr<ICQGOrders>& spOrders)
{
   // Enable filtering
   m_chkFilter.EnableWindow(TRUE);

   long count;
   HRESULT hr = spOrders->get_Count(&count);
   AssertCOMError(spOrders, hr);

   if (count == 0)
   {
      // No orders
      return;
   }

   // Iterate over orders collection and add them to the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   hr = spOrders->get__NewEnum(&spEnumVariant);
   AssertCOMError(spOrders, hr);

   std::vector<ATL::CComVariant> pItems(count);

   ULONG celtFetched;
   hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      assert(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGOrder> spOrder;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGOrder), (void** )&spOrder);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      // Add current order to the list
      ShowOrder(m_lvOrders, spOrder);
   }

   // Select first item
   m_lvOrders.SetItemState(0, LVIS_SELECTED, LVIS_SELECTED);
}

void COrderPropertiesDlg::ShowOrder(
                                 CListCtrl& listView,            
                                 const ATL::CComPtr<ICQGOrder>& spOrder)
{
   ATL::CComBSTR gwOrderID;
   HRESULT hr = spOrder->get_GWOrderID(&gwOrderID);
   AssertCOMError(spOrder, hr);

   // Find row for specified order
   CString gwOrderIDCStr = ATL::CW2T(gwOrderID);
   LVFINDINFO findInfo;
   findInfo.flags = LVFI_STRING;
   findInfo.psz = gwOrderIDCStr;
   int row = listView.FindItem(&findInfo);

   // Add new one if not found
   if (row == -1)
   {
      row = listView.InsertItem(listView.GetItemCount(), _T(""));
   }

   ATL::CComBSTR instrumentName;
   hr = spOrder->get_InstrumentName(&instrumentName);
   AssertCOMError(spOrder, hr);

   eOrderSide side;
   hr = spOrder->get_Side(&side);
   AssertCOMError(spOrder, hr);

   eOrderType type;
   hr = spOrder->get_Type(&type);
   AssertCOMError(spOrder, hr);

   eOrderStatus status;
   hr = spOrder->get_GWStatus(&status);
   AssertCOMError(spOrder, hr);

   listView.SetItemText(row, ocID, gwOrderIDCStr);
   listView.SetItemText(row, ocInstrument, GetValueAsString(instrumentName));
   listView.SetItemText(row, ocSide, OrderSideToString(side));
   listView.SetItemText(row, ocType, OrderTypeToString(type));
   listView.SetItemText(row, ocState, OrderStatusToString(status));
}

void COrderPropertiesDlg::ShowProperties(
                                 const ATL::CComPtr<ICQGOrder>& spOrder)
{
   // Clear old data
   ClearProperties();

   // All other properties are accessible from properties collection
   ATL::CComPtr<ICQGOrderProperties> spProperties;
   HRESULT hr = spOrder->get_Properties(&spProperties);
   AssertCOMError(spOrder, hr);

   long count;
   hr = spProperties->get_Count(&count);
   AssertCOMError(spProperties, hr);

   // Iterate over order's properties collection and show them in the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   hr = spProperties->get__NewEnum(&spEnumVariant);
   AssertCOMError(spProperties, hr);

   std::vector<ATL::CComVariant> pItems(count);
   ULONG celtFetched;
   hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   int row = 0;
   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      ATLASSERT(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGOrderProperty> spProperty;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGOrderProperty), (void** )&spProperty);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      try
      {
         eOrderProperty propType;
         hr = spProperty->get_Type(&propType);
         AssertCOMError(spProperty, hr);

         ATL::CComVariant propValue;
         hr = spProperty->get_Value(&propValue);
         AssertCOMError(spProperty, hr);

         ATL::CComBSTR propName;
         hr = spProperty->get_Name(&propName);
         AssertCOMError(spProperty, hr);

         CString propValueStr;
         switch (propType)
         {
            case opParked:
            case opSynthetic:
            case opErrorFlag:
            case opAggressive:
               // Boolean values
               propValueStr = (propValue.boolVal ? m_Yes : m_No);
               break;
            case opOrderType:
               propValueStr = OrderTypeToString(static_cast<eOrderType>(propValue.intVal));
               break;
            case opDurationType:
               propValueStr = OrderDurationToString(static_cast<eOrderDuration>(propValue.intVal));
               break;
            case opSide:
               propValueStr = OrderSideToString(static_cast<eOrderSide>(propValue.intVal));
               break;
            case opGWStatus:
               propValueStr = OrderStatusToString(static_cast<eOrderStatus>(propValue.intVal));
               break;
            case opState:
               propValueStr = OrderLocalStateToString(static_cast<eOrderLocalState>(propValue.intVal));
               break;
            case opLastEvent:
               propValueStr = OrderEventToString(static_cast<eOrderEvent>(propValue.intVal));
               break;
            default:
               propValueStr = GetValueAsString(propValue);
               break;
         }

         m_lvOrderProps.InsertItem(row, ATL::CW2T(propName));
         m_lvOrderProps.SetItemText(row, opcValue, propValueStr);
         ++row;
      }
      catch (const std::exception&)
      {
      }
   }

   // The following properties aren't accessible from properties collection
   VARIANT_BOOL isFinal;
   hr = spOrder->get_IsFinal(&isFinal);
   AssertCOMError(spOrder, hr);

   VARIANT_BOOL canBeModified;
   hr = spOrder->get_CanBeModified(&canBeModified);
   AssertCOMError(spOrder, hr);

   VARIANT_BOOL isExternal;
   hr = spOrder->get_IsExternal(&isExternal);
   AssertCOMError(spOrder, hr);

   DATE timestamp;
   hr = spOrder->get_Timestamp(&timestamp);
   AssertCOMError(spOrder, hr);

   DATE serverTimestamp;
   hr = spOrder->get_ServerTimestamp(&serverTimestamp);
   AssertCOMError(spOrder, hr);

   // Show them in the list
   row = m_lvOrderProps.InsertItem(row, _T("IsFinal"));
   m_lvOrderProps.SetItemText(row, opcValue, (isFinal ? m_Yes : m_No));

   row = m_lvOrderProps.InsertItem(row + 1, _T("CanBeModified"));
   m_lvOrderProps.SetItemText(row, opcValue, (canBeModified ? m_Yes : m_No));

   row = m_lvOrderProps.InsertItem(row + 1, _T("IsExternal"));
   m_lvOrderProps.SetItemText(row, opcValue, (isExternal ? m_Yes : m_No));

   row = m_lvOrderProps.InsertItem(row + 1, _T("Timestamp"));
   m_lvOrderProps.SetItemText(row, opcValue, GetValueAsString(timestamp, VT_DATE));

   row = m_lvOrderProps.InsertItem(row + 1, _T("ServerTimestamp"));
   m_lvOrderProps.SetItemText(row, opcValue, GetValueAsString(serverTimestamp, VT_DATE));
}

void COrderPropertiesDlg::ShowFills(
                                 const ATL::CComPtr<ICQGOrder>& spOrder)
{
   // Get order fills
   ATL::CComPtr<ICQGFills> spFills;
   HRESULT hr = spOrder->get_Fills(&spFills);
   AssertCOMError(spOrder, hr);

   long count;
   hr = spFills->get_Count(&count);
   AssertCOMError(spFills, hr);

   // Add order fills to the list
   for (int i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGFill> spFill;
      hr = spFills->get_ItemByIndex(i, &spFill);
      AssertCOMError(spFills, hr);
      
      ShowFill(spOrder, spFill);
   }
}

void COrderPropertiesDlg::ShowFill(
                                 const ATL::CComPtr<ICQGOrder>& spOrder,
                                 const ATL::CComPtr<ICQGFill>& spFill)
{
   DATE timeStamp;
   HRESULT hr = spFill->get_ServerTimestamp(&timeStamp);
   AssertCOMError(spFill, hr);

   ATL::CComBSTR fillID;
   hr = spFill->get_Id(&fillID);
   AssertCOMError(spFill, hr);

   eFillStatus status;
   hr = spFill->get_Status(&status);
   AssertCOMError(spFill, hr);

   eOrderSide side;
   hr = spFill->get_Side(0, &side);
   AssertCOMError(spFill, hr);

   long quantity;
   hr = spFill->get_Quantity(0, &quantity);
   AssertCOMError(spFill, hr);

   ATL::CComBSTR fullName;
   hr = spOrder->get_InstrumentName(&fullName);
   AssertCOMError(spOrder, hr);

   // Get and validate price
   double price;
   hr = spFill->get_Price(0, &price);
   AssertCOMError(spFill, hr);

   // Show fill data
   int row = m_lvOrderFills.InsertItem(m_lvOrderFills.GetItemCount(), _T(""));
   m_lvOrderFills.SetItemText(row, ofcPrice, GetValueAsString(price));
   m_lvOrderFills.SetItemText(row, ofcInstrument, ATL::CW2T(fullName));
   m_lvOrderFills.SetItemText(row, ofcTimestamp, GetValueAsString(timeStamp, VT_DATE));
   m_lvOrderFills.SetItemText(row, ofcFillID, ATL::CW2T(fillID));
   m_lvOrderFills.SetItemText(row, ofcStatus, FillStatusToString(status));
   
   if (side == osdBuy)
   {
      m_lvOrderFills.SetItemText(row, ofcBought, GetValueAsString(quantity));
   }
   else /*osdSell*/
   {
      // If UseOrderSide in APIConfiguration is set, order side can be only osdBuy and osdSell
      m_lvOrderFills.SetItemText(row, ofcSold, GetValueAsString(quantity));
   }

   // Show legs' data
   long legCount;
   hr = spFill->get_LegCount(&legCount);
   AssertCOMError(spFill, hr);

   for (int i = 1; i <= legCount; ++i)
   {
      hr = spFill->get_Quantity(i, &quantity);
      AssertCOMError(spFill, hr);

      hr = spFill->get_Side(i, &side);
      AssertCOMError(spFill, hr);

      ATL::CComBSTR displayPrice;
      hr = spFill->get_DisplayPrice(i, &displayPrice);
      AssertCOMError(spFill, hr);

      ATL::CComBSTR currency;
      hr = spFill->get_Currency(i, &currency);
      AssertCOMError(spFill, hr);

      ATL::CComBSTR gwTradeID;
      hr = spFill->get_GWTradeId(i, &gwTradeID);
      AssertCOMError(spFill, hr);

      VARIANT_BOOL isAggressive;
      hr = spFill->get_Aggressive(i, &isAggressive);
      AssertCOMError(spFill, hr);

      hr = spFill->get_InstrumentName(i, &fullName);
      AssertCOMError(spFill, hr);

      row = m_lvOrderFills.InsertItem(row + 1, _T(""));

      if (side == osdBuy)
      {
         m_lvOrderFills.SetItemText(row, ofcBought, GetValueAsString(quantity));
      }
      else
      {
         m_lvOrderFills.SetItemText(row, ofcSold, GetValueAsString(quantity));   
      }
      m_lvOrderFills.SetItemText(row, ofcInstrument, ATL::CW2T(fullName));
      m_lvOrderFills.SetItemText(row, ofcPrice, ATL::CW2T(displayPrice));
      m_lvOrderFills.SetItemText(row, ofcCurrency, GetValueAsString(currency));
      m_lvOrderFills.SetItemText(row, ofcGWTradeID, GetValueAsString(gwTradeID));
      m_lvOrderFills.SetItemText(row, ofcAggressive, (isAggressive ? m_Yes : m_No));
   }

   row = m_lvOrderFills.InsertItem(row + 1, _T(""));
}

void COrderPropertiesDlg::ShowTransactions(
                                 const ATL::CComPtr<ICQGOrder>& spOrder)
{
   // Get order transactions
   ATL::CComPtr<ICQGTransactions> spTransactions;
   HRESULT hr = spOrder->get_Transactions(&spTransactions);
   AssertCOMError(spOrder, hr);

   long count;
   hr = spTransactions->get_Count(&count);
   AssertCOMError(spTransactions, hr);

   // Add order transactions to the list
   for (int i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGTransaction> spTransaction;
      hr = spTransactions->get_ItemByIndex(i, &spTransaction);
      AssertCOMError(spTransactions, hr);

      ShowTransaction(spTransaction);
   }
}

void COrderPropertiesDlg::ShowTransaction(
                                 const ATL::CComPtr<ICQGTransaction>& spTransaction)
{
   DATE timeStamp;
   HRESULT hr = spTransaction->get_ServerTimestamp(&timeStamp);
   AssertCOMError(spTransaction, hr);

   ATL::CComBSTR transID;
   hr = spTransaction->get_Id(&transID);
   AssertCOMError(spTransaction, hr);

   eOrderEvent status;
   hr = spTransaction->get_Status(&status);
   AssertCOMError(spTransaction, hr);

   ATL::CComBSTR rejReason;
   hr = spTransaction->get_RejectReason(&rejReason);
   AssertCOMError(spTransaction, hr);

   // Get corresponding fill object
   ATL::CComPtr<ICQGFill> spFill;
   hr = spTransaction->get_Fill(&spFill);
   AssertCOMError(spTransaction, hr);

   // Check whether transaction is fill transaction
   CString strFillID = m_N_A;
   if (spFill)
   {
      ATL::CComBSTR fillID;
      hr = spFill->get_Id(&fillID);
      AssertCOMError(spFill, hr);

      strFillID = ATL::CW2T(fillID);
   }
      
   // Show transaction data
   int row = m_lvOrderTransactions.InsertItem(m_lvOrderTransactions.GetItemCount(), _T(""));
   m_lvOrderTransactions.SetItemText(row, otcTimestamp, GetValueAsString(timeStamp, VT_DATE));
   m_lvOrderTransactions.SetItemText(row, otcTransactionID, GetValueAsString(transID));
   m_lvOrderTransactions.SetItemText(row, otcStatus, OrderEventToString(status));
   m_lvOrderTransactions.SetItemText(row, otcFillID, strFillID);
   m_lvOrderTransactions.SetItemText(row, otcRejectReason, ATL::CW2T(rejReason));

   // Show modified properties of the modification transaction
   ShowModifiedProperties(row, spTransaction);

   m_lvOrderTransactions.InsertItem(m_lvOrderTransactions.GetItemCount(), _T(""));
}

void COrderPropertiesDlg::ShowModifiedProperties(
                                 int startRow,
                                 const ATL::CComPtr<ICQGTransaction>& spTransaction)
{
   // Get transaction properties
   ATL::CComPtr<ICQGModifiedProperties> spProperties;
   HRESULT hr = spTransaction->get_Properties(&spProperties);
   AssertCOMError(spTransaction, hr);

   long count;
   hr = spProperties->get_Count(&count);
   AssertCOMError(spProperties, hr);

   // Add transaction modified properties to the list
   for (int i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGModifiedProperty> spProperty;
      hr = spProperties->get_ItemByIndex(i, &spProperty);
      AssertCOMError(spProperties, hr);

      startRow = m_lvOrderTransactions.InsertItem(startRow + 1, _T(""));
      ShowModifiedProperty(startRow, spProperty);

   }
}

void COrderPropertiesDlg::ShowModifiedProperty(
                                 int row,
                                 const ATL::CComPtr<ICQGModifiedProperty>& spProperty)
{
   ATL::CComBSTR name;
   HRESULT hr = spProperty->get_Name(&name);
   AssertCOMError(spProperty, hr);

   ATL::CComVariant newValue;
   hr = spProperty->get_NewValue(&newValue);
   AssertCOMError(spProperty, hr);

   ATL::CComVariant prevValue;
   hr = spProperty->get_PreviousValue(&prevValue);
   AssertCOMError(spProperty, hr);

   m_lvOrderTransactions.SetItemText(row, otcPropName, ATL::CW2T(name));
   m_lvOrderTransactions.SetItemText(row, otcPropCurrentValue, GetValueAsString(newValue));
   m_lvOrderTransactions.SetItemText(row, otcPropPreviousValue, GetValueAsString(prevValue));
}

void COrderPropertiesDlg::ShowStrategyLegFills(
                                 const ATL::CComPtr<ICQGTransaction>& spTransaction)
{
   if (!spTransaction)
   {
      return;
   }

   // Get strategy leg fills for transaction
   ATL::CComPtr<ICQGStrategyLegFills> spStrategyLegFills;
   HRESULT hr = spTransaction->get_StrategyLegFills(&spStrategyLegFills);
   AssertCOMError(spTransaction, hr);

   // Get transaction strategy leg fills count
   long count;
   hr = spStrategyLegFills->get_Count(&count);
   AssertCOMError(spStrategyLegFills, hr);

   // Show strategy leg fills data
   for (int index = 0; index < count; ++index)
   {
      // Get strategy leg fill by its index
      ATL::CComPtr<ICQGStrategyLegFill> spStrategyLegfill;
      hr = spStrategyLegFills->get_ItemByIndex(index, &spStrategyLegfill);
      AssertCOMError(spStrategyLegFills, hr);

      // Show strategy leg fill
      ShowStrategyLegFill(spStrategyLegfill);
   }
}

void COrderPropertiesDlg::ShowStrategyLegFill(
                                 const ATL::CComPtr<ICQGStrategyLegFill>& spStrategyLegFill)
{
   // Get strategy leg fill original order Id
   ATL::CComBSTR originalOrderId;
   HRESULT hr = spStrategyLegFill->get_OriginalOrderID(&originalOrderId);
   AssertCOMError(spStrategyLegFill, hr);

   // Get strategy leg fill execution Id
   ATL::CComBSTR executionId;
   hr = spStrategyLegFill->get_ExecutionId(&executionId);
   AssertCOMError(spStrategyLegFill, hr);

   // Get quantity
   long quantity;
   hr = spStrategyLegFill->get_Quantity(&quantity);
   AssertCOMError(spStrategyLegFill, hr);

   // Show strategy leg fill data
   const int row = m_lvStrategyLegFills.InsertItem(m_lvStrategyLegFills.GetItemCount(), _T(""));
   m_lvStrategyLegFills.SetItemText(row, slfcOriginalOrderId, ATL::CW2T(originalOrderId));
   m_lvStrategyLegFills.SetItemText(row, slfcExecutionId, ATL::CW2T(executionId));
   m_lvStrategyLegFills.SetItemText(row, slfcQuantity, GetValueAsString(quantity));
}

void COrderPropertiesDlg::ShowStrategyOrderLegOrders(
                                 const ATL::CComPtr<ICQGOrder>& spOrder)
{
   // Get GUID of strategy order
   ATL::CComBSTR strategyOrderGUID;
   HRESULT hr = spOrder->get_GUID(&strategyOrderGUID);
   AssertCOMError(spOrder, hr);

   // Get all orders
   ATL::CComPtr<ICQGOrders> spOrders;
   hr = m_spOrdersQuery->get_Orders(&spOrders);
   AssertCOMError(m_spOrdersQuery, hr);

   // Get orders sub collection by StrategyOrderGUID property
   ATL::CComPtr<ICQGOrders> spLegOrders;
   hr = spOrders->SelectByProperty(opStrategyOrderGUID, ATL::CComVariant(strategyOrderGUID), &spLegOrders);
   AssertCOMError(spOrders, hr);

   // Get count of leg orders
   long count;
   hr = spLegOrders->get_Count(&count);
   AssertCOMError(spLegOrders, hr);

   if (count > 0)
   {
      // Iterate over order's collection
      ATL::CComPtr<IEnumVARIANT> spEnumVariant;
      hr = spLegOrders->get__NewEnum(&spEnumVariant);
      AssertCOMError(spLegOrders, hr);

      std::vector<ATL::CComVariant> pItems(count);

      ULONG celtFetched;
      hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
      AssertCOMError(spEnumVariant, hr);

      for (std::vector<ATL::CComVariant>::const_iterator it = pItems.begin(); it != pItems.end(); ++it)
      {
         ATLASSERT(it->vt == VT_DISPATCH);
         // Retrieve order
         ATL::CComPtr<ICQGOrder> spLegOrder;
         hr = it->pdispVal->QueryInterface(__uuidof(ICQGOrder), (void**)&spLegOrder);
         AssertCOMError<IDispatch>(it->pdispVal, hr);

         // Show order
         ShowOrder(m_lvStrategyLegOrders, spLegOrder);
      }
   }
}

void COrderPropertiesDlg::ShowHangOrderIds(
                                 const ATL::CComPtr<ICQGTransaction> spTransaction)
{
   if (!spTransaction)
   {
      return;
   }

   // Get hang orders ids
   ATL::CComPtr<ICQGStringCollection> spHangOrderIds;
   HRESULT hr = spTransaction->get_HangOrderIds(&spHangOrderIds);
   AssertCOMError(spTransaction, hr);

   // Get count of hang orders ids
   long count;
   hr = spHangOrderIds->get_Count(&count);
   AssertCOMError(spHangOrderIds, hr);

   for (int index = 0; index < count; ++index)
   {
      ATL::CComBSTR hangOrderId;
      hr = spHangOrderIds->get_Item(index, &hangOrderId);
      AssertCOMError(spHangOrderIds, hr);

      // Show hang order id
      const int row = m_lvHangOrderIds.InsertItem(m_lvHangOrderIds.GetItemCount(), _T(""));
      m_lvHangOrderIds.SetItemText(row, oidcHangOrderId, ATL::CW2T(hangOrderId));
   }
}

CString COrderPropertiesDlg::GetValueAsString(
                                 ATL::CComVariant value,
                                 VARENUM valueType/*= VT_EMPTY*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   AssertCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return VariantToString(value);
   }
   return m_N_A;
}

CString COrderPropertiesDlg::OrderTypeToString(
                                 eOrderType type)
{
   CString typeString;

   switch (type)
   {
      case otLimit:
         typeString = _T("Limit");
         break;
      case otMarket:
         typeString = _T("Market");
         break;
      case otStop:
         typeString = _T("Stop");
         break;
      case otStopLimit:
         typeString = _T("Stop Limit");
         break;
      case otUndefined:
         typeString = _T("Undefined");
         break;
      default:
         assert(!"Unknown order type");
   }

   return typeString;
}

CString COrderPropertiesDlg::OrderLocalStateToString(
                                 eOrderLocalState state)
{
   CString stateString;

   switch (state)
   {
      case olsActivateSent:
         stateString = _T("Activate Sent");
         break;
      case olsAtGW:
         stateString = _T("At GW");
         break;
      case olsCancelSent:
         stateString = _T("Cancel Sent");
         break;
      case olsInClient:
         stateString = _T("In Client");
         break;
      case olsInQueue:
         stateString = _T("In Queue");
         break;
      case olsModifySent:
         stateString = _T("Modify Sent");
         break;
      case olsNotSent:
         stateString = _T("Not Sent");
         break;
      case olsQueueTimeout:
         stateString = _T("Queue Timeout");
         break;
      case olsRemoved:
         stateString = _T("Removed");
         break;
      default:
         assert(!"Unknown order local state");
   }

   return stateString;
}

CString COrderPropertiesDlg::OrderStatusToString(
                                 eOrderStatus status)
{
   CString statusString;

   switch (status)
   {
      case osActiveAt:
         statusString = _T("Active At");
         break;
      case osBusted:
         statusString = _T("Busted");
         break;
      case osCanceled:
         statusString = _T("Canceled");
         break;
      case osContingent:
         statusString = _T("Contingent");
         break;
      case osDisconnected:
         statusString = _T("Disconnected");
         break;
      case osExpired:
         statusString = _T("Expired");
         break;
      case osFilled:
         statusString = _T("Filled");
         break;
      case osInCancel:
         statusString = _T("In Cancel");
         break;
      case osInClient:
         statusString = _T("In Client");
         break;
      case osInModify:
         statusString = _T("In Modify");
         break;
      case osInOrderBook:
         statusString = _T("In Order Book");
         break;
      case osInTransit:
         statusString = _T("In Transit");
         break;
      case osInTransitTimeout:
         statusString = _T("In Transit Timeout");
         break;
      case osNotSent:
         statusString = _T("Not Sent");
         break;
      case osParked:
         statusString = _T("Parked");
         break;
      case osRejectFCM:
         statusString = _T("Reject FCM");
         break;
      case osRejectGW:
         statusString = _T("Reject GW");
         break;
      default:
         assert(!"Unknown order status");
   }

   return statusString;
}

CString COrderPropertiesDlg::OrderStatusFilterToString(
                                 eOrderStatusFilter statusFilter)
{
   CString statusFilterString;

   switch (statusFilter)
   {
      case osfAll:
         statusFilterString = _T("All");
         break;
      case osfCanceled:
         statusFilterString = _T("Canceled");
         break;
      case osfException:
         statusFilterString = _T("Exception");
         break;
      case osfFilled:
         statusFilterString = _T("Filled");
         break;
      case osfFinal:
         statusFilterString = _T("Final");
         break;
      case osfNotFinal:
         statusFilterString = _T("NotFinal");
         break;
      case osfParked:
         statusFilterString = _T("Parked");
         break;
      default:
         assert(!"Unknown order status filter");
   }

   return statusFilterString;
}

CString COrderPropertiesDlg::OrderEventToString(
                                 eOrderEvent event)
{
   CString eventString;

   switch (event)
   {
      case oeAckCan:
         eventString = _T("AckCan");
         break;
      case oeAckMod:
         eventString = _T("AckMod");
         break;
      case oeAckPlace:
         eventString = _T("AckPlace");
         break;
      case oeActiveAt:
         eventString = _T("ActiveAt");
         break;
      case oeCancelQueueTimeout:
         eventString = _T("CancelQueueTimeout");
         break;
      case oeCancelSent:
         eventString = _T("CancelSent");
         break;
      case oeDisconnected:
         eventString = _T("Disconnected");
         break;
      case oeExpired:
         eventString = _T("Expired");
         break;
      case oeFill:
         eventString = _T("Fill");
         break;
      case oeFillBust:
         eventString = _T("FillBust");
         break;
      case oeFillCan:
         eventString = _T("FillCan");
         break;
      case oeFillMod:
         eventString = _T("FillMod");
         break;
      case oeInCan:
         eventString = _T("InCan");
         break;
      case oeInCanTmout:
         eventString = _T("InCanTmout");
         break;
      case oeInClient:
         eventString = _T("InClient");
         break;
      case oeInMod:
         eventString = _T("InMod");
         break;
      case oeInModTmout:
         eventString = _T("InModTmout");
         break;
      case oeInQueue:
         eventString = _T("InQueue");
         break;
      case oeInTransit:
         eventString = _T("InTransit");
         break;
      case oeInTransTmout:
         eventString = _T("InTransTmout");
         break;
      case oeLinkChg:
         eventString = _T("LinkChg");
         break;
      case oeModifyQueueTimeout:
         eventString = _T("ModifyQueueTimeout");
         break;
      case oeModifySent:
         eventString = _T("ModifySent");
         break;
      case oePark:
         eventString = _T("Park");
         break;
      case oeQueueTimeout:
         eventString = _T("QueueTimeout");
         break;
      case oeRejCan:
         eventString = _T("RejCan");
         break;
      case oeRejectFCM:
         eventString = _T("RejectFCM");
         break;
      case oeRejectGW:
         eventString = _T("RejectGW");
         break;
      case oeRejMod:
         eventString = _T("RejMod");
         break;
      case oeRemoved:
         eventString = _T("Removed");
         break;
      case oeSyntheticActivated:
         eventString = _T("SyntheticActivated");
         break;
      case oeUndefined:
         eventString = _T("Undefined");
         break;
      case oeSyntheticFailed:
         eventString = _T("SyntheticFailed");
         break;
      case oeSyntheticOverFill:
         eventString = _T("SyntheticOverFill");
         break;
      case oeSyntheticHang:
         eventString = _T("SyntheticHang");
         break;
      case oeInfoChanged:
         eventString = _T("InfoChanged");
         break;
      default:
         assert(!"Unknown order event");
   }

   return eventString;
}

CString COrderPropertiesDlg::OrderDurationToString(
                                 eOrderDuration duration)
{
   CString durationString;

   switch (duration)
   {
      case odDay:
         durationString = _T("Day");
         break;
      case odFAK:
         durationString = _T("FAK");
         break;
      case odFOK:
         durationString = _T("FOK");
         break;
      case odGoodTillCanceled:
         durationString = _T("Good Till Canceled");
         break;
      case odGoodTillDate:
         durationString = _T("Good Till Date");
         break;
      case odUndefined:
         durationString = _T("Undefined");
         break;
      default:
         assert(!"Unknown order duration");
   }

   return durationString;
}

CString COrderPropertiesDlg::OrderSideToString(
                                 eOrderSide side)
{
   CString sideString;

   switch (side)
   {
      case osdBuy:
         sideString = _T("Buy");
         break;
      case osdSell:
         sideString = _T("Sell");
         break;
      case osdUndefined:
         sideString = _T("Undefined");
         break;
      default:
         assert(!"Unknown order side");
   }

   return sideString;
}

CString COrderPropertiesDlg::RequestStatusToString(
                                 eRequestStatus status)
{
   CString  statusString;

   switch (status)
   {
      case rsInProgress:
         statusString = _T("In Progress");
         break;
      case rsSuccess:
         statusString = _T("Success");
         break;
      case rsFailed:
         statusString = _T("Failed");
         break;
      case rsCanceled:
         statusString = _T("Canceled");
         break;
      default:
         assert(!"Unknown request status");
   }

   return statusString;
}

CString COrderPropertiesDlg::FillStatusToString(
                                 eFillStatus fillStatus)
{
   CString  statusString;

   switch (fillStatus)
   {
      case fsBusted:
         statusString = _T("Busted");
         break;
      case fsCanceled:
         statusString = _T("Canceled");
         break;
      case fsModified:
         statusString = _T("Modified");
         break;
      case fsNormal:
         statusString = _T("Normal");
         break;
      default:
         assert(!"Unknown fill status");
   }

   return statusString;
}

CString COrderPropertiesDlg::ConnectionStatusToString(
                                 eConnectionStatus status)
{
   CString  statusString;

   switch (status)
   {
   case csConnectionDelayed:
      statusString = _T("delayed");
      break;
   case csConnectionDown:
      statusString = _T("down");
      break;
   case csConnectionUp:
      statusString = _T("up");
      break;
   case csConnectionTrouble:
      statusString = _T("trouble");
      break;
   case csConnectionNotLoggedOn:
      statusString = _T("not logged on");
      break;
   default:
      assert(!"Unknown connection status");
   }

   return statusString;
}