// OrderPropertiesDlg.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include "Common.h"
#include "OrderProperties.h"
#include "OrderPropertiesDlg.h"
#include "QueryParametersDlg.h"
#include "OrdersFilterParametersDlg.h"
#include "GWLogonDlg.h"

const CString COrderPropertiesDlg::m_N_A = _T("N/A");
const CString COrderPropertiesDlg::m_Yes = _T("Yes");
const CString COrderPropertiesDlg::m_No = _T("No");
const CString COrderPropertiesDlg::m_FilterText = _T("Filter...");

COrderPropertiesDlg::COrderPropertiesDlg(CWnd* pParent /*=NULL*/)
   : CDialog(COrderPropertiesDlg::IDD, pParent)
{
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void COrderPropertiesDlg::DoDataExchange(
                                 CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_ST_DATA_CONNECTION, m_lblDataConnection);
   DDX_Control(pDX, IDC_ST_GW_CONNECTION, m_lblGWConnection);
   DDX_Control(pDX, IDC_CMB_ACCOUNTS, m_cmbAccounts);
   DDX_Control(pDX, IDC_BTN_QUERY_ORDERS, m_btnQueryOrders);
   DDX_Control(pDX, IDC_BTN_QUERY_CANCEL, m_btnCancelQuery);
   DDX_Control(pDX, IDC_ST_QUERY_STATUS, m_lblQueryStatus);
   DDX_Control(pDX, IDC_LSV_ORDERS, m_lvOrders);
   DDX_Control(pDX, IDC_LSV_ORDER_PROPS, m_lvOrderProps);
   DDX_Control(pDX, IDC_LSV_ORDER_FILLS, m_lvOrderFills);
   DDX_Control(pDX, IDC_LSV_ORDER_TRANSACTIONS, m_lvOrderTransactions);
   DDX_Control(pDX, IDC_CHK_FILTER, m_chkFilter);
   DDX_Control(pDX, IDC_BTN_LOGONGW, m_btnGWLogOnOff);
   DDX_Control(pDX, IDC_ST_GWUSERNAME, m_lblGWUsername);
   DDX_Control(pDX, IDC_LSV_STRATEGY_LEG_FILLS, m_lvStrategyLegFills);
   DDX_Control(pDX, IDC_LSV_STRATEGY_ORDERS_LEG_ORDERS, m_lvStrategyLegOrders);
   DDX_Control(pDX, IDC_LSV_HANG_ORDER_IDS, m_lvHangOrderIds);
}

BEGIN_MESSAGE_MAP(COrderPropertiesDlg, CDialog)
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_BN_CLICKED(IDC_BTN_QUERY_ORDERS, OnBnClickedBtnQueryOrders)
   ON_BN_CLICKED(IDC_BTN_QUERY_CANCEL, OnBnClickedBtnQueryCancel)
   ON_NOTIFY(LVN_ITEMCHANGED, IDC_LSV_ORDERS, OnLvnItemChangedLsvOrders)
   ON_BN_CLICKED(IDC_CHK_FILTER, OnBnClickedChkFilter)
   ON_WM_DESTROY()
   ON_BN_CLICKED(IDC_BTN_LOGONGW, OnBnClickedBtnLogongw)
   ON_NOTIFY(LVN_ITEMCHANGED, IDC_LSV_ORDER_TRANSACTIONS, OnLvnItemchangedLsvOrderTransactions)
END_MESSAGE_MAP()

BOOL COrderPropertiesDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   // Set the icon for this dialog.  The framework does this automatically
   //  when the application's main window is not a dialog
   SetIcon(m_hIcon, TRUE);       // Set big icon
   SetIcon(m_hIcon, FALSE);      // Set small icon

   try
   {
      InitializeControls();
      InitializeCQGCEL();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
  
   return TRUE;  // return TRUE  unless you set the focus to a control
}

void COrderPropertiesDlg::OnPaint() 
{
   if (IsIconic())
   {
      CPaintDC dc(this); // device context for painting

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      // Center icon in client rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      // Draw the icon
      dc.DrawIcon(x, y, m_hIcon);
   }
   else
   {
      CDialog::OnPaint();
   }
}

HCURSOR COrderPropertiesDlg::OnQueryDragIcon()
{
   return static_cast<HCURSOR>(m_hIcon);
}

void COrderPropertiesDlg::OnDestroy()
{
   try
   {
      if (m_spCQGCEL)
      {
         HRESULT hr = ICQGCELDispEventImpl::DispEventUnadvise(m_spCQGCEL);
         assert(SUCCEEDED(hr));

         hr = m_spCQGCEL->Shutdown();
         AssertCOMError(m_spCQGCEL, hr);
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}

void COrderPropertiesDlg::OnOK()
{
   // Override to prevent dialog from closing on Enter
}

void COrderPropertiesDlg::OnBnClickedBtnQueryOrders()
{
   try
   {
      CQueryParametersDlg dlg(m_spCQGCEL, GetSelectedAccount(), this);
      if (dlg.DoModal() == IDOK)
      {
         ClearOrders();

         m_spOrdersQuery = dlg.GetOrdersQuery();

         m_lblQueryStatus.SetWindowText(RequestStatusToString(rsInProgress));

         m_IsInQuery = true;
         EnableControls(true);
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void COrderPropertiesDlg::OnBnClickedBtnQueryCancel()
{
   try
   {
      if (!m_IsInQuery)
      {
         return;
      }

      HRESULT hr = m_spOrdersQuery->Cancel();
      AssertCOMError(m_spOrdersQuery, hr);

      m_IsInQuery = false;
      EnableControls(true);
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void COrderPropertiesDlg::OnLvnItemChangedLsvOrders(NMHDR *pNMHDR, LRESULT *pResult)
{
   try
   {
      // Clear old data
      ClearProperties();
      ClearFills();
      ClearTransactions();
      ClearStrategyLegOrders();

      LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);
      
      if ((pNMLV->uNewState & LVIS_SELECTED) != LVIS_SELECTED)
      {
         // If there is no new item selected then do nothing
         return;
      }

      // Get selected order
      ATL::CComPtr<ICQGOrders> spOrders;
      HRESULT hr = m_spOrdersQuery->get_Orders(&spOrders);
      AssertCOMError(m_spOrdersQuery, hr);

      ATL::CComBSTR gwOrderIDCStr = ATL::CT2W(m_lvOrders.GetItemText(pNMLV->iItem, ocID));
      ATL::CComPtr<ICQGOrder> spOrder;
      
      hr = spOrders->get_Item(gwOrderIDCStr, &spOrder);
      AssertCOMError(spOrders, hr);

      // Store the selected order
      m_spSelectedOrder = spOrder;

      // Show properties for selected order
      ShowProperties(spOrder);

      // Add selected order's fills to the list
      ShowFills(spOrder);

      // Add selected order's transactions to the list
      ShowTransactions(spOrder);

      // If it is strategy order,add selected order's leg orders to the list
      ShowStrategyOrderLegOrders(spOrder);

      *pResult = 0;
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void COrderPropertiesDlg::OnBnClickedChkFilter()
{
   try
   {
      if (m_spOrdersQuery == NULL)
      {
         // No orders
         return;
      }

      if (m_chkFilter.GetCheck() == BST_UNCHECKED)
      {
         COrdersFilterParametersDlg dlg(m_spCQGCEL, this);
         if (dlg.DoModal() == IDOK)
         {        
            // Get all orders
            ATL::CComPtr<ICQGOrders> spAllOrders;
            HRESULT hr = m_spOrdersQuery->get_Orders(&spAllOrders);
            AssertCOMError(m_spOrdersQuery, hr);

            // Get filter parameters
            COrdersFilterParametersDlg::eFilterType filterType = dlg.GetFilterType();
            
            // Get filtered orders
            ATL::CComPtr<ICQGOrders> spOrders;
            CString filterText = _T("Filtered by ");
            switch(filterType)
            {
               case COrdersFilterParametersDlg::ftInstrument:
                  {
                     CString fullName = dlg.GetFullName();
                     hr = spAllOrders->SelectByInstrumentName(ATL::CT2W(dlg.GetFullName()), &spOrders);
                     AssertCOMError(m_spOrdersQuery, hr);

                     filterText += _T("instrument name : ");
                     filterText += fullName;
                     break;
                  }
               case COrdersFilterParametersDlg::ftType:
                  {
                     eOrderType type = dlg.GetOrderType();
                     hr = spAllOrders->SelectByOrderType(type, &spOrders);
                     AssertCOMError(m_spOrdersQuery, hr);

                     filterText += _T("order type : ");
                     filterText += OrderTypeToString(type);
                     break;
                  }
               case COrdersFilterParametersDlg::ftStatus:
                  {
                     eOrderStatus status = dlg.GetOrderStatus();
                     hr = spAllOrders->SelectByOrderStatus(status, &spOrders);
                     AssertCOMError(m_spOrdersQuery, hr);

                     filterText += _T("order status : ");
                     filterText += OrderStatusToString(status);
                     break;
                  }
               case COrdersFilterParametersDlg::ftSide:
                  {
                     eOrderSide side = dlg.GetOrderSide();
                     hr = spAllOrders->SelectBySide(side, &spOrders);
                     AssertCOMError(m_spOrdersQuery, hr);

                     filterText += _T("order side : ");
                     filterText += OrderSideToString(side);
                     break;
                  }
               default:
                  assert(!"Unknown orders filter parameter.");
            }
            // Clear old data
            ClearOrders();

            // Show filtered orders
            ShowOrders(spOrders);

            m_chkFilter.SetWindowText(filterText);
            m_chkFilter.SetCheck(BST_CHECKED);
         }
      }
      else
      {
         // Get all orders
         ATL::CComPtr<ICQGOrders> spOrders;
         HRESULT hr = m_spOrdersQuery->get_Orders(&spOrders);
         AssertCOMError(m_spOrdersQuery, hr);

         // Clear old data
         ClearOrders();

         // Show all orders
         ShowOrders(spOrders);

         ClearFilterCheckBox();
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void COrderPropertiesDlg::OnBnClickedBtnLogongw()
{
   try
   {
      if (!m_IsGWLoggedOn)
      {
         VARIANT_BOOL vBool;
         HRESULT hr = m_spCQGCEL->get_OTPRequired(&vBool);
         AssertCOMError(m_spCQGCEL, hr);
       
         CGWLogonDlg dlg(this, vBool == VARIANT_TRUE);
         if (dlg.DoModal() == IDOK)
         {        
            hr = m_spCQGCEL->GWOTPLogon(dlg.GetUserName(), dlg.GetPassword(), dlg.GetOTPassword());
            AssertCOMError(m_spCQGCEL, hr);
         }
      }
      else
      {
         HRESULT hr = m_spCQGCEL->GWLogoff();
         AssertCOMError(m_spCQGCEL, hr);
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void COrderPropertiesDlg::OnLvnItemchangedLsvOrderTransactions(NMHDR *pNMHDR, LRESULT *pResult)
{
   try
   {
		// Clear strategy leg fills list
      ClearStrategyLegFills();
      ClearHangOrderIds();
		
      LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);

      if ((pNMLV->uNewState & LVIS_SELECTED) != LVIS_SELECTED)
      {
         // If there is no new item selected then do nothing
         return;
      }

      // Get selected order's transactions
      ATL::CComPtr<ICQGTransactions> spTransactions;
      HRESULT hr = m_spSelectedOrder->get_Transactions(&spTransactions);
      AssertCOMError(m_spSelectedOrder, hr);

      // Get selected transaction
      ATL::CComBSTR transactionId = ATL::CT2W(m_lvOrderTransactions.GetItemText(pNMLV->iItem, otcTransactionID));
      ATL::CComPtr<ICQGTransaction> spTransaction;
      hr = spTransactions->get_Item(transactionId, &spTransaction);
      AssertCOMError(spTransaction, hr);

      // Show selected transactions strategy leg fills and hang orders ids
      ShowStrategyLegFills(spTransaction);
      ShowHangOrderIds(spTransaction);

      *pResult = 0;
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}
