// OrderPropertiesDlg.h : header file
//

#pragma once

class COrderPropertiesDlg;
/**
\typedef ATL::IDispEventImpl<1, COrderPropertiesDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              COrderPropertiesDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

/// \brief COrderPropertiesDlg dialog. 
/// \brief Used to show order properties.
class COrderPropertiesDlg : public CDialog, public ICQGCELDispEventImpl
{

public:
   /// \brief Constructor
   /// \param pParent - Parent window
   COrderPropertiesDlg(CWnd* pParent = NULL);

   /// \enum Dialog Data
   enum { IDD = IDD_ORDER_PROPERTIES_DIALOG };

   /// \brief Prepares the container for painting.
   afx_msg void OnPaint();

   /// \brief Used to unadvise from CQGCEL events and shutdown it.
   afx_msg void OnDestroy();

   /// \brief Called by a minimized (iconic) window that does not
   /// \brief have an icon defined for its class.
   afx_msg HCURSOR OnQueryDragIcon();

   /// \brief Opens new dialog to select the property
   /// \brief by which orders will be filtered.
   afx_msg void OnBnClickedChkFilter();

   /// \brief Changes selected order, and dumps it's properties and fills.
   afx_msg void OnLvnItemChangedLsvOrders(NMHDR *pNMHDR, LRESULT *pResult);

   /// \brief Opens new dialog to select the property
   /// \brief by which orders will be requested.
   afx_msg void OnBnClickedBtnQueryOrders();

   /// \brief Cancels orders query.
   afx_msg void OnBnClickedBtnQueryCancel();

   /// \brief Log On/Off to Gateway
   afx_msg void OnBnClickedBtnLogongw();

   /// \brief Changes selected transaction, and dumps it's strategy leg fills and hang orders ids.
   afx_msg void OnLvnItemchangedLsvOrderTransactions(NMHDR *pNMHDR, LRESULT *pResult);

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

protected:

   /// \brief Dialog icon.
   HICON m_hIcon;

   /// \brief Generated message map functions
   virtual void DoDataExchange(
                  CDataExchange* pDX);	// DDX/DDV support

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the application has set the input 
   /// \return focus to one of the controls in the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter.
   virtual void OnOK();

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(COrderPropertiesDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 10, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 2, OnGWConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 3, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 7, OnAccountChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 102, OnInstrumentResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 16, OnQueryProgress)
   END_SINK_MAP()

protected:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - String describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                  LPDISPATCH obj,
                  BSTR errorDescription);

   /// \brief Fired when some changes occurred
   /// \brief in the connection with the CQG Gateway.
   /// \param newStatus - New connection status
   /// \return S_OK
   STDMETHOD(OnGWConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when some changes occurred
   /// \brief in the connection with the CQG data server.
   /// \param newStatus - New connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when new account is added, changed or removed.
   /// \param change - Account change type
   /// \param account - Changed account instance
   /// \param position - Changed position instance
   /// \return S_OK
   STDMETHOD(OnAccountChanged)(
                  eAccountChangeType change,
                  ICQGAccount* account,
                  ICQGPosition* position);

   /// \brief Fired when orders query had some progress.
   /// \param query - Orders query object
   /// \param error - CQGError object to describe the last error which occurred during query, if any
   /// \return S_OK
   STDMETHOD(OnQueryProgress)(
                  ICQGOrdersQuery* query, 
                  ICQGError* error);

   /// \brief Fired when a new instrument is resolved and subscribed or the requested symbol is not available.
   /// \param symbol - Requested symbol
   /// \param cqg_instrument - Subscribed instrument object
   /// \param cqg_error - Error occured during a symbol resolution process
   /// \return S_OK on success
   STDMETHOD(OnInstrumentResolved)(BSTR symbol, ICQGInstrument* cqg_instrument, ICQGError* cqg_error);

private:

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes orders list control.
   void InitializeOrdersList();

   /// \brief Initializes order properties list control.
   void InitializeOrderPropList();

   /// \brief Initializes fills list control.
   void InitializeFillsList();

   /// \brief Initializes transactions list control.
   void InitializeTransactionsList();

   /// \breif Initializes strategy leg fills list control.
   void InitializeStrategyLegFillsList();

   /// \brief Initializes strategy order's leg orders list control.
   void InitializeStrategyLegOrdersList();

   /// \brief Initializes transaction's hang order ids list control.
   void InitializeHangOrderIdsList();

   /// \brief Initializes CQGCEL object, and starts the CQGCEL.
   void InitializeCQGCEL();

   /// \brief Clears all queried orders from the list control.
   void ClearOrders();

   /// \brief Clears all properties' data from the list control.
   void ClearProperties();

   /// \brief Clears all fills' data from the list control.
   void ClearFills();

   /// \brief Clears all transactions' data from the list control.
   void ClearTransactions();

   /// \brief Clears all strategy leg fill data from the list control.
   void ClearStrategyLegFills();

   /// \brief Clears all strategy leg orders data from the list control.
   void ClearStrategyLegOrders();

   /// \brief Clears transaction's hang order ids from the list control.
   void ClearHangOrderIds();

   /// \brief Clears filter check box text, and sets status to unchecked.
   void ClearFilterCheckBox();

   /// \brief Enables/disables controls on the form.
   /// \param enable - enable status
   void EnableControls(
                  bool enable);

   /// \brief Enables/disables GW login/logoff control on the form.
   /// \param status - GW connection new status
   void SetGWLogonButtonStatus(
                  eConnectionStatus status);

   /// \brief Shows specified query status on the form.
   /// \param spOrdersQuery - ICQGOrdersQuery instance
   void ShowQueryStatus(
                  const ATL::CComPtr<ICQGOrdersQuery>& spOrdersQuery);

   /// \brief Fills all accounts to combo box control and changes 
   /// \brief their subscription level.
   void ReloadAccounts();

   /// \brief Adds account GW name and GWID to the accounts combo box control.
   /// \param spAccount - ICQGAccount instance
   void InsertAccount(
                  const ATL::CComPtr<ICQGAccount>& spAccount);

   /// \brief Shows specified orders in the list control.
   /// \param spOrders - ICQGOrders instance
   void ShowOrders(
                  const ATL::CComPtr<ICQGOrders>& spOrders);

   /// \brief Adds order to the list control.
   /// \param listView - list control
   /// \param spOrder - ICQGOrder instance
   void ShowOrder(
                  CListCtrl& listView,            
                  const ATL::CComPtr<ICQGOrder>& spOrder);

   /// \brief Shows specified order's properties in the list control.
   /// \param spOrder - ICQGOrder instance
   void ShowProperties(
                  const ATL::CComPtr<ICQGOrder>& spOrder);

   /// \brief Shows specified order's all fills in the list control.
   /// \param spOrder - ICQGOrder instance
   void ShowFills(
                  const ATL::CComPtr<ICQGOrder>& spOrder);

   /// \brief Shows specified order's all transactions in the list control.
   /// \param spOrder - ICQGOrder instance
   void ShowTransactions(
                  const ATL::CComPtr<ICQGOrder>& spOrder);

   /// \brief Shows specified transaction's modified properties in the list control.
   /// \param spTransaction - ICQGTransaction instance
   void ShowModifiedProperties(
                  int startRow,
                  const ATL::CComPtr<ICQGTransaction>& spTransaction);

   /// \brief Shows specified fill data in the list control.
   /// \param spOrder - ICQGOrder instance
   /// \param spFill - ICQGFill instance
   void ShowFill(
                  const ATL::CComPtr<ICQGOrder>& spOrder,
                  const ATL::CComPtr<ICQGFill>& spFill);

   /// \brief Shows specified transaction data to the list control.
   /// \param spTransaction - ICQGTransaction instance
   void ShowTransaction(
                  const ATL::CComPtr<ICQGTransaction>& spTransaction);

   /// \brief Shows specified property data in the list control.
   /// \param spProperty - ICQGModifiedProperty instance
   void ShowModifiedProperty(
                  int row,
                  const ATL::CComPtr<ICQGModifiedProperty>& spProperty);

   /// \brief Shows Strategy leg fills of strategy order's transaction in the list control.
   /// \param spTransaction - ICQGTransaction instance
   void ShowStrategyLegFills(
                  const ATL::CComPtr<ICQGTransaction>& spTransaction);

   /// \brief Shows strategy leg fill in the list control.
   /// \param spStrategyLegFill - ICQGStrategyLegFill instance
   void ShowStrategyLegFill(
                  const ATL::CComPtr<ICQGStrategyLegFill>& spStrategyLegFill);

   /// \brief Shows leg orders of selected strategy order in the list control.
   /// \param spOrder - ICQGOrder instance
   void ShowStrategyOrderLegOrders(
                  const ATL::CComPtr<ICQGOrder>& spOrder);

   /// \brief Shows transaction's hang orders ids in the list control.
   /// \param spTransaction - ICQGTransaction instance
   void ShowHangOrderIds(
                  const ATL::CComPtr<ICQGTransaction> spTransaction);

   /// \brief Used for getting CQGCEL properties values string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - variant value
   /// \param valueType - variant type
   /// \return String representation of the value
   CString GetValueAsString(
                  ATL::CComVariant value,
                  VARENUM valueType = VT_EMPTY) const;

   /// \brief Returns selected account in the combo.
   /// \return ICQGAccount instance
   ATL::CComPtr<ICQGAccount> GetSelectedAccount() const;

   /// \brief Validates and returns display format of specified price.
   /// \param spOrder - Order which price is formated 
   /// \param price - Price which is formated
   /// \return String representation of price
   CString GetDisplayPrice(
                  const ATL::CComPtr<ICQGOrder> spOrder,
                  double price) const;

   /// \brief Returns string representation of the specified order type.
   /// \param type - order type
   /// \return String representation of the order type
   static CString OrderTypeToString(
                  eOrderType type);

   /// \brief Returns string representation of the specified order local state.
   /// \param state - order local state
   /// \return String representation of the order local state
   static CString OrderLocalStateToString(
                  eOrderLocalState state);

   /// \brief Returns string representation of the specified order status.
   /// \param status - order status
   /// \return String representation of the order status
   static CString OrderStatusToString(
                  eOrderStatus status);
   
   /// \brief Returns string representation of the specified order status filter.
   /// \param statusFilter - order status filter
   /// \return String representation of the order status filter
   static CString OrderStatusFilterToString(
                  eOrderStatusFilter statusFilter);

   /// \brief Returns string representation of the specified order event.
   /// \param event - order event
   /// \return String representation of the order event
   static CString OrderEventToString(
                  eOrderEvent event);

   /// \brief Returns string representation of the specified order duration.
   /// \param duration - order duration
   /// \return String representation of the order duration
   static CString OrderDurationToString(
                  eOrderDuration duration);

   /// \brief Returns string representation of the specified order side.
   /// \param side - order side
   /// \return String representation of the order side
   static CString OrderSideToString(
                  eOrderSide side);

   /// \brief Returns string representation of the specified fill status.
   /// \param fillStatus - fill status
   /// \return String representation of the fill status
   static CString FillStatusToString(
                  eFillStatus fillStatus);

   /// \brief Returns string representation of the request status.    
   /// \param status - request status
   /// \return String representation of the request status
   static CString RequestStatusToString(
                  eRequestStatus status);

   /// \brief Returns string representation of the connection status.    
   /// \param status - connection status
   /// \return String representation of the connection status
   static CString ConnectionStatusToString(
                  eConnectionStatus status);
						
private:

   /// \enum eOrdersColumn. Represents orders columns in the list.
   enum eOrdersColumn;

   /// \enum eOrderPropertiesColumn. Represents order properties columns in the list.
   enum eOrderPropertiesColumn;

   /// \enum eOrderFillColumn. Represents order fill columns in the list.
   enum eOrderFillColumn;

   /// \enum eOrderTransactionColumn. Represents order transaction columns in the list.
   enum eOrderTransactionColumn;

   /// \enum eStrategyLegFillsColumn. Represents strategy leg fills columns in the list.
   enum eStrategyLegFillsColumn;

   /// \enum eHangOrderIdsColumn. Represents transaction's hang orders ids columns in the list.
   enum eHangOrderIdsColumn;

private:

   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Orders query object.
   ATL::CComPtr<ICQGOrdersQuery> m_spOrdersQuery;

   /// \brief Selected order object. 
   /// \brief Used to update order data when order's instrument is subscribed.
   ATL::CComPtr<ICQGOrder> m_spSelectedOrder;

   /// \brief Not available value.
   static const CString m_N_A;
   
   /// \brief Yes string.
   static const CString m_Yes;

   /// \brief No string.
   static const CString m_No;

   /// \brief Filter check box text.
   static const CString m_FilterText;

   /// \brief Used to determine is there a query in progress.
   bool m_IsInQuery;

   /// \brief Used to determine whether user is logged on or off to GW.
   bool m_IsGWLoggedOn;

   /// \brief Accounts combo box control
   /// \brief Contains account's GW names, as well as the GWID as item data.
   /// \brief GWID is used to get account instance from CQGCEL accounts collection.
   CComboBox m_cmbAccounts;

   /// \brief Data connection indicator label control.
   CStatic m_lblDataConnection;

   /// \brief GW connection indicator label control.
   CStatic m_lblGWConnection;
   
   /// \brief Query for orders button control.
   CButton m_btnQueryOrders;
   
   /// \brief Cancel current query button control.
   CButton m_btnCancelQuery;

   /// \brief Current query status label control.
   CStatic m_lblQueryStatus;

   /// \brief Queried orders list control.
   CListCtrl m_lvOrders;

   /// \brief Selected order's properties list control.
   CListCtrl m_lvOrderProps;
   
   /// \brief Selected order's fills list control.
   CListCtrl m_lvOrderFills;

   /// \brief Selected order's transactions list control.
   CListCtrl m_lvOrderTransactions;

   /// \brief Strategy leg fills list control.
   CListCtrl m_lvStrategyLegFills;

   /// \brief Strategy order's leg orders list control.
   CListCtrl m_lvStrategyLegOrders;

   /// \brief Transaction's hang order ids list control.
   CListCtrl m_lvHangOrderIds;

   /// \brief Orders filter check box control.
   CButton m_chkFilter;

   /// \brief Gateway Log On/Off control.
   CButton m_btnGWLogOnOff;

   /// \brief Gateway username
   CStatic m_lblGWUsername;
};

enum COrderPropertiesDlg::eOrdersColumn
{
   ocID,
   ocInstrument,
   ocSide,
   ocType,
   ocState
};

enum COrderPropertiesDlg::eOrderPropertiesColumn
{
   opcName,
   opcValue
};

enum COrderPropertiesDlg::eOrderFillColumn
{
   ofcTimestamp,
   ofcFillID,
   ofcStatus,
   ofcBought,
   ofcSold,
   ofcInstrument,
   ofcPrice,
   ofcCurrency,
   ofcGWTradeID,
   ofcAggressive
};

enum COrderPropertiesDlg::eOrderTransactionColumn
{
   otcTimestamp,
   otcTransactionID,
   otcStatus,
   otcFillID,
   otcRejectReason,
   otcPropName,
   otcPropPreviousValue,
   otcPropCurrentValue
};

enum COrderPropertiesDlg::eStrategyLegFillsColumn
{
   slfcOriginalOrderId,
   slfcExecutionId,
   slfcQuantity
};

enum COrderPropertiesDlg::eHangOrderIdsColumn
{
   oidcHangOrderId
};