// OrdersFilterParametersDlg.cpp : implementation file
//

#include "stdafx.h"
#include <vector>
#include <cassert>
#include <stdexcept>
#include "Common.h"
#include "OrderProperties.h"
#include "OrdersFilterParametersDlg.h"

COrdersFilterParametersDlg::COrdersFilterParametersDlg(
                        const ATL::CComPtr<ICQGCEL>& spCEL,
                        CWnd* pParent /*=NULL*/) 
                              : CDialog(COrdersFilterParametersDlg::IDD, pParent), 
                                m_spCQGCEL(spCEL)
{
   // Now advise the connection, to get events
   assert(m_spCQGCEL != NULL);
   HRESULT hr = ICQGCELDispEventImplOF::DispEventAdvise(m_spCQGCEL);
   AssertCOMError(m_spCQGCEL, hr);
}

void COrdersFilterParametersDlg::DoDataExchange(
                                          CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_CMB_STATUS, m_cmbStatuses);
   DDX_Control(pDX, IDC_CMB_SIDE, m_cmbSides);
   DDX_Control(pDX, IDC_CMB_TYPE, m_cmbTypes);
   DDX_Control(pDX, IDC_CMB_INSTRUMENTS, m_cmbInstruments);
   DDX_Control(pDX, IDC_RD_INSTRUMENT, m_rdInstrument);
   DDX_Control(pDX, IDC_RD_TYPE, m_rdType);
   DDX_Control(pDX, IDC_RD_STATUS, m_rdStatus);
   DDX_Control(pDX, IDC_RD_SIDE, m_rdSide);
   DDX_Control(pDX, IDOK, m_btnOk);
}

BOOL COrdersFilterParametersDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   InitializeControls();

   return TRUE;
}

void COrdersFilterParametersDlg::UnadviseFromCQGCEL()
{
   HRESULT hr = ICQGCELDispEventImplOF::DispEventUnadvise(m_spCQGCEL);
   AssertCOMError(m_spCQGCEL, hr);
}

void COrdersFilterParametersDlg::InitializeControls()
{
   InitializeInstrumentsCombo();
   InitializeSidesCombo();
   InitializeFilterStatusesCombo();
   InitializeTypesCombo();
}

void COrdersFilterParametersDlg::InitializeInstrumentsCombo()
{
   ATL::CComPtr<ICQGInstruments> spInstruments;
   HRESULT hr = m_spCQGCEL->get_Instruments(&spInstruments);
   AssertCOMError(m_spCQGCEL, hr);

   long count;
   hr = spInstruments->get_Count(&count);
   AssertCOMError(spInstruments, hr);

   if (count == 0)
   {
      // Permit from filtering by instrument
      m_rdInstrument.EnableWindow(FALSE);

      // Set filter by type
      m_rdType.SetCheck(BST_CHECKED);
      OnBnClickedRdType();
      return;
   }
   else
   {
      // Set filter by instrument
      m_rdInstrument.SetCheck(BST_CHECKED);
      OnBnClickedRdInstrument();
   }
      
   // Iterate over instruments collection and add them to the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   hr = spInstruments->get__NewEnum(&spEnumVariant);
   AssertCOMError(spInstruments, hr);

   std::vector<ATL::CComVariant> pItems(count);
   ULONG celtFetched;
   hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      ATLASSERT(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGInstrument> spInstrument;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGInstrument), (void** )&spInstrument);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      // Add current instrument to the list
      ATL::CComBSTR fullName;
      hr = spInstrument->get_FullName(&fullName);
      AssertCOMError(spInstrument, hr);

      m_cmbInstruments.AddString(ATL::CW2T(fullName));
   }

   m_cmbInstruments.SetCurSel(0);
}

void COrdersFilterParametersDlg::InitializeSidesCombo()
{
   int row = m_cmbSides.AddString(_T("Sell"));
   m_cmbSides.SetItemData(row, osdSell);

   row = m_cmbSides.AddString(_T("Buy"));
   m_cmbSides.SetItemData(row, osdBuy);

   m_cmbSides.SetCurSel(0);
}

void COrdersFilterParametersDlg::InitializeFilterStatusesCombo()
{
   int row = m_cmbStatuses.AddString(_T("ActiveAt"));
   m_cmbStatuses.SetItemData(row, osActiveAt);

   row = m_cmbStatuses.AddString(_T("Busted"));
   m_cmbStatuses.SetItemData(row, osBusted);

   row = m_cmbStatuses.AddString(_T("Canceled"));
   m_cmbStatuses.SetItemData(row, osCanceled);

   row = m_cmbStatuses.AddString(_T("Contingent"));
   m_cmbStatuses.SetItemData(row, osContingent);

   row = m_cmbStatuses.AddString(_T("Disconnected"));
   m_cmbStatuses.SetItemData(row, osDisconnected);

   row = m_cmbStatuses.AddString(_T("Expired"));
   m_cmbStatuses.SetItemData(row, osExpired);

   row = m_cmbStatuses.AddString(_T("Filled"));
   m_cmbStatuses.SetItemData(row, osFilled);

   row = m_cmbStatuses.AddString(_T("InCancel"));
   m_cmbStatuses.SetItemData(row, osInCancel);

   row = m_cmbStatuses.AddString(_T("InClient"));
   m_cmbStatuses.SetItemData(row, osInClient);

   row = m_cmbStatuses.AddString(_T("InModify"));
   m_cmbStatuses.SetItemData(row, osInModify);

   row = m_cmbStatuses.AddString(_T("InOrderBook"));
   m_cmbStatuses.SetItemData(row, osInOrderBook);

   row = m_cmbStatuses.AddString(_T("InTransit"));
   m_cmbStatuses.SetItemData(row, osInTransit);

   row = m_cmbStatuses.AddString(_T("InTransitTimeout"));
   m_cmbStatuses.SetItemData(row, osInTransitTimeout);

   row = m_cmbStatuses.AddString(_T("NotSent"));
   m_cmbStatuses.SetItemData(row, osNotSent);

   row = m_cmbStatuses.AddString(_T("Parked"));
   m_cmbStatuses.SetItemData(row, osParked);

   row = m_cmbStatuses.AddString(_T("RejectFCM"));
   m_cmbStatuses.SetItemData(row, osRejectFCM);

   row = m_cmbStatuses.AddString(_T("RejectGW"));
   m_cmbStatuses.SetItemData(row, osRejectGW);

   m_cmbStatuses.SetCurSel(0);
}

void COrdersFilterParametersDlg::InitializeTypesCombo()
{
   int row = m_cmbTypes.AddString(_T("Market"));
   m_cmbTypes.SetItemData(row, otMarket);

   row = m_cmbTypes.AddString(_T("Limit"));
   m_cmbTypes.SetItemData(row, otLimit);

   row = m_cmbTypes.AddString(_T("Stop"));
   m_cmbTypes.SetItemData(row, otStop);

   row = m_cmbTypes.AddString(_T("StopLimit"));
   m_cmbTypes.SetItemData(row, otStopLimit);

   m_cmbTypes.SetCurSel(0);
}

STDMETHODIMP COrdersFilterParametersDlg::OnInstrumentResolved(
   BSTR symbol, ICQGInstrument* instrument, ICQGError* cqg_error)
{
   try
   {
      VARIANT_BOOL errorOccured = VARIANT_FALSE;
      auto hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &errorOccured);
      AssertCOMError<ICQGError>(cqg_error, hr);

      if (errorOccured)
      {
         // Show short message and don't add the combo item
         CString msg;
         msg.AppendFormat(_T("%s resolution failed."), symbol);
         AfxMessageBox(msg, MB_ICONWARNING);
         return S_OK;
      }

      ATL::CComBSTR fullName;
      hr = instrument->get_FullName(&fullName);
      AssertCOMError<ICQGInstrument>(instrument, hr);

      // If instrument is already subscribed do not add it to the list
      if (m_cmbInstruments.FindStringExact(0, ATL::CW2T(fullName)) != CB_ERR)
      {
         return S_OK;
      }

      // Add name to the list
      m_cmbInstruments.AddString(ATL::CW2T(fullName));

      // Now it's possible to filter by instrument
      m_rdInstrument.EnableWindow(TRUE);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

BEGIN_MESSAGE_MAP(COrdersFilterParametersDlg, CDialog)
   ON_BN_CLICKED(IDC_RD_INSTRUMENT, OnBnClickedRdInstrument)
   ON_BN_CLICKED(IDC_RD_TYPE, OnBnClickedRdType)
   ON_BN_CLICKED(IDC_RD_STATUS, OnBnClickedRdStatus)
   ON_BN_CLICKED(IDC_RD_SIDE, OnBnClickedRdSide)
   ON_WM_DESTROY()
END_MESSAGE_MAP()

void COrdersFilterParametersDlg::OnOK()
{
   try
   {
      if (m_rdInstrument.GetCheck())
      {
         // Get selected instrument name, if checked
         m_cmbInstruments.GetLBText(m_cmbInstruments.GetCurSel(), m_FullName);
         m_FilterType = ftInstrument;
      }
      else if (m_rdType.GetCheck())
      {
         // Get selected order type, if checked
         m_OrderType = static_cast<eOrderType>
                                 (m_cmbTypes.GetItemData(m_cmbTypes.GetCurSel()));
         m_FilterType = ftType;
      }
      else if (m_rdStatus.GetCheck())
      {
         // Get selected order status, if checked
         m_OrderStatus = static_cast<eOrderStatus>
                                 (m_cmbStatuses.GetItemData(m_cmbStatuses.GetCurSel()));
         m_FilterType = ftStatus;
      }
      else if (m_rdSide.GetCheck())
      {
         // Get selected order side, if checked
         m_OrderSide = static_cast<eOrderSide>
                                 (m_cmbSides.GetItemData(m_cmbSides.GetCurSel()));
         m_FilterType = ftSide;
      }

      CDialog::OnOK();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void COrdersFilterParametersDlg::OnDestroy()
{
   try
   {
      UnadviseFromCQGCEL();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}

COrdersFilterParametersDlg::eFilterType COrdersFilterParametersDlg::GetFilterType() const
{
   return m_FilterType;
}

CString COrdersFilterParametersDlg::GetFullName() const
{
   return m_FullName;
}

eOrderType COrdersFilterParametersDlg::GetOrderType() const
{
   return m_OrderType;
}

eOrderStatus COrdersFilterParametersDlg::GetOrderStatus() const
{
   return m_OrderStatus;
}

eOrderSide COrdersFilterParametersDlg::GetOrderSide() const
{
   return m_OrderSide;
}

void COrdersFilterParametersDlg::OnBnClickedRdInstrument()
{
   m_cmbInstruments.EnableWindow(TRUE);
   m_cmbSides.EnableWindow(FALSE);
   m_cmbStatuses.EnableWindow(FALSE);
   m_cmbTypes.EnableWindow(FALSE);
}

void COrdersFilterParametersDlg::OnBnClickedRdType()
{
   m_cmbInstruments.EnableWindow(FALSE);
   m_cmbSides.EnableWindow(FALSE);
   m_cmbStatuses.EnableWindow(FALSE);
   m_cmbTypes.EnableWindow(TRUE);
}

void COrdersFilterParametersDlg::OnBnClickedRdStatus()
{
   m_cmbInstruments.EnableWindow(FALSE);
   m_cmbSides.EnableWindow(FALSE);
   m_cmbStatuses.EnableWindow(TRUE);
   m_cmbTypes.EnableWindow(FALSE);
}

void COrdersFilterParametersDlg::OnBnClickedRdSide()
{
   m_cmbInstruments.EnableWindow(FALSE);
   m_cmbSides.EnableWindow(TRUE);
   m_cmbStatuses.EnableWindow(FALSE);
   m_cmbTypes.EnableWindow(FALSE);
}
