// QueryParametersDlg.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include <vector>
#include "Common.h"
#include "OrderProperties.h"
#include "QueryParametersDlg.h"

const CString CQueryParametersDlg::m_AllInstruments = _T("All Instruments");

CQueryParametersDlg::CQueryParametersDlg(
                        const ATL::CComPtr<ICQGCEL>& spCEL,
                        const ATL::CComPtr<ICQGAccount>& spAccount,
                        CWnd* pParent /*=NULL*/) 
                              : CDialog(CQueryParametersDlg::IDD, pParent), 
                                m_spCQGCEL(spCEL),
                                m_spAccount(spAccount)
{
   // Now advise the connection, to get events
   assert(m_spCQGCEL != NULL);
   HRESULT hr = ICQGCELDispEventImplQP::DispEventAdvise(m_spCQGCEL);
   AssertCOMError(m_spCQGCEL, hr);
}

void CQueryParametersDlg::DoDataExchange(
                                    CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_CMB_INSTRUMENTS, m_cmbInstruments);
   DDX_Control(pDX, IDC_CMB_SIDES, m_cmbSides);
   DDX_Control(pDX, IDC_CMB_STATUSES, m_cmbStatuses);
   DDX_Control(pDX, IDC_DTP_DATE, m_dtpDate);
   DDX_Control(pDX, IDC_BTN_SUBSCRIBE_INSTRUMENT, m_btnSubscribeInstrument);
   DDX_Control(pDX, IDOK, m_btnOk);
   DDX_Control(pDX, IDC_CHK_INCLUDE_TRANSACTIONS, m_chkTransactions);
}

void CQueryParametersDlg::UnadviseFromCQGCEL()
{
   if (m_spCQGCEL)
   {
      HRESULT hr = ICQGCELDispEventImplQP::DispEventUnadvise(m_spCQGCEL);
      ATLVERIFY(SUCCEEDED(hr));
   }
}

void CQueryParametersDlg::InitializeControls()
{
   InitializeInstrumentsCombo();
   InitializeSidesCombo();
   InitializeFilterStatusesCombo();

   m_dtpDate.SetFormat(_T("dd/MM/yy"));
   m_chkTransactions.SetCheck(BST_CHECKED);

   m_btnSubscribeInstrument.EnableWindow(FALSE);
}

BOOL CQueryParametersDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   InitializeControls();

   return TRUE;
}

void CQueryParametersDlg::InitializeInstrumentsCombo()
{
   m_cmbInstruments.AddString(m_AllInstruments);
   m_cmbInstruments.SetCurSel(0);

   ATL::CComPtr<ICQGInstruments> spInstruments;
   HRESULT hr = m_spCQGCEL->get_Instruments(&spInstruments);
   AssertCOMError(m_spCQGCEL, hr);

   long count;
   hr = spInstruments->get_Count(&count);
   AssertCOMError(spInstruments, hr);

   if (count == 0)
   {
      // No instruments
      return;
   }
      
   // Iterate over instruments collection and add them to the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   hr = spInstruments->get__NewEnum(&spEnumVariant);
   AssertCOMError(spInstruments, hr);

   std::vector<ATL::CComVariant> pItems(count);
   ULONG celtFetched;
   hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      ATLASSERT(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGInstrument> spInstrument;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGInstrument), (void** )&spInstrument);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      // Add current instrument name to the list
      ATL::CComBSTR fullName;
      hr = spInstrument->get_FullName(&fullName);
      AssertCOMError(spInstrument, hr);

      m_cmbInstruments.AddString(ATL::CW2T(fullName));
   }
}

void CQueryParametersDlg::InitializeSidesCombo()
{
   int row = m_cmbSides.AddString(_T("All Sides"));
   m_cmbSides.SetItemData(row, osdUndefined);

   row = m_cmbSides.AddString(_T("Sell"));
   m_cmbSides.SetItemData(row, osdSell);

   row = m_cmbSides.AddString(_T("Buy"));
   m_cmbSides.SetItemData(row, osdBuy);

   m_cmbSides.SetCurSel(0);
}

void CQueryParametersDlg::InitializeFilterStatusesCombo()
{
   int row = m_cmbStatuses.AddString(_T("All Statuses"));
   m_cmbStatuses.SetItemData(row, osfAll);

   row = m_cmbStatuses.AddString(_T("Canceled"));
   m_cmbStatuses.SetItemData(row, osfCanceled);

   row = m_cmbStatuses.AddString(_T("Exception"));
   m_cmbStatuses.SetItemData(row, osfException);

   row = m_cmbStatuses.AddString(_T("Filled"));
   m_cmbStatuses.SetItemData(row, osfFilled);

   row = m_cmbStatuses.AddString(_T("Final"));
   m_cmbStatuses.SetItemData(row, osfFinal);

   row = m_cmbStatuses.AddString(_T("Not Final"));
   m_cmbStatuses.SetItemData(row, osfNotFinal);

   row = m_cmbStatuses.AddString(_T("Parked"));
   m_cmbStatuses.SetItemData(row, osfParked);

   m_cmbStatuses.SetCurSel(0);
}

ATL::CComPtr<ICQGOrdersQuery> CQueryParametersDlg::GetOrdersQuery() const
{
   return m_spOrdersQuery;
}

void CQueryParametersDlg::OnInstrumentSubscribed(const CString& symbol)
{
   int index = m_cmbInstruments.FindStringExact(0, symbol);

   // If instrument is already subscribed do not add it to the list
   if (index == CB_ERR)
   {
      // Add full name to the list
      index = m_cmbInstruments.AddString(symbol);
   }

   // Select newly subscribed instrument in the list
   CString comboSymbol;
   m_cmbInstruments.GetWindowText(comboSymbol);
   if (symbol == comboSymbol.MakeUpper())
   {
      m_cmbInstruments.SetCurSel(index);
      // Call this to enable/disable buttons. Is not called automatically.
      OnCbnSelchangeCmbInstruments();
   }     
}

STDMETHODIMP CQueryParametersDlg::OnInstrumentResolved(
   BSTR symbol, ICQGInstrument* cqg_instrument, ICQGError* cqg_error)
{
   try
   {
      VARIANT_BOOL errorOccured = VARIANT_FALSE;
      auto hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &errorOccured);
      AssertCOMError(m_spCQGCEL, hr);

      if (errorOccured)
      {
         // Show message
         ATL::CComBSTR errDescription;
         hr = cqg_error->get_Description(&errDescription);
         AssertCOMError<ICQGError>(cqg_error, hr);

         OnResolutionFailed(symbol, CString(errDescription));
         return S_OK;
      }

      ATL::CComBSTR fullName;
      hr = cqg_instrument->get_FullName(&fullName);
      AssertCOMError<ICQGInstrument>(cqg_instrument, hr);

      OnInstrumentSubscribed(CString(fullName));
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
   return S_OK;
}

void CQueryParametersDlg::OnResolutionFailed(const CString& symbol, const CString& errorDescription)
{
   // Clear text
   m_cmbInstruments.SetWindowText(_T(""));
   OnCbnEditchangeCmbInstruments();

   CString msg;
   msg.AppendFormat(_T("%s resolution failed. %s"), symbol.GetString(), errorDescription.GetString());
   AfxMessageBox(msg, MB_ICONWARNING);
}

BEGIN_MESSAGE_MAP(CQueryParametersDlg, CDialog)
   ON_BN_CLICKED(IDC_BTN_SUBSCRIBE_INSTRUMENT, OnBnClickedBtnSubscribeInstrument)
   ON_CBN_EDITCHANGE(IDC_CMB_INSTRUMENTS, OnCbnEditchangeCmbInstruments)
   ON_CBN_SELCHANGE(IDC_CMB_INSTRUMENTS, OnCbnSelchangeCmbInstruments)
   ON_WM_CLOSE()
   ON_WM_DESTROY()
END_MESSAGE_MAP()

void CQueryParametersDlg::OnOK()
{
   try
   {
      // Get selected order status
      eOrderStatusFilter status = static_cast<eOrderStatusFilter>
                              (m_cmbStatuses.GetItemData(m_cmbStatuses.GetCurSel()));
      
      // Get selected order side
      eOrderSide side = static_cast<eOrderSide>
                              (m_cmbSides.GetItemData(m_cmbSides.GetCurSel()));
      
      // Get selected time
      COleDateTime oleDateTime;
      m_dtpDate.GetTime(oleDateTime);
      
      // Get selected instrument name
      CString fullName;
      m_cmbInstruments.GetLBText(m_cmbInstruments.GetCurSel(), fullName);

      // Set the transactions query flag
      ATL::CComPtr<ICQGAPIConfig> spConf;
      HRESULT hr = m_spCQGCEL->get_APIConfiguration(&spConf);
      AssertCOMError(m_spCQGCEL, hr);
      hr = spConf->put_IncludeOrderTransactions(m_chkTransactions.GetCheck() == BST_CHECKED);
      AssertCOMError(spConf, hr);

      if (fullName == m_AllInstruments)
      {
         // No instrument selected

         // Query orders
         hr = m_spCQGCEL->QueryOrders(m_spAccount,
                                      NULL, 
                                      status, 
                                      oleDateTime, 
                                      side, 
                                      &m_spOrdersQuery);
         AssertCOMError(m_spCQGCEL, hr);
      }
      else
      {
         // Get selected instrument object
         ATL::CComPtr<ICQGInstruments> spInstruments;
         hr = m_spCQGCEL->get_Instruments(&spInstruments);
         AssertCOMError(m_spCQGCEL, hr);

         ATL::CComPtr<ICQGInstrument> spInstrument;
         hr = spInstruments->get_Item(ATL::CComVariant(fullName), &spInstrument);
         AssertCOMError(spInstruments, hr);

         // Query orders
         hr = m_spCQGCEL->QueryOrders(m_spAccount, 
                                      spInstrument, 
                                      status, 
                                      oleDateTime, 
                                      side, 
                                      &m_spOrdersQuery);
         AssertCOMError(m_spCQGCEL, hr);
      }

      CDialog::OnOK();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CQueryParametersDlg::OnDestroy()
{
   try
   {
      UnadviseFromCQGCEL();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}

void CQueryParametersDlg::OnBnClickedBtnSubscribeInstrument()
{
   try
   {
      CString symbol;
      m_cmbInstruments.GetWindowText(symbol);

      ATL::CComPtr<ICQGInstrumentSubscriptionRequest> request;
      HRESULT hr = m_spCQGCEL->CreateInstrumentRequest(&request);
      AssertCOMError(m_spCQGCEL, hr);

      hr = request->put_Symbol(ATL::CT2W(symbol));
      AssertCOMError(m_spCQGCEL, hr);

      hr = m_spCQGCEL->SubscribeNewInstrument(request);
      AssertCOMError(m_spCQGCEL, hr);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CQueryParametersDlg::OnCbnEditchangeCmbInstruments()
{
   CString symbol;
   m_cmbInstruments.GetWindowText(symbol);

   // Enable subscribe button if text is not in the combo list
   bool enable = (m_cmbInstruments.FindStringExact(0, symbol) == CB_ERR);
   m_btnSubscribeInstrument.EnableWindow(enable && (symbol.Trim().GetLength() > 0));
   m_btnOk.EnableWindow(!enable);
}

void CQueryParametersDlg::OnCbnSelchangeCmbInstruments()
{
   bool enable = (m_cmbInstruments.GetCurSel() == -1);
   m_btnSubscribeInstrument.EnableWindow(enable);
   m_btnOk.EnableWindow(!enable);
}
