// QueryParametersDlg.h : header file
//

#pragma once

class CQueryParametersDlg;
/**
\typedef ATL::IDispEventImpl<1, CQueryParametersDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImplQP
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CQueryParametersDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImplQP;

/// \brief CQueryParametersDlg dialog. 
/// \brief Used to show orders query parameters.
class CQueryParametersDlg : public CDialog, public ICQGCELDispEventImplQP
{
public:
   /// \brief Constructor
   /// \param spCel - CQGCEL instance
   /// \param spAccount - Account used for query
   /// \param pParent - Parent window
   CQueryParametersDlg(
               const ATL::CComPtr<ICQGCEL>& spCEL,
               const ATL::CComPtr<ICQGAccount>& spAccount,
               CWnd* pParent = NULL);

   /// \enum Dialog Data
   enum { IDD = IDD_QUERY_PARAMS_DIALOG };

   /// \brief Subscribes instrument.
   afx_msg void OnBnClickedBtnSubscribeInstrument();

   /// \brief enables/disables m_btnSubscribeInstrument and m_btnOk.
   afx_msg void OnCbnEditchangeCmbInstruments();

   /// \brief enables/disables m_btnSubscribeInstrument and m_btnOk.
   afx_msg void OnCbnSelchangeCmbInstruments();

   /// \brief Used to unadvise from CQGCEL events on dialog destroy.
   afx_msg void OnDestroy();

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

protected:

   /// \brief Generated message map functions
   virtual void DoDataExchange(
                  CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the application has set the input 
   /// \return focus to one of the controls in the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter.
   virtual void OnOK();

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CQueryParametersDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 102, OnInstrumentResolved)
   END_SINK_MAP()

protected:

   /// \brief Fired when a new instrument is resolved and subscribed or the requested symbol is not available.
   /// \param symbol - Requested symbol
   /// \param cqg_instrument - Subscribed instrument object
   /// \param cqg_error - Error occured during a symbol resolution process
   /// \return S_OK on success
   STDMETHOD(OnInstrumentResolved)(BSTR symbol, ICQGInstrument* cqg_instrument, ICQGError* cqg_error);

public:
   
   /// \brief Returns orders query object.
   ATL::CComPtr<ICQGOrdersQuery> GetOrdersQuery() const;

private:

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes instruments combo box.
   void InitializeInstrumentsCombo();

   /// \brief Initializes sides combo box.
   void InitializeSidesCombo();

   /// \brief Initializes statuses combo box.
   void InitializeFilterStatusesCombo();

   /// \brief Unadvises from CQGCEL events.
   void UnadviseFromCQGCEL();

   /// \brief Called when new instrument is resolved and subscribed.
   /// \param symbol - Requested symbol
   void OnInstrumentSubscribed(const CString& symbol);

   /// \brief Called when the request by the incorrect symbol is made.
   /// \param symbol - Requested symbol
   /// \param errorDescription - Error text
   void OnResolutionFailed(const CString& symbol, const CString& errorDescription);

private:

   /// \brief CQGCEL object
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief CQGAccount object
   ATL::CComPtr<ICQGAccount> m_spAccount;
   
   /// \brief CQGOrdersQuery object
   ATL::CComPtr<ICQGOrdersQuery> m_spOrdersQuery;

   /// \brief Order query instrument full name combo box control.
   CComboBox m_cmbInstruments;

   /// \brief Order query side combo box control.
   CComboBox m_cmbSides;

   /// \brief Order query status combo box control.
   CComboBox m_cmbStatuses;

   /// \brief Order query date datetime control.
   CDateTimeCtrl m_dtpDate;

   /// \brief All Instruments string
   static const CString m_AllInstruments;

   /// \brief Subscribe instruments button control.
   CButton m_btnSubscribeInstrument;

   /// \brief OK button control.
   CButton m_btnOk;
   
   /// \brief IncludeTransactions checkbox control
   CButton m_chkTransactions;
};
