// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "PointAndFigureBars.h"
#include "PointAndFigureBarsDlg.h"

void CPointAndFigureBarsDlg::InitializeCQGCEL()
{
   // Create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }
   

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   Common::ThrowOnCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   Common::ThrowOnCOMError(m_spCQGCEL, hr);
}

void CPointAndFigureBarsDlg::InitializeControls()
{
   InitializeList(m_lvLastPointAndFigureBars);
   InitializeList(m_lvPointAndFigureBars);

   ClearAllData();

   OnDataConnectionStatusChanged(csConnectionDown);

   FillIndices();
}

void CPointAndFigureBarsDlg::InitializeList(
                        CListCtrl& listCtrl)
{
   listCtrl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   listCtrl.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   listCtrl.InsertColumn(rcIndex, _T("#"), LVCFMT_LEFT, 40);
   listCtrl.InsertColumn(rcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 150);
   listCtrl.InsertColumn(rcMid, _T("Mid"), LVCFMT_LEFT, 120);
   listCtrl.InsertColumn(rcHigh, _T("High"), LVCFMT_LEFT, 120);
   listCtrl.InsertColumn(rcLow, _T("Low"), LVCFMT_LEFT, 120);
   listCtrl.InsertColumn(rcUp, _T("Up"), LVCFMT_LEFT, 120);
}

void CPointAndFigureBarsDlg::ClearAllData()
{
   m_lblStatus.SetWindowText(_T(""));

   ClearRequestParams();

   ClearTags();

   ClearOutputParams();
   ClearIndices();

   ClearRecords(m_lvLastPointAndFigureBars);
   ClearRecords(m_lvPointAndFigureBars);

   m_lblAllRecsCount.SetWindowText(_T(""));
   m_lblFltrRecsCount.SetWindowText(_T(""));
}

void CPointAndFigureBarsDlg::ClearRequestParams()
{
   UpdateData();

   m_Symbol = _T("");
   m_RangeStart = _T("");
   m_RangeEnd = _T("");
   m_UpdatesEnabled = _T("");
   m_IncludeEnd = _T("");
   m_BoxUnits = _T("");
   m_BoxSize = _T("");
   m_Reversal = _T("");
   m_DataSource = _T("");
   m_SessionsFilter = _T("");
   m_Continuation = _T("");
   m_EqualizeCloses = _T("");
   m_DaysBeforeExpiration = _T("");
   m_SessionFlags = _T("");
   m_SubscriptionLevel = _T("");
   m_RecalcPeriod = _T("");

   UpdateData(FALSE);

   m_lblError.SetWindowText(_T(""));
}

void CPointAndFigureBarsDlg::ClearOutputParams()
{
   UpdateData();

   m_OutputStart = _T("");
   m_OutputEnd = _T("");

   UpdateData(FALSE);
}

void CPointAndFigureBarsDlg::ClearTags()
{
   m_edTagName.SetWindowText(_T(""));
   m_edTagValue.SetWindowText(_T(""));
}

void CPointAndFigureBarsDlg::ClearRecords(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();
}

void CPointAndFigureBarsDlg::ShowAllData()
{
   ShowRequestParams();

   eRequestStatus status;
   HRESULT hr = m_spPointAndFigureBars->get_Status(&status);
   Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

   m_lblStatus.SetWindowText(CHelper::RequestStatusToString(status));

   if (status == rsSuccess)
   {
      ShowOutputParams();

      FillIndices();

      long count;
      hr = m_spPointAndFigureBars->get_Count(&count);
      Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

      // Show records
      const long startIndex = max(count - m_ShowCount + 1, 1);
      const long endIndex = count;
      ShowRecords(startIndex, endIndex, m_lvLastPointAndFigureBars);

      // Show all records count
      m_lblAllRecsCount.SetWindowText(GetValueAsString(count));
   }
}

void CPointAndFigureBarsDlg::ShowChangedData()
{
   // Clear the list box
   ClearRecords(m_lvLastPointAndFigureBars);

   // Get records count
   long count;
   HRESULT hr = m_spPointAndFigureBars->get_Count(&count);
   Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

   // Show records
   const long startIndex = max(count - m_ShowCount + 1, 1);
   const long endIndex = count;
   ShowRecords(startIndex, endIndex, m_lvLastPointAndFigureBars);

   // Show all records count
   m_lblAllRecsCount.SetWindowText(GetValueAsString(count));

   // Show output parameters
   ShowOutputParams();
}

void CPointAndFigureBarsDlg::ShowRequestParams()
{
   UpdateData();

   // Get request from point and figure bars collection
   ATL::CComPtr<ICQGPointAndFigureBarsRequest> spRequest;
   HRESULT hr = m_spPointAndFigureBars->get_Request(&spRequest);
   Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

   // Show request parameters
   ATL::CComBSTR symbol;
   hr = spRequest->get_Symbol(&symbol);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeStart;
   hr = spRequest->get_RangeStart(&rangeStart);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeEnd;
   hr = spRequest->get_RangeEnd(&rangeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL updatesEnabled;
   hr = spRequest->get_UpdatesEnabled(&updatesEnabled);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL includeEnd;
   hr = spRequest->get_IncludeEnd(&includeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   ePfBoxUnits boxUnits;
   hr = spRequest->get_BoxUnits(&boxUnits);
   Common::ThrowOnCOMError(spRequest, hr);

   long boxSize;
   hr = spRequest->get_BoxSize(&boxSize);
   Common::ThrowOnCOMError(spRequest, hr);

   long reversal;
   hr = spRequest->get_Reversal(&reversal);
   Common::ThrowOnCOMError(spRequest, hr);

   ePfDataSource dataSource;
   hr = spRequest->get_DataSource(&dataSource);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant sessionsFilter;
   hr = spRequest->get_SessionsFilter(&sessionsFilter);
   Common::ThrowOnCOMError(spRequest, hr);

   eTimeSeriesContinuationType continuation;
   hr = spRequest->get_Continuation(&continuation);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL equalizeCloses;
   hr = spRequest->get_EqualizeCloses(&equalizeCloses);
   Common::ThrowOnCOMError(spRequest, hr);

   long daysBeforeExpiration;
   hr = spRequest->get_DaysBeforeExpiration(&daysBeforeExpiration);
   Common::ThrowOnCOMError(spRequest, hr);

   eSessionFlag sessionFlags;
   hr = spRequest->get_SessionFlags(&sessionFlags);
   Common::ThrowOnCOMError(spRequest, hr);
   
   eTimeSeriesSubscriptionLevel subscriptionLevel;
   hr = spRequest->get_SubscriptionLevel(&subscriptionLevel);
   Common::ThrowOnCOMError(spRequest, hr);
   
   long recalcPeriod;
   hr = spRequest->get_RecalcPeriod(&recalcPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   m_Symbol = ATL::CW2T(symbol);
   m_RangeStart = GetValueAsString(rangeStart);
   m_RangeEnd = GetValueAsString(rangeEnd);
   m_UpdatesEnabled = GetValueAsString(updatesEnabled, VT_BOOL);
   m_IncludeEnd = GetValueAsString(includeEnd, VT_BOOL);
   m_BoxUnits = CHelper::BoxUnitsToString(boxUnits);
   m_BoxSize = GetValueAsString(boxSize);
   m_Reversal = GetValueAsString(reversal);
   m_DataSource = CHelper::DataSourceToString(dataSource);
   CString filter = GetValueAsString(sessionsFilter);

   if (filter == "31")
   {
      filter = "All sessions (31)";
   }
   else if (filter == "0")
   {
      filter = "Primary session only (0)";
   }
   m_SessionsFilter = filter;

   m_Continuation = CHelper::ContinuationTypeToString(continuation);
   m_EqualizeCloses = GetValueAsString(equalizeCloses, VT_BOOL);
   m_DaysBeforeExpiration = GetValueAsString(daysBeforeExpiration);
   m_SessionFlags = CHelper::SessionFlagToString(sessionFlags);
   m_SubscriptionLevel = CHelper::SubscriptionLevelToString(subscriptionLevel);
   if (subscriptionLevel == tslEndOfBarAndPeriod)
   {
      m_RecalcPeriod = GetValueAsString(recalcPeriod);
   }
   
   UpdateData(FALSE);

   // Get error from point and figure bars collection and show it if exist
   ATL::CComPtr<ICQGError> spError;
   hr = m_spPointAndFigureBars->get_LastError(&spError);
   Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

   if (spError)
   {
      ATL::CComBSTR errDescription;
      hr = spError->get_Description(&errDescription);
      Common::ThrowOnCOMError(spError, hr);

      m_lblError.SetWindowText(ATL::CW2T(errDescription));
   }
}

void CPointAndFigureBarsDlg::ShowOutputParams()
{
   UpdateData();

   // Get output parameters from point and figure bars collection and show them
   DATE startTimestamp;
   HRESULT hr = m_spPointAndFigureBars->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

   DATE endTimestamp;
   hr = m_spPointAndFigureBars->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

   m_OutputStart = GetValueAsString(startTimestamp, VT_DATE);
   m_OutputEnd = GetValueAsString(endTimestamp, VT_DATE);

   UpdateData(FALSE);
}

void CPointAndFigureBarsDlg::ShowRecords(
                        long startIndex,
                        long endIndex,
                        CListCtrl& listCtrl)
{
   for (long i = startIndex; i <= endIndex; ++i)
   {
      AddRecord(i, listCtrl);
   }
}

void CPointAndFigureBarsDlg::AddRecord(
                        long recordIndex,
                        CListCtrl& listCtrl)
{
   // Add empty row
   int row = listCtrl.InsertItem(listCtrl.GetItemCount(), _T(""));

   ShowRecord(recordIndex, listCtrl, row);
}

void CPointAndFigureBarsDlg::ShowRecord(
                        long recordIndex,
                        CListCtrl& listCtrl,
                        long row)
{
   // Get specified PointAndFigureBar from point and figure bars collection by its index
   ATL::CComPtr<ICQGPointAndFigureBar> spPointAndFigureBar;
   HRESULT hr = m_spPointAndFigureBars->get_Item(recordIndex - 1, &spPointAndFigureBar);
   Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

   // Get PointAndFigureBar timestamp to show it
   DATE timestamp;
   hr = spPointAndFigureBar->get_Timestamp(&timestamp);
   Common::ThrowOnCOMError(spPointAndFigureBar, hr);

   listCtrl.SetItemText(row, rcIndex, GetValueAsString(recordIndex));
   listCtrl.SetItemText(row, rcTimestamp, GetValueAsString(timestamp, VT_DATE, true));

   // Show parameters into given listbox
   ShowProperty(spPointAndFigureBar, &ICQGPointAndFigureBar::get_Mid, listCtrl, row, rcMid);
   ShowProperty(spPointAndFigureBar, &ICQGPointAndFigureBar::get_High, listCtrl, row, rcHigh);
   ShowProperty(spPointAndFigureBar, &ICQGPointAndFigureBar::get_Low, listCtrl, row, rcLow);
   
   VARIANT_BOOL up;
   hr = spPointAndFigureBar->get_Up(&up);
   Common::ThrowOnCOMError(spPointAndFigureBar, hr);

   listCtrl.SetItemText(row, rcUp, GetValueAsString(up, VT_BOOL));
}

void CPointAndFigureBarsDlg::ShowProperty(
                        const ATL::CComPtr<ICQGPointAndFigureBar>& spPointAndFigureBar,
                        HRESULT (__stdcall ICQGPointAndFigureBar::*get_Property)(double*),
                        CListCtrl& listCtrl,
                        int row,
                        eRecordsColumns col)
{
   double value;
   HRESULT hr = (spPointAndFigureBar->*get_Property)(&value);
   Common::ThrowOnCOMError(spPointAndFigureBar, hr);

   listCtrl.SetItemText(row, col, GetValueAsString(value));
}

void CPointAndFigureBarsDlg::ClearIndices()
{
   m_edStart.SetValue(0);
   m_edEnd.SetValue(0);
}

void CPointAndFigureBarsDlg::FillIndices()
{
   if (m_spPointAndFigureBars == NULL)
   {
      return;
   }

   eRequestStatus status;
   HRESULT hr = m_spPointAndFigureBars->get_Status(&status);
   Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

   if (status != rsSuccess)
   {
      return;
   }

   // Fill start and end indices controls.
   m_edStart.SetValue(1);

   long count;
   hr = m_spPointAndFigureBars->get_Count(&count);
   Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

   m_edEnd.SetValue(count);
}

void CPointAndFigureBarsDlg::EnableControls()
{
   bool isDataConnectionUp = false;
   HRESULT hr;
   VARIANT_BOOL isStarted = VARIANT_FALSE;

   if (m_spCQGCEL)
   {
      hr = m_spCQGCEL->get_IsStarted(&isStarted);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }

   if (isStarted)
   {
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      hr = m_spCQGCEL->get_Environment(&spEnvironment);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      eConnectionStatus dataConnectionStatus;
      hr = spEnvironment->get_DataConnectionStatus(&dataConnectionStatus);
      Common::ThrowOnCOMError(spEnvironment, hr);

      isDataConnectionUp = (dataConnectionStatus == csConnectionUp);
   }

   m_btnRequest.EnableWindow(isDataConnectionUp);

   bool isPointAndFigureBarsSelected = (m_spPointAndFigureBars != NULL);

   m_btnRemove.EnableWindow(isPointAndFigureBarsSelected && isDataConnectionUp);
   m_btnRemoveAll.EnableWindow(isPointAndFigureBarsSelected && isDataConnectionUp);

   m_edTagName.EnableWindow(isPointAndFigureBarsSelected);
   m_edTagValue.EnableWindow(isPointAndFigureBarsSelected);

   bool isPointAndFigureBarsSucceeded = false;
   if (isPointAndFigureBarsSelected)
   {
      eRequestStatus status;
      hr = m_spPointAndFigureBars->get_Status(&status);
      Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

      isPointAndFigureBarsSucceeded = (status == rsSuccess);
   }

   m_edStart.EnableWindow(isPointAndFigureBarsSucceeded);
   m_spnStart.EnableWindow(isPointAndFigureBarsSucceeded);
   m_edEnd.EnableWindow(isPointAndFigureBarsSucceeded);
   m_spnEnd.EnableWindow(isPointAndFigureBarsSucceeded);

   m_btnShow.EnableWindow(isPointAndFigureBarsSucceeded);
}

CString CPointAndFigureBarsDlg::GetValueAsString(
                        ATL::CComVariant value,
                        VARENUM valueType/*= VT_EMPTY*/,
                        bool withMilliseconds /*= false*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return Common::VariantToString(value, withMilliseconds);
   }
   return CHelper::m_N_A;
}