// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include <cassert>
#include "Common.h"
#include "Helper.h"
#include "PointAndFigureBars.h"
#include "PointAndFigureBarsDlg.h"

STDMETHODIMP CPointAndFigureBarsDlg::OnDataError(
                           LPDISPATCH /*obj*/,
                           BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CPointAndFigureBarsDlg::OnDataConnectionStatusChanged(
                           eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));
      EnableControls();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CPointAndFigureBarsDlg::OnPointAndFigureBarsResolved(
                           ICQGPointAndFigureBars* cqg_point_and_figure_bars,
                           ICQGError* /*cqg_error*/)
{
   try
   {
      if (!m_spPointAndFigureBars)
      {
         return S_OK;
      }

      // Get selected point and figure bars ID
      ATL::CComBSTR selPointAndFigureBarsID;
      HRESULT hr = m_spPointAndFigureBars->get_Id(&selPointAndFigureBarsID);
      Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

      // Get changed point and figure bars ID
      ATL::CComBSTR pointAndFigureBarsID;
      hr = cqg_point_and_figure_bars->get_Id(&pointAndFigureBarsID);
      Common::ThrowOnCOMError<ICQGPointAndFigureBars>(cqg_point_and_figure_bars, hr);

      // Check if point and figure bars resolved in the selected request
      if (selPointAndFigureBarsID == pointAndFigureBarsID)
      {
         ClearAllData();
         ShowAllData();

         EnableControls();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CPointAndFigureBarsDlg::OnPointAndFigureBarsAdded(
                           ICQGPointAndFigureBars* cqg_point_and_figure_bars)
{
   try
   {
      if (!m_spPointAndFigureBars)
      {
         return S_OK;
      }

      // Get selected point and figure bars ID
      ATL::CComBSTR selPointAndFigureBarsID;
      HRESULT hr = m_spPointAndFigureBars->get_Id(&selPointAndFigureBarsID);
      Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

      // Get changed point and figure bars ID
      ATL::CComBSTR pointAndFigureBarsID;
      hr = cqg_point_and_figure_bars->get_Id(&pointAndFigureBarsID);
      Common::ThrowOnCOMError<ICQGPointAndFigureBars>(cqg_point_and_figure_bars, hr);

      // Check if record added in the selected request
      if (selPointAndFigureBarsID == pointAndFigureBarsID)
      {
         // Show changed records
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CPointAndFigureBarsDlg::OnPointAndFigureBarsInserted(
                           ICQGPointAndFigureBars* cqg_point_and_figure_bars,
                           long /*index_*/)
{
   try
   {
      if (!m_spPointAndFigureBars)
      {
         return S_OK;
      }

      // Get selected point and figure bars ID
      ATL::CComBSTR selPointAndFigureBarsID;
      HRESULT hr = m_spPointAndFigureBars->get_Id(&selPointAndFigureBarsID);
      Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

      // Get changed point and figure bars ID
      ATL::CComBSTR pointAndFigureBarsID;
      hr = cqg_point_and_figure_bars->get_Id(&pointAndFigureBarsID);
      Common::ThrowOnCOMError<ICQGPointAndFigureBars>(cqg_point_and_figure_bars, hr);

      // Check if record inserted in the selected request
      if (selPointAndFigureBarsID == pointAndFigureBarsID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
   
   return S_OK;
}

STDMETHODIMP CPointAndFigureBarsDlg::OnPointAndFigureBarsRemoved(
                           ICQGPointAndFigureBars* cqg_point_and_figure_bars,
                           long /*index_*/)
{
   try
   {
      if (!m_spPointAndFigureBars)
      {
         return S_OK;
      }

      // Get selected point and figure bars ID
      ATL::CComBSTR selPointAndFigureBarsID;
      HRESULT hr = m_spPointAndFigureBars->get_Id(&selPointAndFigureBarsID);
      Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

      // Get changed point and figure bars ID
      ATL::CComBSTR pointAndFigureBarsID;
      hr = cqg_point_and_figure_bars->get_Id(&pointAndFigureBarsID);
      Common::ThrowOnCOMError<ICQGPointAndFigureBars>(cqg_point_and_figure_bars, hr);

      // Check if record removed from the selected request
      if (selPointAndFigureBarsID == pointAndFigureBarsID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CPointAndFigureBarsDlg::OnPointAndFigureBarsUpdated(
                           ICQGPointAndFigureBars* cqg_point_and_figure_bars,
                           long index_)
{
   try
   {
      if (!m_spPointAndFigureBars)
      {
         return S_OK;
      }

      // Get selected point and figure bars ID
      ATL::CComBSTR selPointAndFigureBarsID;
      HRESULT hr = m_spPointAndFigureBars->get_Id(&selPointAndFigureBarsID);
      Common::ThrowOnCOMError(m_spPointAndFigureBars, hr);

      // Get changed point and figure bars ID
      ATL::CComBSTR pointAndFigureBarsID;
      hr = cqg_point_and_figure_bars->get_Id(&pointAndFigureBarsID);
      Common::ThrowOnCOMError<ICQGPointAndFigureBars>(cqg_point_and_figure_bars, hr);

      // Check if record updated in the selected request
      if (selPointAndFigureBarsID == pointAndFigureBarsID)
      {
         // Find row for specified point and figure bar
         CString indexStr;
         indexStr.Format(_T("%d"), index_ + 1);
         LVFINDINFO findInfo;
         findInfo.flags = LVFI_STRING;
         findInfo.psz = indexStr;
         int row = m_lvLastPointAndFigureBars.FindItem(&findInfo);

         // If found change it's properties
         if (row > -1)
         {
            ShowRecord(index_ + 1, m_lvLastPointAndFigureBars, row);
         }
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}