// PointAndFigureBars.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include <cassert>
#include "Helper.h"

const CString CHelper::m_N_A = _T("N/A");
const CString CHelper::DateFormat = _T("dd/MM/yyyy HH:mm");

void CHelper::SetComboBoxItem(
                        CComboBox& comboBoxCtrl,
                        DWORD_PTR itemData)
{
   for (int i = 0; i < comboBoxCtrl.GetCount(); ++i)
   {
      if (comboBoxCtrl.GetItemData(i) == itemData)
      {
         comboBoxCtrl.SetCurSel(i);
         return;
      }
   }

   assert(!"Invalid item data");
}

int CHelper::CompareDates(
                        const COleDateTime& date1,
                        const COleDateTime& date2)
{
   // Check if dates are equal (seconds are ignored)
   if (date1.GetYear() == date2.GetYear() && date1.GetMonth() == date2.GetMonth() &&
      date1.GetDay() == date2.GetDay() && date1.GetHour() == date2.GetHour() &&
      date1.GetMinute() == date2.GetMinute())
   {
      return 0;
   }

   return (date1 < date2 ? -1 : 1);
}

CString CHelper::ConnectionStatusToString(
                        eConnectionStatus connectionStatus)
{
   CString connectionStatusStr;

   switch (connectionStatus)
   {
   case csConnectionUp:
      connectionStatusStr = _T("Connection Up");
      break;
   case csConnectionDown:
      connectionStatusStr = _T("Connection Down");
      break;
   case csConnectionDelayed:
      connectionStatusStr = _T("Connection Delayed");
      break;
   default:
      assert(!"Unknown connection status");
   }

   return connectionStatusStr;
}

CString CHelper::RequestStatusToString(
                        eRequestStatus requestStatus)
{
   CString requestStatusStr;

   switch (requestStatus)
   {
   case rsInProgress:
      requestStatusStr = _T("InProgress");
      break;
   case rsSuccess:
      requestStatusStr = _T("Success");
      break;
   case rsCanceled:
      requestStatusStr = _T("Canceled");
      break;
   case rsFailed:
      requestStatusStr = _T("Failed");
      break;
   default:
      assert(!"Unknown request status");
   }

   return requestStatusStr;
}

CString CHelper::BoxUnitsToString(
                        ePfBoxUnits pfBoxUnits)
{
   CString pfBoxUnitsStr;

   switch (pfBoxUnits)
   {
   case pfbuPrice:
      pfBoxUnitsStr = _T("Price");
      break;
   case pfbuTicks:
      pfBoxUnitsStr = _T("Ticks");
      break;
   default:
      assert(!"Unknown box units");
   }

   return pfBoxUnitsStr;
}

CString CHelper::DataSourceToString(
                     ePfDataSource pfDataSource)
{
   CString pfDataSourceStr;

   switch (pfDataSource)
   {
   case pfdsTicks:
      pfDataSourceStr = _T("Ticks");
      break;
   case pfds1Min:
      pfDataSourceStr = _T("1Min");
      break;
   case pfds5Min:
      pfDataSourceStr = _T("5Min");
      break;
   case pfds15Min:
      pfDataSourceStr = _T("15Min");
      break;
   case pfds30Min:
      pfDataSourceStr = _T("30Min");
      break;
   case pfds60Min:
      pfDataSourceStr = _T("60Min");
      break;
   case pfdsDaily:
      pfDataSourceStr = _T("Daily");
      break;
   default:
      assert(!"Unknown data source");
   }

   return pfDataSourceStr;
}

CString CHelper::ContinuationTypeToString(
                        eTimeSeriesContinuationType continuationType)
{
   CString continuationTypeStr;

   switch (continuationType)
   {
   case tsctNoContinuation:
      continuationTypeStr = _T("NoContinuation");
      break;
   case tsctStandard:
      continuationTypeStr = _T("Standard");
      break;
   case tsctStandardByMonth:
      continuationTypeStr = _T("StandardByMonth");
      break;
   case tsctActive:
      continuationTypeStr = _T("Active");
      break;
   case tsctActiveByMonth:
      continuationTypeStr = _T("ActiveByMonth");
      break;
   case tsctAdjusted:
      continuationTypeStr = _T("Adjusted");
      break;
   case tsctAdjustedByMonth:
      continuationTypeStr = _T("AdjustedByMonth");
      break;
   default:
      assert(!"Unknown continuation type");
   }

   return continuationTypeStr;
}

CString CHelper::SessionFlagToString(
                        eSessionFlag sessionFlag)
{
   CString sessionFlagStr = _T("Undefined");

   if ((sessionFlag & sfDailyFromIntraday) == sfDailyFromIntraday)
   {
      sessionFlagStr = _T("DailyFromIntraday");
   }

   return sessionFlagStr;
}

CString CHelper::SubscriptionLevelToString(
                        eTimeSeriesSubscriptionLevel subscriptionLevel)
{
   CString subscriptionLevelStr;

   switch (subscriptionLevel)
   {
   case tslEachBar:
      subscriptionLevelStr = _T("EachBar");
      break;
   case tslEachTick:
      subscriptionLevelStr = _T("EachTick");
      break;
   case tslEndOfBarAndPeriod:
      subscriptionLevelStr = _T("EndOfBarAndPeriod");
      break;
   default:
      assert(!"Unknown subscription level");
   }

   return subscriptionLevelStr;
}