// PointAndFigureBarsReqDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "PointAndFigureBars.h"
#include "PointAndFigureBarsReqDlg.h"

namespace
{
   VARIANT_BOOL boolToVarBool(BOOL value)
   {
      return value ? VARIANT_TRUE : VARIANT_FALSE;
   }
}

CPointAndFigureBarsReqDlg::CPointAndFigureBarsReqDlg(
                              const ATL::CComPtr<ICQGPointAndFigureBarsRequest>& spRequest,
                              const DATE lineTime,
                              CWnd* pParent /*=NULL*/)
                              : CDialog(CPointAndFigureBarsReqDlg::IDD, pParent),
                                m_spRequest(spRequest),
                                m_LineTime(lineTime),
                                m_Symbol(_T("")),
                                m_RangeStartDate(COleDateTime::GetCurrentTime()),
                                m_RangeEndDate(COleDateTime::GetCurrentTime()),
                                m_RequestType(rtIntInt),
                                m_EqualizeCloses(FALSE),
                                m_UpdatesEnabled(FALSE),
                                m_DailyFromIntraday(FALSE),
                                m_CustomSession(FALSE)
{
   // Initialize local variables
   m_edRangeStart.NegativeAllowed(true);
   m_edRangeEnd.NegativeAllowed(true);
}

void CPointAndFigureBarsReqDlg::DoDataExchange(
                              CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);

   DDX_Text(pDX, IDC_ED_SYMBOL, m_Symbol);

   DDX_Control(pDX, IDC_DTP_RANGE_START, m_dtpRangeStart);
   DDX_DateTimeCtrl(pDX, IDC_DTP_RANGE_START, m_RangeStartDate);
   DDX_Control(pDX, IDC_ED_RANGE_START, m_edRangeStart);
   DDX_Control(pDX, IDC_SPN_RANGE_START, m_spnRangeStart);
   
   DDX_Control(pDX, IDC_DTP_RANGE_END, m_dtpRangeEnd);
   DDX_DateTimeCtrl(pDX, IDC_DTP_RANGE_END, m_RangeEndDate);
   DDX_Control(pDX, IDC_ED_RANGE_END, m_edRangeEnd);
   DDX_Control(pDX, IDC_SPN_RANGE_END, m_spnRangeEnd);

   DDX_Radio(pDX, IDC_RD_INT_INT, m_RequestType);

   DDX_Control(pDX, IDC_CHK_INCLUDE_END, m_chkIncludeEnd);
   DDX_Check(pDX, IDC_CHK_INCLUDE_END, m_IncludeEnd);

   DDX_Control(pDX, IDC_CMB_BOX_UNITS, m_cmbBoxUnits);

   DDX_Control(pDX, IDC_ED_BOX_SIZE, m_edBoxSize);
   DDX_Control(pDX, IDC_SPN_BOX_SIZE, m_spnBoxSize);

   DDX_Control(pDX, IDC_CMB_DATA_SOURCE, m_cmbDataSource);

   DDX_Control(pDX, IDC_ED_REVERSAL, m_edReversal);
   DDX_Control(pDX, IDC_SPN_REVERSAL, m_spnReversal);

   DDX_Control(pDX, IDC_CMB_CONTINUATION, m_cmbContinuation);

   DDX_Control(pDX, IDC_CHK_EQUALIZE_CLOSES, m_chkEqualizeCloses);
   DDX_Check(pDX, IDC_CHK_EQUALIZE_CLOSES, m_EqualizeCloses);

   DDX_Control(pDX, IDC_ED_DAYS_BEFORE_EXP, m_edDaysBeforeExp);
   DDX_Control(pDX, IDC_SPN_DAYS_BEFORE_EXP, m_spnDaysBeforeExp);

   DDX_Check(pDX, IDC_CHK_UPDATES_ENABLED, m_UpdatesEnabled);

   DDX_Check(pDX, IDC_CHK_DAILY_FROM_INTR, m_DailyFromIntraday);

   DDX_Control(pDX, IDC_ED_SESSIONS_FILTER, m_edSessionsFilter);
   DDX_Control(pDX, IDC_SPN_SESSIONS_FILTER, m_spnSessionsFilter);
   
   DDX_Control(pDX, IDC_ED_CUSTOM_SESSIONS_FILTER, m_edCustomSessionsFilter);
   DDX_Text(pDX, IDC_ED_CUSTOM_SESSIONS_FILTER, m_CustomSessionsFilter);

   DDX_Check(pDX, IDC_CHK_CUSTOM_SESSION, m_CustomSession);
   
   DDX_Control(pDX, IDC_CMB_SUBSCRIPTION, m_cmbSubscription);
   
   DDX_Control(pDX, IDC_ST_RECALC_PERIOD, m_lblRecalcPeriod);
   DDX_Control(pDX, IDC_ED_RECALC_PERIOD, m_edRecalcPeriod);
   DDX_Control(pDX, IDC_SPN_RECALC_PERIOD, m_spnRecalcPeriod);
}

BOOL CPointAndFigureBarsReqDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      InitializeControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;
}

BEGIN_MESSAGE_MAP(CPointAndFigureBarsReqDlg, CDialog)
   ON_BN_CLICKED(IDOK, OnBnClickedOk)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_SESSIONS_FILTER, OnDeltaposSpnSessionsFilter)
   ON_BN_CLICKED(IDC_CHK_CUSTOM_SESSION, OnBnClickedChkCustomSession)
   ON_CBN_SELCHANGE(IDC_CMB_CONTINUATION, OnCbnSelChangeCmbContinuation)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_BOX_SIZE, OnDeltaposSpnBoxSize)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RANGE_END, OnDeltaposSpnRangeEnd)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RANGE_START, OnDeltaposSpnRangeStart)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_DAYS_BEFORE_EXP, OnDeltaposSpnDaysBeforeExp)
   ON_BN_CLICKED(IDC_RD_INT_INT, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_RD_DATE_INT, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_RD_DATE_DATE, OnBnClickedRdRequestType)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_REVERSAL, OnDeltaposSpnReversal)
   ON_CBN_SELCHANGE(IDC_CMB_SUBSCRIPTION, OnCbnSelChangeCmbSubscription)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RECALC_PERIOD, OnDeltaposSpnRecalcPeriod)
END_MESSAGE_MAP()

void CPointAndFigureBarsReqDlg::InitializeControls()
{
   RequestTypeChanged();

   InitializeBoxUnitsCombo();
   InitializeDataSourceCombo();
   InitializeContinuationCombo();
   InitializeSubscriptionCombo();

   m_dtpRangeStart.SetFormat(CHelper::DateFormat);
   m_dtpRangeEnd.SetFormat(CHelper::DateFormat);

   // Set default parameters
   SetDefaultParameters();
}

void CPointAndFigureBarsReqDlg::SetDefaultParameters()
{
   UpdateData();

   ePfBoxUnits defBoxUnits;
   HRESULT hr = m_spRequest->get_BoxUnits(&defBoxUnits);
   Common::ThrowOnCOMError(m_spRequest, hr);

   long defBoxSize;
   hr = m_spRequest->get_BoxSize(&defBoxSize);
   Common::ThrowOnCOMError(m_spRequest, hr);

   ePfDataSource defDataSource;
   hr = m_spRequest->get_DataSource(&defDataSource);
   Common::ThrowOnCOMError(m_spRequest, hr);

   long defReversal;
   hr = m_spRequest->get_Reversal(&defReversal);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defUpdatesEnabled;
   hr = m_spRequest->get_UpdatesEnabled(&defUpdatesEnabled);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defIncludeEnd;
   hr = m_spRequest->get_IncludeEnd(&defIncludeEnd);
   Common::ThrowOnCOMError(m_spRequest, hr);

   ATL::CComVariant defSessionsFilter;
   hr = m_spRequest->get_SessionsFilter(&defSessionsFilter);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eTimeSeriesContinuationType defContinuation;
   hr = m_spRequest->get_Continuation(&defContinuation);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defEqualizeCloses;
   hr = m_spRequest->get_EqualizeCloses(&defEqualizeCloses);
   Common::ThrowOnCOMError(m_spRequest, hr);

   long defDaysBeforeExp;
   hr = m_spRequest->get_DaysBeforeExpiration(&defDaysBeforeExp);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eSessionFlag defSessionFlags;
   hr = m_spRequest->get_SessionFlags(&defSessionFlags);
   Common::ThrowOnCOMError(m_spRequest, hr);
   
   eTimeSeriesSubscriptionLevel defSubscription;
   hr = m_spRequest->get_SubscriptionLevel(&defSubscription);
   Common::ThrowOnCOMError(m_spRequest, hr);
   
   long defRecalcPeriod;
   hr = m_spRequest->get_RecalcPeriod(&defRecalcPeriod);
   Common::ThrowOnCOMError(m_spRequest, hr);

   // Set controls default values
   m_edBoxSize.SetValue(defBoxSize);
   CHelper::SetComboBoxItem(m_cmbBoxUnits, (DWORD_PTR)defBoxUnits);
   m_edReversal.SetValue(defReversal);
   CHelper::SetComboBoxItem(m_cmbDataSource, (DWORD_PTR)defDataSource);
   m_UpdatesEnabled = defUpdatesEnabled;
   m_IncludeEnd = defIncludeEnd;
   CHelper::SetComboBoxItem(m_cmbContinuation, (DWORD_PTR)defContinuation);
   m_EqualizeCloses = defEqualizeCloses;
   m_edDaysBeforeExp.SetValue(defDaysBeforeExp);
   CHelper::SetComboBoxItem(m_cmbSubscription, (DWORD_PTR)defSubscription);
   m_edRecalcPeriod.SetValue(defRecalcPeriod);
   SetSessionFlags(defSessionFlags);
   if (defSessionsFilter.vt == VT_UI4)
   {
      m_CustomSession = FALSE;
      m_edSessionsFilter.SetValue(defSessionsFilter.intVal);
      m_CustomSessionsFilter = _T("");
   }
   else if (defSessionsFilter.vt == VT_BSTR)
   {
      m_CustomSession = TRUE;
      m_edSessionsFilter.SetValue(0);
      m_CustomSessionsFilter = defSessionsFilter.bstrVal;
   }

   OnCbnSelChangeCmbContinuation();
   OnBnClickedChkCustomSession();
   OnCbnSelChangeCmbSubscription();

   m_edRangeStart.SetValue(0);
   m_RangeStartDate = m_LineTime - 1;
   m_edRangeEnd.SetValue(-10);
   m_RangeEndDate = m_LineTime;

   UpdateData(FALSE);
}

void CPointAndFigureBarsReqDlg::OnBnClickedOk()
{
   try
   {
      UpdateData();

      // Set request parameters
      ATL::CComBSTR symbol(m_Symbol);
      HRESULT hr = m_spRequest->put_Symbol(symbol);
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_RequestType == rtIntInt)
      {
         hr = m_spRequest->put_RangeStart(ATL::CComVariant(m_edRangeStart.GetValue()));
      }
      else
      {
         hr = m_spRequest->put_RangeStart(ATL::CComVariant(m_RangeStartDate.m_dt, VT_DATE));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_RequestType == rtDateDate)
      {
         hr = m_spRequest->put_RangeEnd(ATL::CComVariant(m_RangeEndDate.m_dt, VT_DATE));
      }
      else
      {
         hr = m_spRequest->put_RangeEnd(ATL::CComVariant(m_edRangeEnd.GetValue()));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_IncludeEnd(boolToVarBool(m_IncludeEnd));
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get box units
      ePfBoxUnits boxUnits = 
         static_cast<ePfBoxUnits>(m_cmbBoxUnits.GetItemData(m_cmbBoxUnits.GetCurSel()));

      hr = m_spRequest->put_BoxUnits(boxUnits);
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_BoxSize(m_edBoxSize.GetValue());
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get data source
      ePfDataSource dataSource = 
         static_cast<ePfDataSource>(m_cmbDataSource.GetItemData(m_cmbDataSource.GetCurSel()));

      hr = m_spRequest->put_DataSource(dataSource);
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_Reversal(m_edReversal.GetValue());
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get continuation type
      eTimeSeriesContinuationType continuationType = 
         static_cast<eTimeSeriesContinuationType>(m_cmbContinuation.GetItemData(m_cmbContinuation.GetCurSel()));

      hr = m_spRequest->put_Continuation(continuationType);
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_EqualizeCloses(boolToVarBool(m_EqualizeCloses));
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_DaysBeforeExpiration(m_edDaysBeforeExp.GetValue());
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_UpdatesEnabled(boolToVarBool(m_UpdatesEnabled));
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_CustomSession)
      {
         hr = m_spRequest->put_SessionsFilter(ATL::CComVariant(m_CustomSessionsFilter));
      }
      else
      {
         hr = m_spRequest->put_SessionsFilter(ATL::CComVariant(m_edSessionsFilter.GetValue()));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_SessionFlags(GetSessionFlags());
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get subscription type
      eTimeSeriesSubscriptionLevel subscriptionLevel = 
         static_cast<eTimeSeriesSubscriptionLevel>(m_cmbSubscription.GetItemData(m_cmbSubscription.GetCurSel()));

      hr = m_spRequest->put_SubscriptionLevel(subscriptionLevel);
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_RecalcPeriod(m_edRecalcPeriod.GetValue());
      Common::ThrowOnCOMError(m_spRequest, hr);
      
      // Close request window
      OnOK();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CPointAndFigureBarsReqDlg::InitializeBoxUnitsCombo()
{
   int row = m_cmbBoxUnits.AddString(_T("Price"));
   m_cmbBoxUnits.SetItemData(row, pfbuPrice);

   row = m_cmbBoxUnits.AddString(_T("Ticks"));
   m_cmbBoxUnits.SetItemData(row, pfbuTicks);
}

void CPointAndFigureBarsReqDlg::InitializeDataSourceCombo()
{
   int row = m_cmbDataSource.AddString(_T("Ticks"));
   m_cmbDataSource.SetItemData(row, pfdsTicks);

   row = m_cmbDataSource.AddString(_T("1Min"));
   m_cmbDataSource.SetItemData(row, pfds1Min);

   row = m_cmbDataSource.AddString(_T("5Min"));
   m_cmbDataSource.SetItemData(row, pfds5Min);

   row = m_cmbDataSource.AddString(_T("15Min"));
   m_cmbDataSource.SetItemData(row, pfds15Min);

   row = m_cmbDataSource.AddString(_T("30Min"));
   m_cmbDataSource.SetItemData(row, pfds30Min);

   row = m_cmbDataSource.AddString(_T("60Min"));
   m_cmbDataSource.SetItemData(row, pfds60Min);

   row = m_cmbDataSource.AddString(_T("Daily"));
   m_cmbDataSource.SetItemData(row, pfdsDaily);
}

void CPointAndFigureBarsReqDlg::InitializeContinuationCombo()
{
   int row = m_cmbContinuation.AddString(_T("NoContinuation"));
   m_cmbContinuation.SetItemData(row, tsctNoContinuation);

   row = m_cmbContinuation.AddString(_T("Standard"));
   m_cmbContinuation.SetItemData(row, tsctStandard);

   row = m_cmbContinuation.AddString(_T("StandardByMonth"));
   m_cmbContinuation.SetItemData(row, tsctStandardByMonth);

   row = m_cmbContinuation.AddString(_T("Active"));
   m_cmbContinuation.SetItemData(row, tsctActive);

   row = m_cmbContinuation.AddString(_T("ActiveByMonth"));
   m_cmbContinuation.SetItemData(row, tsctActiveByMonth);

   row = m_cmbContinuation.AddString(_T("Adjusted"));
   m_cmbContinuation.SetItemData(row, tsctAdjusted);

   row = m_cmbContinuation.AddString(_T("AdjustedByMonth"));
   m_cmbContinuation.SetItemData(row, tsctAdjustedByMonth);
}

void CPointAndFigureBarsReqDlg::InitializeSubscriptionCombo()
{
   int row = m_cmbSubscription.AddString(_T("EachBar"));
   m_cmbSubscription.SetItemData(row, tslEachBar);

   row = m_cmbSubscription.AddString(_T("EachTick"));
   m_cmbSubscription.SetItemData(row, tslEachTick);

   row = m_cmbSubscription.AddString(_T("EndOfBarAndPeriod"));
   m_cmbSubscription.SetItemData(row, tslEndOfBarAndPeriod);
}

void CPointAndFigureBarsReqDlg::OnBnClickedRdRequestType()
{
   try
   {
      RequestTypeChanged();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CPointAndFigureBarsReqDlg::OnDeltaposSpnBoxSize(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edBoxSize.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CPointAndFigureBarsReqDlg::OnDeltaposSpnReversal(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edReversal.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CPointAndFigureBarsReqDlg::OnDeltaposSpnRangeStart(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRangeStart.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CPointAndFigureBarsReqDlg::OnDeltaposSpnRangeEnd(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRangeEnd.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CPointAndFigureBarsReqDlg::OnDeltaposSpnDaysBeforeExp(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edDaysBeforeExp.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CPointAndFigureBarsReqDlg::OnDeltaposSpnSessionsFilter(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edSessionsFilter.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CPointAndFigureBarsReqDlg::OnBnClickedChkCustomSession()
{
   try
   {
      UpdateData();

      m_spnSessionsFilter.ShowWindow(m_CustomSession ? SW_HIDE : SW_SHOW);
      m_edSessionsFilter.ShowWindow(m_CustomSession ? SW_HIDE : SW_SHOW);

      m_edCustomSessionsFilter.ShowWindow(m_CustomSession ? SW_SHOW : SW_HIDE);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CPointAndFigureBarsReqDlg::OnCbnSelChangeCmbContinuation()
{
   try
   {
      // Get continuation type
      eTimeSeriesContinuationType continuationType = 
         static_cast<eTimeSeriesContinuationType>(m_cmbContinuation.GetItemData(m_cmbContinuation.GetCurSel()));

      switch (continuationType)
      {
      case tsctNoContinuation:
      case tsctStandard:
      case tsctStandardByMonth:
         m_chkEqualizeCloses.EnableWindow(FALSE);
         m_edDaysBeforeExp.EnableWindow(FALSE);
         m_spnDaysBeforeExp.EnableWindow(FALSE);
         break;
      case tsctActive:
      case tsctActiveByMonth:
         m_chkEqualizeCloses.EnableWindow(TRUE);
         m_edDaysBeforeExp.EnableWindow(FALSE);
         m_spnDaysBeforeExp.EnableWindow(FALSE);
         break;
      default:
         m_chkEqualizeCloses.EnableWindow(TRUE);
         m_edDaysBeforeExp.EnableWindow(TRUE);
         m_spnDaysBeforeExp.EnableWindow(TRUE);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CPointAndFigureBarsReqDlg::RequestTypeChanged()
{
   UpdateData();

   m_dtpRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_HIDE : SW_SHOW);
   m_edRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_SHOW : SW_HIDE);
   m_spnRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_SHOW : SW_HIDE);

   m_dtpRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_SHOW : SW_HIDE);
   m_edRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_HIDE : SW_SHOW);
   m_spnRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_HIDE : SW_SHOW);

   m_chkIncludeEnd.EnableWindow(m_RequestType == rtDateDate);
}

void CPointAndFigureBarsReqDlg::SetSessionFlags(
                              eSessionFlag sessionFlag)
{
   UpdateData();
   m_DailyFromIntraday = ((sessionFlag & sfDailyFromIntraday) == sfDailyFromIntraday);
   UpdateData(FALSE);
}

eSessionFlag CPointAndFigureBarsReqDlg::GetSessionFlags()
{
   UpdateData();

   int sessionFlag = sfUndefined;

   if (m_DailyFromIntraday)
   {
      sessionFlag = (sessionFlag | sfDailyFromIntraday);
   }

   return (eSessionFlag)sessionFlag;
}

void CPointAndFigureBarsReqDlg::OnCbnSelChangeCmbSubscription()
{
    try
   {
      // Get subscription level
      eTimeSeriesSubscriptionLevel subscriptionLevel = 
         static_cast<eTimeSeriesSubscriptionLevel>(m_cmbSubscription.GetItemData(m_cmbSubscription.GetCurSel()));

      if (subscriptionLevel == tslEndOfBarAndPeriod)
      {
         m_lblRecalcPeriod.EnableWindow(TRUE);
         m_edRecalcPeriod.EnableWindow(TRUE);
         m_spnRecalcPeriod.EnableWindow(TRUE);
      }
      else
      {
         m_lblRecalcPeriod.EnableWindow(FALSE);
         m_edRecalcPeriod.EnableWindow(FALSE);
         m_spnRecalcPeriod.EnableWindow(FALSE);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CPointAndFigureBarsReqDlg::OnDeltaposSpnRecalcPeriod(
                                NMHDR *pNMHDR,
                                LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRecalcPeriod.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}