// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include <vector>
#include "Common.h"
#include "QFormulasDlg.h"
#include "StrategyOrderPlacing.h"
#include "StrategyOrderPlacingDlg.h"

namespace
{
   inline CString getAccountName(const ATL::CComPtr<ICQGAccount>& spAccount)
   {
      ATL::CComBSTR accName;
      GET_PARAM(spAccount, accName, get_GWAccountName);

      ATL::CComBSTR FcmAccID;
      GET_PARAM(spAccount, FcmAccID, get_FcmAccountID);

      CString accString = ATL::CW2T(accName);
      accString = accString + _T(" (") + ATL::CW2T(FcmAccID) + _T(")");

      return accString;
   }
}

void CStrategyOrderPlacingDlg::InitializeCQGCEL()
{
   // create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   AssertCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   AssertCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   AssertCOMError(spConf, hr);

   hr = spConf->put_UseOrderSide(VARIANT_TRUE);
   AssertCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   AssertCOMError(m_spCQGCEL, hr);
}

void CStrategyOrderPlacingDlg::InitializeOrdersList(CListCtrl& lvOrders)
{
   lvOrders.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   lvOrders.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   lvOrders.InsertColumn(ocID, _T("Order #"), LVCFMT_LEFT, 55);
   lvOrders.InsertColumn(ocSide, _T("Buy/Sell"), LVCFMT_LEFT, 50);
   lvOrders.InsertColumn(ocSize, _T("Size"), LVCFMT_LEFT, 50);
   lvOrders.InsertColumn(ocState, _T("State"), LVCFMT_LEFT, 100);
   lvOrders.InsertColumn(ocInstrument, _T("Instrument"), LVCFMT_LEFT, 100);
   lvOrders.InsertColumn(ocType, _T("Type"), LVCFMT_LEFT, 58);
   lvOrders.InsertColumn(ocLimitPrice, _T("Limit Price"), LVCFMT_LEFT, 65);
   lvOrders.InsertColumn(ocStopPrice, _T("Stop Price"), LVCFMT_LEFT, 65);
   lvOrders.InsertColumn(ocDuration, _T("Duration"), LVCFMT_LEFT, 60);
   lvOrders.InsertColumn(ocAvgFillPrice, _T("Avg Fill Price"), LVCFMT_LEFT, 75);
   lvOrders.InsertColumn(ocPlaceTime, _T("Place Time"), LVCFMT_LEFT, 115);
   lvOrders.InsertColumn(ocLastFillTime, _T("Last Fill Time"), LVCFMT_LEFT, 115);
   lvOrders.InsertColumn(ocCanceledTime, _T("Canceled Time"), LVCFMT_LEFT, 115);
   
   // Hidden column
   lvOrders.InsertColumn(ocGWOrderID, _T("GWOrderID"), LVCFMT_LEFT, 0);
}

void CStrategyOrderPlacingDlg::InitializeTypesCombo()
{
   int row = m_cmbType.AddString(_T("Market"));
   m_cmbType.SetItemData(row, otMarket);

   row = m_cmbType.AddString(_T("Limit"));
   m_cmbType.SetItemData(row, otLimit);

   row = m_cmbType.AddString(_T("Stop"));
   m_cmbType.SetItemData(row, otStop);

   m_cmbPattern.AddString(_T(""));

   // Select first element
   m_cmbType.SetCurSel(0);
}

void CStrategyOrderPlacingDlg::InitializeCancelationInstructionCombo()
{
   // used enum values as row index and item data only for facility.
   m_cmbCancelationInstruction.InsertString(octUndefined, _T("Undefined"));
   m_cmbCancelationInstruction.SetItemData(octUndefined, octUndefined);

   m_cmbCancelationInstruction.InsertString(octCancelAllLegOrders, _T("CancelAllLegOrders"));
   m_cmbCancelationInstruction.SetItemData(octCancelAllLegOrders, octCancelAllLegOrders);

   m_cmbCancelationInstruction.InsertString(octCancelPrimaryLegOrders, _T("CancelPrimaryLegOrders"));
   m_cmbCancelationInstruction.SetItemData(octCancelPrimaryLegOrders, octCancelPrimaryLegOrders);

   m_cmbCancelationInstruction.EnableWindow(FALSE);
}

void CStrategyOrderPlacingDlg::InitializeDurationsCombo()
{
   int row = m_cmbDuration.AddString(_T("DAY"));
   m_cmbDuration.SetItemData(row, odDay);

   row = m_cmbDuration.AddString(_T("GTC"));
   m_cmbDuration.SetItemData(row, odGoodTillCanceled);
   
   row = m_cmbDuration.AddString(_T("GTD"));
   m_cmbDuration.SetItemData(row, odGoodTillDate);

   row = m_cmbDuration.AddString(_T("FAK"));
   m_cmbDuration.SetItemData(row, odFAK);

   row = m_cmbDuration.AddString(_T("FOK"));
   m_cmbDuration.SetItemData(row, odFOK);
   
   row = m_cmbDuration.AddString(_T("ATC"));
   m_cmbDuration.SetItemData(row, odATC);

   row = m_cmbDuration.AddString(_T("ATO"));
   m_cmbDuration.SetItemData(row, odATO);
   
   row = m_cmbDuration.AddString(_T("GTT"));
   m_cmbDuration.SetItemData(row, odGoodTillTime);

   // Select first element
   m_cmbDuration.SetCurSel(0);
}

void CStrategyOrderPlacingDlg::InitializeBooleans()
{
   m_DataConnectionUp = false;
   m_GWConnectionUp = false;
   m_LimitPriceEnabled = false;
   m_StopPriceEnabled = false;
   m_GTDEnabled = false;
}

void CStrategyOrderPlacingDlg::InitializeControls()
{
   ClearAllInstrumentData();

   OnDataConnectionStatusChanged(csConnectionDown);
   OnGWConnectionStatusChanged(csConnectionDown);

   InitializeOrdersList(m_lvOrders);
   InitializeOrdersList(m_lvLegOrders);
   InitializeTypesCombo();
   InitializeDurationsCombo();
   InitializeCancelationInstructionCombo();

   m_dtpGTD.SetFormat(m_SHORT_DATE);
}

namespace
{
   bool IsValidDefinition(const ATL::CComPtr<ICQGStrategyDefinition>& spDef)
   {
      if(spDef == NULL)
      {
         return false;
      }
      eStrategyRequestStatus status = srsNotActive;
      if(spDef != NULL)
      {
         GET_PARAM(spDef, status, get_Status);
      }
      return (status != srsFailed && status != srsNotActive);
   }
}

bool CStrategyOrderPlacingDlg::EnableDefinitionControls()
{
   ATL::CComPtr<ICQGInstrument> spInstrument = GetSelectedInstrument();
   BOOL isInstrumentSelected = spInstrument != NULL;

   bool definitioExists = false;

   if(isInstrumentSelected)
   {
      ATL::CComBSTR fullName;
      GET_PARAM(spInstrument, fullName, get_FullName);

      definitioExists = (m_expressionToGuid.find(fullName) != m_expressionToGuid.end());
   }

   CString instName;
   m_cmbSpreadExpr.GetWindowText(instName);
   
   m_btnSubscribe.EnableWindow(m_DataConnectionUp && !isInstrumentSelected && !instName.IsEmpty());
   m_btnDefine.EnableWindow(m_GWConnectionUp && !definitioExists && isInstrumentSelected);
   m_cmbSpreadExpr.EnableWindow(m_DataConnectionUp);

   m_spDefinition = GetCurrentStrategyDefinition();
   bool definitionStatus = definitionStatus = IsValidDefinition(m_spDefinition);

   return (m_spCQGCEL && isInstrumentSelected && definitionStatus && definitioExists);
}

void CStrategyOrderPlacingDlg::EnableControls()
{
   m_cmbAccounts.EnableWindow(m_GWConnectionUp);
   
   const bool isDefinitionSelected = EnableDefinitionControls();
   const bool isAccountSelected = (m_spCQGCEL && GetSelectedAccount());
   const bool isIcebergStrategy = m_chkIcebergStrategy.GetCheck() == BST_CHECKED;

   m_btnBuy.EnableWindow(m_GWConnectionUp && isAccountSelected && isDefinitionSelected);
   m_btnSell.EnableWindow(m_GWConnectionUp && isAccountSelected && isDefinitionSelected);

   m_edQuantity.EnableWindow(m_GWConnectionUp && isDefinitionSelected);
   m_spnQuantity.EnableWindow(m_GWConnectionUp && isDefinitionSelected);

   m_edLimitPrice.EnableWindow(m_GWConnectionUp && isDefinitionSelected && m_LimitPriceEnabled);
   m_spnLimitPrice.EnableWindow(m_GWConnectionUp && isDefinitionSelected && m_LimitPriceEnabled);
   
   m_edStopPrice.EnableWindow(m_GWConnectionUp && isDefinitionSelected && m_StopPriceEnabled);
   m_spnStopPrice.EnableWindow(m_GWConnectionUp && isDefinitionSelected && m_StopPriceEnabled);
   
   m_cmbType.EnableWindow(m_GWConnectionUp && isDefinitionSelected && !isIcebergStrategy);

   m_cmbDuration.EnableWindow(m_GWConnectionUp && isDefinitionSelected);
   m_dtpGTD.EnableWindow(m_GWConnectionUp && isDefinitionSelected && m_GTDEnabled);

   m_chkIcebergStrategy.EnableWindow(m_GWConnectionUp && isDefinitionSelected);
   m_edVisibleQuantity.EnableWindow(m_GWConnectionUp && isDefinitionSelected && isIcebergStrategy);

   BOOL isOrderSelected = (m_spCQGCEL && GetSelectedOrder() != NULL);

   CString patternName;
   m_cmbPattern.GetWindowText(patternName);
   m_btnPatternModify.EnableWindow(!patternName.IsEmpty());
   m_edPatternString.EnableWindow(patternName.IsEmpty());

   m_btnModify.EnableWindow(m_GWConnectionUp && isOrderSelected);
   m_btnCancel.EnableWindow(m_GWConnectionUp && isOrderSelected);
   m_btnCancelAll.EnableWindow(m_GWConnectionUp && isAccountSelected && (m_lvOrders.GetItemCount() > 0));
   m_btnTransection.EnableWindow(isOrderSelected);

   VARIANT_BOOL isOrderCanBeCanceled = FALSE;
   if (isOrderSelected)
   {
      GET_PARAM(GetSelectedOrder(), isOrderCanBeCanceled, get_CanBeCanceled);
   }
   
   m_cmbCancelationInstruction.EnableWindow(isOrderCanBeCanceled);
}

void CStrategyOrderPlacingDlg::ClearAllDefinitionData()
{
   m_edGwAlias.SetWindowText(m_N_A);
   m_edClientId.SetWindowText(m_N_A);
   m_edStrategyStatus.SetWindowText(m_N_A);
}

void CStrategyOrderPlacingDlg::ClearAllInstrumentData()
{
   ClearQuotes();

   ClearPosition();

   m_edStopPrice.SetWindowText(m_N_A);
   m_edLimitPrice.SetWindowText(m_N_A);
      
   SetQuantity(m_edQuantity, m_MIN_QUANTITY);
   SetQuantity(m_edVisibleQuantity, m_MIN_QUANTITY);
}

void CStrategyOrderPlacingDlg::ClearQuotes()
{
   // Set all quotes texts to N/A
   m_lblAskPrice.SetWindowText(m_N_A);
   m_lblAskVolume.SetWindowText(m_N_A);
   m_lblBidPrice.SetWindowText(m_N_A);
   m_lblBidVolume.SetWindowText(m_N_A);
}

void CStrategyOrderPlacingDlg::ClearPosition()
{
   // Set position texts to N/A
   m_lblLongPosition.SetWindowText(m_N_A);
   m_lblShortPosition.SetWindowText(m_N_A);
}

void CStrategyOrderPlacingDlg::ClearOrders()
{
   // Clear orders
   m_lvOrders.DeleteAllItems();
   m_lvLegOrders.DeleteAllItems();
}

void CStrategyOrderPlacingDlg::ReloadAccounts()
{
   // Clear all previous data
   m_cmbAccounts.ResetContent();
   ClearOrders();

   // Clear positions
   ClearPosition();
   m_positions.clear();
   m_failedAccounts.clear();

   // Get accounts from the CQGCEL
   ATL::CComPtr<ICQGAccounts> spAccounts;
   GET_PARAM(m_spCQGCEL, spAccounts, get_Accounts);

   long count;
   GET_PARAM(spAccounts, count, get_Count);

   if (count == 0)
   {
      // No accounts
      EnableControls();
      return;
   }

   // Iterate over accounts collection and add them to the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   GET_PARAM(spAccounts, spEnumVariant, get__NewEnum);

   std::vector<ATL::CComVariant> pItems(count);

   ULONG celtFetched;
   HRESULT hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      ATLASSERT(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGAccount> spAccount;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGAccount), (void** )&spAccount);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      // Add current account to the list
      InsertAccount(spAccount);

      // Query for filled orders
      QueryForPositions(spAccount);
   }

   // Select the first account in the list
   m_cmbAccounts.SetCurSel(0);

   EnableControls();
}

void CStrategyOrderPlacingDlg::QueryForPositions(const ATL::CComPtr<ICQGAccount>& spAccount)
{
   ATL::CComPtr<ICQGOrdersQuery> spOrdersQuery;
   HRESULT hr = m_spCQGCEL->QueryOrders(spAccount, NULL, osfFilled, ATL::COleDateTime::GetCurrentTime(), 
                                          osdUndefined, &spOrdersQuery);
   AssertCOMError(m_spCQGCEL, hr);
}

void CStrategyOrderPlacingDlg::UpdatePositions(const ATL::CComPtr<ICQGOrdersQuery>& spQuery)
{
   long queryID;
   GET_PARAM(spQuery, queryID, get_QueryID);

   eRequestStatus reqStatus;
   GET_PARAM(spQuery, reqStatus, get_Status);

   if (reqStatus != rsSuccess)
   {
      return;
   }

   // Get account data
   ATL::CComPtr<ICQGAccount> spAccount;
   GET_PARAM(spQuery, spAccount, get_Account);

   long accID;
   GET_PARAM(spAccount, accID, get_GWAccountID);

   if (reqStatus == rsFailed || reqStatus == rsCanceled)
   {
      m_failedAccounts.insert(accID);

      CString accName = getAccountName(spAccount);
      accName.Insert(0, _T("Failed to get positions for account "));
      throw std::runtime_error(ATL::CT2A(accName.GetString()));
   }

   // Get orders
   ATL::CComPtr<ICQGOrders> spOrders;
   GET_PARAM(spQuery, spOrders, get_Orders);

   long count;
   GET_PARAM(spOrders, count, get_Count);

   if (count > 0)
   {
      // Iterate over orders collection and add them to the list
      ATL::CComPtr<IEnumVARIANT> spEnumVariant;
      GET_PARAM(spOrders, spEnumVariant, get__NewEnum);

      std::vector<ATL::CComVariant> pItems(count);
      ULONG celtFetched;
      HRESULT hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
      AssertCOMError(spEnumVariant, hr);

      for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
      {
         assert(it->vt == VT_DISPATCH);

         ATL::CComPtr<ICQGOrder> spOrder;
         hr = it->pdispVal->QueryInterface(__uuidof(ICQGOrder), (void** )&spOrder);
         AssertCOMError<IDispatch>(it->pdispVal, hr);

         // Update position for specified order
         UpdatePosition(spOrder);
      }
   }
}

void CStrategyOrderPlacingDlg::UpdatePosition(const ATL::CComPtr<ICQGOrder>& spOrder)
{
   if(!IsStrategyOrder(spOrder))
   {
      return;
   }

   ATL::CComPtr<ICQGAccount> spAccount;
   GET_PARAM(spOrder, spAccount, get_Account);

   long accID;
   GET_PARAM(spAccount, accID, get_GWAccountID);

   if (m_failedAccounts.find(accID) != m_failedAccounts.end())
   {
      // Initial request for this account was failed
      return;
   }

   ATL::CComBSTR instrumentName;
   GET_PARAM(spOrder, instrumentName, get_InstrumentName);

   eOrderSide side;
   GET_PARAM(spOrder, side, get_Side);

   long quantity;
   GET_PARAM(spOrder, quantity, get_FilledQuantity);

   if (side == osdSell)
   {
      m_positions[accID][instrumentName].Short += quantity;
   }
   else if (side == osdBuy)
   {
      m_positions[accID][instrumentName].Long += quantity;
   }

   ShowPosition();   
}

void CStrategyOrderPlacingDlg::ShowPosition()
{
   // Get account
   ATL::CComPtr<ICQGAccount> spAccount = GetSelectedAccount();
   ATL::CComPtr<ICQGStrategyDefinition> spDefinition = GetCurrentStrategyDefinition();
   if (spAccount == NULL || spDefinition == NULL)
   {
      ClearPosition();
      return;
   }

   long accID;
   GET_PARAM(spAccount, accID, get_GWAccountID);

   ATL::CComBSTR alias;
   GET_PARAM(spDefinition, alias, get_Symbol);

   if ((alias.Length() == 0) || (m_failedAccounts.find(accID) != m_failedAccounts.end()))
   {
      // Initial request for this account was failed
      ClearPosition();
      return;
   }

   CString str;
   str.Format(_T("%d"), m_positions[accID][alias].Long);
   m_lblLongPosition.SetWindowText(str);

   str.Format(_T("%d"), m_positions[accID][alias].Short);
   m_lblShortPosition.SetWindowText(str);
}

void CStrategyOrderPlacingDlg::InsertAccount(
                                 const ATL::CComPtr<ICQGAccount>& spAccount)
{
   // Create string for specified account to add to the combo
   CString accString = getAccountName(spAccount);

   // Get account ID to map with item
   long accID;
   GET_PARAM(spAccount, accID, get_GWAccountID);

   // Add new account name to the combo and map ID
   int row = m_cmbAccounts.AddString(accString);
   m_cmbAccounts.SetItemData(row, accID);
}

void CStrategyOrderPlacingDlg::ShowStrategyProperties(
                                 const ATL::CComPtr<ICQGStrategyDefinition>& spDefinition)
{
   if(spDefinition == NULL)
   {
      return;
   }
   ShowPosition();

   ATL::CComBSTR defExpression;
   GET_PARAM(spDefinition, defExpression, get_RequestString);

   eStrategyRequestStatus status;
   GET_PARAM(spDefinition, status, get_Status);
   
   m_edStrategyStatus.SetWindowText(StatusToString(status));

   ATL::CComBSTR id;
   GET_PARAM(spDefinition,id, get_ClientId);
   
   m_edClientId.SetWindowText((ATL::CW2T)id);

   ATL::CComBSTR alias;
   GET_PARAM(spDefinition, alias, get_Symbol);

   m_edGwAlias.SetWindowText((ATL::CW2T)alias);
}

double CStrategyOrderPlacingDlg::GetValidPrice(
                                 const ATL::CComPtr<ICQGInstrument>& spInstrument)
{
   ATL::CComPtr<ICQGQuotes> spQuotes;
   GET_PARAM(spInstrument, spQuotes, get_Quotes);
 
   // Check quotes to get valid price
   ATL::CComPtr<ICQGQuote> spQuote;
   HRESULT hr = spQuotes->get_Item(qtTrade, &spQuote);
   AssertCOMError(spQuotes, hr);
   
   VARIANT_BOOL isValid;
   hr = spQuote->get_IsValid(&isValid);
   AssertCOMError(spQuote, hr);
   
   if (!isValid)
   {
      spQuote = NULL;
      hr = spQuotes->get_Item(qtAsk, &spQuote);
      AssertCOMError(spQuotes, hr);
   
      hr = spQuote->get_IsValid(&isValid);
      AssertCOMError(spQuote, hr);
      
      if (!isValid)
      {
         spQuote = NULL;
         hr = spQuotes->get_Item(qtBid, &spQuote);
         AssertCOMError(spQuotes, hr);
      }
   }
   
   double price = m_INVALID_DOUBLE;
   
   if (spQuote)
   {
      GET_PARAM(spQuote, price, get_Price);
      AssertCOMError(spQuote, hr);
   }
   
   return price;
}

void CStrategyOrderPlacingDlg::ShowValidPrice(
                                 const ATL::CComPtr<ICQGInstrument>& spInstrument)
{   
   m_edLimitPrice.SetWindowText(m_N_A);
   m_edStopPrice.SetWindowText(m_N_A);
   
   // Get order type
   eOrderType orderType = static_cast<eOrderType>
                                    (m_cmbType.GetItemData(m_cmbType.GetCurSel()));

   m_LimitPriceEnabled = (orderType == otLimit);
   m_StopPriceEnabled = (orderType == otStop);
   
   if (m_LimitPriceEnabled || m_StopPriceEnabled)
   {
      VARIANT_BOOL isValid;
      double price = GetValidPrice(spInstrument);
      
      HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(price), &isValid);
      AssertCOMError(m_spCQGCEL, hr);
      
      if (isValid == VARIANT_TRUE)
      {
         ATL::CComBSTR displayPrice;
         hr = spInstrument->ToDisplayPrice(price, &displayPrice);
         AssertCOMError(spInstrument, hr);
         
         if (m_LimitPriceEnabled)
         {
            m_edLimitPrice.SetWindowText(ATL::CW2T(displayPrice));
         }
         if (m_StopPriceEnabled)
         {
            m_edStopPrice.SetWindowText(ATL::CW2T(displayPrice));
         }
      }
   }
   
   EnableControls();
}

void CStrategyOrderPlacingDlg::ShowQuotes(
                                 const ATL::CComPtr<ICQGInstrument>& spInstrument)
{
   ATL::CComPtr<ICQGQuotes> spQuotes;
   GET_PARAM(spInstrument, spQuotes, get_Quotes);

   ShowQuote(spQuotes, qtAsk, m_lblAskPrice, m_lblAskVolume);
   ShowQuote(spQuotes, qtBid, m_lblBidPrice, m_lblBidVolume);
}

void CStrategyOrderPlacingDlg::ShowQuote(
                                 const ATL::CComPtr<ICQGQuotes>& spQuotes,
                                 eQuoteType quoteType,
                                 CStatic& priceLabel,
                                 CStatic& volumeLabel)
{
   // Update quote
   ATL::CComPtr<ICQGQuote> spQuote;
   HRESULT hr = spQuotes->get_Item(quoteType, &spQuote);
   if (SUCCEEDED(hr))
   {
      CString displayPrice;
      CString displayVolume;
      GetQuoteDisplayValues(spQuote, displayPrice, displayVolume); 
      priceLabel.SetWindowText(displayPrice);
      volumeLabel.SetWindowText(displayVolume);

      spQuote.Release();
   }
}

void CStrategyOrderPlacingDlg::GetQuoteDisplayValues(
                                 const ATL::CComPtr<ICQGQuote>& spQuote,
                                 CString& displayPrice,
                                 CString& displayVolume)
{
   displayPrice = m_N_A;
   displayVolume = m_N_A;

   if (spQuote != NULL)
   {
      VARIANT_BOOL isValid;
      HRESULT hr = spQuote->get_IsValid(&isValid);
      AssertCOMError(spQuote, hr);
      if (isValid)
      {
         double price;
         hr = spQuote->get_Price(&price);
         AssertCOMError(spQuote, hr);

         long volume;
         hr = spQuote->get_Volume(&volume);
         AssertCOMError(spQuote, hr);

         ATL::CComPtr<ICQGInstrument> spInstrument;
         hr = spQuote->get_Instrument(&spInstrument);
         AssertCOMError(spQuote, hr);

         // Get the display price from price
         ATL::CComBSTR dispPrice;
         hr = spInstrument->ToDisplayPrice(price, &dispPrice);
         AssertCOMError(spInstrument, hr);

         // Convert volume to string
         CString dispVolume;
         dispVolume.Format(_T("%d"), volume);

         displayPrice = ATL::CW2T(dispPrice);
         displayVolume = dispVolume;
      }
   }
}

void CStrategyOrderPlacingDlg::ShowFilteredOrders()
{
   // Clear old data
   ClearOrders();

   // Get account and it's all orders
   ATL::CComPtr<ICQGAccount> spAccount = GetSelectedAccount();
   
   if (spAccount == NULL)
   {
      // No account selected
      return;
   }

   ATL::CComPtr<ICQGOrders> spOrders;
   GET_PARAM(spAccount, spOrders, get_Orders);

   // No filter, show all orders
   ShowStrategyOrders(spOrders);
}

void CStrategyOrderPlacingDlg::UnselectAllOrders()
{
   for (int i = 0; i < m_lvOrders.GetItemCount(); ++i)
   {
      m_lvOrders.SetItemState(i, 0,  LVIS_SELECTED);
   }
}

void CStrategyOrderPlacingDlg::ShowStrategyOrders(const ATL::CComPtr<ICQGOrders>& spOrders)
{
   long count;
   GET_PARAM(spOrders, count, get_Count);

   if (count == 0)
   {
      // No orders
      return;
   }

   // Iterate over orders collection and add them to the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   GET_PARAM(spOrders, spEnumVariant, get__NewEnum);

   std::vector<ATL::CComVariant> pItems(count);

   ULONG celtFetched;
   HRESULT hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      assert(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGOrder> spOrder;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGOrder), (void** )&spOrder);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      // Add current order to the list
      ShowStrategyOrder(spOrder, ctAdded);
   }
}

void CStrategyOrderPlacingDlg::ShowLegOrders(const ATL::CComPtr<ICQGOrder>& spOrder)
{
   ATL::CComPtr<ICQGOrder> spSelectedOrder = GetSelectedOrder();
   if (spSelectedOrder == NULL)
   {
      return;
   }

   ATL::CComBSTR guid;
   GET_PARAM(spOrder, guid, get_GUID);

   ATL::CComBSTR selectedGuid;
   GET_PARAM(spSelectedOrder, selectedGuid, get_GUID);

   if(guid != selectedGuid)
   {
      return;
   }

   // Clear all legs data from list
   m_lvLegOrders.DeleteAllItems();

   // Leg orders are placed form CQGI so they are externals
   ATL::CComPtr<ICQGOrders> spOrders;
   GET_PARAM(m_spCQGCEL, spOrders, get_Orders);

   ATL::CComPtr<ICQGOrders> spLegOrders;
   HRESULT hr = spOrders->SelectByProperty(opStrategyOrderGUID, CComVariant(guid), &spLegOrders);
   AssertCOMError(spOrders, hr);

   // Iterate over leg orders
   long count;
   GET_PARAM(spLegOrders, count, get_Count);

   if (count == 0)
   {
      // No orders
      return;
   }

   // Iterate over orders collection and add them to the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   GET_PARAM(spLegOrders, spEnumVariant, get__NewEnum);

   std::vector<ATL::CComVariant> pItems(count);

   ULONG celtFetched;
   hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      assert(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGOrder> spLegOrder;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGOrder), (void** )&spLegOrder);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      // Add current order to the list
      ShowOrder(spLegOrder, ctAdded, m_lvLegOrders);
   }
}

void CStrategyOrderPlacingDlg::UpdateDefinitionInfo()
{
   ClearAllDefinitionData();

   CString currentName;
   m_cmbSpreadExpr.GetWindowText(currentName);

   if(currentName.IsEmpty() || m_cmbSpreadExpr.GetCurSel() == LB_ERR)
   {
      return;
   }

   CString selectedName;
   m_cmbSpreadExpr.GetLBText(m_cmbSpreadExpr.GetCurSel(),selectedName);
   
   int index = m_cmbSpreadExpr.FindStringExact(0, selectedName);

   if(selectedName != currentName && index == LB_ERR)
   {
      m_btnSubscribe.EnableWindow(!currentName.IsEmpty());
      ShowStrategyProperties(NULL);
      return;
   }

   ATL::CComPtr<ICQGStrategyDefinition> spDefinition = GetCurrentStrategyDefinition();
   ShowStrategyProperties(spDefinition);
}

void CStrategyOrderPlacingDlg::UpdateDefinitionHolder(
                        const ATL::CComPtr<ICQGStrategyDefinition>& spDefinition)
{
   m_spDefinition = spDefinition;

   eStrategyRequestStatus status;
   GET_PARAM(m_spDefinition, status, get_Status);

   if (status != srsSuccess)
   {
      return;
   }

   ATL::CComBSTR alias;
   GET_PARAM(m_spDefinition, alias, get_Symbol);
   
   ATL::CComBSTR instrumentName;
   GET_PARAM(m_spDefinition, instrumentName, get_RequestString);

   m_aliasToName[alias] = instrumentName;
}

int CStrategyOrderPlacingDlg::GetOrderRow(const ATL::CComPtr<ICQGOrder>& spOrder, CListCtrl& lvOrders)
{
   ATL::CComBSTR orderID;
   GET_PARAM(spOrder, orderID, get_OriginalOrderID);

   // Find row for specified order
   LVFINDINFO findInfo;
   findInfo.flags = LVFI_STRING;
   findInfo.psz = ATL::CW2T(orderID);

   return lvOrders.FindItem(&findInfo);
}

void CStrategyOrderPlacingDlg::ShowStrategyOrder(const ATL::CComPtr<ICQGOrder>& spOrder,
                                                 eChangeType change)
{
   if (IsStrategyOrder(spOrder))
   {
      VARIANT_BOOL isExternal;
      GET_PARAM(spOrder, isExternal, get_IsExternal);

      if(isExternal == VARIANT_TRUE)
      {
         return;
      }

      ShowOrder(spOrder, change, m_lvOrders);
      ShowLegOrders(spOrder);
   }
}

void CStrategyOrderPlacingDlg::ShowLegOrder(const ATL::CComPtr<ICQGOrder>& spOrder,
                                            eChangeType change)
{

   ATL::CComPtr<ICQGOrder> spSelectedOrder = GetSelectedOrder();
   if (spSelectedOrder == NULL)
   {
      return;
   }

   ATL::CComPtr<ICQGOrderProperties> spProperties;
   GET_PARAM(spOrder, spProperties, get_Properties);

   ATL::CComPtr<ICQGOrderProperty> spProperty;
   HRESULT hr = spProperties->get_Item(opStrategyOrderGUID, &spProperty);
   AssertCOMError(spProperties, hr);

   if (spProperty == NULL)
   {
      return;
   }

   ATL::CComVariant strategyGUID;
   GET_PARAM(spProperty, strategyGUID, get_Value);

   ATL::CComBSTR guid;
   GET_PARAM(spSelectedOrder, guid, get_GUID);

   if (guid != strategyGUID.bstrVal)
   {
      return;
   }

   ShowOrder(spOrder, change, m_lvLegOrders);
}

void CStrategyOrderPlacingDlg::ShowOrder(const ATL::CComPtr<ICQGOrder>& spOrder,
                                         eChangeType change,
                                         CListCtrl& lvOrders)
{
   // Get specified order's row
   int row = GetOrderRow(spOrder, lvOrders);

   // If change type is removed, remove order from list
   if (change == ctRemoved && row > -1)
   {
      lvOrders.DeleteItem(row);
      return;
   }

   // Add new one if not found
   if (row == -1)
   {
      row = lvOrders.InsertItem(lvOrders.GetItemCount(), _T(""));
   }
   
   ATL::CComBSTR orderID;
   GET_PARAM(spOrder, orderID, get_OriginalOrderID);

   eOrderSide side;
   GET_PARAM(spOrder, side, get_Side);

   long size;
   GET_PARAM(spOrder, size, get_Quantity);

   eOrderStatus status;
   GET_PARAM(spOrder, status, get_GWStatus);

   ATL::CComBSTR instrumentName;
   GET_PARAM(spOrder, instrumentName, get_InstrumentName);

   eOrderType type;
   GET_PARAM(spOrder, type, get_Type);

   double limitPrice;
   GET_PARAM(spOrder, limitPrice, get_LimitPrice);

   double stopPrice;
   GET_PARAM(spOrder, stopPrice, get_StopPrice);

   eOrderDuration duration;
   GET_PARAM(spOrder, duration, get_DurationType);

   DATE placeTime;
   GET_PARAM(spOrder, placeTime, get_PlaceTime);

   
   lvOrders.SetItemText(row, ocID, ATL::CW2T(orderID));
   lvOrders.SetItemText(row, ocSide, OrderSideToString(side));
   lvOrders.SetItemText(row, ocSize, GetValueAsString(size));
   lvOrders.SetItemText(row, ocState, OrderStatusToString(status));
   lvOrders.SetItemText(row, ocInstrument, ATL::CW2T(instrumentName));
   lvOrders.SetItemText(row, ocType, OrderTypeToString(type));
   lvOrders.SetItemText(row, ocLimitPrice, GetDisplayPrice(spOrder, limitPrice));
   lvOrders.SetItemText(row, ocStopPrice, GetDisplayPrice(spOrder, stopPrice));
   lvOrders.SetItemText(row, ocDuration, OrderDurationToString(duration));
   lvOrders.SetItemText(row, ocPlaceTime, GetValueAsString(placeTime, VT_DATE));
  
   //Get order properties to get average fill price
   ATL::CComPtr<ICQGOrderProperties> spProperties;
   GET_PARAM(spOrder, spProperties, get_Properties);

   // Get average fill price
   ATL::CComPtr<ICQGOrderProperty> spProperty;
   HRESULT hr = spProperties->get_Item(opAverageFillPrice, &spProperty);
   AssertCOMError(spProperties, hr);

   ATL::CComVariant avgFillPriceVar;
   GET_PARAM(spProperty, avgFillPriceVar, get_Value);

   double avgFillPrice = avgFillPriceVar.dblVal;
   lvOrders.SetItemText(row, ocAvgFillPrice, GetDisplayPrice(spOrder, avgFillPrice));

   // Get last fill time
   spProperty.Release();
   hr = spProperties->get_Item(opLastFillTime, &spProperty);
   AssertCOMError(spProperties, hr);

   CString lastFillTimeStr = m_N_A;
   if (spProperty)
   {
      ATL::CComVariant lastFillTimeVar;
      GET_PARAM(spProperty, lastFillTimeVar, get_Value);
      lastFillTimeStr = GetValueAsString(lastFillTimeVar.date, VT_DATE);
   }
   lvOrders.SetItemText(row, ocLastFillTime, lastFillTimeStr);

   // Get canceled time
   spProperty.Release();
   hr = spProperties->get_Item(opCanceledTime, &spProperty);
   AssertCOMError(spProperties, hr);

   CString canceledTimeStr = m_N_A;
   if (spProperty)
   {
      ATL::CComVariant canceledTimeVar;
      GET_PARAM(spProperty, canceledTimeVar, get_Value);
      canceledTimeStr = GetValueAsString(canceledTimeVar.date, VT_DATE);
   }
   lvOrders.SetItemText(row, ocCanceledTime, canceledTimeStr);

   // Map order GW ID with corresponding item
   ATL::CComBSTR gwOrderID;
   GET_PARAM(spOrder, gwOrderID, get_GWOrderID);
   
   lvOrders.SetItemText(row, ocGWOrderID, ATL::CW2T(gwOrderID));

   lvOrders.SetItemState(row, LVIS_SELECTED, LVIS_SELECTED);
}

CString CStrategyOrderPlacingDlg::GetDisplayPrice(
                                 const ATL::CComPtr<ICQGOrder> spOrder,
                                 double price) const
{
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(price), &isValid);
   AssertCOMError(m_spCQGCEL, hr);

   // Get instrument to get price display format
   ATL::CComBSTR alias;
   GET_PARAM(spOrder, alias, get_InstrumentName);

   // Get corresponding instrument for order alias
   ATL::CComPtr<ICQGInstrument> spInstrument = GetInstrumentByAlias(alias);

   CString displayPrice;
   if (isValid && spInstrument != NULL)
   {
      // Get display price
      ATL::CComBSTR displayPriceBSTR;
      hr = spInstrument->ToDisplayPrice(price, &displayPriceBSTR);
      AssertCOMError(spInstrument, hr);

      displayPrice = ATL::CW2T(displayPriceBSTR);
   }
   else
   {
      displayPrice = m_N_A;
   }

   return displayPrice;
}

CString CStrategyOrderPlacingDlg::GetValueAsString(
                                 ATL::CComVariant value,
                                 VARENUM valueType/*= VT_EMPTY*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   AssertCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return VariantToString(value);
   }
   return m_N_A;
}

ATL::CComPtr<ICQGAccount> CStrategyOrderPlacingDlg::GetSelectedAccount() const
{
   try
   {
      ATL::CComPtr<ICQGAccounts> spAccounts;
      GET_PARAM(m_spCQGCEL, spAccounts, get_Accounts);

      long accID = static_cast<long>(m_cmbAccounts.GetItemData(m_cmbAccounts.GetCurSel()));

      ATL::CComPtr<ICQGAccount> spAccount;
      HRESULT hr = spAccounts->get_Item(accID, &spAccount);
      AssertCOMError(spAccounts, hr);

      return spAccount;
   }
   catch (...)
   {
   }

   return NULL;
}

ATL::CComPtr<ICQGOrder> CStrategyOrderPlacingDlg::GetSelectedOrder() const
{
   try
   {
      ATL::CComPtr<ICQGOrders> spOrders;
      GET_PARAM(m_spCQGCEL, spOrders, get_Orders);

      int row = m_lvOrders.GetNextItem(-1, LVIS_SELECTED);
      if (row == -1)
      {
         return NULL;
      }

      ATL::CComBSTR gwOrderID = ATL::CT2W(m_lvOrders.GetItemText(row, ocGWOrderID));

      ATL::CComPtr<ICQGOrder> spOrder;
      HRESULT hr = spOrders->get_Item(gwOrderID, &spOrder);
      AssertCOMError(spOrders, hr);

      return spOrder;
   }
   catch (...)
   {
   }

   return NULL;
}

ATL::CComPtr<ICQGInstrument> CStrategyOrderPlacingDlg::GetSelectedInstrument() const
{
   try
   {
      // Get selected instrument's index
      int index = m_cmbSpreadExpr.GetCurSel();
      if (index != LB_ERR)
      {
         CString currentName;
         m_cmbSpreadExpr.GetWindowText(currentName);
         
         // Get text from the selected index
         CString selectedName;
         m_cmbSpreadExpr.GetLBText(index, selectedName);

         int index = m_cmbSpreadExpr.FindStringExact(0, selectedName);

         if(currentName != selectedName && index == LB_ERR)
         {
            return NULL;
         }

         ATL::CComBSTR fullName = ATL::CT2W(selectedName);

         // Get instruments collection
         ATL::CComPtr<ICQGInstruments> spInstruments;
         GET_PARAM(m_spCQGCEL, spInstruments, get_Instruments);

         // Get instrument from collection by it's full name
         ATL::CComVariant idx(fullName);
         ATL::CComPtr<ICQGInstrument> spInstrument;
         HRESULT hr = spInstruments->get_Item(idx, &spInstrument);
         AssertCOMError(m_spCQGCEL, hr);

         return spInstrument;
      }
   }
   catch (...)
   {
   }

   return NULL;
}

ATL::CComPtr<ICQGInstrument> CStrategyOrderPlacingDlg::GetInstrumentByAlias(
                                 const ATL::CComBSTR& alias) const
{
   try
   {
      AliasToName::const_iterator it = m_aliasToName.find(alias);
      if(it == m_aliasToName.end())
      {
         return NULL;
      }
      
      ATL::CComPtr<ICQGInstruments> spInstruments;
      GET_PARAM(m_spCQGCEL, spInstruments, get_Instruments);

      ATL::CComBSTR instrumentName(it->second);

      ATL::CComPtr<ICQGInstrument> spInstrument;
      HRESULT hr = spInstruments->get_Item(ATL::CComVariant(instrumentName), &spInstrument);
      AssertCOMError(spInstruments, hr);

      return spInstrument;
   }
   catch (...)
   {
   }

   return NULL;
}

void CStrategyOrderPlacingDlg::PlaceOrder(
                                 eOrderSide orderSide)
{
   ATL::CComPtr<ICQGAccount> spAccount = GetSelectedAccount();

   ATL::CComPtr<ICQGInstrument> spInstrument = GetSelectedInstrument();

   // Get order type
   eOrderType orderType = static_cast<eOrderType>
                                     (m_cmbType.GetItemData(m_cmbType.GetCurSel()));

   // Get quantity
   long quantity = GetQuantity(m_edQuantity);

   // Get limit price and stop price 
   double limitPrice = 0;
   double stopPrice = 0;

   switch(orderType)
   {
      case otLimit:
         limitPrice = GetPrice(m_edLimitPrice, spInstrument);
         break;
      case otStop:
         stopPrice = GetPrice(m_edStopPrice, spInstrument);
         break;
   }

   if (limitPrice == m_INVALID_DOUBLE || stopPrice == m_INVALID_DOUBLE)
   {
      // Limit price or/and stop price is incorrect
      AfxMessageBox(_T("Incorrect price for '") + OrderTypeToString(orderType) + _T("' order"));
      return;
   }

   // Create order
   ATL::CComPtr<ICQGOrder> spOrder;
   HRESULT hr = m_spCQGCEL->CreateStrategyOrder(orderType, m_spDefinition, spAccount, NULL,
                           quantity, orderSide, limitPrice, 
                           stopPrice, L"", &spOrder);
   AssertCOMError(m_spCQGCEL, hr);

   // Put duration type of the order
   eOrderDuration orderDuration = static_cast<eOrderDuration>
                                    (m_cmbDuration.GetItemData(m_cmbDuration.GetCurSel()));
   hr = spOrder->put_DurationType(orderDuration);
   AssertCOMError(m_spCQGCEL, hr);

   // Get order properties to change some values
   ATL::CComPtr<ICQGOrderProperties> spProperties;
   hr = spOrder->get_Properties(&spProperties);
   AssertCOMError(spOrder, hr);

   if (orderDuration == odGoodTillDate || orderDuration == odGoodTillTime)
   {
      // Set GTD time property if order duration is GTD or GTT
      COleDateTime oleDateTime;
      m_dtpGTD.GetTime(oleDateTime);

      ATL::CComVariant date(oleDateTime);
      date.vt = VT_DATE;

      SetPropertyValue(spProperties, opGTDTime, date);
   }

   if(!IsValidDefinition(m_spDefinition))
   {
      AfxMessageBox(_T("Invalid strategy definition"));
      return;
   }

   // Set pattern string
   ATL::CComBSTR patternString;

   CString temp;
   m_edPatternString.GetWindowText(temp);
   patternString = ATL::CT2W(temp.Trim());

   if (patternString.Length() == 0)
   {
      ATL::CComPtr<ICQGExecutionPattern> spPattern;
      hr = m_spCQGCEL->CreateExecutionPattern(m_spDefinition, orderType, &spPattern);
      AssertCOMError(m_spCQGCEL, hr);

      GET_PARAM(spPattern, patternString, get_PatternString);
   } 

   if (m_chkIcebergStrategy.GetCheck() == BST_CHECKED)
   {
      SetPropertyValue(spProperties, opExecutionInstructions, oeiIceberg);
      SetPropertyValue(spProperties, opVisibleQuantity, GetQuantity(m_edVisibleQuantity));
   }

   SetPropertyValue(spProperties, opExecutionPattern, ATL::CComVariant(patternString));

   // Place order
   hr = spOrder->Place();
   AssertCOMError(spOrder, hr);
}

CString CStrategyOrderPlacingDlg::OrderDurationToString(
                                 eOrderDuration duration)
{
   CString durationString;

   switch (duration)
   {
      case odATC:
         durationString = _T("ATC");
         break;
      case odATO:
         durationString = _T("ATO");
         break;
      case odDay:
         durationString = _T("DAY");
         break;
      case odFAK:
         durationString = _T("FAK");
         break;
      case odFOK:
         durationString = _T("FOK");
         break;
      case odGoodTillCanceled:
         durationString = _T("GTC");
         break;
      case odGoodTillDate:
         durationString = _T("GTD");
         break;
      case odGoodTillTime:
         durationString = _T("GTT");
         break;
      case odUndefined:
         durationString = _T("Undefined");
         break;
      default:
         assert(!"Unknown order duration");
   }

   return durationString;
}

CString CStrategyOrderPlacingDlg::OrderSideToString(
                                 eOrderSide side)
{
   CString sideString;

   switch (side)
   {
      case osdBuy:
         sideString = _T("Buy");
         break;
      case osdSell:
         sideString = _T("Sell");
         break;
      case osdUndefined:
         sideString = _T("Undefined");
         break;
      default:
         assert(!"Unknown order side");
   }

   return sideString;
}

CString CStrategyOrderPlacingDlg::OrderTypeToString(
                                 eOrderType type)
{
   CString typeString;

   switch (type)
   {
      case otLimit:
         typeString = _T("Limit");
         break;
      case otMarket:
         typeString = _T("Market");
         break;
      case otStop:
         typeString = _T("Stop");
         break;
      case otStopLimit: 
         typeString = _T("Stop Limit");
         break;
      case otUndefined:
         typeString = _T("Undefined");
         break;
      default:
         assert(!"Unknown order type");
   }

   return typeString;
}

CString CStrategyOrderPlacingDlg::OrderStatusToString(
                                 eOrderStatus status) const
{
   CString statusString;

   switch (status)
   {
      case osActiveAt:
         statusString = _T("Active At");
         break;
      case osBusted:
         statusString = _T("Busted");
         break;
      case osCanceled:
         statusString = _T("Canceled");
         break;
      case osContingent:
         statusString = _T("Contingent");
         break;
      case osDisconnected:
         statusString = _T("Disconnected");
         break;
      case osExpired:
         statusString = _T("Expired");
         break;
      case osFilled:
         statusString = _T("Filled");
         break;
      case osInCancel:
         statusString = _T("In Cancel");
         break;
      case osInClient:
         statusString = _T("In Client");
         break;
      case osInModify:
         statusString = _T("In Modify");
         break;
      case osInOrderBook:
         statusString = _T("In Order Book");
         break;
      case osInTransit:
         statusString = _T("In Transit");
         break;
      case osInTransitTimeout:
         statusString = _T("In Transit Timeout");
         break;
      case osNotSent:
         statusString = _T("Not Sent");
         break;
      case osParked:
         statusString = _T("Parked");
         break;
      case osRejectFCM:
         statusString = _T("Reject FCM");
         break;
      case osRejectGW:
         statusString = _T("Reject GW");
         break;
      default:
         assert(!"Unknown order status");
   }

   return statusString;
}

CString CStrategyOrderPlacingDlg::ConnectionStatusToString(
                                 eConnectionStatus status)
{
   CString  statusString;

   switch (status)
   {
   case csConnectionDelayed:
      statusString = _T("delayed");
      break;
   case csConnectionDown:
      statusString = _T("down");
      break;
   case csConnectionUp:
      statusString = _T("up");
      break;
   case csConnectionTrouble:
      statusString = _T("trouble");
      break;
   case csConnectionNotLoggedOn:
      statusString = _T("not logged on");
      break;
   default:
      assert(!"Unknown connection status");
   }

   return statusString;
}

CString CStrategyOrderPlacingDlg::StatusToString(const eStrategyRequestStatus status)
{
   CString caption = _T("Undefined Status");
   switch (status)
   {
   case srsNotActive:
      caption = _T("not active");
      break;
   case srsPending:
      caption = _T("pending");
      break;
   case srsSuccess:
      caption = _T("success");
      break;
   case srsFailed:
      caption = _T("failed");
      break;
   default:
      assert(!"Unknown eStrategyRequestStatus value");
   }
   return caption;
}

ATL::CComPtr<ICQGStrategyDefinition> CStrategyOrderPlacingDlg::GetCurrentStrategyDefinition()
{
   if(m_cmbSpreadExpr.GetCurSel() == LB_ERR)
   {
      return NULL;
   }

   ATL::CComPtr<ICQGStrategyDefinition> spDef;
   CString name;
   m_cmbSpreadExpr.GetLBText(m_cmbSpreadExpr.GetCurSel(),name);
   try
   {
      ATL::CComPtr<ICQGInstrument> spInstrument = GetSelectedInstrument();

      if(!spInstrument)
      {
         return NULL;
      }

      ATL::CComBSTR fullName;
      GET_PARAM(spInstrument, fullName, get_FullName);

      NameToGuid::iterator it = m_expressionToGuid.find(fullName);

      if(it != m_expressionToGuid.end())
      {
         // Get all definitions 
         ATL::CComPtr<ICQGStrategyDefinitions> spDefinitions;
         GET_PARAM(m_spCQGCEL, spDefinitions, get_AllStrategyDefinitions);

         HRESULT hr = spDefinitions->get_Item(ATL::CComBSTR(it->second), &spDef);
         AssertCOMError(spDefinitions, hr);
      }
   }
   catch(...)
   {
   }

   return spDef;
}

bool CStrategyOrderPlacingDlg::IsStrategyOrder(const ATL::CComPtr<ICQGOrder>& spOrder) const
{
   try
   {
      CComBSTR instrumentName;
      GET_PARAM(spOrder, instrumentName, get_InstrumentName);
            
      CString name(instrumentName);

      if(name.Left(2) == _T("Y."))
      {
         return true;
      }
   }
   catch(...)
   {
   }
   return false;
}


void CStrategyOrderPlacingDlg::ShowCancelationInstruction(const ATL::CComPtr<ICQGOrder>& spOrder)
{
   if (spOrder == NULL)
   {
      return;
   }

   eOrderCancelationType cancelationType;
   GET_PARAM(spOrder, cancelationType, get_CancelationInstruction);
   
   m_cmbCancelationInstruction.SetCurSel(cancelationType);
}

void CQFormulasDlg::ClearAllData()
{
   m_cmbDefinition.ResetContent();
   m_lblQFormulaNumber.SetWindowText(CStrategyOrderPlacingDlg::m_N_A);
   m_edQFormulaExpression.SetWindowText(CStrategyOrderPlacingDlg::m_N_A);
   m_edQFormulaExecPattern.SetWindowText(CStrategyOrderPlacingDlg::m_N_A);
   m_qFormula = _T("");
}

const CString& CQFormulasDlg::GetQFormulaName() const
{
   return m_qFormula;
}

const CString& CQFormulasDlg::GetExecutionPattern() const
{
   return m_executionPattern;
}