// ExecutionPatternDlg.h : header file
//
#pragma once
#include "afxcmn.h"
#include "afxwin.h"

#include <map>
#include <stack>


// ExecutionPatternDlg dialog

class CExecutionPatternDlg : public CDialog
{
	DECLARE_DYNAMIC(CExecutionPatternDlg)

public:
   // public typedefs
   typedef ATL::CComPtr<ICQGExecutionPatternParameter> PatternParameterPtr;
   typedef ATL::CComPtr<ICQGExecutionPatternParameters> PatternParametersPtr;

public:
   /// \brief Constructor
   /// \param spCel ICQGCEL instance
   /// \param spPattern Execution pattern instance
   /// \param patternName Name of pattern
   /// \param patterns collection of paterns
   /// \param spDef StrategyDefinition for default pattern creation
   /// \param pParent Parent window
	CExecutionPatternDlg(ATL::CComPtr<ICQGCEL>& spCel,
                        ATL::CComPtr<ICQGExecutionPattern>& spPattern,
                        const CString& patternName,
                        std::map<CString, ATL::CComPtr<ICQGExecutionPattern> >& patterns,
                        ATL::CComPtr<ICQGStrategyDefinition> spDef = NULL, 
                        CWnd* pParent = NULL);   // standard constructor

	virtual ~CExecutionPatternDlg();

   // Dialog Data
	enum { IDD = IDD_EXECUTION_PATTERN_DIALOG };

public:
   /// \brief Returns execution pattern
   /// \return execution pattern if presence or NULL otherwise  
   ATL::CComPtr<ICQGExecutionPattern> GetPattern() const;

   /// \brief Returns execution pattern name
   /// \return executin pattern name
   const CString& GetPatternName() const;

protected:
   /// \brief Dialog icon.
   HICON m_hIcon;

   ///brief DDX/DDV support
	virtual void DoDataExchange(CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

   DECLARE_MESSAGE_MAP()
   
public:
   /// \brief Enable/disable default pattern creation flag
   afx_msg void OnBnClickedChdefaultPattern();

   /// \brief Observes pattern name change
   afx_msg void OnEnChangeEdpatternName();

   /// \brief Creates new pattern
   afx_msg void OnBnClickedButtonCreate();

   /// \brief Adds parameter to the execution pattern
   afx_msg void OnBnClickedButtonAdd();

   /// \brief Removes current selected param
   afx_msg void OnBnClickedButtonParamRemove();

   /// \brief Parameters name input control response processor
   afx_msg void OnEnChangeEdparamName();

   /// \brief Response current selection change in TreeCtrl
   afx_msg void OnTvnSelchangedPatternsTree(NMHDR *pNMHDR, LRESULT *pResult);

   /// \brief Ok button pressed responder
   afx_msg void OnBnClickedOk();

   /// \brief Remove pattern's all params
   afx_msg void OnBnClickedButtonParamRemoveAll();

   /// \brief Process parameter type changes
   afx_msg void OnCbnSelchangeCmbparamType();

private:

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Shows execution pattern tree in treeView
   /// \param spPatternParams Pattern parameters to show
   /// \param parentItem Parent item in the TreeCtrl for given parameters  
   void ShowPatternParams(PatternParametersPtr spPatternParams, HTREEITEM parentItem);

   /// \brief Adds given param to the tree view
   /// \param spPatternParams Pattern parameter to show
   /// \param parentItem Parent item in the TreeCtrl for given parameter  
   void AddParamToView(PatternParameterPtr& spParam, HTREEITEM parentItem);

   /// \brief Adds tree item with given text
   /// \param itemText Text to show in view
   /// \param parentItem Parent item of new item in TreeCtrl
   HTREEITEM AddItemToTree(const CString& itemText, HTREEITEM parentItem);
   
   /// \brief Search pattern param appropriate to given TreeCtrl item and fill stack of parents of the item
   /// \param item Item of tree to search
   /// \param path Stack of parent parameters of the found execution pattern
   /// \note top of the stack is parameter which linked with given tree item
   /// \return Parameter accossiated with given tree item
   PatternParameterPtr FindParamPath(const HTREEITEM& item, 
                                       std::stack<PatternParameterPtr >& path ) const;
   
   /// \brief Creates clone of given execution pattern
   /// \param spPattern pattern to clone
   ATL::CComPtr<ICQGExecutionPattern> ClonePattern(const ATL::CComPtr<ICQGExecutionPattern>& spPattern) const;

   /// \brief Redraw execution pattern 
   void RedrawTree();

   /// \brief Enables/disables controls on the form.
   void EnableControls();

 private:
   
    /// \brief Execution pattern tree view control
   CTreeCtrl m_trPatternView;

   /// \brief Execution pattern name control
   CEdit m_edPatternName;

   /// \brief Execution pattern type combobox control
   CComboBox m_cmbPatternType;

   /// \brief Create button control
   CButton m_btnCreate;

   // / \brief to recognize creation default execution pattern
   CButton m_chDefaultPattern;

   /// \brief OK button control
   CButton m_btnOk;

   /// \brief Execution pattern parameter name input control
   CEdit m_edParamName;
   
   /// \brief Add parameter to pattern button control
   CButton m_btnParamAdd;

   /// \brief Execution pattern parameter type input control
   CComboBox m_cmbParamType;

   /// \brief Remove parameter from pattern button control
   CButton m_btnParamRemove;
   
   /// \brief Remove all parameters from pattern button control
   CButton m_btnParamRemoveAll;

   /// \brief Execution pattern parameter value input control
   CEdit m_edParamValue;

private:

   /// \brief ICQGCEL instance holder
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Execution pattern holder for creating or modifying
   ATL::CComPtr<ICQGExecutionPattern> m_spPattern;

   /// \brief current pattern name holder
   CString m_patternName;

   /// \brief Associated container of name to execution patterns reference
   std::map<CString, ATL::CComPtr<ICQGExecutionPattern> >& m_patterns;

   /// \brief Strategy definition holder for default pattern creation
   ATL::CComPtr<ICQGStrategyDefinition> m_spDefinition;

   /// \brief Flag which represents is dialog opened for creating or modifying pattern(true/false)
   BOOL m_isNewPattern;
   
   /// \brief Holder of selected param of execution pattern
   PatternParameterPtr m_spCurrentParam;
};
