// StrategyOrderPlacingDlg.h : header file
//

#pragma once
#include "afxwin.h"
#include "afxcmn.h"

#include <map>
#include <set>
#include <utility>

class CStrategyOrderPlacingDlg;

/**
\typedef ATL::IDispEventImpl<1, CStrategyOrderPlacingDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CStrategyOrderPlacingDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

// CStrategyOrderPlacingDlg dialog
class CStrategyOrderPlacingDlg : public CDialog, public ICQGCELDispEventImpl
{

public:

   /// \brief Constructor
   /// \param pParent Parent window ponter
   CStrategyOrderPlacingDlg(CWnd* pParent = NULL);

   /// \enum Dialog Data
   enum { IDD = IDD_STRATEGYORDERPLACING_DIALOG };

protected:

   /// \brief Dialog icon.
   HICON m_hIcon;

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
                  CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter.
   virtual void OnOK();

protected:

   /// \brief Prepares the container for painting.
   afx_msg void OnPaint();

   /// \brief The system calls this function to obtain the cursor to
   /// \brief display while the user drags the minimized window.
   afx_msg HCURSOR OnQueryDragIcon();

   /// \brief Used to unadvise from CQGCEL events and shutdown it.
   afx_msg void OnDestroy();

   /// \brief Subscribes instrument.
   afx_msg void OnBnClickedBtnSubscribe();

   /// \brief Launches QFormula dialog.
   afx_msg void OnBnClickedBtnQFormula();

   /// \brief Changes selected instrument, and shows it's data.
   afx_msg void OnCbnSelchangeCmbSpread();

   /// \brief Changes selected account, and shows it's orders.
   afx_msg void OnCbnSelChangeCmbAccounts();

   /// \brief Opens dialog to modify selected order.
   afx_msg void OnBnClickedBtnModify();

   /// \brief Define selected strategy.
   afx_msg void OnBnClickedBtnDefineStrategy();

   /// \brief Cancels selected order.
   afx_msg void OnBnClickedBtnCancel();

   /// \brief Cancels all orders.
   afx_msg void OnBnClickedBtnCancelAll();

   /// \brief Changes and validates quantity.
   /// \param pNMHDR - Contains information about a notification message
   /// \param pResult - Signed result of message processing
   afx_msg void OnDeltaposSpQuantity(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Validates quantity.
   afx_msg void OnEnKillfocusEdQuantity();

   /// \brief Changes stop price.
   /// \param pNMHDR - Contains information about a notification message
   /// \param pResult - Signed result of message processing
   afx_msg void OnDeltaposSpStopPrice(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Changes limit price.
   /// \param pNMHDR - Contains information about a notification message
   /// \param pResult - Signed result of message processing
   afx_msg void OnDeltaposSpLimitPrice(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Places sell order.
   afx_msg void OnBnClickedBtnSell();

   /// \brief Places buy order.
   afx_msg void OnBnClickedBtnBuy();

   /// \brief Changes selected order.
   /// \param pNMHDR - Contains information about a notification message
   /// \param pResult - Signed result of message processing
   afx_msg void OnLvnItemChangedLsvOrders(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Changes order type.
   afx_msg void OnCbnSelChangeCmbType();

   /// \brief Changes order duration type.
   afx_msg void OnCbnSelChangeCmbDuration();

   /// \brief Opens dialog to modify pattern.
   afx_msg void OnBnClickedBtnPatternModify();

   /// \brief Opens dialog to create new pattern
   afx_msg void OnBnClickedBtnPatternAdd();

   /// \brief Process pattern selection from combobox.
   afx_msg void OnCbnSelchangeCmbPattern();

   /// \brief Process editing of strategy name in combobox.
   afx_msg void OnCbnEditchangeCmbSpread();

   /// \brief Opens transactions dialog
   afx_msg void OnBnClickedBtnTransactions();
   
   /// \brief Process cancellation instruction change.
   afx_msg void OnCbnSelchangeCmbCancelationType();

   /// \brief Process Iceberg strategy selection from check box.
   afx_msg void OnBnClickedCheck();

   /// \brief Changes and validates visible quantity.
   /// \param pNMHDR - Contains information about a notification message
   /// \param pResult - Signed result of message processing
   afx_msg void OnDeltaposSpnVisibleQuantity(
                  NMHDR *pNMHDR, 
                  LRESULT *pResult);
   
   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CStrategyOrderPlacingDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 10, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 2, OnGWConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 3, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 7, OnAccountChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 102, OnInstrumentResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 5, OnInstrumentChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 17, OnOrderChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 78, OnEventStrategyDefinitionProgress)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 16, OnQueryProgress)
   END_SINK_MAP()

protected:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - String describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                  LPDISPATCH obj,
                  BSTR errorDescription);

   /// \brief Fired when some changes occurred
   /// \brief in the connection with the CQG Gateway.
   /// \param newStatus - New connection status
   /// \return S_OK
   STDMETHOD(OnGWConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when some changes occurred
   /// \brief in the connection with the CQG data server.
   /// \param newStatus - New connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when new account is added, changed or removed.
   /// \param change - Account change type
   /// \param account - Changed account instance
   /// \param position - Changed position instance
   /// \return S_OK
   STDMETHOD(OnAccountChanged)(
                  eAccountChangeType change,
                  ICQGAccount* account,
                  ICQGPosition* position);

   /// \brief Fired when a new instrument is resolved and subscribed or the requested symbol is not available.
   /// \param symbol - Requested symbol
   /// \param cqg_instrument - Subscribed instrument object
   /// \param cqg_error - Error occured during a symbol resolution process
   /// \return S_OK on success
   STDMETHOD(OnInstrumentResolved)(BSTR symbol, ICQGInstrument* cqg_instrument, ICQGError* cqg_error);

   /// \brief Fired when any of instrument quotes or
   /// \brief dynamic instrument properties are changed.
   /// \param instrument - Changed instrument object
   /// \param quotes - Changed quotes collection
   /// \param props - Changed properties collection
   /// \return S_OK
   STDMETHOD(OnInstrumentChanged)(
                  ICQGInstrument* instrument,
                  ICQGQuotes* quotes,
                  ICQGInstrumentProperties* props);

   /// \brief Fired when new order was added, order status was 
   /// \brief changed or the order was removed.
   /// \param change - The type of the occurred change
   /// \param order - CQGOrder object representing the order to which the change refers
   /// \param oldProperties - CQGOrderProperties collection representing 
   /// \param the old values of the changed order properties
   /// \param fill - CQGFill object representing the last fill of the order
   /// \param cqgerr - CQGError object to describe the last error, if any
   /// \return S_OK
   STDMETHOD(OnOrderChanged)(
                  eChangeType change, 
                  ICQGOrder* order, 
                  ICQGOrderProperties* oldProperties, 
                  ICQGFill* fill, 
                  ICQGError* cqgerr);

   /// \brief Fired when the strategy definition status changed
   /// \param definition - Strategy definition wich status was changed
   /// \param cqgerr Object describing last error occurred during processing strategy definition request
   STDMETHOD (OnEventStrategyDefinitionProgress)(
                     ICQGStrategyDefinition* definition,
                     ICQGError* cqgerr);

   /// \brief Fired when orders query had some progress.
   /// \param query - Orders query object
   /// \param error - CQGError object to describe the last error which occurred during query, if any
   /// \return S_OK
   STDMETHOD(OnQueryProgress)(
                  ICQGOrdersQuery* query, 
                  ICQGError* error);

private:

   /// \enum eOrdersColumn. Represents orders columns in the list.
   enum eOrdersColumn;

   /// \enum eFilterType. Filter statuses.
   enum eFilterType;

   /// \brief Position. Used to hold position info.
   struct Position
   {
      Position() : Long(0), Short(0) 
      {}

      long Long;
      long Short;
   };

   /// \brief Type definition of mapping from instrument name to definition guid
   typedef std::map<ATL::CAdapt<ATL::CComBSTR>, ATL::CAdapt<ATL::CComBSTR> > NameToGuid;

   /// \brief Type definition of mapping from definition GWAlias to instrument name
   typedef std::map<ATL::CAdapt<ATL::CComBSTR>, ATL::CAdapt<ATL::CComBSTR> > AliasToName;

   /// \brief Type definition of mapping from execution pattern name to pattern
   typedef std::map<CString, CComPtr<ICQGExecutionPattern> > PatternsHolder;

   /// \brief Mapping from instrument name to position data
   typedef std::map<ATL::CAdapt<ATL::CComBSTR>, Position> PositionHolder;

   /// \brief Mapping from account Id to its positions
   typedef std::map<long, PositionHolder> PositionsHolder;

private:

   /// \brief Initializes CQGCEL object, and starts the CQGCEL.
   void InitializeCQGCEL();

   /// \brief Initializes all enable/disable booleans.
   void InitializeBooleans();

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes given orders list control.
   /// \param CListCtrl object to initialize.
   void InitializeOrdersList(
                  CListCtrl& lvOrders);

   /// \brief Initializes types combo control.
   void InitializeTypesCombo();

   /// \brief Initializes order cancelation instruction combo control.
   void InitializeCancelationInstructionCombo();

   /// \brief Initializes durations combo control.
   void InitializeDurationsCombo();

   /// \brief Initializes positions
   void InitializePositions();

   /// \brief Clears quotes data shown on the form.
   void ClearQuotes();

   /// \brief Clears position data shown on the form.
   void ClearPosition();

   /// \brief Clears instrument's description label control.
   void ClearDescription();

   /// \brief Clears definition's description control.
   void ClearAllDefinitionData();

   /// \brief Clears instrument's all data shown on the form.
   void ClearAllInstrumentData();

   /// \brief Clears all orders from the list control.
   void ClearOrders();

   /// \brief Shows all current quotes on the form for specified instrument.
   /// \param spInstrument - ICQGInstrument instance
   void ShowQuotes(
                  const ATL::CComPtr<ICQGInstrument>& spInstrument);

   /// \brief Shows specified quote in the labels.
   /// \param spQuotes - ICQGQuotes instance
   /// \param quoteType - Quote type to show
   /// \param priceLabel - Price label control
   /// \param volumeLabel - Volume label control
   void ShowQuote(
                  const ATL::CComPtr<ICQGQuotes>& spQuotes,
                  eQuoteType quoteType,
                  CStatic& priceLabel,
                  CStatic& volumeLabel);
   
   /// \brief Shows specified strategy  definition properties.
   /// \param spDefinition - ICQGStrategyDefinition instance
   void ShowStrategyProperties(
                  const ATL::CComPtr<ICQGStrategyDefinition>& spDefinition);

   /// \brief Returns price of specified instrument in Trade/Ask/Bid quotes order.
   /// \param spInstrument - ICQGInstrument instance
   double GetValidPrice(
                  const ATL::CComPtr<ICQGInstrument>& spInstrument);

   /// \brief Shows specified instrument's valid price in LimitPrice and/or StopPrice label control(s).
   /// \param spInstrument - ICQGInstrument instance
   void CStrategyOrderPlacingDlg::ShowValidPrice(
                  const ATL::CComPtr<ICQGInstrument>& spInstrument);

   /// \brief Enables/disables controls on the form.
   void EnableControls();

   /// \brief Enables/disables strategy definition related controls on the form.
   /// \return true if instrument selected and definition status is acceptable
   /// false - otherwise
   bool EnableDefinitionControls();

   /// \brief Returns display format of the price and volume from specified quote.
   /// \param spQuote [in] - ICQGQuote instance
   /// \param displayPrice [out] - Formatted price string
   /// \param displayVolume [out] - Formatted volume string
   static void GetQuoteDisplayValues(
                  const ATL::CComPtr<ICQGQuote>& spQuote,
                  CString& displayPrice,
                  CString& displayVolume);

   /// \brief Fills all accounts to combo box control and changes 
   /// \brief their subscription levels.
   void ReloadAccounts();

   /// \brief Adds account's description to the accounts combo box control.
   /// \param spAccount - ICQGAccount instance
   void InsertAccount(
                  const ATL::CComPtr<ICQGAccount>& spAccount);

   /// \brief Returns instrument from CQGCEL currently selected in the list.
   /// \return ICQGInstrument instance
   ATL::CComPtr<ICQGInstrument> GetSelectedInstrument() const;

   /// \brief Returns selected account in the combo.
   /// \return ICQGAccount instance
   ATL::CComPtr<ICQGAccount> GetSelectedAccount() const;

   /// \brief Returns selected order in the list.
   /// \return ICQGOrder instance
   ATL::CComPtr<ICQGOrder> GetSelectedOrder() const;

   /// \brief Returns instrument by specified GWAlias.
   /// \param alias - GWAlias of the instrument 
   /// \return ICQGInstrument instance
   ATL::CComPtr<ICQGInstrument> GetInstrumentByAlias(const ATL::CComBSTR& alias) const;

   /// \brief Shows orders by selected status in the list control.
   void ShowFilteredOrders();

   /// \brief Unselects all orders.
   void UnselectAllOrders();

   /// \brief Shows specified orders in the list control.
   /// \param spOrders - ICQGOrders instance
   void ShowStrategyOrders(
                  const ATL::CComPtr<ICQGOrders>& spOrders);

   /// \brief Shows leg orders of the selected order
   /// \param spOrder - ICQGOrder instance
   void ShowLegOrders(const ATL::CComPtr<ICQGOrder>& spOrder);

   /// \brief Updates strategy definition info fields
   void CStrategyOrderPlacingDlg::UpdateDefinitionInfo();

   /// \brief Updates strategy definitions holders
   /// \param spDefinition - Strategy definition instance
   void UpdateDefinitionHolder(
                  const ATL::CComPtr<ICQGStrategyDefinition>& spDefinition);

   /// \brief Returns specified orders row in the list control.
   /// \param spOrder - ICQGOrder instance
   /// \param lvOrders - CListCtrl view to show orders
   /// \return Orders row as integer
   int GetOrderRow(
                  const ATL::CComPtr<ICQGOrder>& spOrder,
                  CListCtrl& lvOrders);

   /// \brief Adds, updates or removes specified order in the list control.
   /// \param spOrder - ICQGOrder instance
   /// \param change - Change type
   /// \param lvOrders - CListCtrl view to show orders
   void ShowOrder(
                  const ATL::CComPtr<ICQGOrder>& spOrder,
                  eChangeType change, 
                  CListCtrl& lvOrders);

   /// \brief Adds, updates or removes specified order in the strategies list control.
   /// \param spOrder - ICQGOrder instance
   /// \param change - Change type
   void ShowStrategyOrder(
                  const ATL::CComPtr<ICQGOrder>& spOrder,
                  eChangeType change);

   /// \brief Adds, updates or removes specified order in the legs list control.
   /// \param spOrder - ICQGOrder instance
   /// \param change - Change type
   void ShowLegOrder(
                  const ATL::CComPtr<ICQGOrder>& spOrder,
                  eChangeType change);

   /// \brief Places order.
   /// \param orderSide - Placed order side.
   void PlaceOrder(
                  eOrderSide orderSide);

   /// \brief Validates and returns display format of specified price.
   /// \param spOrder - Order which price is formated 
   /// \param price - Price which is formated
   /// \return String representation of price
   CString GetDisplayPrice(
                  const ATL::CComPtr<ICQGOrder> spOrder,
                  double price) const;

   /// \brief Used for getting CQGCEL properties values string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - variant value
   /// \param valueType - variant type
   /// \return String representation of the value
   CString GetValueAsString(
                  ATL::CComVariant value,
                  VARENUM valueType = VT_EMPTY) const;

   /// \brief Returns order status, by which orders will be filtered.
   /// \param filterType - filter type
   /// \return Order status as integer
   int FilterTypeToOrderStatus(
                  eFilterType filterType);

   /// \brief Returns string representation of the specified order status.
   /// \param status - order status
   /// \return String representation of the order status
   CString OrderStatusToString(
                  eOrderStatus status) const;

   /// \brief Returns current selected strategy definition.
   /// \return strategy definition or NULL if no selected definition.
   ATL::CComPtr<ICQGStrategyDefinition> GetCurrentStrategyDefinition();

   /// \brief Recognize is order strategy or no
   /// \param spOrder order to recognize
   /// \return true if given order is strategy order, false - otherwise
   bool IsStrategyOrder(
                  const ATL::CComPtr<ICQGOrder>& spOrder) const;

   /// \brief Queries for orders to calculate initial positions
   /// \param spAccount account to query
   void QueryForPositions(
                  const ATL::CComPtr<ICQGAccount>& spAccount);

   /// \brief Updates positions data.
   /// \param spOrders - ICQGOrders instance
   void UpdatePositions(
                  const ATL::CComPtr<ICQGOrdersQuery>& spOrders);

   /// \brief Updates position data.
   /// \param spOrder - ICQGOrder instance
   void UpdatePosition(
                  const ATL::CComPtr<ICQGOrder>& spOrder);

   /// \brief Shows position for selected account and instrument
   void ShowPosition();

   /// \brief Returns string representation of the specified order type.
   /// \param type - order type
   /// \return String representation of the order type
   static CString OrderTypeToString(
                  eOrderType type );

   /// \brief Returns string representation of the specified order duration.
   /// \param duration - order duration
   /// \return String representation of the order duration
   static CString OrderDurationToString(
                  eOrderDuration duration);

   /// \brief Returns string representation of the specified order side.
   /// \param side - order side
   /// \return String representation of the order side
   static CString OrderSideToString(
                  eOrderSide side);

   /// \brief Returns string representation of the connection status.    
   /// \param status - connection status
   /// \return String representation of the connection status
   static CString ConnectionStatusToString(
                  eConnectionStatus status);

   /// \brief Convert strategy definition status to string.
   /// \param status - eStrategyRequestStatus status
   /// \return String representation of the status
   static CString StatusToString(
                  const eStrategyRequestStatus status);

public:

   /// \brief Changes specified property's value.
   /// \param spProperties - ICQGOrderProperties instance
   /// \param propertyType - Property type, which value must be changed
   /// \param propertyValue - Property value
   static void SetPropertyValue(
                  ATL::CComPtr<ICQGOrderProperties>& spProperties,
                  eOrderProperty propertyType,
                  ATL::CComVariant propertyValue);

   /// \brief Changes specified edit box's price.
   /// \param editCtrl - Edit box control, which price will be changed
   /// \param spInstrument - ICQGInstrument instance
   /// \param delta - Change type
   static void ChangePrice(
                  CEdit& editCtrl,
                  ATL::CComPtr<ICQGInstrument> spInstrument,
                  int delta);

   /// \brief Returns specified edit box's price.
   /// \param editCtrl - Edit box control, which price will be returned
   /// \param spInstrument - ICQGInstrument instance
   /// \return Price
   static double GetPrice(
                  CEdit& editCtrl,
                  ATL::CComPtr<ICQGInstrument> spInstrument);

   /// \brief Returns quantity from the specified edit box.
   /// \param editCtrl - Edit box control, which quantity will be returned.
   /// \return Quantity
   static long GetQuantity(
                  CEdit& editCtrl);

   /// \brief Returns valid value for specified quantity.
   /// \param quantity - Quantity.
   /// \return Valid quantity
   static long GetValidQuantity(
                  long quantity);

   /// \brief Sets quantity to edit box.
   /// \param editCtrl - Edit box control, which quantity will be changed.
   /// \param quantity - Quantity.
   static void SetQuantity(
                  CEdit& editCtrl,
                  long quantity);

   /// \brief Shows selected order cancelation instruction
   /// \param spOrder - ICQGOrder instance
   void ShowCancelationInstruction(const ATL::CComPtr<ICQGOrder>& spOrder);

public:

   /// \brief Not available value.
   static const CString m_N_A;
private:

   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Quantity maximum value.
   static const long m_MAX_QUANTITY;

   /// \brief Quantity minimum value.
   static const long m_MIN_QUANTITY;

   /// \brief Invalid double value.
   static const double m_INVALID_DOUBLE;

   /// \brief Invalid integer value.
   static const int m_INVALID_INTEGER;
   
   /// \brief Short date format.
   static const CString m_SHORT_DATE;
   
   /// \brief Short time format.
   static const CString m_SHORT_TIME;

   /// \brief Used to enable/disable controls,
   /// \brief True, when data connection is up.
   bool m_DataConnectionUp;

   /// \brief Used to enable/disable controls,
   /// \brief True, when GW connection is up.
   bool m_GWConnectionUp;

   /// \brief Used to enable/disable controls,
   /// \brief True, when limit price is enabled.
   bool m_LimitPriceEnabled;

   /// \brief Used to enable/disable controls,
   /// \brief True, when stop price is enabled.
   bool m_StopPriceEnabled;

   /// \brief Used to enable/disable controls,
   /// \brief True, when good till date is enabled.
   bool m_GTDEnabled;

   /// \brief Associated container of strategy expression to id;
   NameToGuid m_expressionToGuid;

   /// \brief Associated container of strategy definition alias to instrument name;
   AliasToName m_aliasToName;

   /// \brief Associated container of name to execution patterns
   PatternsHolder m_patterns;

   /// \brief Holder of current execution pattern
   ATL::CComPtr<ICQGExecutionPattern> m_spCurrentPattern;

   /// \brief Holder of current strategy definition 
   ATL::CComPtr<ICQGStrategyDefinition> m_spDefinition;

   /// \brief Collection of positions for strategy symbols
   PositionsHolder m_positions;

   /// \brief Set of account IDs for which orders query was failed
   std::set<long> m_failedAccounts;

private:

   /// \brief Data connection indicator label control
   CStatic m_lblDataConnection;

   /// \brief GW connection indicator label control
   CStatic m_lblGWConnection;

   /// \brief Accounts combo box control
   /// \brief Contains account's GW names, as well as the GWID as item data
   /// \brief GWID is used to get account instance from CQGCEL accounts collection.
   CComboBox m_cmbAccounts;

   /// \brief Ask price label control
   CStatic m_lblAskPrice;

   /// \brief Bid price label control
   CStatic m_lblBidPrice;

   /// \brief Ask volume label control
   CStatic m_lblAskVolume;

   /// \brief Bid volume label control
   CStatic m_lblBidVolume;

   /// \brief Spread to subscribe edit box control
   /// \note Contains all user defined spreads
   CComboBox m_cmbSpreadExpr;

   /// \brief Instrument subscription button control
   CButton m_btnSubscribe;

   /// \brief QFormulas request button control
   CButton m_btnQFormula;

   /// \brief Orders list view control
   CListCtrl m_lvOrders;

   /// \brief Orders types combo box control
   CComboBox m_cmbType;

   /// \brief Orders durations combo box control
   CComboBox m_cmbDuration;

   /// \brief Limit price spin button control
   CSpinButtonCtrl m_spnLimitPrice;

   /// \brief Stop price spin button control
   CSpinButtonCtrl m_spnStopPrice;

   /// \brief Quantity spin button control
   CSpinButtonCtrl m_spnQuantity;

   /// \brief Modify order button control
   CButton m_btnModify;

   /// \brief Cancel order button control
   CButton m_btnCancel;

   /// \brief Cancel all orders button control
   CButton m_btnCancelAll;

   /// \brief Define strategy button control
   CButton m_btnDefine;

   /// \brief Good till date date-time control
   CDateTimeCtrl m_dtpGTD;

   /// \brief Order quantity edit box control
   CEdit m_edQuantity;

   /// \brief Order limit price edit box control
   CEdit m_edLimitPrice;

   /// \brief Order stop price edit box control
   CEdit m_edStopPrice;

   /// \brief Buy order button control
   CButton m_btnBuy;
   
   /// \brief Defined strategy status control
   CEdit m_edStrategyStatus;

   /// \brief Sell order button control
   CButton m_btnSell;

   /// \brief Defined strategy client id view control
   CEdit m_edClientId;

   /// \brief Defined strategy GW alias view control
   CEdit m_edGwAlias;

   /// \brief Opens dialog to modify selected pattern button control
   CButton m_btnPatternModify;

   /// \brief Opens dialog to create new execution pattern button control
   CButton m_btnPatternAdd;

   /// \brief Combobox control for selecting execution pattern
   CComboBox m_cmbPattern;

   /// \brief Leg orders list view control
   CListCtrl m_lvLegOrders;

   /// \brief Long position label control
   CStatic m_lblLongPosition;

   /// \brief Short position label control
   CStatic m_lblShortPosition;

   /// \brief Transaction show button control
   CButton m_btnTransection;
   
   /// \brief Pattern string edit box control
   CEdit m_edPatternString;

   /// \brief Cancellation instruction type
   CComboBox m_cmbCancelationInstruction;

   /// \brief Visible Quantity spin button control
   CSpinButtonCtrl m_spnVisibleQuantity;

   /// \brief Order visible quantity edit box control
   CEdit m_edVisibleQuantity;

   /// \brief Iceberg strategy order CheckBox control
   CButton m_chkIcebergStrategy;
};

enum CStrategyOrderPlacingDlg::eOrdersColumn
{
   ocID,
   ocSide,
   ocSize,
   ocState,
   ocInstrument,
   ocType,
   ocLimitPrice,
   ocStopPrice,
   ocDuration,
   ocAvgFillPrice,
   ocPlaceTime,
   ocLastFillTime,
   ocCanceledTime,
   ocGWOrderID
};
