// TransactionDlg.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include "Common.h"
#include "StrategyOrderPlacing.h"
#include "TransactionDlg.h"
#include ".\transactiondlg.h"


const CString CTransactionDlg::m_N_A = _T("N/A");

// CTransactionDlg dialog

IMPLEMENT_DYNAMIC(CTransactionDlg, CDialog)
CTransactionDlg::CTransactionDlg(const ATL::CComPtr<ICQGCEL>& spCEL, 
                                 ATL::CComPtr<ICQGOrder> spOrder, 
                                 CWnd* pParent /*=NULL*/)
	: CDialog(CTransactionDlg::IDD, pParent)
   , m_spCQGCEL(spCEL)
   , m_spOrder(spOrder)
{
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

CTransactionDlg::~CTransactionDlg()
{
}

void CTransactionDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_LSV_TRANSACTIONS, m_lvOrderTransactions);
   DDX_Control(pDX, IDC_LSV_HANGS, m_lvHangOrderIds);
}

STDMETHODIMP CTransactionDlg::OnOrderChanged(
                                             eChangeType /*change*/, 
                                             ICQGOrder* order, 
                                             ICQGOrderProperties* /*oldProperties*/, 
                                             ICQGFill* /*fill*/, 
                                             ICQGError* /*cqgerr*/)
{
   try
   {
      ATL::CComBSTR orderGuid;
      HRESULT hr = order->get_GUID(&orderGuid);
      AssertCOMError<ICQGOrder>(order, hr);

      ATL::CComBSTR currentOrderGuid;
      GET_PARAM(m_spOrder, currentOrderGuid, get_GUID);

      if (orderGuid == currentOrderGuid)
      {
         // clear transactions list
         m_lvOrderTransactions.DeleteAllItems();
         ShowTransactions(order);

         // clear hanged ids
         ClearHangOrderIds();
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

BOOL CTransactionDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   // Set the icon for this dialog.  The framework does this automatically
   //  when the application's main window is not a dialog
   SetIcon(m_hIcon, TRUE);       // Set big icon
   SetIcon(m_hIcon, FALSE);      // Set small icon

   try
   {
      // Now advise the connection, to get events
      assert(m_spCQGCEL != NULL);
      HRESULT hr = ICQGCELDispEventImplTransaction::DispEventAdvise(m_spCQGCEL);
      AssertCOMError(m_spCQGCEL, hr);

      InitializeControls();
      ShowTransactions(m_spOrder);
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
}


BEGIN_MESSAGE_MAP(CTransactionDlg, CDialog)
   ON_NOTIFY(LVN_ITEMCHANGED, IDC_LSV_TRANSACTIONS, OnLvnItemchangedLsvTransactions)
   ON_WM_DESTROY()
END_MESSAGE_MAP()


void CTransactionDlg::ShowTransactions(
   const ATL::CComPtr<ICQGOrder>& spOrder)
{
   if(spOrder == NULL)
   {
      return;
   }
   // Get order transactions
   ATL::CComPtr<ICQGTransactions> spTransactions;
   HRESULT hr = spOrder->get_Transactions(&spTransactions);
   AssertCOMError(spOrder, hr);

   long count;
   hr = spTransactions->get_Count(&count);
   AssertCOMError(spTransactions, hr);

   // Add order transactions to the list
   for (int i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGTransaction> spTransaction;
      hr = spTransactions->get_ItemByIndex(i, &spTransaction);
      AssertCOMError(spTransactions, hr);

      ShowTransaction(spTransaction);
   }
}

void CTransactionDlg::ShowTransaction(
   const ATL::CComPtr<ICQGTransaction>& spTransaction)
{
   DATE timeStamp;
   HRESULT hr = spTransaction->get_ServerTimestamp(&timeStamp);
   AssertCOMError(spTransaction, hr);

   ATL::CComBSTR transID;
   hr = spTransaction->get_Id(&transID);
   AssertCOMError(spTransaction, hr);

   eOrderEvent status;
   hr = spTransaction->get_Status(&status);
   AssertCOMError(spTransaction, hr);

   ATL::CComBSTR rejReason;
   hr = spTransaction->get_RejectReason(&rejReason);
   AssertCOMError(spTransaction, hr);

   // Get corresponding fill object
   ATL::CComPtr<ICQGFill> spFill;
   hr = spTransaction->get_Fill(&spFill);
   AssertCOMError(spTransaction, hr);

   // Check whether transaction is fill transaction
   CString strFillID = m_N_A;
   if (spFill)
   {
      ATL::CComBSTR fillID;
      hr = spFill->get_Id(&fillID);
      AssertCOMError(spFill, hr);

      strFillID = ATL::CW2T(fillID);
   }

   // Show transaction data
   int row = m_lvOrderTransactions.InsertItem(m_lvOrderTransactions.GetItemCount(), _T(""));
   m_lvOrderTransactions.SetItemText(row, otcTimestamp, GetValueAsString(timeStamp, VT_DATE));
   m_lvOrderTransactions.SetItemText(row, otcTransactionID, GetValueAsString(transID));
   m_lvOrderTransactions.SetItemText(row, otcStatus, OrderEventToString(status));
   m_lvOrderTransactions.SetItemText(row, otcFillID, strFillID);
   m_lvOrderTransactions.SetItemText(row, otcRejectReason, ATL::CW2T(rejReason));

   m_lvOrderTransactions.InsertItem(m_lvOrderTransactions.GetItemCount(), _T(""));
}

void CTransactionDlg::ShowHangOrderIds(
   const ATL::CComPtr<ICQGTransaction>& spTransaction)
{
   if (!spTransaction)
   {
      return;
   }

   // Get hang orders ids
   ATL::CComPtr<ICQGStringCollection> spHangOrderIds;
   GET_PARAM(spTransaction, spHangOrderIds, get_HangOrderIds);

   // Get count of hang orders ids
   long count;
   GET_PARAM(spHangOrderIds, count, get_Count);

   for (int index = 0; index < count; ++index)
   {
      ATL::CComBSTR hangOrderId;
      HRESULT hr = spHangOrderIds->get_Item(index, &hangOrderId);
      AssertCOMError(spHangOrderIds, hr);

      // Show hang order id
      const int row = m_lvHangOrderIds.InsertItem(m_lvHangOrderIds.GetItemCount(), _T(""));
      m_lvHangOrderIds.SetItemText(row, oidcHangOrderId, ATL::CW2T(hangOrderId));
   }
}

void CTransactionDlg::ClearHangOrderIds()
{
   m_lvHangOrderIds.DeleteAllItems();
}

void CTransactionDlg::InitializeControls()
{
   InitializeTransactionsList();
   InitializeHangOrderIdsList();
}

CString CTransactionDlg::GetValueAsString(
                         ATL::CComVariant value,
                         VARENUM valueType/*= VT_EMPTY*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   AssertCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return VariantToString(value);
   }
   return m_N_A;
}

void CTransactionDlg::InitializeTransactionsList()
{
   m_lvOrderTransactions.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvOrderTransactions.InsertColumn(otcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 110);
   m_lvOrderTransactions.InsertColumn(otcTransactionID, _T("Trans. ID"), LVCFMT_LEFT, 60);
   m_lvOrderTransactions.InsertColumn(otcStatus, _T("Status"), LVCFMT_LEFT, 55);
   m_lvOrderTransactions.InsertColumn(otcFillID, _T("Fill ID"), LVCFMT_LEFT, 60);
   m_lvOrderTransactions.InsertColumn(otcRejectReason, _T("Rej. Reason"), LVCFMT_LEFT, 80);

   m_lvOrderTransactions.InsertColumn(otcPropName, _T("Prop. Name"), LVCFMT_LEFT, 70);
   m_lvOrderTransactions.InsertColumn(otcPropPreviousValue, _T("Old Value"), LVCFMT_LEFT, 70);
   m_lvOrderTransactions.InsertColumn(otcPropCurrentValue, _T("New Value"), LVCFMT_LEFT, 70);
}

void CTransactionDlg::InitializeHangOrderIdsList()
{
   m_lvHangOrderIds.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvHangOrderIds.InsertColumn(oidcHangOrderId, _T("HangOrderIds"), LVCFMT_LEFT, 120);
}

CString CTransactionDlg::OrderEventToString(
   eOrderEvent event)
{
   CString eventString;

   switch (event)
   {
   case oeAckCan:
      eventString = _T("AckCan");
      break;
   case oeAckMod:
      eventString = _T("AckMod");
      break;
   case oeAckPlace:
      eventString = _T("AckPlace");
      break;
   case oeActiveAt:
      eventString = _T("ActiveAt");
      break;
   case oeCancelQueueTimeout:
      eventString = _T("CancelQueueTimeout");
      break;
   case oeCancelSent:
      eventString = _T("CancelSent");
      break;
   case oeDisconnected:
      eventString = _T("Disconnected");
      break;
   case oeExpired:
      eventString = _T("Expired");
      break;
   case oeFill:
      eventString = _T("Fill");
      break;
   case oeFillBust:
      eventString = _T("FillBust");
      break;
   case oeFillCan:
      eventString = _T("FillCan");
      break;
   case oeFillMod:
      eventString = _T("FillMod");
      break;
   case oeInCan:
      eventString = _T("InCan");
      break;
   case oeInCanTmout:
      eventString = _T("InCanTmout");
      break;
   case oeInClient:
      eventString = _T("InClient");
      break;
   case oeInMod:
      eventString = _T("InMod");
      break;
   case oeInModTmout:
      eventString = _T("InModTmout");
      break;
   case oeInQueue:
      eventString = _T("InQueue");
      break;
   case oeInTransit:
      eventString = _T("InTransit");
      break;
   case oeInTransTmout:
      eventString = _T("InTransTmout");
      break;
   case oeLinkChg:
      eventString = _T("LinkChg");
      break;
   case oeModifyQueueTimeout:
      eventString = _T("ModifyQueueTimeout");
      break;
   case oeModifySent:
      eventString = _T("ModifySent");
      break;
   case oePark:
      eventString = _T("Park");
      break;
   case oeQueueTimeout:
      eventString = _T("QueueTimeout");
      break;
   case oeRejCan:
      eventString = _T("RejCan");
      break;
   case oeRejectFCM:
      eventString = _T("RejectFCM");
      break;
   case oeRejectGW:
      eventString = _T("RejectGW");
      break;
   case oeRejMod:
      eventString = _T("RejMod");
      break;
   case oeRemoved:
      eventString = _T("Removed");
      break;
   case oeSyntheticActivated:
      eventString = _T("SyntheticActivated");
      break;
   case oeUndefined:
      eventString = _T("Undefined");
      break;
   case oeSyntheticFailed:
      eventString = _T("SyntheticFailed");
      break;
   case oeSyntheticOverFill:
      eventString = _T("SyntheticOverFill");
      break;
   case oeSyntheticHang:
      eventString = _T("SyntheticHang");
      break;
   case oeInfoChanged:
      eventString = _T("InfoChanged");
      break;
   default:
      assert(!"Unknown order event");
   }

   return eventString;
}
// CTransactionDlg message handlers

void CTransactionDlg::OnLvnItemchangedLsvTransactions(NMHDR *pNMHDR, LRESULT *pResult)
{
   try
   {
      // Clear hanged order's ids list
      ClearHangOrderIds();

      LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);

      if ((pNMLV->uNewState & LVIS_SELECTED) != LVIS_SELECTED)
      {
         // If there is no new item selected then do nothing
         return;
      }

      // Get selected order's transactions
      ATL::CComPtr<ICQGTransactions> spTransactions;
      HRESULT hr = m_spOrder->get_Transactions(&spTransactions);
      AssertCOMError(m_spOrder, hr);

      // Get selected transaction
      ATL::CComBSTR transactionId = ATL::CT2W(m_lvOrderTransactions.GetItemText(pNMLV->iItem, otcTransactionID));
      ATL::CComPtr<ICQGTransaction> spTransaction;
      hr = spTransactions->get_Item(transactionId, &spTransaction);
      AssertCOMError(spTransaction, hr);

      // Show selected transactions strategy hang orders ids
      ShowHangOrderIds(spTransaction);

      *pResult = 0;
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTransactionDlg::OnDestroy()
{
   try
   {
      if (m_spCQGCEL)
      {
         HRESULT hr = ICQGCELDispEventImplTransaction::DispEventUnadvise(m_spCQGCEL);
         VERIFY(SUCCEEDED(hr));
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}
