﻿using System;
using System.Windows.Forms;
using CQG;

namespace StrategyOrderPlacing
{
    public partial class frmQFormula : Form
    {
        /// <summary>
        /// CQGCEL object.
        /// </summary>
        private CQGCEL m_CEL;

        /// <summary>
        /// CQGQFormulaDefinitions object
        /// </summary>
        private CQGQFormulaDefinitions m_QFormulaDefintions;

        /// <summary>
        /// Selected QFormula definition name
        /// </summary>
        private string m_QFormulaDefName;

        /// <summary>
        /// Initializes QFormula controls.
        /// </summary>
        /// <param name="cel">CQGCEL object.</param>
        public frmQFormula(CQGCEL cel)
        {
            // Required for Windows Form Designer support
            InitializeComponent();

            m_CEL = cel;

            // Handle following event
            m_CEL.QFormulaDefinitionsResolved +=
                new _ICQGCELEvents_QFormulaDefinitionsResolvedEventHandler(cel_QFormulaDefinitionsResolved);
        }

        /// <summary>
        /// Gets selected QFormula definition name
        /// </summary>
        public string QFormulaDefinitionName
        {
            get { return m_QFormulaDefName; }
        }

        /// <summary>
        /// Gets selected QFormula definition execution pattern
        /// </summary>
        public string QFormulaDefinitionExecutionPattern
        {
            get { return txtExecutionPattern.Text; }
        }

        #region Form controls event handlers

        /// <summary>
        /// Requests QFormula definitions when the button is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data. </param>
        private void btnRequestDefinitions_Click(object sender, EventArgs e)
        {
            try
            {
                m_CEL.RequestQFormulaDefinitions();
                txtStatus.Text = "InProgress";
                btnRequestDefinitions.Enabled = false;
            }
            catch (System.Exception ex)
            {
                ErrorHandler.HandleError("frmQFormula", "btnRequestDefinitions_Click", ex);
            }
        }

        /// <summary>
        /// Used to display newly selected QFormula definition data.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data. </param>
        private void cmbQFormulaDef_SelectedIndexChanged(object sender, EventArgs e)
        {
            CQGQFormulaDefinition QFormulaDef = m_QFormulaDefintions.get_ItemByName(cmbQFormulaDef.SelectedItem.ToString());
            if (QFormulaDef != null)
            {
                txtNumber.Text = QFormulaDef.Number.ToString();
                rtxtExpression.Text = QFormulaDef.Expression;
                txtExecutionPattern.Text = QFormulaDef.ExecutionPattern;
                m_QFormulaDefName = "Q." + QFormulaDef.Name;
            }
        }

        #endregion

        #region CQGCEL event handlers

        /// <summary>
        /// This event is fired when the QFormualDefinitions are resolved.
        /// </summary>
        /// <param name="cqg_qformula_definitions">QForluma defintions</param>
        /// <param name="cqg_error">Error ocurred during the request</param>
        private void cel_QFormulaDefinitionsResolved(CQGQFormulaDefinitions cqg_qformula_definitions,
                                                     CQGError cqg_error)
        {
            if (m_CEL.IsValid(cqg_error))
            {
                txtStatus.Text = "Failed";
                btnRequestDefinitions.Enabled = true;

                ErrorHandler.ShowErrorMessage(cqg_error.Description, "QFormulaDefinitions resolved", this);
                return;
            }

            txtStatus.Text = "Success";
            m_QFormulaDefintions = cqg_qformula_definitions;
            FillQFormulaDefinitions();
        }

        #endregion

        #region Auxiliary methods

        /// <summary>
        /// Fills all QFormula definitions to the combo box.
        /// </summary
        private void FillQFormulaDefinitions()
        {
            cmbQFormulaDef.Items.Clear();
            if (m_QFormulaDefintions.Count > 0)
            {
                foreach (CQGQFormulaDefinition qformulaDefin in m_QFormulaDefintions)
                {
                    cmbQFormulaDef.Items.Add(qformulaDefin.Name);
                }

                // Select first definition
                cmbQFormulaDef.SelectedIndex = 0;
            }
        }
        #endregion
    }
}
