﻿using System;
using System.Windows.Forms;
using CQG;

namespace StrategyOrderPlacing
{
    public partial class frmTransactions : Form
    {
        /// <summary>
        /// CQGOrder object.
        /// </summary>
        private CQGOrder m_Order;

        /// <summary>
        /// Form constructor.
        /// </summary>
        /// <param name="order">CQGOrder object.</param>
        public frmTransactions(CQGOrder order)
        {
            InitializeComponent();
            InitializeTransactionsList();
            InitializeHangOrderIdsList();
            m_Order = order;

            ShowTransactions();
        }

        #region Auxiliary methods

        // <summary>
        /// Initializes transaction list view control.
        /// </summary>
        private void InitializeTransactionsList()
        {
            // Add columns to list (this columns represent the sub-items of the list item)
            Type enumType = typeof(eOrderTransactionColumn);
            Array enumValues = Enum.GetValues(enumType);
            foreach (eOrderTransactionColumn val in enumValues)
            {
                lsvTransactions.Columns.Add(val.ToString(), 100, HorizontalAlignment.Center);
            }
        }

        /// <summary>
        /// Initializes transaction hang order ids list view control.
        /// </summary>
        private void InitializeHangOrderIdsList()
        {
            // First column contains the hang order IDs of the transaction
            lsvHangOrderIDs.Columns.Add("HangOrderID", 100, HorizontalAlignment.Center);
        }

        /// <summary>
        /// Shows order's all transactions.
        /// </summary>
        private void ShowTransactions()
        {
            lsvTransactions.Items.Clear();
            int row;
            ListViewItem item;
            foreach (CQGTransaction transaction in m_Order.Transactions)
            {
                row = lsvTransactions.Items.Count;
                item = lsvTransactions.Items.Insert(row, transaction.ServerTimestamp.ToString("G"));

                // Add order as a tag
                item.Tag = transaction;

                // Add sub items for each column
                for (int i = 0; i < Enum.GetNames(typeof(eOrderTransactionColumn)).Length; i++)
                {
                    item.SubItems.Add(string.Empty);
                }

                item.SubItems[(int)eOrderTransactionColumn.TransactionID].Text = transaction.Id;
                item.SubItems[(int)eOrderTransactionColumn.Status].Text = transaction.Status.ToString().Substring(2);
                item.SubItems[(int)eOrderTransactionColumn.FillID].Text = transaction.Fill != null ? transaction.Fill.Id : frmStrategyOrderPlacing.N_A;
                item.SubItems[(int)eOrderTransactionColumn.RejectReason].Text = transaction.RejectReason;
            }
        }

        #endregion

        #region Form controls event handlers

        /// <summary>
        /// Shows order transactions when form is loaded.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data. </param>
        private void frmTransactions_Load(object sender, EventArgs e)
        {
            ShowTransactions();
        }

        /// <summary>
        /// Used to display newly selected transactions hang order ids.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data. </param>
        private void lsvTransactions_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Retrieve currently selected transaction
            if (lsvTransactions.SelectedItems.Count == 1)
            {
                CQGTransaction transaction = lsvTransactions.SelectedItems[0].Tag as CQGTransaction;
                if (transaction != null)
                {
                    CQGStringCollection hangOrderIDs = transaction.HangOrderIds;
                    foreach (string hangOrderID in hangOrderIDs)
                    {
                        lsvHangOrderIDs.Items.Insert(lsvHangOrderIDs.Items.Count, hangOrderID);
                    }
                }
            }
        }

        #endregion

        #region Nested types

        /// <summary>
        /// Specifies columns in transactions list view control.
        /// </summary>
        private enum eOrderTransactionColumn
        {
            Timestamp = 0,
            TransactionID,
            Status,
            FillID,
            RejectReason
        }

        #endregion
    }
}
