// InstrumentTypesDlg.cpp : implementation file
//

#include "stdafx.h"
#include "SymbolResolution.h"
#include "InstrumentTypesDlg.h"

// CInstrumentTypesDlg dialog
CInstrumentTypesDlg::CInstrumentTypesDlg(CWnd* pParent /*=NULL*/)
   : CDialog(CInstrumentTypesDlg::IDD, pParent)
   , m_CallOptionsChecked(FALSE)
   , m_PutOptionsChecked(FALSE)
   , m_FuturesChecked(FALSE)
   , m_StocksChecked(FALSE)
   , m_TreasuriesChecked(FALSE)
   , m_OtherChecked(FALSE)
   , m_OnlyTradableChecked(FALSE)
{
}

void CInstrumentTypesDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Check(pDX, IDC_CHKCALLOPTIONS, m_CallOptionsChecked);
   DDX_Check(pDX, IDC_CHKPUTOPTIONS, m_PutOptionsChecked);
   DDX_Check(pDX, IDC_CHKFUTURES, m_FuturesChecked);
   DDX_Check(pDX, IDC_CHKSTOCKS, m_StocksChecked);
   DDX_Check(pDX, IDC_CHKTREASURIES, m_TreasuriesChecked);
   DDX_Check(pDX, IDC_CHKOTHER, m_OtherChecked);
   DDX_Check(pDX, IDC_CHKONLYTRADABLE, m_OnlyTradableChecked);
   DDX_Control(pDX, IDOK, m_btnOk);
}

BOOL CInstrumentTypesDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   m_btnOk.EnableWindow(FALSE);

   return TRUE;  // return TRUE  unless you set the focus to a control
}

BEGIN_MESSAGE_MAP(CInstrumentTypesDlg, CDialog)
   ON_BN_CLICKED(IDC_BTNSELECTALL, OnBnClickedBtnSelectAll)
   ON_BN_CLICKED(IDC_BTNSELECTOPTIONS, OnBnClickedBtnSelectOptions)
   ON_BN_CLICKED(IDC_CHKCALLOPTIONS, OnCheckBoxClick)
   ON_BN_CLICKED(IDC_CHKPUTOPTIONS, OnCheckBoxClick)
   ON_BN_CLICKED(IDC_CHKFUTURES, OnCheckBoxClick)
   ON_BN_CLICKED(IDC_CHKSTOCKS, OnCheckBoxClick)
   ON_BN_CLICKED(IDC_CHKTREASURIES, OnCheckBoxClick)
   ON_BN_CLICKED(IDC_CHKOTHER, OnCheckBoxClick)
END_MESSAGE_MAP()

void CInstrumentTypesDlg::GetTypes(long& instrumentTypes, VARIANT_BOOL& tradeableOnly) const
{
   // NOTE: instrument type is a bit mask, here we collect all types
   // which user has selected to request for commodity.
   // User can select tradable only check box to get only instruments which
   // are tradable. This requires connection with Gateway server.
   tradeableOnly = m_OnlyTradableChecked ? VARIANT_TRUE : VARIANT_FALSE;

   instrumentTypes = itUndefined;

   if (m_CallOptionsChecked && m_PutOptionsChecked &&
         m_FuturesChecked && m_StocksChecked && 
         m_TreasuriesChecked && m_OtherChecked)
   {
      instrumentTypes = itAllInstruments;
      return;
   }

   if (m_CallOptionsChecked) 
      instrumentTypes |= itOptionCall;

   if (m_PutOptionsChecked) 
      instrumentTypes |= itOptionPut;

   if (m_FuturesChecked)
      instrumentTypes |=  itFuture;

   if (m_StocksChecked) 
      instrumentTypes |=  itStock;

   if (m_TreasuriesChecked) 
      instrumentTypes |=  itTreasure;

   if (m_OtherChecked) 
      instrumentTypes |=  itOther;   
}

void CInstrumentTypesDlg::OnBnClickedBtnSelectAll()
{
   UpdateData(TRUE);

   m_CallOptionsChecked = TRUE;
   m_PutOptionsChecked = TRUE;
   m_FuturesChecked = TRUE;
   m_StocksChecked = TRUE;
   m_TreasuriesChecked = TRUE;
   m_OtherChecked = TRUE;

   m_btnOk.EnableWindow(TRUE);
   UpdateData(FALSE);
}

void CInstrumentTypesDlg::OnBnClickedBtnSelectOptions()
{
   UpdateData(TRUE);

   m_CallOptionsChecked = TRUE;
   m_PutOptionsChecked = TRUE;

   m_btnOk.EnableWindow(TRUE);
   UpdateData(FALSE);
}

void CInstrumentTypesDlg::OnCheckBoxClick()
{
   UpdateData(TRUE);
   m_btnOk.EnableWindow(m_CallOptionsChecked ||
                        m_PutOptionsChecked ||
                        m_FuturesChecked ||
                        m_StocksChecked ||
                        m_TreasuriesChecked ||
                        m_OtherChecked);
}
