// SymbolResolutionDlg.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <sstream>
#include <vector>
#include <cassert>
#include "Common.h"
#include "SymbolResolution.h"
#include "SymbolResolutionDlg.h"
#include "InstrumentTypesDlg.h"

CSymbolResolutionDlg::CSymbolResolutionDlg(
                                 CWnd* pParent /*=NULL*/)
   : CDialog(CSymbolResolutionDlg::IDD, pParent)
{
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}


//////////////////////////////////////////////////////////////////////////
//////////////////// CQGCEL events handlers
//////////////////////////////////////////////////////////////////////////
STDMETHODIMP CSymbolResolutionDlg::OnDataError(
                                 LPDISPATCH /*obj*/,
                                 BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CSymbolResolutionDlg::OnGWConnectionStatusChanged(
                                 eConnectionStatus newStatus)
{
   m_lblGWConnection.SetWindowText(_T("GW connection : ") + ConnectionStatusToString(newStatus));

   if (newStatus == csConnectionUp)
   {
      m_spCQGCEL->put_AccountSubscriptionLevel(aslAccountsOnly);
   }

   // Enable/disable buttons 
   m_btnGetCommodities.EnableWindow((newStatus == csConnectionUp) && 
                                    (GetCheckedAccountsCount() > 0));

   return S_OK;
}

STDMETHODIMP CSymbolResolutionDlg::OnDataConnectionStatusChanged(
                                 eConnectionStatus newStatus)
{
   m_lblDataConnection.SetWindowText(_T("Data connection ") + ConnectionStatusToString(newStatus));

   // Enable/disable buttons
   m_btnGetInstruments.EnableWindow((m_edSymbol.GetWindowTextLength() > 0) && 
                                    (newStatus == csConnectionUp));
   m_edSymbol.EnableWindow(newStatus == csConnectionUp);

   return S_OK;
}

STDMETHODIMP CSymbolResolutionDlg::OnCommodityInstrumentsResolved(
                                 BSTR commodityName, 
                                 eInstrumentType instrumentTypes, 
                                 ICQGCommodityInstruments* instrumentNames)
{
   try
   {
      // Create node with commodity name
      CString nodeName;
      nodeName = ATL::CW2T(commodityName);
      nodeName += _T(" (") + InstrumentTypeToString(instrumentTypes) + _T(")");
      
      VARIANT_BOOL isTradableOnly;
      HRESULT hr = instrumentNames->get_IsTradableOnly(&isTradableOnly);
      AssertCOMError<ICQGCommodityInstruments>(instrumentNames, hr);
      
      if (isTradableOnly == VARIANT_TRUE)
      {
         nodeName += _T(" : Only tradables");
      }

      // Add node
      HTREEITEM hTrItem = m_trFullNames.InsertItem(nodeName);
      
      // Add all resolved instruments as subnodes to list
      long count;
      hr = instrumentNames->get_Count(&count);
      AssertCOMError<ICQGCommodityInstruments>(instrumentNames, hr);

      for (long i = 0; i < count; ++i)
      {
         ATL::CComBSTR instrumentName;
         hr = instrumentNames->get_Item(i, &instrumentName);
         AssertCOMError<ICQGCommodityInstruments>(instrumentNames, hr);

         m_trFullNames.InsertItem(ATL::CW2T(instrumentName), hTrItem);
      }

      // Enable remove all button
      m_btnRemoveAll.EnableWindow(TRUE);
   }
   catch (std::runtime_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
   return S_OK;
}

STDMETHODIMP CSymbolResolutionDlg::OnAccountChanged(
                                 eAccountChangeType change, 
                                 ICQGAccount* /*account*/, 
                                 ICQGPosition* /*position*/)
{
   try
   {
      if (change != actAccountsReloaded)
      {
         return S_OK;
      }

      ATL::CComPtr<ICQGAccounts> spAccounts;
      HRESULT hr = m_spCQGCEL->get_Accounts(&spAccounts);
      AssertCOMError(m_spCQGCEL, hr);

      long count;
      hr = spAccounts->get_Count(&count);
      AssertCOMError(spAccounts, hr);

      if (count == 0)
      {
         // No accounts
         return S_OK;
      }

      // Iterate over accounts collection and add them to list
      ATL::CComPtr<IEnumVARIANT> spEnumVariant;
      hr = spAccounts->get__NewEnum(&spEnumVariant);
      AssertCOMError(spAccounts, hr);

      std::vector<ATL::CComVariant> pItems(count);
      ULONG celtFetched;
      hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
      AssertCOMError(spEnumVariant, hr);

      for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
      {
         ATLASSERT(it->vt == VT_DISPATCH);

         ATL::CComPtr<ICQGAccount> spAccount;
         hr = it->pdispVal->QueryInterface(__uuidof(ICQGAccount), (void** )&spAccount);
         AssertCOMError<IDispatch>(it->pdispVal, hr);

         // Add current account to list
         InsertAccount(spAccount);
      }
   }
   catch (std::runtime_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CSymbolResolutionDlg::OnTradableCommoditiesResolved(
                                 long gwAccountID, 
                                 ICQGCommodities* commodityNames, 
                                 ICQGError* err)
{
   try
   {
      HRESULT hr;

      if (err != NULL)
      {
         // Showing message box with error description which 
         // occurs during commodities request process
         long errCode;
         hr = err->get_Code(&errCode);
         AssertCOMError(m_spCQGCEL, hr);

         ATL::CComBSTR errDesc;
         hr = err->get_Description(&errDesc);
         AssertCOMError(m_spCQGCEL, hr);

         // Create error message
         CString codeString;
         codeString.Format(_T("%d"), errCode);

         CString errMsg(_T("Error "));
         errMsg += _T(" : ");
         errMsg += ATL::CW2T(errDesc);
         AfxMessageBox(errMsg, MB_ICONASTERISK);

         // Note, that CQGCEL is not concerned with errors in the events
         // so we alway return S_OK
         return S_OK;
      }

      // Add account as node to tree
      HTREEITEM hTrItem;
      if (gwAccountID == 0)
      {
         hTrItem = m_trCommodities.InsertItem(_T("All Accounts"));
      }
      else
      {
         ATL::CComPtr<ICQGAccounts> spAccounts;
         hr = m_spCQGCEL->get_Accounts(&spAccounts);
         AssertCOMError(m_spCQGCEL, hr);

         ATL::CComPtr<ICQGAccount> spAccount;
         hr = spAccounts->get_Item(gwAccountID, &spAccount);
         AssertCOMError(spAccounts, hr);

         ATL::CComBSTR accNameGW;
         hr = spAccount->get_GWAccountName(&accNameGW);
         AssertCOMError(spAccount, hr);

         hTrItem = m_trCommodities.InsertItem(ATL::CW2T(accNameGW));
      }

      // Insert all tradable commodities as subnodes to added account node 
      long count;
      hr = commodityNames->get_Count(&count);
      AssertCOMError<ICQGCommodities>(commodityNames, hr);

      for (int i = 0; i < count; ++i)
      {
         ATL::CComBSTR cmdtyName;
         hr = commodityNames->get_Item(i, &cmdtyName);
         AssertCOMError<ICQGCommodities>(commodityNames, hr);

         m_trCommodities.InsertItem(ATL::CW2T(cmdtyName), hTrItem);
      }
   }
   catch (std::runtime_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
   return S_OK;
}

//////////////////////////////////////////////////////////////////////////
//////////////////// Dialog controls events handlers
//////////////////////////////////////////////////////////////////////////
void CSymbolResolutionDlg::OnBnClickedBtnGetCommodities()
{
   try
   {
      HRESULT hr;

      m_trCommodities.DeleteAllItems();

      if (GetCheckedAccountsCount() == m_lvAccounts.GetItemCount())
      {
         // To get all accounts commodities request with argument 0
         hr = m_spCQGCEL->RequestTradableCommodities(0);
         AssertCOMError(m_spCQGCEL, hr);
      }
      else
      {
         // Iterate over checked accounts and request for each separately
         for (int i = 0; i < m_lvAccounts.GetItemCount(); ++i)
         {
            if (m_lvAccounts.GetCheck(i))
            {
               CString GWAccIDStr = m_lvAccounts.GetItemText(i, 2).GetString();
               TCHAR* ptrToEnd = NULL;
               long GWAccID = _tcstol(GWAccIDStr, &ptrToEnd, 10);
               hr = m_spCQGCEL->RequestTradableCommodities(GWAccID);
               AssertCOMError(m_spCQGCEL, hr);
            }
         }
      }
   }
   catch (std::runtime_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CSymbolResolutionDlg::OnBnClickedBtnGetResolution()
{
   try
   {
      // Open dialog to allow user to select 
      // instrument types for request 
      CInstrumentTypesDlg dlg;
      if (dlg.DoModal() == IDOK)
      {
         CString symbol;
         long instrumentTypes;
         VARIANT_BOOL tradeableOnly;

         // To request for only tradable instruments
         // Gateway server connection must be up
         m_edSymbol.GetWindowText(symbol);
         dlg.GetTypes(instrumentTypes, tradeableOnly);
         HRESULT hr = m_spCQGCEL->RequestCommodityInstruments(ATL::CT2W(symbol), 
                                          (eInstrumentType)instrumentTypes, tradeableOnly);
         AssertCOMError(m_spCQGCEL, hr);
      }
   }
   catch (std::runtime_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CSymbolResolutionDlg::OnTvnSelChangedTrFullNames(
                                 NMHDR *pNMHDR, 
                                 LRESULT *pResult)
{
   LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);

   // Enable/disable remove button based on instrument 
   // or commodity is selected
   HTREEITEM hTrItem = pNMTreeView->itemNew.hItem;
   m_btnRemove.EnableWindow(m_trFullNames.GetParentItem(hTrItem) == NULL);

   *pResult = 0;
}

void CSymbolResolutionDlg::OnTvnSelChangedTrCommodities(
                                 NMHDR *pNMHDR,
                                 LRESULT *pResult)
{
   LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);

   // Fill selected commodity to the commodity edit box
   HTREEITEM hTrItem = pNMTreeView->itemNew.hItem;
   if (m_trCommodities.GetParentItem(hTrItem) != NULL)
   {
      CString symbol = m_trCommodities.GetItemText(hTrItem);
      m_edSymbol.SetWindowText(symbol);
   }

   *pResult = 0;
}

void CSymbolResolutionDlg::OnEnChangeEdSymbol()
{
   m_btnGetInstruments.EnableWindow((m_edSymbol.GetWindowTextLength() != 0) &&
                                    IsDataConnectionUp());
}

void CSymbolResolutionDlg::OnBnClickedBtnRemove()
{
   HTREEITEM hTrItem = m_trFullNames.GetSelectedItem();

   // Only root nodes can be removed
   ATLASSERT(m_trFullNames.GetParentItem(hTrItem) == NULL);

   m_trFullNames.DeleteItem(hTrItem);
   m_trFullNames.SetFocus();

   m_btnRemove.EnableWindow(m_trFullNames.GetCount() != 0);
   m_btnRemoveAll.EnableWindow(m_trFullNames.GetCount() != 0);
}

void CSymbolResolutionDlg::OnBnClickedBtnRemoveAll()
{
   m_trFullNames.DeleteAllItems();

   m_btnRemove.EnableWindow(FALSE);
   m_btnRemoveAll.EnableWindow(FALSE);
}

void CSymbolResolutionDlg::OnLvnItemchangedLsvaccounts(
                                 NMHDR * /*pNMHDR*/,
                                 LRESULT *pResult)
{
   m_btnGetCommodities.EnableWindow((GetCheckedAccountsCount() > 0) && 
                                    IsGWConnectionUp());
   *pResult = 0;
}

void CSymbolResolutionDlg::OnBnClickedBtnSelectAllAccs()
{
   CheckAllAccounts(TRUE);
}

void CSymbolResolutionDlg::OnBnClickedBtnUnselectAllAccs()
{
   CheckAllAccounts(FALSE);
}

//////////////////////////////////////////////////////////////////////////
//////////////////// Private methods
//////////////////////////////////////////////////////////////////////////
void CSymbolResolutionDlg::InitializeControls()
{
   OnDataConnectionStatusChanged(csConnectionDown);
   OnGWConnectionStatusChanged(csConnectionDown);

   m_btnRemove.EnableWindow(FALSE);
   m_btnRemoveAll.EnableWindow(FALSE);

   m_lvAccounts.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_CHECKBOXES);
   m_lvAccounts.InsertColumn(0, _T("Check"), LVCFMT_LEFT, 20);
   m_lvAccounts.InsertColumn(1, _T("Account Name"), LVCFMT_LEFT, 130);
   m_lvAccounts.InsertColumn(2, _T("Account GWID"), LVCFMT_LEFT, 50);
}

BOOL CSymbolResolutionDlg::IsDataConnectionUp() const
{
   // Check connection status   
   ATL::CComPtr<ICQGEnvironment> spEnv;
   HRESULT hr = m_spCQGCEL->get_Environment(&spEnv);
   AssertCOMError(m_spCQGCEL, hr);

   eConnectionStatus connStatus;
   hr = spEnv->get_DataConnectionStatus(&connStatus);
   AssertCOMError(spEnv, hr);

   return (connStatus == csConnectionUp);
}

BOOL CSymbolResolutionDlg::IsGWConnectionUp() const
{
   // Connection status can be accessed via environment.
   // So at first we need to get it from CQGCEL.
   ATL::CComPtr<ICQGEnvironment> spEnv;
   HRESULT hr = m_spCQGCEL->get_Environment(&spEnv);
   AssertCOMError(m_spCQGCEL, hr);

   eConnectionStatus connStatus;
   hr = spEnv->get_GWConnectionStatus(&connStatus);
   AssertCOMError(spEnv, hr);

   // Check connection status
   return (connStatus == csConnectionUp);
}

void CSymbolResolutionDlg::InitializeCQGCEL()
{
   // create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Change configuration in order not to check Ready status, 
   // and let CQCCEL know that it is used from ATL client.
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   AssertCOMError<ICQGCEL>(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   AssertCOMError<ICQGAPIConfig>(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   AssertCOMError<ICQGAPIConfig>(spConf, hr);

   hr = spConf->put_DefaultInstrumentSubscriptionLevel(dsQuotesAndDOM);
   AssertCOMError<ICQGAPIConfig>(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   AssertCOMError<ICQGCEL>(m_spCQGCEL, hr);
}

int CSymbolResolutionDlg::GetCheckedAccountsCount() const
{
   int count = m_lvAccounts.GetItemCount();
   int checkedCount = 0;
   for (int i = 0; i < count; ++i)
   {
      if (m_lvAccounts.GetCheck(i))
      {
         ++checkedCount;
      }
   }
   return checkedCount;
}

CString CSymbolResolutionDlg::InstrumentTypeToString(
                                 eInstrumentType type)
{
   // NOTE: Instrument type is a bit mask
   if ((type & itAllInstruments) == itAllInstruments)
   {
      return _T("All instruments");
   }

   CString typeString;
   if ((type & itAllOptions) == itAllOptions)
   {
      typeString += _T("All options, ");
   }
   else
   {
      if ((type & itOptionCall) == itOptionCall)
         typeString += _T("Call Options, ");

      if ((type & itOptionPut) == itOptionPut)
         typeString += _T("Put Options, ");
   }

   if ((type & itFuture) == itFuture)
      typeString += _T("Futures, ");

   if ((type & itStock) == itStock)
      typeString += _T("Stocks, ");

   if ((type & itTreasure) == itTreasure)
      typeString += _T("Treasuries, ");

   if ((type & itOther) == itOther)
      typeString += _T("Other, ");

   ATLASSERT(typeString.GetLength() != 0);
   typeString = typeString.Left(typeString.GetLength() - 2);

   return typeString;
}

void CSymbolResolutionDlg::InsertAccount(
                                 const ATL::CComPtr<ICQGAccount>& spAccount)
{
   // Create string for specified account to add to the list
   ATL::CComBSTR accName;
   HRESULT hr = spAccount->get_GWAccountName(&accName);
   AssertCOMError(spAccount, hr);

   long accID;
   hr = spAccount->get_GWAccountID(&accID);
   AssertCOMError(spAccount, hr);

   CString accIDString;
   accIDString.Format(_T("%d"), accID);

   // Get last row to insert new account name to the end 
   int row = m_lvAccounts.GetItemCount();

   m_lvAccounts.InsertItem(row, _T(""));
   m_lvAccounts.SetItemText(row, 1, ATL::CW2T(accName));
   m_lvAccounts.SetItemText(row, 2, accIDString);
}

void CSymbolResolutionDlg::CheckAllAccounts(
                                 BOOL check)
{
   int count = m_lvAccounts.GetItemCount();
   for (int i = 0; i < count; ++i)
   {
      m_lvAccounts.SetCheck(i, check);
   }
}

//////////////////////////////////////////////////////////////////////////
//////////////////// Overridden methods and message handlers
//////////////////////////////////////////////////////////////////////////
void CSymbolResolutionDlg::DoDataExchange(
                                 CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_LSVACCOUNTS, m_lvAccounts);
   DDX_Control(pDX, IDC_BTNGETCOMMODITIES, m_btnGetCommodities);
   DDX_Control(pDX, IDC_STDATACONNECTION, m_lblDataConnection);
   DDX_Control(pDX, IDC_STGWCONNECTION, m_lblGWConnection);
   DDX_Control(pDX, IDC_TRCOMMODITIES, m_trCommodities);
   DDX_Control(pDX, IDC_BTNGETRESOLUTION, m_btnGetInstruments);
   DDX_Control(pDX, IDC_EDSYMBOL, m_edSymbol);
   DDX_Control(pDX, IDC_TRFULLNAMES, m_trFullNames);
   DDX_Control(pDX, IDC_BTNREMOVE, m_btnRemove);
   DDX_Control(pDX, IDC_BTNREMOVEALL, m_btnRemoveAll);
}

BEGIN_MESSAGE_MAP(CSymbolResolutionDlg, CDialog)
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_BN_CLICKED(IDC_BTNGETCOMMODITIES, OnBnClickedBtnGetCommodities)
   ON_BN_CLICKED(IDC_BTNGETRESOLUTION, OnBnClickedBtnGetResolution)
   ON_EN_CHANGE(IDC_EDSYMBOL, OnEnChangeEdSymbol)
   ON_BN_CLICKED(IDC_BTNREMOVE, OnBnClickedBtnRemove)
   ON_BN_CLICKED(IDC_BTNREMOVEALL, OnBnClickedBtnRemoveAll)
   ON_BN_CLICKED(IDC_BTNSELECTALLACCS, OnBnClickedBtnSelectAllAccs)
   ON_BN_CLICKED(IDC_BTNUNSELECTALLACCS, OnBnClickedBtnUnselectAllAccs)
   ON_NOTIFY(LVN_ITEMCHANGED, IDC_LSVACCOUNTS, OnLvnItemchangedLsvaccounts)
   ON_NOTIFY(TVN_SELCHANGED, IDC_TRCOMMODITIES, OnTvnSelChangedTrCommodities)
   ON_NOTIFY(TVN_SELCHANGED, IDC_TRFULLNAMES, OnTvnSelChangedTrFullNames)
   ON_WM_DESTROY()
END_MESSAGE_MAP()

BOOL CSymbolResolutionDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   // Set the icon for this dialog.  The framework does this automatically
   //  when the application's main window is not a dialog
   SetIcon(m_hIcon, TRUE);       // Set big icon
   SetIcon(m_hIcon, FALSE);      // Set small icon

   try
   {
      InitializeControls();
      InitializeCQGCEL();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
}

void CSymbolResolutionDlg::OnDestroy()
{
   try
   {
      if (m_spCQGCEL)
      {
         HRESULT hr = ICQGCELDispEventImpl::DispEventUnadvise(m_spCQGCEL);
         AssertCOMError<ICQGCEL>(m_spCQGCEL, hr);

         hr = m_spCQGCEL->Shutdown();
         AssertCOMError<ICQGCEL>(m_spCQGCEL, hr);
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}

void CSymbolResolutionDlg::OnOK()
{
   // Override to prevent dialog closing on Enter
}

// If you add a minimize button to your dialog, you will need the code below
// to draw the icon.  For MFC applications using the document/view model,
// this is automatically done for you by the framework.
void CSymbolResolutionDlg::OnPaint() 
{
   if (IsIconic())
   {
      CPaintDC dc(this); // device context for painting

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      // Center icon in client rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      // Draw the icon
      dc.DrawIcon(x, y, m_hIcon);
   }
   else
   {
      CDialog::OnPaint();
   }
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CSymbolResolutionDlg::OnQueryDragIcon()
{
   return static_cast<HCURSOR>(m_hIcon);
}

CString CSymbolResolutionDlg::ConnectionStatusToString(eConnectionStatus status)
{
   CString  statusString;

   switch (status)
   {
   case csConnectionDelayed:
      statusString = _T("delayed");
      break;
   case csConnectionDown:
      statusString = _T("down");
      break;
   case csConnectionUp:
      statusString = _T("up");
      break;
   case csConnectionTrouble:
      statusString = _T("trouble");
      break;
   case csConnectionNotLoggedOn:
      statusString = _T("not logged on");
      break;
   default:
      assert(!"Unknown connection status");
   }

   return statusString;
}