// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "TFlowBars.h"
#include "TFlowBarsDlg.h"

void CTFlowBarsDlg::InitializeCQGCEL()
{
   // Create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   Common::ThrowOnCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   Common::ThrowOnCOMError(m_spCQGCEL, hr);
}

void CTFlowBarsDlg::InitializeControls()
{
   InitializeList(m_lvLastTFlowBars);
   InitializeList(m_lvTFlowBars);

   ClearAllData();

   OnDataConnectionStatusChanged(csConnectionDown);

   FillIndices();
}

void CTFlowBarsDlg::InitializeList(
                        CListCtrl& listCtrl)
{
   listCtrl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   listCtrl.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   listCtrl.InsertColumn(rcIndex, _T("#"), LVCFMT_LEFT, 40);
   listCtrl.InsertColumn(rcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 150);
   listCtrl.InsertColumn(rcOpen, _T("Open"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcHigh, _T("High"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcLow, _T("Low"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcClose, _T("Close"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcMid, _T("Mid"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTrueHigh, _T("True High"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTrueLow, _T("True Low"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTrueMid, _T("True Mid"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcAskVolume, _T("Ask Volume"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcBidVolume, _T("Bid Volume"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcBestAskPrice, _T("Best Ask Price"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcBestBidPrice, _T("Best Bid Price"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcDomBidVolume, _T("Dom Bid Volume"), LVCFMT_LEFT, 150);
   listCtrl.InsertColumn(rcDomAskVolume, _T("Dom Ask Volume"), LVCFMT_LEFT, 150);
}

void CTFlowBarsDlg::ClearAllData()
{
   m_lblStatus.SetWindowText(_T(""));

   ClearRequestParams();

   ClearTags();

   ClearOutputParams();
   ClearIndices();

   ClearRecords(m_lvLastTFlowBars);
   ClearRecords(m_lvTFlowBars);

   m_lblAllRecsCount.SetWindowText(_T(""));
   m_lblFltrRecsCount.SetWindowText(_T(""));
}

void CTFlowBarsDlg::ClearRequestParams()
{
   UpdateData();

   m_Symbol = _T("");
   m_RangeStart = _T("");
   m_RangeEnd = _T("");
   m_AggregationLevel = _T("");
   m_Aggregation = _T("");
   m_DomAskVolLevel = _T("");
   m_DomBidVolLevel = _T("");
   m_UpdatesEnabled = _T("");
   m_IncludeEnd = _T("");
   m_SessionsFilter = _T("");
   m_Continuation = _T("");
   m_EqualizeCloses = _T("");
   m_DaysBeforeExpiration = _T("");
   m_SessionFlags = _T("");
   m_SubscriptionLevel = _T("");
   m_RecalcPeriod = _T("");

   UpdateData(FALSE);

   m_lblError.SetWindowText(_T(""));
}

void CTFlowBarsDlg::ClearOutputParams()
{
   UpdateData();

   m_OutputStart = _T("");
   m_OutputEnd = _T("");

   UpdateData(FALSE);
}

void CTFlowBarsDlg::ClearTags()
{
   m_edTagName.SetWindowText(_T(""));
   m_edTagValue.SetWindowText(_T(""));
}

void CTFlowBarsDlg::ClearRecords(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();
}

void CTFlowBarsDlg::ShowAllData()
{
   ShowRequestParams();

   eRequestStatus status;
   HRESULT hr = m_spTFlowBars->get_Status(&status);
   Common::ThrowOnCOMError(m_spTFlowBars, hr);

   m_lblStatus.SetWindowText(CHelper::RequestStatusToString(status));

   if (status == rsSuccess)
   {
      ShowOutputParams();

      FillIndices();

      long count;
      hr = m_spTFlowBars->get_Count(&count);
      Common::ThrowOnCOMError(m_spTFlowBars, hr);

      // Show records
      const long startIndex = max(count - m_ShowCount + 1, 1);
      const long endIndex = count;
      ShowRecords(startIndex, endIndex, m_lvLastTFlowBars);

      // Show all records count
      m_lblAllRecsCount.SetWindowText(GetValueAsString(count));
   }
}

void CTFlowBarsDlg::ShowChangedData()
{
   // Clear the list box
   ClearRecords(m_lvLastTFlowBars);

   // Get records count
   long count;
   HRESULT hr = m_spTFlowBars->get_Count(&count);
   Common::ThrowOnCOMError(m_spTFlowBars, hr);

   // Show records
   const long startIndex = max(count - m_ShowCount + 1, 1);
   const long endIndex = count;
   ShowRecords(startIndex, endIndex, m_lvLastTFlowBars);

   // Show all records count
   m_lblAllRecsCount.SetWindowText(GetValueAsString(count));

   // Show output parameters
   ShowOutputParams();
}

void CTFlowBarsDlg::ShowRequestParams()
{
   UpdateData();

   // Get request from tflow bars collection
   ATL::CComPtr<ICQGTFlowBarsRequest> spRequest;
   HRESULT hr = m_spTFlowBars->get_Request(&spRequest);
   Common::ThrowOnCOMError(m_spTFlowBars, hr);

   // Show request parameters
   ATL::CComBSTR symbol;
   hr = spRequest->get_Symbol(&symbol);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeStart;
   hr = spRequest->get_RangeStart(&rangeStart);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeEnd;
   hr = spRequest->get_RangeEnd(&rangeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   long aggregationLevel;
   hr = spRequest->get_AggregationLevel(&aggregationLevel);
   Common::ThrowOnCOMError(spRequest, hr);

   eTfAggregation aggregation;
   hr = spRequest->get_Aggregation(&aggregation);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL updatesEnabled;
   hr = spRequest->get_UpdatesEnabled(&updatesEnabled);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL includeEnd;
   hr = spRequest->get_IncludeEnd(&includeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant sessionsFilter;
   hr = spRequest->get_SessionsFilter(&sessionsFilter);
   Common::ThrowOnCOMError(spRequest, hr);

   eTimeSeriesContinuationType continuation;
   hr = spRequest->get_Continuation(&continuation);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL equalizeCloses;
   hr = spRequest->get_EqualizeCloses(&equalizeCloses);
   Common::ThrowOnCOMError(spRequest, hr);

   long daysBeforeExpiration;
   hr = spRequest->get_DaysBeforeExpiration(&daysBeforeExpiration);
   Common::ThrowOnCOMError(spRequest, hr);

   eSessionFlag sessionFlags;
   hr = spRequest->get_SessionFlags(&sessionFlags);
   Common::ThrowOnCOMError(spRequest, hr);
   
   eTimeSeriesSubscriptionLevel subscriptionLevel;
   hr = spRequest->get_SubscriptionLevel(&subscriptionLevel);
   Common::ThrowOnCOMError(spRequest, hr);
   
   long recalcPeriod;
   hr = spRequest->get_RecalcPeriod(&recalcPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   eTFlowRequestDomVolumeLevel askLevel;
   hr = spRequest->get_DomAskVolumeLevel(&askLevel);
   Common::ThrowOnCOMError(spRequest, hr);

   eTFlowRequestDomVolumeLevel bidLevel;
   hr = spRequest->get_DomBidVolumeLevel(&bidLevel);
   Common::ThrowOnCOMError(spRequest, hr);

   m_Symbol = ATL::CW2T(symbol);
   m_RangeStart = GetValueAsString(rangeStart);
   m_RangeEnd = GetValueAsString(rangeEnd);
   m_AggregationLevel = GetValueAsString(aggregationLevel);
   m_Aggregation = CHelper::AggregationToString(aggregation);
   m_DomAskVolLevel = CHelper::DomVolumeLevelToString(askLevel);
   m_DomBidVolLevel = CHelper::DomVolumeLevelToString(bidLevel);
   m_UpdatesEnabled = GetValueAsString(updatesEnabled, VT_BOOL);
   m_IncludeEnd = GetValueAsString(includeEnd, VT_BOOL);

   CString filter = GetValueAsString(sessionsFilter);
   if (filter == "31")
   {
      filter = "All sessions (31)";
   }
   else if (filter == "0")
   {
      filter = "Primary session only (0)";
   }
   m_SessionsFilter = filter;

   m_Continuation = CHelper::ContinuationTypeToString(continuation);
   m_EqualizeCloses = GetValueAsString(equalizeCloses, VT_BOOL);
   m_DaysBeforeExpiration = GetValueAsString(daysBeforeExpiration);
   m_SessionFlags = CHelper::SessionFlagToString(sessionFlags);
   m_SubscriptionLevel = CHelper::SubscriptionLevelToString(subscriptionLevel);
   if (subscriptionLevel == tslEndOfBarAndPeriod)
   {
      m_RecalcPeriod = GetValueAsString(recalcPeriod);
   }

   UpdateData(FALSE);

   // Get error from tflow bars collection and show it if exist
   ATL::CComPtr<ICQGError> spError;
   hr = m_spTFlowBars->get_LastError(&spError);
   Common::ThrowOnCOMError(m_spTFlowBars, hr);

   if (spError)
   {
      ATL::CComBSTR errDescription;
      hr = spError->get_Description(&errDescription);
      Common::ThrowOnCOMError(spError, hr);

      m_lblError.SetWindowText(ATL::CW2T(errDescription));
   }
}

void CTFlowBarsDlg::ShowOutputParams()
{
   UpdateData();

   // Get output parameters from tflow bars collection and show them
   DATE startTimestamp;
   HRESULT hr = m_spTFlowBars->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spTFlowBars, hr);

   DATE endTimestamp;
   hr = m_spTFlowBars->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spTFlowBars, hr);

   m_OutputStart = GetValueAsString(startTimestamp, VT_DATE);
   m_OutputEnd = GetValueAsString(endTimestamp, VT_DATE);

   UpdateData(FALSE);
}

void CTFlowBarsDlg::ShowRecords(
                        long startIndex,
                        long endIndex,
                        CListCtrl& listCtrl)
{
   for (long i = startIndex; i <= endIndex; ++i)
   {
      AddRecord(i, listCtrl);
   }
}

void CTFlowBarsDlg::AddRecord(
                        long recordIndex,
                        CListCtrl& listCtrl)
{
   // Add empty row
   int row = listCtrl.InsertItem(listCtrl.GetItemCount(), _T(""));

   ShowRecord(recordIndex, listCtrl, row);
}

void CTFlowBarsDlg::ShowRecord(
                        long recordIndex,
                        CListCtrl& listCtrl,
                        long row)
{
   // Get specified TFlowBar from tflow bars collection by its index
   ATL::CComPtr<ICQGTFlowBar> spTFlowBar;
   HRESULT hr = m_spTFlowBars->get_Item(recordIndex - 1, &spTFlowBar);
   Common::ThrowOnCOMError(m_spTFlowBars, hr);

   // Get TFlowBar timestamp to show it
   DATE timestamp;
   hr = spTFlowBar->get_Timestamp(&timestamp);
   Common::ThrowOnCOMError(spTFlowBar, hr);

   listCtrl.SetItemText(row, rcIndex, GetValueAsString(recordIndex));
   listCtrl.SetItemText(row, rcTimestamp, GetValueAsString(timestamp, VT_DATE, true));

   // Show parameters into given listbox
   ShowProperty<double>(spTFlowBar, &ICQGTFlowBar::get_Open, listCtrl, row, rcOpen);
   ShowProperty<double>(spTFlowBar, &ICQGTFlowBar::get_High, listCtrl, row, rcHigh);
   ShowProperty<double>(spTFlowBar, &ICQGTFlowBar::get_Low, listCtrl, row, rcLow);
   ShowProperty<double>(spTFlowBar, &ICQGTFlowBar::get_Close, listCtrl, row, rcClose);
   ShowProperty<double>(spTFlowBar, &ICQGTFlowBar::get_Mid, listCtrl, row, rcMid);
   ShowProperty<double>(spTFlowBar, &ICQGTFlowBar::get_TrueHigh, listCtrl, row, rcTrueHigh);
   ShowProperty<double>(spTFlowBar, &ICQGTFlowBar::get_TrueLow, listCtrl, row, rcTrueLow);
   ShowProperty<double>(spTFlowBar, &ICQGTFlowBar::get_TrueMid, listCtrl, row, rcTrueMid);
   ShowProperty<long>(spTFlowBar, &ICQGTFlowBar::get_AskVolume, listCtrl, row, rcAskVolume);
   ShowProperty<long>(spTFlowBar, &ICQGTFlowBar::get_BidVolume, listCtrl, row, rcBidVolume);
   ShowProperty<double>(spTFlowBar, &ICQGTFlowBar::get_BestAskPrice, listCtrl, row, rcBestAskPrice);
   ShowProperty<double>(spTFlowBar, &ICQGTFlowBar::get_BestBidPrice, listCtrl, row, rcBestBidPrice);

   listCtrl.SetItemText(row, rcDomAskVolume, GetDomString(spTFlowBar, true));
   listCtrl.SetItemText(row, rcDomBidVolume, GetDomString(spTFlowBar, false));
}

CString CTFlowBarsDlg::GetDomString(
                        const ATL::CComPtr<ICQGTFlowBar>& spTFlowBar,
                        bool isAsk)
{
   static const CString DELIMITER = _T("; ");

   long count;
   HRESULT hr;
   if (isAsk)
   {
      hr = spTFlowBar->get_DomAskCount(&count);
   }
   else
   {
      hr = spTFlowBar->get_DomBidCount(&count);
   }

   Common::ThrowOnCOMError(spTFlowBar, hr);

   if (count == 0)
   {
      return CHelper::m_N_A;
   }

   CString domStr;
   for (long i = 0; i < count; ++i)
   {
      long domVol;
      if (isAsk)
      {
         hr = spTFlowBar->get_DomAskVolume(i, &domVol);
      }
      else
      {
         hr = spTFlowBar->get_DomBidVolume(i, &domVol);
      }
      Common::ThrowOnCOMError(spTFlowBar, hr);

      domStr.Append(GetValueAsString(domVol));
      domStr.Append(DELIMITER);
   }
   
   return domStr.Left(domStr.GetLength() - DELIMITER.GetLength());
}

template <typename T>
void CTFlowBarsDlg::ShowProperty(
                        const ATL::CComPtr<ICQGTFlowBar>& spTFlowBar,
                        HRESULT (__stdcall ICQGTFlowBar::*get_Property)(T*),
                        CListCtrl& listCtrl,
                        int row,
                        eRecordsColumns col)
{
   T value;
   HRESULT hr = (spTFlowBar->*get_Property)(&value);
   Common::ThrowOnCOMError(spTFlowBar, hr);

   listCtrl.SetItemText(row, col, GetValueAsString(value));
}

void CTFlowBarsDlg::ClearIndices()
{
   m_edStart.SetValue(0);
   m_edEnd.SetValue(0);
}

void CTFlowBarsDlg::FillIndices()
{
   if (m_spTFlowBars == NULL)
   {
      return;
   }

   eRequestStatus status;
   HRESULT hr = m_spTFlowBars->get_Status(&status);
   Common::ThrowOnCOMError(m_spTFlowBars, hr);

   if (status != rsSuccess)
   {
      return;
   }

   // Fill start and end indices controls.
   m_edStart.SetValue(1);

   long count;
   hr = m_spTFlowBars->get_Count(&count);
   Common::ThrowOnCOMError(m_spTFlowBars, hr);

   m_edEnd.SetValue(count);
}

void CTFlowBarsDlg::EnableControls()
{
   bool isDataConnectionUp = false;
   HRESULT hr;
   VARIANT_BOOL isStarted = VARIANT_FALSE;

   if (m_spCQGCEL)
   {
      hr = m_spCQGCEL->get_IsStarted(&isStarted);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }
   
   if (isStarted)
   {
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      hr = m_spCQGCEL->get_Environment(&spEnvironment);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      eConnectionStatus dataConnectionStatus;
      hr = spEnvironment->get_DataConnectionStatus(&dataConnectionStatus);
      Common::ThrowOnCOMError(spEnvironment, hr);

      isDataConnectionUp = (dataConnectionStatus == csConnectionUp);
   }

   m_btnRequest.EnableWindow(isDataConnectionUp);

   bool isTFlowBarsSelected = (m_spTFlowBars != NULL);

   m_btnRemove.EnableWindow(isTFlowBarsSelected && isDataConnectionUp);
   m_btnRemoveAll.EnableWindow(isTFlowBarsSelected && isDataConnectionUp);

   m_edTagName.EnableWindow(isTFlowBarsSelected);
   m_edTagValue.EnableWindow(isTFlowBarsSelected);

   bool isTFlowBarsSucceeded = false;
   if (isTFlowBarsSelected)
   {
      eRequestStatus status;
      hr = m_spTFlowBars->get_Status(&status);
      Common::ThrowOnCOMError(m_spTFlowBars, hr);

      isTFlowBarsSucceeded = (status == rsSuccess);
   }

   m_edStart.EnableWindow(isTFlowBarsSucceeded);
   m_edEnd.EnableWindow(isTFlowBarsSucceeded);
   m_spnStart.EnableWindow(isTFlowBarsSucceeded);
   m_spnEnd.EnableWindow(isTFlowBarsSucceeded);

   m_btnShow.EnableWindow(isTFlowBarsSucceeded);
}

CString CTFlowBarsDlg::GetValueAsString(
                        ATL::CComVariant value,
                        VARENUM valueType/*= VT_EMPTY*/,
                        bool withMilliseconds /*= false*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return Common::VariantToString(value, withMilliseconds);
   }
   return CHelper::m_N_A;
}
