/*
* =====================================================================
*  Filename: Helper.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

class CHelper
{

private:

   /// \brief CHelper constructor.
   CHelper();

public:

   /// \brief Converts enum of connection status to appropriate string.
   /// \param connectionStatus - enum which will be converted
   /// \return String which describes tflow bars request type.
   static CString ConnectionStatusToString(
						eConnectionStatus connectionStatus);

   /// \brief Converts enum of request status to appropriate string.
   /// \param requestStatus - enum which will be converted
   /// \return String which describes request status.
   static CString RequestStatusToString(
						eRequestStatus requestStatus);

   /// \brief Converts enum of aggregation to appropriate string.
   /// \param aggregation - enum which will be converted
   /// \return String which describes aggregation.
   static CString AggregationToString(
						eTfAggregation aggregation);

   /// \brief Converts enum of DOM level to appropriate string.
   /// \param level - enum which will be converted
   /// \return String which describes DOM Volume Level.
   static CString DomVolumeLevelToString(
                  eTFlowRequestDomVolumeLevel level);
   
   /// \brief Converts enum of continuation type to appropriate string.
   /// \param continuationType - enum which will be converted
   /// \return String which describes continuation type.
   static CString ContinuationTypeToString(
						eTimeSeriesContinuationType continuationType);

   /// \brief Converts bitmap of session flag to appropriate string.
   /// \param sessionFlag - bitmap which will be converted
   /// \return String which describes session flag.
   static CString SessionFlagToString(
						eSessionFlag sessionFlag);
						
   /// \brief Converts enum of subscription level to appropriate string.
   /// \param subscriptionLevel - enum which will be converted
   /// \return String which describes subscription level.
   static CString SubscriptionLevelToString(
						eTimeSeriesSubscriptionLevel continuationType);

   /// \brief Compares two dates : COleDateTime comparison is not enough,
   /// \brief because we need to ignore seconds.
   /// \param date1 - first date
   /// \param date2 - second date
   /// \return Integer which means :
   /// \return -1 : first date is less than second,
   /// \return  0 : dates are equal,
   /// \return  1 : first date is greater than second.
   static int CompareDates(
                  const COleDateTime& date1,
                  const COleDateTime& date2);


public:

   /// \brief Selects an item of the specified combo box, which item data matches with given.
   /// \param comboBoxCtrl - combo box, which item will be selected
   /// \param itemData - item data, which must match with selected item data
   static void SetComboBoxItem(
						CComboBox& comboBoxCtrl,
						DWORD_PTR itemData);

public:

   /// \brief Custom date format.
   static const CString DateFormat;

   /// \brief Not available string.
   static const CString m_N_A;

};
