// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "Ticks.h"
#include "TicksDlg.h"

void CTicksDlg::InitializeCQGCEL()
{
   // Create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   Common::ThrowOnCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   Common::ThrowOnCOMError(m_spCQGCEL, hr);
}

void CTicksDlg::InitializeControls()
{
   InitializeList(m_lvLastTicks);
   InitializeList(m_lvTicks);

   ClearAllData();

   OnDataConnectionStatusChanged(csConnectionDown);

   m_rdIndices.SetCheck(BST_CHECKED);
   OnBnClickedRdIndices();

   m_dtpStart.SetFormat(CHelper::DateFormat);
   m_dtpEnd.SetFormat(CHelper::DateFormat);
}

void CTicksDlg::InitializeList(
                        CListCtrl& listCtrl)
{
   listCtrl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   listCtrl.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   listCtrl.InsertColumn(tcIndex, _T("#"), LVCFMT_LEFT, 40);
   listCtrl.InsertColumn(tcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 150);
   listCtrl.InsertColumn(tcPrice, _T("Price"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(tcVolume, _T("Volume"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(tcPriceType, _T("Price Type"), LVCFMT_LEFT, 100);
   listCtrl.InsertColumn(tcCorrectionType, _T("Correction Type"), LVCFMT_LEFT, 100);
   listCtrl.InsertColumn(tcMarketState, _T("Market State"), LVCFMT_LEFT, 100);
   listCtrl.InsertColumn(tcSalesConditionCode, _T("Sales Condition Code"), LVCFMT_LEFT, 100);
   listCtrl.InsertColumn(tcSalesConditionLabel, _T("Sales Condition Label"), LVCFMT_LEFT, 100);
   listCtrl.InsertColumn(tcContributorIdCode, _T("Contributor Id Code"), LVCFMT_LEFT, 100);
   listCtrl.InsertColumn(tcContributorId, _T("Contributor Id"), LVCFMT_LEFT, 100);
}

void CTicksDlg::ClearAllData()
{
   m_lblStatus.SetWindowText(_T(""));

   ClearRequestParams();

   ClearTags();

   ClearOutputParams();
   ClearDatesAndIndices();

   m_lvLastTicks.SetItemCountEx(0);
   m_lvTicks.SetItemCountEx(0);

   m_lblAllRecsCount.SetWindowText(_T(""));
   m_lblFltrRecsCount.SetWindowText(_T(""));
   m_lblLastRecsCount.SetWindowText(_T(""));
}

void CTicksDlg::ClearRequestParams()
{
   m_lblSymbol.SetWindowText(_T(""));
   m_lblType.SetWindowText(_T(""));
   m_lblRangeStart.SetWindowText(_T(""));
   m_lblRangeEnd.SetWindowText(_T(""));
   m_lblSessionsFilter.SetWindowText(_T(""));
   m_lblTicksFilter.SetWindowText(_T(""));
   m_lblLimit.SetWindowText(_T(""));

   m_lblError.SetWindowText(_T(""));
}

void CTicksDlg::ClearOutputParams()
{
   m_lblOutputStart.SetWindowText(_T(""));
   m_lblOutputEnd.SetWindowText(_T(""));
}

void CTicksDlg::ClearTags()
{
   m_edTagName.SetWindowText(_T(""));
   m_edTagValue.SetWindowText(_T(""));
}

void CTicksDlg::ShowAllData()
{
   ShowRequestParams();

   eRequestStatus status;
   HRESULT hr = m_spTicks->get_Status(&status);
   Common::ThrowOnCOMError(m_spTicks, hr);

   m_lblStatus.SetWindowText(CHelper::RequestStatusToString(status));

   if (status == rsSuccess)
   {
      ShowOutputParams();

      FillDatesAndIndices();

      long count;
      hr = m_spTicks->get_Count(&count);
      Common::ThrowOnCOMError(m_spTicks, hr);

      m_lvLastTicks.SetItemCountEx(count, LVSICF_NOSCROLL);

      // Show all records and last chunk counts
      m_lblAllRecsCount.SetWindowText(GetValueAsString(count));
      m_lblLastRecsCount.SetWindowText(GetValueAsString(count));
   }
}

void CTicksDlg::ShowRequestParams()
{
   // Get request from Ticks collection
   ATL::CComPtr<ICQGTicksRequest> spRequest;
   HRESULT hr = m_spTicks->get_Request(&spRequest);
   Common::ThrowOnCOMError(m_spTicks, hr);

   // Show request parameters
   ATL::CComBSTR symbol;
   hr = spRequest->get_Symbol(&symbol);
   Common::ThrowOnCOMError(spRequest, hr);
   m_lblSymbol.SetWindowText(ATL::CW2T(symbol));

   eTicksRequestType type;
   hr = spRequest->get_Type(&type);
   Common::ThrowOnCOMError(spRequest, hr);
   m_lblType.SetWindowText(CHelper::TicksRequestTypeToString(type));

   DATE rangeStart;
   hr = spRequest->get_RangeStart(&rangeStart);
   Common::ThrowOnCOMError(spRequest, hr);
   m_lblRangeStart.SetWindowText(GetValueAsString(rangeStart, VT_DATE));

   DATE rangeEnd;
   hr = spRequest->get_RangeEnd(&rangeEnd);
   Common::ThrowOnCOMError(spRequest, hr);
   m_lblRangeEnd.SetWindowText(GetValueAsString(rangeEnd, VT_DATE));

   long sessionsFilter;
   hr = spRequest->get_SessionsFilter(&sessionsFilter);
   Common::ThrowOnCOMError(spRequest, hr);

   CString filter = GetValueAsString(sessionsFilter);
   if (filter == "31")
   {
      filter = "All sessions (31)";
   }
   else if (filter == "0")
   {
      filter = "Primary session only (0)";
   }
   m_lblSessionsFilter.SetWindowText(filter);

   long limit = 0;
   hr = spRequest->get_Limit(&limit);
   if (SUCCEEDED(hr))
   {
      m_lblLimit.SetWindowText(GetValueAsString(limit));
   }
   else
   {
      m_lblLimit.SetWindowText(_T("Unlimited"));
   }

   eTickFilter tickFilter;
   hr = spRequest->get_TickFilter(&tickFilter);
   Common::ThrowOnCOMError(spRequest, hr);
   m_lblTicksFilter.SetWindowText(CHelper::TicksFilterToString(tickFilter));

   // Get error from Ticks collection and show it if exist
   ATL::CComPtr<ICQGError> spError;
   hr = m_spTicks->get_LastError(&spError);
   Common::ThrowOnCOMError(m_spTicks, hr);

   VARIANT_BOOL isValid;
   hr = m_spCQGCEL->IsValid(ATL::CComVariant(spError), &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      ATL::CComBSTR errDescription;
      hr = spError->get_Description(&errDescription);
      Common::ThrowOnCOMError(spError, hr);

      m_lblError.SetWindowText(ATL::CW2T(errDescription));
   }
}

void CTicksDlg::ShowOutputParams()
{
   // Get output parameters from Ticks collection and show them
   DATE startTimestamp;
   HRESULT hr = m_spTicks->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spTicks, hr);
   m_lblOutputStart.SetWindowText(GetValueAsString(startTimestamp, VT_DATE));

   DATE endTimestamp;
   hr = m_spTicks->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spTicks, hr);
   m_lblOutputEnd.SetWindowText(GetValueAsString(endTimestamp, VT_DATE));
}

void CTicksDlg::ClearDatesAndIndices()
{
   m_edStart.SetWindowText(_T(""));
   m_edEnd.SetWindowText(_T(""));

   static COleDateTime nullDate;

   m_dtpStart.SetTime(nullDate);
   m_dtpEnd.SetTime(nullDate);
}

void CTicksDlg::FilterTypeChanged()
{
   FillDatesAndIndices();

   int indicesShowStatus = (m_rdIndices.GetCheck() == BST_CHECKED ? SW_SHOW : SW_HIDE);
   m_edStart.ShowWindow(indicesShowStatus);
   m_spnStart.ShowWindow(indicesShowStatus);
   m_spnEnd.ShowWindow(indicesShowStatus);
   m_edEnd.ShowWindow(indicesShowStatus);

   int datesShowStatus = (m_rdIndices.GetCheck() == BST_CHECKED ? SW_HIDE : SW_SHOW);
   m_dtpStart.ShowWindow(datesShowStatus);
   m_dtpEnd.ShowWindow(datesShowStatus);
}

void CTicksDlg::FillDatesAndIndices()
{
   if (m_spTicks == NULL)
   {
      return;
   }

   eRequestStatus status;
   HRESULT hr = m_spTicks->get_Status(&status);
   Common::ThrowOnCOMError(m_spTicks, hr);

   if (status != rsSuccess)
   {
      return;
   }

   // Get start\end timestamps from Ticks collection and fill them into start\end time controls
   DATE startTimestamp;
   hr = m_spTicks->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spTicks, hr);
   m_dtpStart.SetTime(startTimestamp);

   DATE endTimestamp;
   hr = m_spTicks->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spTicks, hr);
   m_dtpEnd.SetTime(endTimestamp);

   // Fill start and end indices controls.
   m_edStart.SetValue(1);

   long count;
   hr = m_spTicks->get_Count(&count);
   Common::ThrowOnCOMError(m_spTicks, hr);

   m_edEnd.SetValue(count);
}

void CTicksDlg::EnableControls()
{
   bool isDataConnectionUp = false;

   VARIANT_BOOL isStarted;
   HRESULT hr = m_spCQGCEL->get_IsStarted(&isStarted);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isStarted)
   {
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      hr = m_spCQGCEL->get_Environment(&spEnvironment);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      eConnectionStatus dataConnectionStatus;
      hr = spEnvironment->get_DataConnectionStatus(&dataConnectionStatus);
      Common::ThrowOnCOMError(spEnvironment, hr);

      isDataConnectionUp = (dataConnectionStatus == csConnectionUp);
   }

   m_btnRequest.EnableWindow(isDataConnectionUp);

   bool isTicksSelected = (m_spTicks != NULL);

   m_btnRemove.EnableWindow(isTicksSelected && isDataConnectionUp);
   m_btnRemoveAll.EnableWindow(isTicksSelected && isDataConnectionUp);

   m_edTagName.EnableWindow(isTicksSelected);
   m_edTagValue.EnableWindow(isTicksSelected);

   bool isFilteringEnabled = false;
   if (isTicksSelected)
   {
      eRequestStatus status;
      hr = m_spTicks->get_Status(&status);
      Common::ThrowOnCOMError(m_spTicks, hr);

      isFilteringEnabled = (status == rsSuccess);

      if (isFilteringEnabled)
      { 
         ATL::CComPtr<ICQGTicksRequest> spRequest;
         HRESULT hr = m_spTicks->get_Request(&spRequest);
         Common::ThrowOnCOMError(m_spTicks, hr);

         long limit = 0;
         isFilteringEnabled = FAILED(spRequest->get_Limit(&limit));
      }
   }

   m_rdIndices.EnableWindow(isFilteringEnabled);
   m_rdRequestType.EnableWindow(isFilteringEnabled);

   m_edStart.EnableWindow(isFilteringEnabled);
   m_edEnd.EnableWindow(isFilteringEnabled);
   m_spnStart.EnableWindow(isFilteringEnabled);
   m_spnEnd.EnableWindow(isFilteringEnabled);
   m_dtpStart.EnableWindow(isFilteringEnabled);
   m_dtpEnd.EnableWindow(isFilteringEnabled);

   m_btnShow.EnableWindow(isFilteringEnabled);
}

CString CTicksDlg::GetValueAsString(
                        ATL::CComVariant value,
                        VARENUM valueType/*= VT_EMPTY*/,
                        bool withMilliseconds /*= false*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return Common::VariantToString(value, withMilliseconds);
   }
   return CHelper::m_N_A;
}
