// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "Ticks.h"
#include "TicksDlg.h"

STDMETHODIMP CTicksDlg::OnDataError(
                           LPDISPATCH /*obj*/,
                           BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CTicksDlg::OnDataConnectionStatusChanged(
                           eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      EnableControls();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTicksDlg::OnTicksResolved(
                           ICQGTicks* cqg_ticks,
                           ICQGError* /*cqg_error*/)
{
   try
   {
      // Get selected ticks ID
      ATL::CComBSTR selTicksID;
      HRESULT hr = m_spTicks->get_Id(&selTicksID);
      Common::ThrowOnCOMError(m_spTicks, hr);

      // Get added ticks ID
      ATL::CComBSTR ticksID;
      hr = cqg_ticks->get_Id(&ticksID);
      Common::ThrowOnCOMError<ICQGTicks>(cqg_ticks, hr);

      // Check if ticks resolved in the selected request
      if (selTicksID == ticksID)
      {
         ClearAllData();
         ShowAllData();

         EnableControls();
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTicksDlg::OnTicksAdded(
                           ICQGTicks* cqg_ticks, 
                           long added_ticks_count)
{
   try
   {
      if (m_spTicks == NULL)
      {
         return S_OK;
      }

      // Get selected ticks ID
      ATL::CComBSTR selTicksID;
      HRESULT hr = m_spTicks->get_Id(&selTicksID);
      Common::ThrowOnCOMError(m_spTicks, hr);

      // Get added ticks ID
      ATL::CComBSTR ticksID;
      hr = cqg_ticks->get_Id(&ticksID);
      Common::ThrowOnCOMError<ICQGTicks>(cqg_ticks, hr);

      // Check if records added in the selected request
      if (selTicksID == ticksID)
      {
         long count;
         HRESULT hr = cqg_ticks->get_Count(&count);
         Common::ThrowOnCOMError<ICQGTicks>(cqg_ticks, hr);

         m_lvLastTicks.SetItemCountEx(count, LVSICF_NOSCROLL);

         // Show records all and last chunk counts
         m_lblLastRecsCount.SetWindowText(GetValueAsString(added_ticks_count));
         m_lblAllRecsCount.SetWindowText(GetValueAsString(count));

         // Show output parameters
         ShowOutputParams();
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTicksDlg::OnTicksRemoved(ICQGTicks* cqg_ticks, long/* index*/)
{
   try
   {
      if (m_spTicks == NULL)
      {
         return S_OK;
      }

      // Get selected ticks ID
      ATL::CComBSTR selTicksID;
      HRESULT hr = m_spTicks->get_Id(&selTicksID);
      Common::ThrowOnCOMError(m_spTicks, hr);

      // Get added ticks ID
      ATL::CComBSTR ticksID;
      hr = cqg_ticks->get_Id(&ticksID);
      Common::ThrowOnCOMError<ICQGTicks>(cqg_ticks, hr);

      // Check if records added in the selected request
      if (selTicksID == ticksID)
      {
         long count;
         HRESULT hr = cqg_ticks->get_Count(&count);
         Common::ThrowOnCOMError<ICQGTicks>(cqg_ticks, hr);

         m_lvLastTicks.SetItemCountEx(count, LVSICF_NOSCROLL);

         // Show records all and last chunk counts
         m_lblAllRecsCount.SetWindowText(GetValueAsString(count));

         // Show output parameters
         ShowOutputParams();
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}
