// TimedBars.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include <cassert>
#include "Helper.h"

const CString CHelper::m_N_A = _T("N/A");
const CString CHelper::DateFormat = _T("dd/MM/yyyy HH:mm");

void CHelper::SetComboBoxItem(
                        CComboBox& comboBoxCtrl,
                        DWORD_PTR itemData)
{
   for (int i = 0; i < comboBoxCtrl.GetCount(); ++i)
   {
      if (comboBoxCtrl.GetItemData(i) == itemData)
      {
         comboBoxCtrl.SetCurSel(i);
         return;
      }
   }

   assert(!"Invalid item data");
}

int CHelper::CompareDates(
                        const COleDateTime& date1,
                        const COleDateTime& date2)
{
   // Check if dates are equal (seconds are ignored)
   if (date1.GetYear() == date2.GetYear() && date1.GetMonth() == date2.GetMonth() &&
      date1.GetDay() == date2.GetDay() && date1.GetHour() == date2.GetHour() &&
      date1.GetMinute() == date2.GetMinute())
   {
      return 0;
   }

   return (date1 < date2 ? -1 : 1);
}

CString CHelper::ConnectionStatusToString(
                        eConnectionStatus connectionStatus)
{
   CString connectionStatusStr;

   switch (connectionStatus)
   {
   case csConnectionUp:
      connectionStatusStr = _T("Connection Up");
      break;
   case csConnectionDown:
      connectionStatusStr = _T("Connection Down");
      break;
   case csConnectionDelayed:
      connectionStatusStr = _T("Connection Delayed");
      break;
   default:
      assert(!"Unknown connection status");
   }

   return connectionStatusStr;
}

CString CHelper::TicksRequestTypeToString(
                        eTicksRequestType ticksRequestType)
{
   CString ticksRequestTypeStr;

   switch (ticksRequestType)
   {
   case trtTimeRange:
      ticksRequestTypeStr = _T("TimeRange");
      break;
   case trtSinceTime:
      ticksRequestTypeStr = _T("SinceTime");
      break;
   case trtSinceTimeNotify:
      ticksRequestTypeStr = _T("SinceTimeNotify");
      break;
   case trtCurrentNotify:
      ticksRequestTypeStr = _T("CurrentNotify");
      break;
   default:
      assert(!"Unknown ticks request type");
   }

   return ticksRequestTypeStr;
}

CString CHelper::TickPriceTypeToString(
                        eTickPriceType tickPriceType)
{
   CString tickPriceTypeStr;

   switch (tickPriceType)
   {
   case tptTrade:
      tickPriceTypeStr = _T("Trade");
      break;
   case tptBid:
      tickPriceTypeStr = _T("Bid");
      break;
   case tptAsk:
      tickPriceTypeStr = _T("Ask");
      break;
   case tptSettlement:
      tickPriceTypeStr = _T("Settlement");
      break;
   default:
      assert(!"Unknown tick price type");
   }

   return tickPriceTypeStr;
}

CString CHelper::TickCorrectionTypeToString(
                        eTickCorrectionType tickCorrectionType)
{
   CString tickCorrectionTypeStr;

   switch (tickCorrectionType)
   {
   case tctRegular:
      tickCorrectionTypeStr = _T("Regular");
      break;
   case tctInsert:
      tickCorrectionTypeStr = _T("Insert");
      break;
   case tctDelete:
      tickCorrectionTypeStr = _T("Delete");
      break;
   case tctUpdate:
      tickCorrectionTypeStr = _T("Update");
      break;
   default:
      assert(!"Unknown tick correction type");
   }

   return tickCorrectionTypeStr;
}

CString CHelper::MarketStateToString(
                        eMarketState marketState)
{
   CString marketStateStr;

   switch (marketState)
   {
   case msFast:
      marketStateStr = _T("Fast");
      break;
   case msNormal:
      marketStateStr = _T("Normal");
      break;
   default:
      assert(!"Unknown market state");
   }

   return marketStateStr;
}

CString CHelper::RequestStatusToString(
                        eRequestStatus requestStatus)
{
   CString requestStatusStr;

   switch (requestStatus)
   {
   case rsInProgress:
      requestStatusStr = _T("InProgress");
      break;
   case rsSuccess:
      requestStatusStr = _T("Success");
      break;
   case rsCanceled:
      requestStatusStr = _T("Canceled");
      break;
   case rsFailed:
      requestStatusStr = _T("Failed");
      break;
   default:
      assert(!"Unknown request status");
   }

   return requestStatusStr;
}

CString CHelper::TicksFilterToString(
                        eTickFilter tickFilter)
{
   switch (tickFilter)
   {
      case tfAll:
         return _T("All");
      case tfAsk:
         return _T("Ask");
      case tfAskLow:
         return _T("AskLow");
      case tfBid:
         return _T("Bid");
      case tfBidHigh:
         return _T("BidHigh");
      case tfDefault:
         return _T("Default");
      case tfSettlement:
         return _T("Settlement");
      case tfTrade:
         return _T("Trade");
      default:
         assert(!"Unknown ticks filter");
         return _T(""); // Added for pass warning
   }
}