/*
* =====================================================================
*  Filename: Helper.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

class CHelper
{

private:

   /// \brief CHelper constructor.
   CHelper();

public:

   /// \brief Converts enum of connection status to appropriate string.
   /// \param connectionStatus - enum which will be converted
   /// \return String which describes ticks request type.
   static CString ConnectionStatusToString(
                     eConnectionStatus connectionStatus);

   /// \brief Converts enum of ticks request type to appropriate string.
   /// \param ticksRequestType - enum which will be converted
   /// \return 
   static CString TicksRequestTypeToString(
                     eTicksRequestType ticksRequestType);

   /// \brief Converts enum of tick price type to appropriate string.
   /// \param tickPriceType - enum which will be converted
   /// \return String which describes tick price type.
   static CString TickPriceTypeToString(
                     eTickPriceType tickPriceType);

   /// \brief Converts enum of tick correction type to appropriate string.
   /// \param tickCorrectionType - enum which will be converted
   /// \return String which describes tick correction type.
   static CString TickCorrectionTypeToString(
                     eTickCorrectionType tickCorrectionType);

   /// \brief Converts enum of market state to appropriate string.
   /// \param marketState - enum which will be converted
   /// \return String which describes market state.
   static CString MarketStateToString(
                     eMarketState marketState);

   /// \brief Converts enum of request status to appropriate string.
   /// \param requestStatus - enum which will be converted
   /// \return String which describes request status.
   static CString RequestStatusToString(
                     eRequestStatus requestStatus);

   /// \brief Converts enum of ticks filter type to appropriate string.
   /// \param tickFilter - enum value which will be converted
   /// \return String which describes ticks filter type.
   static CString TicksFilterToString(
                     eTickFilter tickFilter);

   /// \brief Compares two dates : COleDateTime comparison is not enough,
   /// \brief because we need to ignore seconds.
   /// \param date1 - first date
   /// \param date2 - second date
   /// \return Integer which means :
   /// \return -1 : first date is less than second,
   /// \return  0 : dates are equal,
   /// \return  1 : first date is greater than second.
   static int CompareDates(
                     const COleDateTime& date1,
                     const COleDateTime& date2);


public:

   /// \brief Selects an item of the specified combo box, which item data matches with given.
   /// \param comboBoxCtrl - combo box, which item will be selected
   /// \param itemData - item data, which must match with selected item data
   static void SetComboBoxItem(
      CComboBox& comboBoxCtrl,
      DWORD_PTR itemData);

public:

   /// \brief Custom date format.
   static const CString DateFormat;

   /// \brief Not available string.
   static const CString m_N_A;

};