// TicksDlg.cpp : implementation file
//

#include "stdafx.h"
#include <cassert>
#include "Common.h"
#include "Helper.h"
#include "Ticks.h"
#include "TicksDlg.h"
#include "TicksReqDlg.h"

const long CTicksDlg::m_MaxIdInterval = 500;
const long CTicksDlg::m_MaxDtInterval = 3;

CTicksDlg::CTicksDlg(CWnd* pParent /*=NULL*/)
   : CDialog(CTicksDlg::IDD, pParent)
{
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CTicksDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_LBL_DATA_CONNECTION, m_lblDataConnection);
   DDX_Control(pDX, IDC_BTN_REQUEST, m_btnRequest);
   DDX_Control(pDX, IDC_BTN_REMOVE, m_btnRemove);
   DDX_Control(pDX, IDC_BTN_REMOVE_ALL, m_btnRemoveAll);
   DDX_Control(pDX, IDC_LBL_STATUS, m_lblStatus);
   DDX_Control(pDX, IDC_LBL_SYMBOL, m_lblSymbol);
   DDX_Control(pDX, IDC_LBL_TYPE, m_lblType);
   DDX_Control(pDX, IDC_LBL_RANGE_START, m_lblRangeStart);
   DDX_Control(pDX, IDC_LBL_RANGE_END, m_lblRangeEnd);
   DDX_Control(pDX, IDC_LBL_SESSIONS_FILTER, m_lblSessionsFilter);
   DDX_Control(pDX, IDC_LBL_TICKS_FILTER, m_lblTicksFilter);
   DDX_Control(pDX, IDC_LBL_TICKS_LIMIT, m_lblLimit);
   DDX_Control(pDX, IDC_ED_START, m_edStart);
   DDX_Control(pDX, IDC_ED_END, m_edEnd);
   DDX_Control(pDX, IDC_BTN_SHOW, m_btnShow);
   DDX_Control(pDX, IDC_LSV_LAST_TICKS, m_lvLastTicks);
   DDX_Control(pDX, IDC_LSV_TICKS, m_lvTicks);
   DDX_Control(pDX, IDC_ED_TAG_NAME, m_edTagName);
   DDX_Control(pDX, IDC_ED_TAG_VALUE, m_edTagValue);
   DDX_Control(pDX, IDC_LBL_OUTPUT_START, m_lblOutputStart);
   DDX_Control(pDX, IDC_LBL_OUTPUT_END, m_lblOutputEnd);
   DDX_Control(pDX, IDC_LBL_ERROR, m_lblError);
   DDX_Control(pDX, IDC_LBL_RECS_COUNT, m_lblFltrRecsCount);
   DDX_Control(pDX, IDC_LBL_LAST_RECS_COUNT, m_lblLastRecsCount);
   DDX_Control(pDX, IDC_CMB_TICKS, m_cmbTicks);
   DDX_Control(pDX, IDC_DTP_END, m_dtpEnd);
   DDX_Control(pDX, IDC_DTP_START, m_dtpStart);
   DDX_Control(pDX, IDC_SPN_START, m_spnStart);
   DDX_Control(pDX, IDC_SPN_END, m_spnEnd);
   DDX_Control(pDX, IDC_RD_DATES, m_rdRequestType);
   DDX_Control(pDX, IDC_RD_INDICES, m_rdIndices);
   DDX_Control(pDX, IDC_LBL_ALL_RECS_COUNT, m_lblAllRecsCount);
}

BEGIN_MESSAGE_MAP(CTicksDlg, CDialog)
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_WM_DESTROY()
   ON_BN_CLICKED(IDC_BTN_REMOVE, OnBnClickedBtnRemove)
   ON_BN_CLICKED(IDC_BTN_REMOVE_ALL, OnBnClickedBtnRemoveAll)
   ON_CBN_SELCHANGE(IDC_CMB_TICKS, OnCbnSelChangeCmbTicks)
   ON_BN_CLICKED(IDC_BTN_REQUEST, OnBnClickedBtnRequest)
   ON_BN_CLICKED(IDC_BTN_SHOW, OnBnClickedBtnShow)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_START, OnDeltaposSpnStart)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_END, OnDeltaposSpnEnd)
   ON_BN_CLICKED(IDC_RD_INDICES, OnBnClickedRdIndices)
   ON_BN_CLICKED(IDC_RD_DATES, OnBnClickedRdDate)
   ON_NOTIFY(LVN_GETDISPINFO, IDC_LSV_LAST_TICKS, OnGetDispInfoTicksList)
   ON_NOTIFY(LVN_GETDISPINFO, IDC_LSV_TICKS, OnGetDispInfoFilteredTicksList)
END_MESSAGE_MAP()

CString CTicksDlg::getTickColumnText(int tickRowNumber, int tickColumnNumber)
{
   try
   {
      // Get specified Tick from Ticks collection by its index
      ATL::CComPtr<ICQGTick> spTick;
      Common::ThrowOnCOMError(m_spTicks, m_spTicks->get_Item(tickRowNumber, &spTick));

      //Which column?
      switch (tickColumnNumber)
      {
      case tcIndex: return GetValueAsString(tickRowNumber + 1);
      case tcTimestamp:
      {
         DATE timestamp;
         Common::ThrowOnCOMError(spTick, spTick->get_Timestamp(&timestamp));
         return GetValueAsString(timestamp, VT_DATE, true);
      }
      case tcPrice:
      {
         double price;
         Common::ThrowOnCOMError(spTick, spTick->get_Price(&price));
         return GetValueAsString(price);
      }
      case tcVolume:
      {
         long volume;
         Common::ThrowOnCOMError(spTick, spTick->get_Volume(&volume));
         return GetValueAsString(volume);
      }
      case tcPriceType:
      {
         eTickPriceType priceType;
         Common::ThrowOnCOMError(spTick, spTick->get_PriceType(&priceType));
         return CHelper::TickPriceTypeToString(priceType);
      }
      case tcCorrectionType:
      {
         eTickCorrectionType correctionType;
         Common::ThrowOnCOMError(spTick, spTick->get_CorrectionType(&correctionType));
         return CHelper::TickCorrectionTypeToString(correctionType);
      }
      case tcMarketState:
      {
         eMarketState marketState;
         Common::ThrowOnCOMError(spTick, spTick->get_MarketState(&marketState));
         return CHelper::MarketStateToString(marketState);
      }
      case tcSalesConditionCode:
      {
         long salesConditionCode;
         Common::ThrowOnCOMError(spTick, spTick->get_SalesConditionCode(&salesConditionCode));
         return GetValueAsString(salesConditionCode);
      }
      case tcSalesConditionLabel:
      {
         CComBSTR salesConditionLabel;
         Common::ThrowOnCOMError(spTick, spTick->get_SalesConditionLabel(&salesConditionLabel));
         return CString(ATL::CW2T(salesConditionLabel));
      }
      case tcContributorIdCode:
      {
         long contributorIdCode;
         Common::ThrowOnCOMError(spTick, spTick->get_ContributorIdCode(&contributorIdCode));
         return GetValueAsString(contributorIdCode);
      }
      case tcContributorId:
      {
         CComBSTR contributorId;
         Common::ThrowOnCOMError(spTick, spTick->get_ContributorId(&contributorId));
         return CString(ATL::CW2T(contributorId));
      }
      default:
         assert(false); // unexpected column
         return CString("unexpected column");
      }
   }
   catch (std::runtime_error& exception)
   {
      return CString(exception.what());
   }
}

void CTicksDlg::OnGetDispInfoTicksList(NMHDR* pNMHDR, LRESULT* pResult)
{
   LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;
   LV_ITEM* pItem = &(pDispInfo)->item;

   if (pItem->mask & LVIF_TEXT)
   {
      lstrcpyn(pItem->pszText, getTickColumnText(pItem->iItem, pItem->iSubItem), pItem->cchTextMax);
   }

   *pResult = 0;
}

void CTicksDlg::OnGetDispInfoFilteredTicksList(NMHDR* pNMHDR, LRESULT* pResult)
{
   LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;
   LV_ITEM* pItem = &(pDispInfo)->item;

   if (pItem->mask & LVIF_TEXT)
   {
      lstrcpyn(pItem->pszText, getTickColumnText(pItem->iItem + m_filteredIndexStart - 1, pItem->iSubItem), pItem->cchTextMax);
   }

   *pResult = 0;
}

BOOL CTicksDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      // Set the icon for this dialog.  The framework does this automatically
      //  when the application's main window is not a dialog
      SetIcon(m_hIcon, TRUE);       // Set big icon
      SetIcon(m_hIcon, FALSE);      // Set small icon

      InitializeCQGCEL();
      InitializeControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
}

void CTicksDlg::OnPaint() 
{
   if (IsIconic())
   {
      CPaintDC dc(this); // device context for painting

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      // Center icon in client rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      // Draw the icon
      dc.DrawIcon(x, y, m_hIcon);
   }
   else
   {
      CDialog::OnPaint();
   }
}

HCURSOR CTicksDlg::OnQueryDragIcon()
{
   return static_cast<HCURSOR>(m_hIcon);
}

void CTicksDlg::OnOK()
{
   // If enter clicked when one of the tag edit boxes is active
   // then call corresponding function
   CWnd* pWnd = GetFocus();

   if (pWnd == &m_edTagName)
   {
      OnEnterEdTagName();
   }
   else if (pWnd == &m_edTagValue)
   {
      OnEnterEdTagValue();
   }
}

void CTicksDlg::OnDestroy()
{
   try
   {
      ClearAllData();

      HRESULT hr = ICQGCELDispEventImpl::DispEventUnadvise(m_spCQGCEL);
      assert(SUCCEEDED(hr));

      hr = m_spCQGCEL->Shutdown();
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}

void CTicksDlg::OnBnClickedBtnRemove()
{
   try
   {
      // Remove selected Ticks from the collection
      HRESULT hr = m_spCQGCEL->RemoveTicks(m_spTicks);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      // Remove item from combo box
      int row = m_cmbTicks.GetCurSel();
      int count = m_cmbTicks.DeleteString(row);

      if (count == 0)
      {
         ClearAllData();
         m_spTicks = NULL;

         // Clear combo box
         m_cmbTicks.ResetContent();
      }
      else
      {
         m_cmbTicks.SetCurSel((row >= count ? (count - 1) : row));
         OnCbnSelChangeCmbTicks();
      }

      EnableControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTicksDlg::OnBnClickedBtnRemoveAll()
{
   try
   {
      // Clear combo box
      m_cmbTicks.ResetContent();

      // Remove all Ticks from CQGCEL
      HRESULT hr = m_spCQGCEL->RemoveAllTicks();
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      m_spTicks = NULL;

      ClearAllData();

      EnableControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTicksDlg::OnCbnSelChangeCmbTicks()
{
   try
   {
      ClearAllData();

      if (m_cmbTicks.GetCount() > 0)
      {
         // Get GUID from combo box
         CString id;
         m_cmbTicks.GetWindowText(id);

         // Get item from CQGCEL by GUID
         ATL::CComPtr<ICQGTicksCollection> spAllTicks;
         HRESULT hr = m_spCQGCEL->get_AllTicks(&spAllTicks);
         Common::ThrowOnCOMError(m_spCQGCEL, hr);

         m_spTicks.Release();
         hr = spAllTicks->get_ItemById(ATL::CT2W(id), &m_spTicks);
         Common::ThrowOnCOMError(spAllTicks, hr);

         // Show corresponding items all data
         ShowAllData();

         // Initialize parameters for filtering
         m_rdIndices.SetCheck(BST_CHECKED);
         OnBnClickedRdIndices();
      }

      EnableControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTicksDlg::OnBnClickedBtnRequest()
{
   try
   {
      // Create empty request
      ATL::CComPtr<ICQGTicksRequest> spRequest;
      HRESULT hr = m_spCQGCEL->CreateTicksRequest(&spRequest);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      spRequest->put_SessionsFilter(31);  // In other words "all sessions". 31 is represented by "11111" bits, where each bit designates a session in "Chart Preference"/"Sessions".

      // Get line time from CQGCEL->Environment
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      hr = m_spCQGCEL->get_Environment(&spEnvironment);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      DATE lineTime;
      hr = spEnvironment->get_LineTime(&lineTime);
      Common::ThrowOnCOMError(spEnvironment, hr);

      // Open form to fill up request data
      CTicksReqDlg dlg(spRequest, lineTime);
      if (dlg.DoModal() == IDOK)
      {
         // Send request to CQGCEL
         ATL::CComPtr<ICQGTicks> spTicks;
         hr = m_spCQGCEL->RequestTicks(spRequest, &spTicks);
         Common::ThrowOnCOMError(m_spCQGCEL, hr);

         // Add ID to the requests combo box
         ATL::CComBSTR ticksID;
         hr = spTicks->get_Id(&ticksID);
         Common::ThrowOnCOMError(spTicks, hr);

         int row = m_cmbTicks.AddString(ATL::CW2T(ticksID));
         m_cmbTicks.SetCurSel(row);
         OnCbnSelChangeCmbTicks();
      }

      EnableControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTicksDlg::OnBnClickedBtnShow()
{
   try
   {
      // Clear records and records count
      m_lvTicks.SetItemCountEx(0);
      m_lblFltrRecsCount.SetWindowText(_T(""));

      if (m_rdIndices.GetCheck())
      {
         m_filteredIndexStart = m_edStart.GetValue();
         m_filteredIndexEnd = m_edEnd.GetValue();

         // Check for valid indices
         if (m_filteredIndexEnd < m_filteredIndexStart)
         {
            AfxMessageBox(_T("Start index must be smaller than end index."), MB_ICONWARNING);
            return;
         }

         if (m_filteredIndexStart < 1)
         {
            AfxMessageBox(_T("Start index must be greater than zero."), MB_ICONWARNING);
            return;
         }

         long count;
         HRESULT hr = m_spTicks->get_Count(&count);
         Common::ThrowOnCOMError<ICQGTicks>(m_spTicks, hr);

         if (m_filteredIndexEnd > count)
         {
            AfxMessageBox(_T("End index must be smaller than records count."), MB_ICONWARNING);
            return;
         }

         if (m_filteredIndexEnd - m_filteredIndexStart >= m_MaxIdInterval)
         {
            CString msgText;
            msgText.Format(_T("Range must indicate to no more than %d records."), m_MaxIdInterval);
            AfxMessageBox(msgText, MB_ICONWARNING);
            return;
         }
      }
      else
      {
         // Get start and end dates and remove seconds from them: to retrieve same data for same requests
         COleDateTime startTime;
         m_dtpStart.GetTime(startTime);

         COleDateTime startTimeNS(startTime.GetYear(), startTime.GetMonth(), startTime.GetDay(), 
                                startTime.GetHour(), startTime.GetMinute(), 0);

         COleDateTime endTime;
         m_dtpEnd.GetTime(endTime);

         COleDateTime endTimeNS(endTime.GetYear(), endTime.GetMonth(), endTime.GetDay(), 
                               endTime.GetHour(), endTime.GetMinute(), 0);

         // Add one minute to end date
         endTimeNS += COleDateTimeSpan(0, 0, 1, 0);

         // Check for valid date range
         if (CHelper::CompareDates(startTimeNS, endTimeNS) >= 0)
         {
            AfxMessageBox(_T("Start time must be earlier than end time."), MB_ICONWARNING);
            return;
         }

         if (CHelper::CompareDates(endTimeNS, startTimeNS + COleDateTimeSpan(0, 0, m_MaxDtInterval, 0)) > 0)
         {
            CString msgText;
            msgText.Format(_T("Range must indicate to no more than %d minutes."), m_MaxDtInterval);
            AfxMessageBox(msgText, MB_ICONWARNING);
            return;
         }

         // Get start and end indices for specified dates
         HRESULT hr = m_spTicks->IndicesForDates(startTimeNS.m_dt, endTimeNS.m_dt, &m_filteredIndexStart, &m_filteredIndexEnd);
         Common::ThrowOnCOMError(m_spTicks, hr);

         // If no record exists, both start and end indices will be invalid
         VARIANT_BOOL isValid;
         hr = m_spCQGCEL->IsValid(ATL::CComVariant(m_filteredIndexStart), &isValid);
         Common::ThrowOnCOMError(m_spCQGCEL, hr);

         if (!isValid)
         {
            AfxMessageBox(_T("There aren't any records in this time range"), MB_ICONINFORMATION);
            return;
         }

         // IndicesForDates function returns 0 based indices, so expanding them by 1
         m_filteredIndexStart += 1;
         m_filteredIndexEnd += 1;
      }

      // Show filtered records and its' count
      auto filteredItemsCount = m_filteredIndexEnd - m_filteredIndexStart + 1;
      m_lblFltrRecsCount.SetWindowText(GetValueAsString(filteredItemsCount));
      m_lvTicks.SetItemCountEx(filteredItemsCount, LVSICF_NOSCROLL);
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTicksDlg::OnDeltaposSpnStart(NMHDR *pNMHDR, LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edStart.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CTicksDlg::OnDeltaposSpnEnd(NMHDR *pNMHDR, LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edEnd.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}
void CTicksDlg::OnBnClickedRdIndices()
{
   try
   {
      FilterTypeChanged();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTicksDlg::OnBnClickedRdDate()
{
   try
   {
      FilterTypeChanged();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTicksDlg::OnEnterEdTagName()
{
   try
   {
      CString tagName;
      m_edTagName.GetWindowText(tagName);

      // If there is no tag name specified, then do nothing
      if (tagName.IsEmpty()) 
      {
         return;
      }

      ATL::CComVariant tagValue;
      HRESULT hr = m_spTicks->get_Tag(ATL::CT2W(tagName), &tagValue);
      Common::ThrowOnCOMError(m_spTicks, hr);

      // If there is no tag with specified tag name, tag value will be VT_EMPTY
      if (tagValue.vt != VT_EMPTY)
      {
         m_edTagValue.SetWindowText(GetValueAsString(tagValue));
      }
      else
      {
         m_edTagValue.SetWindowText(_T(""));
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTicksDlg::OnEnterEdTagValue()
{
   try
   {
      CString tagName;
      m_edTagName.GetWindowText(tagName);

      // If there is no tag name specified, then do nothing
      if (tagName.IsEmpty())
      {
         return;
      }

      CString tagValue;
      m_edTagValue.GetWindowText(tagValue);

      // If there is no tag value specified, then remove tag with specified tag name
      HRESULT hr;
      if (tagValue.IsEmpty())
      {
         hr = m_spTicks->put_Tag(ATL::CT2W(tagName), ATL::CComVariant());
      }
      else
      {
         hr = m_spTicks->put_Tag(ATL::CT2W(tagName), ATL::CComVariant(tagValue));
      }
      Common::ThrowOnCOMError(m_spTicks, hr);

      m_edTagName.SetWindowText(_T(""));
      m_edTagValue.SetWindowText(_T(""));

      m_edTagName.SetFocus();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}
