/*
* =====================================================================
*  Filename: TicksDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

#include "afxwin.h"
#include "afxcmn.h"
#include "afxdtctl.h"
#include "NumEdit.h"

class CTicksDlg;

/**
\typedef ATL::IDispEventImpl<1, COrderPlacingDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CTicksDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

// CTicksDlg dialog
class CTicksDlg : public CDialog, public ICQGCELDispEventImpl
{

public:

   explicit CTicksDlg(CWnd* pParent = NULL); 

   /// \enum Dialog Data
   enum { IDD = IDD_TICKS_DIALOG };

protected:

   /// \brief Dialog icon.
   HICON m_hIcon;

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
                  CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter
   /// \brief and catch Enters in tag name and tag value edit controls.
   virtual void OnOK();

protected:

   /// \brief Prepares the container for painting.
   afx_msg void OnPaint();

   /// \brief The system calls this function to obtain the cursor to
   /// \brief display while the user drags the minimized window.
   afx_msg HCURSOR OnQueryDragIcon();

   /// \brief Used to unadvise from CQGCEL events and shutdown it.
   afx_msg void OnDestroy();

   /// \brief Changes selected ticks object, and shows its data.
   afx_msg void OnCbnSelChangeCmbTicks();

   /// \brief Opens request settings form to fill up request parameters.
   afx_msg void OnBnClickedBtnRequest();

   /// \brief Removes selected ticks from CQGCEL and combo box control.
   afx_msg void OnBnClickedBtnRemove();

   /// \brief Removes all ticks from CQGCEL and combo box control.
   afx_msg void OnBnClickedBtnRemoveAll();

   /// \brief Changes filtering type to indices.
   afx_msg void OnBnClickedRdIndices();

   /// \brief Changes filtering type to dates.
   afx_msg void OnBnClickedRdDate();

   /// \brief Used to increment or decrement start range value for filtering.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnStart(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Used to increment or decrement end range value for filtering.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnEnd(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Shows filtered ticks.
   afx_msg void OnBnClickedBtnShow();

   afx_msg void OnGetDispInfoTicksList(NMHDR* pNMHDR, LRESULT* pResult);
   afx_msg void OnGetDispInfoFilteredTicksList(NMHDR* pNMHDR, LRESULT* pResult);

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CTicksDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000000a, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000003, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000016, OnTicksResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000017, OnTicksAdded)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000077, OnTicksRemoved)
   END_SINK_MAP()

protected:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - string describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                  LPDISPATCH obj,
                  BSTR errorDescription);

   /// \brief Fired when some changes occurred in the connection with the CQG data server.
   /// \param newStatus - new connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when the collection of ticks (CQGTicks) is resolved
   /// \brief or when some error occurred during ticks request processing
   /// \param cqg_ticks - reference to resolved CQGTicks
   /// \param cqg_error - CQGError object describing last error occurred during processing ticks request.
   /// \return S_OK
   STDMETHOD(OnTicksResolved)(
                  ICQGTicks* cqg_ticks,
                  ICQGError* cqg_error);

   /// \brief Fired when CQGTick items are added to ticks collection.
   /// \param cqg_ticks - reference to changed CQGTicks
   /// \param added_ticks_count - specifies the number of added ticks towards the end of CQGTicks
   /// \return S_OK
   STDMETHOD(OnTicksAdded)(
                  ICQGTicks* cqg_ticks, 
                  long added_ticks_count);

   STDMETHOD(OnTicksRemoved)(ICQGTicks* cqg_ticks, long index);

private:

   /// \enum eTickssColumn. Represents ticks columns in the list.
   enum eTicksColumn;

private:

   /// \brief Initializes CQGCEL object, and starts the CQGCEL.
   void InitializeCQGCEL();

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes specified list control's columns.
   /// \param listCtrl - list control which columns will be configured
   static void InitializeList(
                  CListCtrl& listCtrl);

   /// \brief Clears all data(parameters\outputs\records) from the form.
   void ClearAllData();

   /// \brief Clears request parameters from the form.
   void ClearRequestParams();

   /// \brief Clears output parameters from the form.
   void ClearOutputParams();
   
   /// \brief Clears request tags.
   void ClearTags();

   /// \brief Shows all data(parameters\outputs\records) connected
   /// \brief with selected request.
   void ShowAllData();

   /// \brief Shows selected request parameters on the form.
   void ShowRequestParams();

   /// \brief Shows selected request outputs on the form.
   void ShowOutputParams();

   /// \brief Clears dates and indices, which are used for filtering.
   void ClearDatesAndIndices();

   /// \brief Shows/Hides controls for filtering.
   void FilterTypeChanged();

   /// \brief Fills dates and indices for filtering.
   void FillDatesAndIndices();

   /// \brief Used for getting CQGCEL properties values string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - variant value
   /// \param valueType - variant type
   /// \param withMilliseconds - add milliseconds to the string, only for VT_DATE
   /// \return String representation of the value.
   CString GetValueAsString(
      ATL::CComVariant value,
      VARENUM valueType = VT_EMPTY,
      bool withMilliseconds = false) const;

   /// \brief Enables/disables controls on the form based on connection status and other parameters.
   void EnableControls();

   /// \brief Shows tag value for specified name.
   void OnEnterEdTagName();

   /// \brief Sets tag value for specified name.
   void OnEnterEdTagValue();

   CString getTickColumnText(int tickRowNumber, int tickColumnNumber);

private:

   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Selected ticks object.
   ATL::CComPtr<ICQGTicks> m_spTicks;

   /// \brief Max indices interval for filtering.
   static const long m_MaxIdInterval;

   /// \brief Max dates interval in minutes for filtering.
   static const long m_MaxDtInterval;

private:

   /// \brief Active ticks combo box control
   CComboBox m_cmbTicks;

   /// \brief Data connection indicator label control.
   CStatic m_lblDataConnection;

   /// \brief Request ticks button control.
   CButton m_btnRequest;

   /// \brief Remove selected ticks button control.
   CButton m_btnRemove;

   /// \brief Remove all ticks button control.
   CButton m_btnRemoveAll;

   /// \brief Last error label control.
   CStatic m_lblError;

   /// \brief Request status label control.
   CStatic m_lblStatus;

   /// \brief Request symbol label control.
   CStatic m_lblSymbol;

   /// \brief Request type label control.
   CStatic m_lblType;

   /// \brief Request range start label control.
   CStatic m_lblRangeStart;

   /// \brief Request range end label control.
   CStatic m_lblRangeEnd;

   /// \brief Request sessions filter label control.
   CStatic m_lblSessionsFilter;

   /// \brief Request ticks filter label control.
   CStatic m_lblTicksFilter;

   /// \brief Request ticks limit label control.
   CStatic m_lblLimit;

   /// \brief Request ticks button control.
   CButton m_btnShow;

   /// \brief Tag name edit control.
   CEdit m_edTagName;

   /// \brief Tag value edit control.
   CEdit m_edTagValue;

   /// \brief Output range start label control.
   CStatic m_lblOutputStart;

   /// \brief Output range end label control.
   CStatic m_lblOutputEnd;

   /// \brief All records count label control.
   CStatic m_lblAllRecsCount;

   /// \brief Filtered records count label control.
   CStatic m_lblFltrRecsCount;

   /// \brief Last chunk records count label control.
   CStatic m_lblLastRecsCount;

   /// \brief Last chunk of ticks list view control.
   CListCtrl m_lvLastTicks;

   /// \brief Filtering by dates radio button control.
   CButton m_rdRequestType;

   /// \brief Filtering by indices radio button control.
   CButton m_rdIndices;

   /// \brief Filtering by indices range start edit control.
   CNumEdit m_edStart;

   /// \brief Filtering by indices range start spin button control.
   CSpinButtonCtrl m_spnStart;

   /// \brief Filtering by indices range end edit control.
   CNumEdit m_edEnd;

   /// \brief Filtering by indices range end spin button control.
   CSpinButtonCtrl m_spnEnd;

   /// \brief Filtering by date range start date time control.
   CDateTimeCtrl m_dtpEnd;

   /// \brief Filtering by date range end date time control.
   CDateTimeCtrl m_dtpStart;

   /// \brief Filtered ticks list view control.
   CListCtrl m_lvTicks;

   long m_filteredIndexStart;
   long m_filteredIndexEnd;
};

enum CTicksDlg::eTicksColumn
{
   tcIndex,
   tcTimestamp,
   tcPrice,
   tcVolume,
   tcPriceType,
   tcCorrectionType,
   tcMarketState,
   tcSalesConditionCode,
   tcSalesConditionLabel,
   tcContributorIdCode,
   tcContributorId
};
