// TicksReqDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "Ticks.h"
#include "TicksReqDlg.h"

CTicksReqDlg::CTicksReqDlg(
                     const ATL::CComPtr<ICQGTicksRequest>& spRequest,
                     DATE lineTime,
                     CWnd* pParent /*=NULL*/)
                     : CDialog(CTicksReqDlg::IDD, pParent),
                       m_spRequest(spRequest),
                       m_LineTime(lineTime),
                       m_ReqType(0),
                       m_useTicksLimit(FALSE)
{
}

void CTicksReqDlg::DoDataExchange(
                     CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_RD_TIME_RANGE, m_rdRequestType);
   DDX_Control(pDX, IDC_ED_SESSIONS_FILTER, m_edSessionsFilter);
   DDX_Control(pDX, IDC_ED_SYMBOL, m_edSymbol);
   DDX_Control(pDX, IDC_DTP_START, m_dtpStart);
   DDX_Control(pDX, IDC_DTP_END, m_dtpEnd);
   DDX_Radio(pDX, IDC_RD_TIME_RANGE, m_ReqType);
   DDX_Control(pDX, IDC_CMB_TICK_FILTER, m_cmbTickFilter);
   DDX_Control(pDX, IDC_ED_TICKS_LIMIT, m_edTicksLimit);
   DDX_Check(pDX, IDC_CHECK_USE_LIMIT, m_useTicksLimit);
}

BOOL CTicksReqDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      InitializeControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;
}

BEGIN_MESSAGE_MAP(CTicksReqDlg, CDialog)
   ON_BN_CLICKED(IDOK, OnBnClickedOk)
   ON_BN_CLICKED(IDC_RD_TIME_RANGE, OnBnClickedRdTimeRange)
   ON_BN_CLICKED(IDC_RD_SINCE_TIME, OnBnClickedRdSinceTime)
   ON_BN_CLICKED(IDC_RD_SINCE_TIME_NOTIFY, OnBnClickedRdSinceTimeNotify)
   ON_BN_CLICKED(IDC_RD_CURRENT_NOTIFY, OnBnClickedRdCurrentNotify)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_SESSIONS_FILTER, OnDeltaposSpnSessionsFilter)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_TICKS_LIMIT, OnDeltaposSpnTicksLimit)
   ON_BN_CLICKED(IDC_CHECK_USE_LIMIT, &CTicksReqDlg::OnBnClickedCheckUseLimit)
END_MESSAGE_MAP()

void CTicksReqDlg::InitializeControls()
{
   InitializeTickFilterCombo();

   m_rdRequestType.SetCheck(BST_CHECKED);
   OnBnClickedRdTimeRange();
   OnBnClickedCheckUseLimit();

   // Set default parameters
   SetDefaultParameters();

   m_dtpStart.SetFormat(CHelper::DateFormat);
   m_dtpEnd.SetFormat(CHelper::DateFormat);
}

void CTicksReqDlg::SetDefaultParameters()
{
   // Get sessions filter default value from CQGTicksRequest
   long defSessionsFilter;
   HRESULT hr = m_spRequest->get_SessionsFilter(&defSessionsFilter);
   Common::ThrowOnCOMError(m_spRequest, hr);

   // Set sessions filter default value
   m_edSessionsFilter.SetValue(defSessionsFilter);

   m_edTicksLimit.SetValue(7);

   // Get ticks filter default value from CQGTicksRequest
   eTickFilter defTickFilter;
   hr = m_spRequest->get_TickFilter(&defTickFilter);
   Common::ThrowOnCOMError(m_spRequest, hr);

   // Set ticks filter default value
   CHelper::SetComboBoxItem(m_cmbTickFilter, (DWORD_PTR)defTickFilter);
}

void CTicksReqDlg::OnBnClickedOk()
{
   // Set request parameters
   eTicksRequestType type = GetTicksReqType();
   HRESULT hr = m_spRequest->put_Type(type);
   Common::ThrowOnCOMError(m_spRequest, hr);

   CString symbolStr;
   m_edSymbol.GetWindowText(symbolStr);
   ATL::CComBSTR symbol(symbolStr);
   hr = m_spRequest->put_Symbol(symbol);
   Common::ThrowOnCOMError(m_spRequest, hr);

   if (type != trtCurrentNotify)
   {
      COleDateTime startTime;
      m_dtpStart.GetTime(startTime);
      hr = m_spRequest->put_RangeStart(startTime);
      Common::ThrowOnCOMError(m_spRequest, hr);
   }

   if (type == trtTimeRange)
   {
      COleDateTime endTime;
      m_dtpEnd.GetTime(endTime);
      hr = m_spRequest->put_RangeEnd(endTime);
      Common::ThrowOnCOMError(m_spRequest, hr);
   }

   hr = m_spRequest->put_SessionsFilter(m_edSessionsFilter.GetValue());
   Common::ThrowOnCOMError(m_spRequest, hr);

   hr = m_spRequest->put_TickFilter(static_cast<eTickFilter>(
                     m_cmbTickFilter.GetItemData(m_cmbTickFilter.GetCurSel())));

   if (m_useTicksLimit)
   {
      hr = m_spRequest->put_Limit(m_edTicksLimit.GetValue());
   }

   Common::ThrowOnCOMError(m_spRequest, hr);

   // Close request window
   OnOK();
}

eTicksRequestType CTicksReqDlg::GetTicksReqType()
{
   UpdateData(TRUE);

   eTicksRequestType reqType;
   switch (m_ReqType)
   {
   case 0:
   	reqType = trtTimeRange;
      break;
   case 1:
      reqType = trtSinceTime;
      break;
   case 2:
      reqType = trtSinceTimeNotify;
      break;
   case 3:
      reqType = trtCurrentNotify;
      break;
   default:
      throw std::invalid_argument("Unknown request type");
   }

   return reqType;
}

void CTicksReqDlg::OnBnClickedRdTimeRange()
{
   RequestTypeChanged();
}

void CTicksReqDlg::OnBnClickedRdSinceTime()
{
   RequestTypeChanged();
}

void CTicksReqDlg::OnBnClickedRdSinceTimeNotify()
{
   RequestTypeChanged();
}

void CTicksReqDlg::OnBnClickedRdCurrentNotify()
{
   RequestTypeChanged();
}

void CTicksReqDlg::RequestTypeChanged()
{
   // Get request type
   eTicksRequestType type = GetTicksReqType();

   m_dtpStart.EnableWindow(type != trtCurrentNotify);
   m_dtpEnd.EnableWindow(type == trtTimeRange);

   // Minus one hour from start time
   COleDateTime startTime = m_LineTime - COleDateTimeSpan(0, 1, 0, 0);

   m_dtpStart.SetTime(startTime);
   m_dtpEnd.SetTime(m_LineTime);
}

void CTicksReqDlg::OnDeltaposSpnSessionsFilter(
                     NMHDR *pNMHDR,
                     LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edSessionsFilter.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CTicksReqDlg::OnDeltaposSpnTicksLimit(
   NMHDR *pNMHDR,
   LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edTicksLimit.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CTicksReqDlg::InitializeTickFilterCombo()
{
	int row = m_cmbTickFilter.AddString(CHelper::TicksFilterToString(tfAll));
	m_cmbTickFilter.SetItemData(row, tfAll);

	row = m_cmbTickFilter.AddString(CHelper::TicksFilterToString(tfAsk));
	m_cmbTickFilter.SetItemData(row, tfAsk);

	row = m_cmbTickFilter.AddString(CHelper::TicksFilterToString(tfAskLow));
	m_cmbTickFilter.SetItemData(row, tfAskLow);

	row = m_cmbTickFilter.AddString(CHelper::TicksFilterToString(tfBid));
	m_cmbTickFilter.SetItemData(row, tfBid);

	row = m_cmbTickFilter.AddString(CHelper::TicksFilterToString(tfBidHigh));
	m_cmbTickFilter.SetItemData(row, tfBidHigh);

	row = m_cmbTickFilter.AddString(CHelper::TicksFilterToString(tfDefault));
	m_cmbTickFilter.SetItemData(row, tfDefault);

	row = m_cmbTickFilter.AddString(CHelper::TicksFilterToString(tfSettlement));
	m_cmbTickFilter.SetItemData(row, tfSettlement);

	row = m_cmbTickFilter.AddString(CHelper::TicksFilterToString(tfTrade));
	m_cmbTickFilter.SetItemData(row, tfTrade);
}



void CTicksReqDlg::OnBnClickedCheckUseLimit()
{
   UpdateData(TRUE);

   m_edTicksLimit.EnableWindow(!!m_useTicksLimit);
   this->GetDlgItem(IDC_SPN_TICKS_LIMIT)->EnableWindow(!!m_useTicksLimit);
}
