/*
* =====================================================================
*  Filename: TicksReqDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

#include "afxwin.h"
#include "afxdtctl.h"
#include "NumEdit.h"

// CTicksReqDlg dialog
class CTicksReqDlg : public CDialog
{

public:

   CTicksReqDlg(
               const ATL::CComPtr<ICQGTicksRequest>& spRequest,
               DATE lineTime,
               CWnd* pParent = NULL);

   /// \enum Dialog Data.
   enum { IDD = IDD_TICKS_REQ_DIALOG };

protected:

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
               CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

protected:

   /// \brief Sets request parameters and closes the form.
   afx_msg void OnBnClickedOk();

   /// \brief Changes request type to Time.
   afx_msg void OnBnClickedRdTimeRange();

   /// \brief Changes request type to SinceTime.
   afx_msg void OnBnClickedRdSinceTime();

   /// \brief Changes request type to SinceTimeNotify.
   afx_msg void OnBnClickedRdSinceTimeNotify();

   /// \brief Changes request type to CurrentNotify.
   afx_msg void OnBnClickedRdCurrentNotify();

   afx_msg void OnBnClickedCheckUseLimit();

   /// \brief Used to increment or decrement sessions filter.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnSessionsFilter(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement ticks limit.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnTicksLimit(
      NMHDR *pNMHDR,
      LRESULT *pResult);

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

private:

   /// \brief Returns selected request type.
   /// \return Selected request type as eTicksRequestType object.
   eTicksRequestType GetTicksReqType();

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Sets all parameters default values taken from CQGTicksRequest.
   void SetDefaultParameters();

   /// \brief Changes request type.
   void RequestTypeChanged();

   /// \brief Initializes tick filter combo box.
   void InitializeTickFilterCombo();

private:

   /// \brief CQGTicksRequest object.
   ATL::CComPtr<ICQGTicksRequest> m_spRequest;

   /// \brief Contains line time when form activated from CQGCEL.
   const DATE m_LineTime;

private:

   /// \brief Time range radio button control.
   CButton m_rdRequestType;

   /// \brief Request Type.
   int m_ReqType;

   /// \brief Request symbol edit control.
   CEdit m_edSymbol;

   /// \brief Request range start date time control.
   CDateTimeCtrl m_dtpStart;

   /// \brief Request range end date time control.
   CDateTimeCtrl m_dtpEnd;

   /// \brief Request sessions filter edit control.
   CNumEdit m_edSessionsFilter;

   /// \brief Request tick filter combo box control
   CComboBox m_cmbTickFilter;

   /// \brief Request ticks limit edit control.
   CNumEdit m_edTicksLimit;

   BOOL m_useTicksLimit;

};
