// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "TimedBars.h"
#include "TimedBarsDlg.h"

void CTimedBarsDlg::InitializeCQGCEL()
{
   // Create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   Common::ThrowOnCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   Common::ThrowOnCOMError(m_spCQGCEL, hr);
}

void CTimedBarsDlg::InitializeControls()
{
   InitializeList(m_lvLastTimedBars);
   InitializeList(m_lvTimedBars);

   ClearAllData();

   OnDataConnectionStatusChanged(csConnectionDown);

   m_dtpStart.SetFormat(CHelper::DateFormat);
   m_dtpEnd.SetFormat(CHelper::DateFormat);

   UpdateData();
   m_FilterType = ftByIndices;
   UpdateData(FALSE);
   OnBnClickedRdIndices();
}

void CTimedBarsDlg::InitializeList(
                        CListCtrl& listCtrl)
{
   listCtrl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   listCtrl.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   listCtrl.InsertColumn(rcIndex, _T("#"), LVCFMT_LEFT, 40);
   listCtrl.InsertColumn(rcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 150);
   listCtrl.InsertColumn(rcOpen, _T("Open"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcHigh, _T("High"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcLow, _T("Low"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcClose, _T("Close"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcMid, _T("Mid"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcHLC3, _T("HLC3"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcAvg, _T("Avg"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTrueHigh, _T("True High"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTrueLow, _T("True Low"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcRange, _T("Range"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTrueRange, _T("True Range"), LVCFMT_LEFT, 80);
   listCtrl.InsertColumn(rcOpenInterest, _T("Open Interest"), LVCFMT_LEFT, 80);
   listCtrl.InsertColumn(rcActualVol, _T("Actual Vol"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcTickVol, _T("Tick Vol"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcAskVol, _T("Ask Vol"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcBidVol, _T("Bid Vol"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcAskFlagVol, _T("Ask Flag Vol"), LVCFMT_LEFT, 60);
   listCtrl.InsertColumn(rcBidFlagVol, _T("Bid Flag Vol"), LVCFMT_LEFT, 60);
}

void CTimedBarsDlg::ClearAllData()
{
   m_lblStatus.SetWindowText(_T(""));

   ClearRequestParams();

   ClearTags();

   ClearOutputParams();
   ClearDatesAndIndices();

   ClearRecords(m_lvLastTimedBars);
   ClearRecords(m_lvTimedBars);

   m_lblAllRecsCount.SetWindowText(_T(""));
   m_lblFltrRecsCount.SetWindowText(_T(""));
}

void CTimedBarsDlg::ClearRequestParams()
{
   UpdateData();

   m_Symbol = _T("");
   m_RangeStart = _T("");
   m_RangeEnd = _T("");
   m_TickFilter = _T("");
   m_IntradayPeriod = _T("");
   m_HistoricalPeriod = _T("");
   m_UpdatesEnabled = _T("");
   m_IncludeEnd = _T("");
   m_SessionsFilter = _T("");
   m_Continuation = _T("");
   m_EqualizeCloses = _T("");
   m_DaysBeforeExpiration = _T("");
   m_SessionFlags = _T("");
   m_DailyBarClose = _T("");

   UpdateData(FALSE);

   m_lblError.SetWindowText(_T(""));
}

void CTimedBarsDlg::ClearOutputParams()
{
   UpdateData();

   m_OutputStart = _T("");
   m_OutputEnd = _T("");

   UpdateData(FALSE);
}

void CTimedBarsDlg::ClearTags()
{
   m_edTagName.SetWindowText(_T(""));
   m_edTagValue.SetWindowText(_T(""));
}

void CTimedBarsDlg::ClearRecords(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();
}

void CTimedBarsDlg::ShowAllData()
{
   ShowRequestParams();

   eRequestStatus status;
   HRESULT hr = m_spTimedBars->get_Status(&status);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   m_lblStatus.SetWindowText(CHelper::RequestStatusToString(status));

   if (status == rsSuccess)
   {
      ShowOutputParams();

      FillDatesAndIndices();

      long count;
      hr = m_spTimedBars->get_Count(&count);
      Common::ThrowOnCOMError(m_spTimedBars, hr);

      // Show records
      const long startIndex = max(count - m_ShowCount + 1, 1);
      const long endIndex = count;
      ShowRecords(startIndex, endIndex, m_lvLastTimedBars);

      // Show all records count
      m_lblAllRecsCount.SetWindowText(GetValueAsString(count));
   }
}

void CTimedBarsDlg::ShowChangedData()
{
   // Clear the list box
   ClearRecords(m_lvLastTimedBars);

   // Get records count
   long count;
   HRESULT hr = m_spTimedBars->get_Count(&count);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   // Show records
   const long startIndex = max(count - m_ShowCount + 1, 1);
   const long endIndex = count;
   ShowRecords(startIndex, endIndex, m_lvLastTimedBars);

   // Show all records count
   m_lblAllRecsCount.SetWindowText(GetValueAsString(count));

   // Show output parameters
   ShowOutputParams();
}

void CTimedBarsDlg::ShowRequestParams()
{
   UpdateData();

   // Get request from timed bars collection
   ATL::CComPtr<ICQGTimedBarsRequest> spRequest;
   HRESULT hr = m_spTimedBars->get_Request(&spRequest);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   // Show request parameters
   ATL::CComBSTR symbol;
   hr = spRequest->get_Symbol(&symbol);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeStart;
   hr = spRequest->get_RangeStart(&rangeStart);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeEnd;
   hr = spRequest->get_RangeEnd(&rangeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   long intradayPeriod;
   hr = spRequest->get_IntradayPeriod(&intradayPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   eHistoricalPeriod historicalPeriod;
   hr = spRequest->get_HistoricalPeriod(&historicalPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   eTickFilter tickFilter;
   hr = spRequest->get_TickFilter(&tickFilter);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL updatesEnabled, updatesNoHistorical;
   hr = spRequest->get_UpdatesEnabled(&updatesEnabled);
   Common::ThrowOnCOMError(spRequest, hr);
   hr = spRequest->get_IgnoreEventsOnHistoricalBars(&updatesNoHistorical);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL includeEnd;
   hr = spRequest->get_IncludeEnd(&includeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant sessionsFilter;
   hr = spRequest->get_SessionsFilter(&sessionsFilter);
   Common::ThrowOnCOMError(spRequest, hr);

   eTimeSeriesContinuationType continuation;
   hr = spRequest->get_Continuation(&continuation);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL equalizeCloses;
   hr = spRequest->get_EqualizeCloses(&equalizeCloses);
   Common::ThrowOnCOMError(spRequest, hr);

   long daysBeforeExpiration;
   hr = spRequest->get_DaysBeforeExpiration(&daysBeforeExpiration);
   Common::ThrowOnCOMError(spRequest, hr);

   eSessionFlag sessionFlags;
   hr = spRequest->get_SessionFlags(&sessionFlags);
   Common::ThrowOnCOMError(spRequest, hr);

   eDailyBarClose dailyBarClose;
   hr = spRequest->get_DailyBarClose(&dailyBarClose);
   Common::ThrowOnCOMError(spRequest, hr);

   m_Symbol = ATL::CW2T(symbol);
   m_RangeStart = GetValueAsString(rangeStart);
   m_RangeEnd = GetValueAsString(rangeEnd);
   m_IntradayPeriod = GetValueAsString(intradayPeriod);
   m_HistoricalPeriod = CHelper::HistoricalPeriodToString(historicalPeriod);
   m_TickFilter = CHelper::TickFilterToString(tickFilter);
   m_UpdatesEnabled = GetValueAsString(updatesEnabled, VT_BOOL);
   if (updatesNoHistorical)
   {
      m_UpdatesEnabled += CString("/NoHist");
   }
   m_IncludeEnd = GetValueAsString(includeEnd, VT_BOOL);
   
   CString filter = GetValueAsString(sessionsFilter);
   if (filter == "31")
   {
      filter = "All sessions (31)";
   }
   else if (filter == "0")
   {
      filter = "Primary session only (0)";
   }
   m_SessionsFilter = filter;

   m_Continuation = CHelper::ContinuationTypeToString(continuation);
   m_EqualizeCloses = GetValueAsString(equalizeCloses, VT_BOOL);
   m_DaysBeforeExpiration = GetValueAsString(daysBeforeExpiration);
   m_SessionFlags = CHelper::SessionFlagToString(sessionFlags);
   m_DailyBarClose = CHelper::DailyBarCloseToString(dailyBarClose);

   UpdateData(FALSE);

   // Get error from timed bars collection and show it if exist
   ATL::CComPtr<ICQGError> spError;
   hr = m_spTimedBars->get_LastError(&spError);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   if (spError)
   {
      ATL::CComBSTR errDescription;
      hr = spError->get_Description(&errDescription);
      Common::ThrowOnCOMError(spError, hr);

      m_lblError.SetWindowText(ATL::CW2T(errDescription));
   }
}

void CTimedBarsDlg::ShowOutputParams()
{
   UpdateData();

   // Get output parameters from timed bars collection and show them
   DATE startTimestamp;
   HRESULT hr = m_spTimedBars->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   DATE endTimestamp;
   hr = m_spTimedBars->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   m_OutputStart = GetValueAsString(startTimestamp, VT_DATE);
   m_OutputEnd = GetValueAsString(endTimestamp, VT_DATE);

   UpdateData(FALSE);
}

void CTimedBarsDlg::ShowRecords(
                        long startIndex,
                        long endIndex,
                        CListCtrl& listCtrl)
{
   for (long i = startIndex; i <= endIndex; ++i)
   {
      AddRecord(i, listCtrl);
   }
}

void CTimedBarsDlg::AddRecord(
                        long recordIndex,
                        CListCtrl& listCtrl)
{
   // Add empty row
   int row = listCtrl.InsertItem(listCtrl.GetItemCount(), _T(""));

   ShowRecord(recordIndex, listCtrl, row);
}

void CTimedBarsDlg::ShowRecord(
                        long recordIndex,
                        CListCtrl& listCtrl,
                        long row)
{
   // Get specified TimedBar from timed bars collection by its index
   ATL::CComPtr<ICQGTimedBar> spTimedBar;
   HRESULT hr = m_spTimedBars->get_Item(recordIndex - 1, &spTimedBar);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   // Get TimedBar timestamp to show it
   DATE timestamp;
   hr = spTimedBar->get_Timestamp(&timestamp);
   Common::ThrowOnCOMError(spTimedBar, hr);

   listCtrl.SetItemText(row, rcIndex, GetValueAsString(recordIndex));
   listCtrl.SetItemText(row, rcTimestamp, GetValueAsString(timestamp, VT_DATE, true));

   // Show parameters into given listbox
   ShowProperty(spTimedBar, &ICQGTimedBar::get_Open, listCtrl, row, rcOpen);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_High, listCtrl, row, rcHigh);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_Low, listCtrl, row, rcLow);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_Close, listCtrl, row, rcClose);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_Mid, listCtrl, row, rcMid);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_HLC3, listCtrl, row, rcHLC3);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_Avg, listCtrl, row, rcAvg);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_TrueHigh, listCtrl, row, rcTrueHigh);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_TrueLow, listCtrl, row, rcTrueLow);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_Range, listCtrl, row, rcRange);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_TrueRange, listCtrl, row, rcTrueRange);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_OpenInterest, listCtrl, row, rcOpenInterest);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_ActualVolume, listCtrl, row, rcActualVol);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_TickVolume, listCtrl, row, rcTickVol);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_AskVolume, listCtrl, row, rcAskVol);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_BidVolume, listCtrl, row, rcBidVol);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_AskFlagVolume, listCtrl, row, rcAskFlagVol);
   ShowProperty(spTimedBar, &ICQGTimedBar::get_BidFlagVolume, listCtrl, row, rcBidFlagVol);
}

template <typename T>
void CTimedBarsDlg::ShowProperty(
                        const ATL::CComPtr<ICQGTimedBar>& spTimedBar,
                        HRESULT (__stdcall ICQGTimedBar::*get_Property)(T*),
                        CListCtrl& listCtrl,
                        int row,
                        eRecordsColumns col)
{
   T value;
   HRESULT hr = (spTimedBar->*get_Property)(&value);
   Common::ThrowOnCOMError(spTimedBar, hr);

   listCtrl.SetItemText(row, col, GetValueAsString(value));
}

void CTimedBarsDlg::ClearDatesAndIndices()
{
   m_edStart.SetValue(0);
   m_edEnd.SetValue(0);

   static const COleDateTime nullDate;

   m_dtpStart.SetTime(nullDate);
   m_dtpEnd.SetTime(nullDate);
}

void CTimedBarsDlg::FilterTypeChanged()
{
   UpdateData();

   FillDatesAndIndices();

   int indicesShowStatus = (m_FilterType == ftByIndices ? SW_SHOW : SW_HIDE);
   m_edStart.ShowWindow(indicesShowStatus);
   m_spnStart.ShowWindow(indicesShowStatus);
   m_spnEnd.ShowWindow(indicesShowStatus);
   m_edEnd.ShowWindow(indicesShowStatus);

   int datesShowStatus = (m_FilterType == ftByIndices ? SW_HIDE : SW_SHOW);
   m_dtpStart.ShowWindow(datesShowStatus);
   m_dtpEnd.ShowWindow(datesShowStatus);
}

void CTimedBarsDlg::FillDatesAndIndices()
{
   if (m_spTimedBars == NULL)
   {
      return;
   }

   eRequestStatus status;
   HRESULT hr = m_spTimedBars->get_Status(&status);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   if (status != rsSuccess)
   {
      return;
   }

   // Get start\end timestamps from timed bars collection and fill them into start\end time controls
   DATE startTimestamp;
   hr = m_spTimedBars->get_StartTimestamp(&startTimestamp);
   Common::ThrowOnCOMError(m_spTimedBars, hr);
   m_dtpStart.SetTime(startTimestamp);

   DATE endTimestamp;
   hr = m_spTimedBars->get_EndTimestamp(&endTimestamp);
   Common::ThrowOnCOMError(m_spTimedBars, hr);
   m_dtpEnd.SetTime(endTimestamp);

   // Fill start and end indices controls.
   m_edStart.SetValue(1);

   long count;
   hr = m_spTimedBars->get_Count(&count);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   m_edEnd.SetValue(count);
}

void CTimedBarsDlg::EnableControls()
{
   bool isDataConnectionUp = false;
   HRESULT hr;
   VARIANT_BOOL isStarted = VARIANT_FALSE;
   
   if (m_spCQGCEL)
   {
      hr = m_spCQGCEL->get_IsStarted(&isStarted);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }

   if (isStarted)
   {
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      hr = m_spCQGCEL->get_Environment(&spEnvironment);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      eConnectionStatus dataConnectionStatus;
      hr = spEnvironment->get_DataConnectionStatus(&dataConnectionStatus);
      Common::ThrowOnCOMError(spEnvironment, hr);

      isDataConnectionUp = (dataConnectionStatus == csConnectionUp);
   }

   m_btnRequest.EnableWindow(isDataConnectionUp);

   bool isTimedBarsSelected = (m_spTimedBars != NULL);

   m_btnRemove.EnableWindow(isTimedBarsSelected && isDataConnectionUp);
   m_btnRemoveAll.EnableWindow(isTimedBarsSelected && isDataConnectionUp);

   m_edTagName.EnableWindow(isTimedBarsSelected);
   m_edTagValue.EnableWindow(isTimedBarsSelected);

   bool isTimedBarsSucceeded = false;
   if (isTimedBarsSelected)
   {
      eRequestStatus status;
      hr = m_spTimedBars->get_Status(&status);
      Common::ThrowOnCOMError(m_spTimedBars, hr);

      isTimedBarsSucceeded = (status == rsSuccess);
   }

   m_rdIndices.EnableWindow(isTimedBarsSucceeded);
   m_rdDates.EnableWindow(isTimedBarsSucceeded);

   m_edStart.EnableWindow(isTimedBarsSucceeded);
   m_edEnd.EnableWindow(isTimedBarsSucceeded);
   m_spnStart.EnableWindow(isTimedBarsSucceeded);
   m_spnEnd.EnableWindow(isTimedBarsSucceeded);
   m_dtpStart.EnableWindow(isTimedBarsSucceeded);
   m_dtpEnd.EnableWindow(isTimedBarsSucceeded);

   m_btnShow.EnableWindow(isTimedBarsSucceeded);
}

CString CTimedBarsDlg::GetValueAsString(
                        ATL::CComVariant value,
                        VARENUM valueType/*= VT_EMPTY*/,
                        bool withMilliseconds /*= false*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return Common::VariantToString(value, withMilliseconds);
   }
   return CHelper::m_N_A;
}
