// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "TimedBars.h"
#include "TimedBarsDlg.h"

STDMETHODIMP CTimedBarsDlg::OnDataError(
                           LPDISPATCH /*obj*/,
                           BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CTimedBarsDlg::OnDataConnectionStatusChanged(
                           eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      EnableControls();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTimedBarsDlg::OnTimedBarsResolved(
                           ICQGTimedBars* cqg_timed_bars,
                           ICQGError* /*cqg_error*/)
{
   try
   {
      if (!m_spTimedBars)
      {
         return S_OK;
      }

      // Get selected timed bars ID
      ATL::CComBSTR selTimedBarsID;
      HRESULT hr = m_spTimedBars->get_Id(&selTimedBarsID);
      Common::ThrowOnCOMError(m_spTimedBars, hr);

      // Get changed timed bars ID
      ATL::CComBSTR timedBarsID;
      hr = cqg_timed_bars->get_Id(&timedBarsID);
      Common::ThrowOnCOMError<ICQGTimedBars>(cqg_timed_bars, hr);

      // Check if timed bars resolved in the selected request
      if (selTimedBarsID == timedBarsID)
      {
         ClearAllData();
         ShowAllData();

         EnableControls();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTimedBarsDlg::OnTimedBarsAdded(
                           ICQGTimedBars* cqg_timed_bars)
{
   try
   {
      if (!m_spTimedBars)
      {
         return S_OK;
      }

      // Get selected timed bars ID
      ATL::CComBSTR selTimedBarsID;
      HRESULT hr = m_spTimedBars->get_Id(&selTimedBarsID);
      Common::ThrowOnCOMError(m_spTimedBars, hr);

      // Get changed timed bars ID
      ATL::CComBSTR timedBarsID;
      hr = cqg_timed_bars->get_Id(&timedBarsID);
      Common::ThrowOnCOMError<ICQGTimedBars>(cqg_timed_bars, hr);

      // Check if record added in the selected request
      if (selTimedBarsID == timedBarsID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTimedBarsDlg::OnTimedBarsInserted(
                           ICQGTimedBars* cqg_timed_bars,
                           long /*index_*/)
{
   try
   {
      if (!m_spTimedBars)
      {
         return S_OK;
      }

      // Get selected timed bars ID
      ATL::CComBSTR selTimedBarsID;
      HRESULT hr = m_spTimedBars->get_Id(&selTimedBarsID);
      Common::ThrowOnCOMError(m_spTimedBars, hr);

      // Get changed timed bars ID
      ATL::CComBSTR timedBarsID;
      hr = cqg_timed_bars->get_Id(&timedBarsID);
      Common::ThrowOnCOMError<ICQGTimedBars>(cqg_timed_bars, hr);

      // Check if record inserted in the selected request
      if (selTimedBarsID == timedBarsID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTimedBarsDlg::OnTimedBarsRemoved(
                           ICQGTimedBars* cqg_timed_bars,
                           long /*index_*/)
{
   if (!m_spTimedBars)
   {
      return S_OK;
   }

   try
   {
      // Get selected timed bars ID
      ATL::CComBSTR selTimedBarsID;
      HRESULT hr = m_spTimedBars->get_Id(&selTimedBarsID);
      Common::ThrowOnCOMError(m_spTimedBars, hr);

      // Get changed timed bars ID
      ATL::CComBSTR timedBarsID;
      hr = cqg_timed_bars->get_Id(&timedBarsID);
      Common::ThrowOnCOMError<ICQGTimedBars>(cqg_timed_bars, hr);

      // Check if record removed from the selected request
      if (selTimedBarsID == timedBarsID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTimedBarsDlg::OnTimedBarsUpdated(
                           ICQGTimedBars* cqg_timed_bars,
                           long index_)
{
   try
   {
      if (!m_spTimedBars)
      {
         return S_OK;
      }

      // Get selected timed bars ID
      ATL::CComBSTR selTimedBarsID;
      HRESULT hr = m_spTimedBars->get_Id(&selTimedBarsID);
      Common::ThrowOnCOMError(m_spTimedBars, hr);

      // Get changed timed bars ID
      ATL::CComBSTR timedBarsID;
      hr = cqg_timed_bars->get_Id(&timedBarsID);
      Common::ThrowOnCOMError<ICQGTimedBars>(cqg_timed_bars, hr);

      // Check if record updated in the selected request
      if (selTimedBarsID == timedBarsID)
      {
         // Find row for specified timed bar
         CString indexStr;
         indexStr.Format(_T("%d"), index_ + 1);
         LVFINDINFO findInfo;
         findInfo.flags = LVFI_STRING;
         findInfo.psz = indexStr;
         int row = m_lvLastTimedBars.FindItem(&findInfo);

         // If found change it's properties
         if (row > -1)
         {
            ShowRecord(index_ + 1, m_lvLastTimedBars, row);
         }
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}