// TimedBarsDlg.cpp : implementation file
//

#include "stdafx.h"
#include <cassert>
#include "Common.h"
#include "Helper.h"
#include "TimedBars.h"
#include "TimedBarsDlg.h"
#include "TimedBarsReqDlg.h"

const long CTimedBarsDlg::m_ShowCount = 5;
const long CTimedBarsDlg::m_MaxIdInterval = 500;

CTimedBarsDlg::CTimedBarsDlg(CWnd* pParent /*=NULL*/)
                           : CDialog(CTimedBarsDlg::IDD, pParent),
                           m_FilterType(ftByIndices)
{
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CTimedBarsDlg::DoDataExchange(
                           CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_LBL_DATA_CONNECTION, m_lblDataConnection);

   DDX_Control(pDX, IDC_CMB_TIMED_BARS, m_cmbTimedBars);

   DDX_Control(pDX, IDC_BTN_REQUEST, m_btnRequest);
   DDX_Control(pDX, IDC_BTN_REMOVE, m_btnRemove);
   DDX_Control(pDX, IDC_BTN_REMOVE_ALL, m_btnRemoveAll);

   DDX_Control(pDX, IDC_LBL_STATUS, m_lblStatus);

   DDX_Text(pDX, IDC_LBL_SYMBOL, m_Symbol);
   DDX_Text(pDX, IDC_LBL_RANGE_START, m_RangeStart);
   DDX_Text(pDX, IDC_LBL_RANGE_END, m_RangeEnd);
   DDX_Text(pDX, IDC_LBL_TICK_FILTER, m_TickFilter);
   DDX_Text(pDX, IDC_LBL_INTRADAY_PERIOD, m_IntradayPeriod);
   DDX_Text(pDX, IDC_LBL_HISTORICAL_PERIOD, m_HistoricalPeriod);
   DDX_Text(pDX, IDC_LBL_UPDATES_ENABLED, m_UpdatesEnabled);
   DDX_Text(pDX, IDC_LBL_INCLUDE_END, m_IncludeEnd);
   DDX_Text(pDX, IDC_LBL_SESSIONS_FILTER, m_SessionsFilter);
   DDX_Text(pDX, IDC_LBL_CONTINUATION, m_Continuation);
   DDX_Text(pDX, IDC_LBL_EQUALIZE_CLOSES, m_EqualizeCloses);
   DDX_Text(pDX, IDC_LBL_DAYS_BEFORE_EXPIRATION, m_DaysBeforeExpiration);
   DDX_Text(pDX, IDC_LBL_SESSION_FLAGS, m_SessionFlags);
   DDX_Text(pDX, IDC_LBL_DAILY_BAR_CLOSE, m_DailyBarClose);

   DDX_Text(pDX, IDC_LBL_OUTPUT_START, m_OutputStart);
   DDX_Text(pDX, IDC_LBL_OUTPUT_END, m_OutputEnd);

   DDX_Control(pDX, IDC_LBL_ERROR, m_lblError);

   DDX_Control(pDX, IDC_LSV_LAST_TIMED_BARS, m_lvLastTimedBars);
   DDX_Control(pDX, IDC_LSV_TIMED_BARS, m_lvTimedBars);

   DDX_Control(pDX, IDC_ED_TAG_NAME, m_edTagName);
   DDX_Control(pDX, IDC_ED_TAG_VALUE, m_edTagValue);

   DDX_Control(pDX, IDC_RD_INDICES, m_rdIndices);
   DDX_Control(pDX, IDC_RD_DATES, m_rdDates);
   DDX_Radio(pDX, IDC_RD_INDICES, m_FilterType);

   DDX_Control(pDX, IDC_DTP_FILTER_START_DATE, m_dtpStart);
   DDX_Control(pDX, IDC_ED_FILTER_START_INDEX, m_edStart);
   DDX_Control(pDX, IDC_SPN_FILTER_START_INDEX, m_spnStart);
   DDX_Control(pDX, IDC_DTP_FILTER_END_DATE, m_dtpEnd);
   DDX_Control(pDX, IDC_ED_FILTER_END_INDEX, m_edEnd);
   DDX_Control(pDX, IDC_SPN_FILTER_END_INDEX, m_spnEnd);

   DDX_Control(pDX, IDC_BTN_SHOW, m_btnShow);

   DDX_Control(pDX, IDC_LBL_RECS_COUNT, m_lblFltrRecsCount);
   DDX_Control(pDX, IDC_LBL_ALL_RECS_COUNT, m_lblAllRecsCount);
}

BEGIN_MESSAGE_MAP(CTimedBarsDlg, CDialog)
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_WM_DESTROY()
   ON_BN_CLICKED(IDC_BTN_REMOVE, OnBnClickedBtnRemove)
   ON_BN_CLICKED(IDC_BTN_REMOVE_ALL, OnBnClickedBtnRemoveAll)
   ON_BN_CLICKED(IDC_BTN_REQUEST, OnBnClickedBtnRequest)
   ON_BN_CLICKED(IDC_BTN_SHOW, OnBnClickedBtnShow)
   ON_BN_CLICKED(IDC_RD_INDICES, OnBnClickedRdIndices)
   ON_BN_CLICKED(IDC_RD_DATES, OnBnClickedRdDate)
   ON_CBN_SELCHANGE(IDC_CMB_TIMED_BARS, OnCbnSelChangeCmbTimedBars)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_FILTER_START_INDEX, OnDeltaposSpnStart)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_FILTER_END_INDEX, OnDeltaposSpnEnd)
END_MESSAGE_MAP()

BOOL CTimedBarsDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   // Set the icon for this dialog.  The framework does this automatically
   //  when the application's main window is not a dialog
   SetIcon(m_hIcon, TRUE);       // Set big icon
   SetIcon(m_hIcon, FALSE);      // Set small icon

   try
   {
      InitializeControls();
      InitializeCQGCEL();
   }
   catch (const std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
}

void CTimedBarsDlg::OnPaint() 
{
   if (IsIconic())
   {
      CPaintDC dc(this); // device context for painting

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      // Center icon in client rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      // Draw the icon
      dc.DrawIcon(x, y, m_hIcon);
   }
   else
   {
      CDialog::OnPaint();
   }
}

HCURSOR CTimedBarsDlg::OnQueryDragIcon()
{
   return static_cast<HCURSOR>(m_hIcon);
}

void CTimedBarsDlg::OnOK()
{
   // If enter clicked when one of the tag edit boxes is active
   // then call corresponding function
   CWnd* pWnd = GetFocus();

   if (pWnd == &m_edTagName)
   {
      OnEnterEdTagName();
   }
   else if (pWnd == &m_edTagValue)
   {
      OnEnterEdTagValue();
   }
}

void CTimedBarsDlg::OnDestroy()
{
   try
   {
      if (m_spCQGCEL)
      {
         HRESULT hr = ICQGCELDispEventImpl::DispEventUnadvise(m_spCQGCEL);
         assert(SUCCEEDED(hr));

         hr = m_spCQGCEL->Shutdown();
         Common::ThrowOnCOMError(m_spCQGCEL, hr);
      }
   }
   catch (const std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   CDialog::OnDestroy();
}

void CTimedBarsDlg::OnBnClickedBtnRemove()
{
   try
   {
      // Remove selected timed bars from the collection
      HRESULT hr = m_spCQGCEL->RemoveTimedBars(m_spTimedBars);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      // Remove item from combo box
      int row = m_cmbTimedBars.GetCurSel();
      int count = m_cmbTimedBars.DeleteString(row);

      if (count == 0)
      {
         ClearAllData();
         m_spTimedBars = NULL;

         // Clear combo box
         m_cmbTimedBars.ResetContent();
      }
      else
      {
         m_cmbTimedBars.SetCurSel((row >= count ? (count - 1) : row));
         OnCbnSelChangeCmbTimedBars();
      }

      EnableControls();
   }
   catch (const std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsDlg::OnBnClickedBtnRemoveAll()
{
   try
   {
      // Clear combo box
      m_cmbTimedBars.ResetContent();

      // Remove all timed bars from CQGCEL
      HRESULT hr = m_spCQGCEL->RemoveAllTimedBars();
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      m_spTimedBars = NULL;

      ClearAllData();

      EnableControls();
   }
   catch (const std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsDlg::OnCbnSelChangeCmbTimedBars()
{
   try
   {
      ClearAllData();

      if (m_cmbTimedBars.GetCount() > 0)
      {
         // Get GUID from combo box
         CString id;
         m_cmbTimedBars.GetWindowText(id);

         // Get item from CQGCEL by GUID
         ATL::CComPtr<ICQGTimedBarsCollection> spAllTimedBars;
         HRESULT hr = m_spCQGCEL->get_AllTimedBars(&spAllTimedBars);
         Common::ThrowOnCOMError(m_spCQGCEL, hr);

         m_spTimedBars.Release();
         hr = spAllTimedBars->get_ItemById(ATL::CT2W(id), &m_spTimedBars);
         Common::ThrowOnCOMError(spAllTimedBars, hr);

         // Show corresponding items all data
         ShowAllData();

         // Initialize parameters for filtering
         UpdateData();
         m_FilterType = ftByIndices;
         UpdateData(FALSE);
         OnBnClickedRdIndices();
      }

      EnableControls();
   }
   catch (const std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsDlg::OnBnClickedBtnRequest()
{
   try
   {
      // Create empty request
      ATL::CComPtr<ICQGTimedBarsRequest> spRequest;
      HRESULT hr = m_spCQGCEL->CreateTimedBarsRequest(&spRequest);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      spRequest->put_SessionsFilter(ATL::CComVariant(31));  // In other words "all sessions". 31 is represented by "11111" bits, where each bit designates a session in "Chart Preference"/"Sessions".

      // Get line time from CQGCEL->Environment
      ATL::CComPtr<ICQGEnvironment> spEnvironment;
      hr = m_spCQGCEL->get_Environment(&spEnvironment);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      DATE lineTime;
      hr = spEnvironment->get_LineTime(&lineTime);
      Common::ThrowOnCOMError(spEnvironment, hr);

      // Open form to fill up request data
      CTimedBarsReqDlg dlg(spRequest, lineTime);
      if (dlg.DoModal() == IDOK)
      {
         // Send request to CQGCEL
         ATL::CComPtr<ICQGTimedBars> spTimedBars;
         hr = m_spCQGCEL->RequestTimedBars(spRequest, &spTimedBars);
         Common::ThrowOnCOMError(m_spCQGCEL, hr);

         // Add ID to the requests combo box
         ATL::CComBSTR timedBarsID;
         hr = spTimedBars->get_Id(&timedBarsID);
         Common::ThrowOnCOMError(spTimedBars, hr);

         int row = m_cmbTimedBars.AddString(ATL::CW2T(timedBarsID));
         m_cmbTimedBars.SetCurSel(row);
         OnCbnSelChangeCmbTimedBars();
      }

      EnableControls();
   }
   catch (const std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsDlg::OnBnClickedBtnShow()
{
   try
   {
      // Clear records and records count
      ClearRecords(m_lvTimedBars);
      m_lblFltrRecsCount.SetWindowText(_T(""));

      long startIndex;
      long endIndex;
      bool isValid;

      UpdateData();

      if (m_FilterType == ftByIndices)
      {
         isValid = GetIndices(startIndex, endIndex);
      }
      else
      {
         isValid = GetIndicesForDates(startIndex, endIndex);
      }

      // Show filtered records and its' count when range is valid
      if (isValid)
      {
         m_lblFltrRecsCount.SetWindowText(GetValueAsString(endIndex - startIndex + 1));
         ShowRecords(startIndex, endIndex, m_lvTimedBars);
      }
   }
   catch (const std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

bool CTimedBarsDlg::GetIndices(
                           long& startIndex,
                           long& endIndex)
{
   startIndex = m_edStart.GetValue();
   endIndex = m_edEnd.GetValue();

   // Check for valid indices
   if (endIndex < startIndex)
   {
      AfxMessageBox(_T("Start index must be smaller than end index."), MB_ICONWARNING);
      return false;
   }

   if (startIndex < 1)
   {
      AfxMessageBox(_T("Start index must be greater than zero."), MB_ICONWARNING);
      return false;
   }

   long count;
   HRESULT hr = m_spTimedBars->get_Count(&count);
   Common::ThrowOnCOMError<ICQGTimedBars>(m_spTimedBars, hr);

   if (endIndex > count)
   {
      AfxMessageBox(_T("End index must be smaller than records count."), MB_ICONWARNING);
      return false;
   }

   if (endIndex - startIndex >= m_MaxIdInterval)
   {
      CString msgText;
      msgText.Format(_T("Range must indicate to no more than %d records."), m_MaxIdInterval);
      AfxMessageBox(msgText, MB_ICONWARNING);
      return false;
   }

   return true;
}

bool CTimedBarsDlg::GetIndicesForDates(
                           long& startIndex,
                           long& endIndex)
{
   // Get start and end dates and remove seconds from them: to retrieve same data for same requests
   COleDateTime startTime;
   m_dtpStart.GetTime(startTime);

   COleDateTime startTimeNS(startTime.GetYear(), startTime.GetMonth(), startTime.GetDay(), 
      startTime.GetHour(), startTime.GetMinute(), 0);

   COleDateTime endTime;
   m_dtpEnd.GetTime(endTime);

   COleDateTime endTimeNS(endTime.GetYear(), endTime.GetMonth(), endTime.GetDay(), 
      endTime.GetHour(), endTime.GetMinute(), 0);

   // Check for valid date range
   if (CHelper::CompareDates(startTimeNS, endTimeNS) > 0)
   {
      AfxMessageBox(_T("Start date must be earlier than end date."), MB_ICONWARNING);
      return false;
   }

   // Get start and end indices for specified dates
   HRESULT hr = m_spTimedBars->IndexForDate(startTimeNS.m_dt, &startIndex);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   hr = m_spTimedBars->IndexForDate(endTimeNS.m_dt, &endIndex);
   Common::ThrowOnCOMError(m_spTimedBars, hr);

   // Check for valid dates
   VARIANT_BOOL isValid;
   hr = m_spCQGCEL->IsValid(ATL::CComVariant(startIndex), &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (!isValid)
   {
      AfxMessageBox(_T("Start date is invalid: there isn't any record with this timestamp"), MB_ICONINFORMATION);
      return false;
   }

   hr = m_spCQGCEL->IsValid(ATL::CComVariant(endIndex), &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (!isValid)
   {
      AfxMessageBox(_T("End date is invalid: there isn't any record with this timestamp"), MB_ICONINFORMATION);
      return false;
   }

   if (endIndex - startIndex >= m_MaxIdInterval)
   {
      CString msgText;
      msgText.Format(_T("Range must indicate to no more than %d records."), m_MaxIdInterval);
      AfxMessageBox(msgText, MB_ICONWARNING);
      return false;
   }

   // IndicesForDates function returns 0 based indices, so expanding them by 1
   startIndex += 1;
   endIndex += 1;

   return true;
}

void CTimedBarsDlg::OnDeltaposSpnStart(
                           NMHDR *pNMHDR,
                           LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edStart.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CTimedBarsDlg::OnDeltaposSpnEnd(
                           NMHDR *pNMHDR,
                           LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edEnd.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CTimedBarsDlg::OnBnClickedRdIndices()
{
   try
   {
      FilterTypeChanged();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsDlg::OnBnClickedRdDate()
{
   try
   {
      FilterTypeChanged();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsDlg::OnEnterEdTagName()
{
   try
   {
      CString tagName;
      m_edTagName.GetWindowText(tagName);

      // If there is no tag name specified, then do nothing
      if (tagName.IsEmpty()) 
      {
         return;
      }

      ATL::CComVariant tagValue;
      HRESULT hr = m_spTimedBars->get_Tag(ATL::CT2W(tagName), &tagValue);
      Common::ThrowOnCOMError(m_spTimedBars, hr);

      // If there is no tag with specified tag name, tag value will be VT_EMPTY
      m_edTagValue.SetWindowText((tagValue.vt != VT_EMPTY) ? GetValueAsString(tagValue) : _T(""));
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsDlg::OnEnterEdTagValue()
{
   try
   {
      CString tagName;
      m_edTagName.GetWindowText(tagName);

      // If there is no tag name specified, then do nothing
      if (tagName.IsEmpty())
      {
         return;
      }

      CString tagValue;
      m_edTagValue.GetWindowText(tagValue);

      // If there is no tag value specified, then remove tag with specified tag name
      HRESULT hr = m_spTimedBars->put_Tag(ATL::CT2W(tagName), 
         tagValue.IsEmpty() ? ATL::CComVariant() : ATL::CComVariant(tagValue));
      Common::ThrowOnCOMError(m_spTimedBars, hr);

      m_edTagName.SetWindowText(_T(""));
      m_edTagValue.SetWindowText(_T(""));

      m_edTagName.SetFocus();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}
