// TimedBarsReqDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "TimedBars.h"
#include "TimedBarsReqDlg.h"

namespace
{
   VARIANT_BOOL boolToVarBool(BOOL value)
   {
      return value ? VARIANT_TRUE : VARIANT_FALSE;
   }
}

CTimedBarsReqDlg::CTimedBarsReqDlg(
                              const ATL::CComPtr<ICQGTimedBarsRequest>& spRequest,
                              const DATE lineTime,
                              CWnd* pParent /*=NULL*/)
                              : CDialog(CTimedBarsReqDlg::IDD, pParent),
                                m_spRequest(spRequest),
                                m_LineTime(lineTime),
                                m_Symbol(_T("")),
                                m_RangeStartDate(COleDateTime::GetCurrentTime()),
                                m_RangeEndDate(COleDateTime::GetCurrentTime()),
                                m_RequestType(rtIntInt),
                                m_EqualizeCloses(FALSE),
                                m_UpdatesEnabled(FALSE),
                                m_DailyFromIntraday(FALSE),
                                m_CustomSession(FALSE),
                                m_NoHistorical(FALSE)
{
   // Initialize local variables
   m_edRangeStart.NegativeAllowed(true);
   m_edRangeEnd.NegativeAllowed(true);
}

void CTimedBarsReqDlg::DoDataExchange(
                              CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);

   DDX_Text(pDX, IDC_ED_SYMBOL, m_Symbol);

   DDX_Control(pDX, IDC_DTP_RANGE_START, m_dtpRangeStart);
   DDX_DateTimeCtrl(pDX, IDC_DTP_RANGE_START, m_RangeStartDate);
   DDX_Control(pDX, IDC_ED_RANGE_START, m_edRangeStart);
   DDX_Control(pDX, IDC_SPN_RANGE_START, m_spnRangeStart);
   
   DDX_Control(pDX, IDC_DTP_RANGE_END, m_dtpRangeEnd);
   DDX_DateTimeCtrl(pDX, IDC_DTP_RANGE_END, m_RangeEndDate);
   DDX_Control(pDX, IDC_ED_RANGE_END, m_edRangeEnd);
   DDX_Control(pDX, IDC_SPN_RANGE_END, m_spnRangeEnd);

   DDX_Radio(pDX, IDC_RD_INT_INT, m_RequestType);

   DDX_Control(pDX, IDC_CHK_INCLUDE_END, m_chkIncludeEnd);
   DDX_Check(pDX, IDC_CHK_INCLUDE_END, m_IncludeEnd);

   DDX_Control(pDX, IDC_CMB_HISTORICAL, m_cmbHistorical);
   DDX_Control(pDX, IDC_CMB_TICK_FILTER, m_cmbTickFilter);

   DDX_Control(pDX, IDC_CMB_DAILY_BAR_CLOSE, m_cmbDailyBarClose);

   DDX_Control(pDX, IDC_ED_INTRADAY, m_edIntraday);
   DDX_Control(pDX, IDC_SPN_INTRADAY, m_spnIntraday);

   DDX_Control(pDX, IDC_CMB_CONTINUATION, m_cmbContinuation);

   DDX_Control(pDX, IDC_CHK_EQUALIZE_CLOSES, m_chkEqualizeCloses);
   DDX_Check(pDX, IDC_CHK_EQUALIZE_CLOSES, m_EqualizeCloses);

   DDX_Control(pDX, IDC_ED_DAYS_BEFORE_EXP, m_edDaysBeforeExp);
   DDX_Control(pDX, IDC_SPN_DAYS_BEFORE_EXP, m_spnDaysBeforeExp);

   DDX_Check(pDX, IDC_CHK_UPDATES_ENABLED, m_UpdatesEnabled);
   DDX_Check(pDX, IDC_CHK_UPDATES_NO_HISTORICAL, m_NoHistorical);
   DDX_Control(pDX, IDC_CHK_UPDATES_NO_HISTORICAL, m_chkNoHistorical);

   DDX_Check(pDX, IDC_CHK_DAILY_FROM_INTR, m_DailyFromIntraday);

   DDX_Control(pDX, IDC_ED_SESSIONS_FILTER, m_edSessionsFilter);
   DDX_Control(pDX, IDC_SPN_SESSIONS_FILTER, m_spnSessionsFilter);
   
   DDX_Control(pDX, IDC_ED_CUSTOM_SESSIONS_FILTER, m_edCustomSessionsFilter);
   DDX_Text(pDX, IDC_ED_CUSTOM_SESSIONS_FILTER, m_CustomSessionsFilter);

   DDX_Check(pDX, IDC_CHK_CUSTOM_SESSION, m_CustomSession);
}

BOOL CTimedBarsReqDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      InitializeControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;
}

BEGIN_MESSAGE_MAP(CTimedBarsReqDlg, CDialog)
   ON_BN_CLICKED(IDOK, OnBnClickedOk)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_SESSIONS_FILTER, OnDeltaposSpnSessionsFilter)
   ON_BN_CLICKED(IDC_CHK_CUSTOM_SESSION, OnBnClickedChkCustomSession)
   ON_CBN_SELCHANGE(IDC_CMB_CONTINUATION, OnCbnSelChangeCmbContinuation)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_INTRADAY, OnDeltaposSpnIntraday)
   ON_CBN_SELCHANGE(IDC_CMB_HISTORICAL, OnCbnSelChangeCmbHistorical)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RANGE_END, OnDeltaposSpnRangeEnd)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RANGE_START, OnDeltaposSpnRangeStart)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_DAYS_BEFORE_EXP, OnDeltaposSpnDaysBeforeExp)
   ON_BN_CLICKED(IDC_RD_INT_INT, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_RD_DATE_INT, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_RD_DATE_DATE, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_CHK_UPDATES_ENABLED, OnBnClickedChkUpdatesEnabled)
END_MESSAGE_MAP()

void CTimedBarsReqDlg::InitializeControls()
{
   RequestTypeChanged();

   InitializeHistoricalCombo();
   InitializeContinuationCombo();
   InitializeTickFilterCombo();
   InitializeDailyBarCloseCombo();

   m_dtpRangeStart.SetFormat(CHelper::DateFormat);
   m_dtpRangeEnd.SetFormat(CHelper::DateFormat);

   // Set default parameters
   SetDefaultParameters();
}

void CTimedBarsReqDlg::SetDefaultParameters()
{
   UpdateData();

   // Get default values form request
   long defIntradayPeriod;
   HRESULT hr = m_spRequest->get_IntradayPeriod(&defIntradayPeriod);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eHistoricalPeriod defHistoricalPeriod;
   hr = m_spRequest->get_HistoricalPeriod(&defHistoricalPeriod);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eTickFilter defTickFilter;
   hr = m_spRequest->get_TickFilter(&defTickFilter);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eDailyBarClose defDailyBarClose;
   hr = m_spRequest->get_DailyBarClose(&defDailyBarClose);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defUpdatesEnabled, defNoHistorical;
   hr = m_spRequest->get_UpdatesEnabled(&defUpdatesEnabled);
   Common::ThrowOnCOMError(m_spRequest, hr);
   hr = m_spRequest->get_IgnoreEventsOnHistoricalBars(&defNoHistorical);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defIncludeEnd;
   hr = m_spRequest->get_IncludeEnd(&defIncludeEnd);
   Common::ThrowOnCOMError(m_spRequest, hr);

   ATL::CComVariant defSessionsFilter;
   hr = m_spRequest->get_SessionsFilter(&defSessionsFilter);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eTimeSeriesContinuationType defContinuation;
   hr = m_spRequest->get_Continuation(&defContinuation);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defEqualizeCloses;
   hr = m_spRequest->get_EqualizeCloses(&defEqualizeCloses);
   Common::ThrowOnCOMError(m_spRequest, hr);

   long defDaysBeforeExp;
   hr = m_spRequest->get_DaysBeforeExpiration(&defDaysBeforeExp);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eSessionFlag defSessionFlags;
   hr = m_spRequest->get_SessionFlags(&defSessionFlags);
   Common::ThrowOnCOMError(m_spRequest, hr);

   // Set controls default values
   m_edIntraday.SetValue(defIntradayPeriod);
   CHelper::SetComboBoxItem(m_cmbHistorical, (DWORD_PTR)defHistoricalPeriod);
   CHelper::SetComboBoxItem(m_cmbTickFilter, (DWORD_PTR)defTickFilter);
   CHelper::SetComboBoxItem(m_cmbDailyBarClose, (DWORD_PTR)defDailyBarClose);
   m_UpdatesEnabled = defUpdatesEnabled;
   m_chkNoHistorical.EnableWindow(m_UpdatesEnabled);
   m_NoHistorical = defNoHistorical;
   m_IncludeEnd = defIncludeEnd;
   CHelper::SetComboBoxItem(m_cmbContinuation, (DWORD_PTR)defContinuation);
   m_EqualizeCloses = defEqualizeCloses;
   m_edDaysBeforeExp.SetValue(defDaysBeforeExp);
   SetSessionFlags(defSessionFlags);
   if (defSessionsFilter.vt == VT_UI4)
   {
      m_CustomSession = FALSE;
      m_edSessionsFilter.SetValue(defSessionsFilter.intVal);
      m_CustomSessionsFilter = _T("");
   }
   else if (defSessionsFilter.vt == VT_BSTR)
   {
      m_CustomSession = TRUE;
      m_edSessionsFilter.SetValue(0);
      m_CustomSessionsFilter = defSessionsFilter.bstrVal;
   }
   OnCbnSelChangeCmbHistorical();
   OnCbnSelChangeCmbContinuation();
   OnBnClickedChkCustomSession();

   m_edRangeStart.SetValue(0);
   m_RangeStartDate = m_LineTime - 1;
   m_edRangeEnd.SetValue(-10);
   m_RangeEndDate = m_LineTime;

   UpdateData(FALSE);
}

void CTimedBarsReqDlg::OnBnClickedOk()
{
   try
   {
      UpdateData();

      // Set request parameters
      ATL::CComBSTR symbol(m_Symbol);
      HRESULT hr = m_spRequest->put_Symbol(symbol);
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_RequestType == rtIntInt)
      {
         hr = m_spRequest->put_RangeStart(ATL::CComVariant(m_edRangeStart.GetValue()));
      }
      else
      {
         hr = m_spRequest->put_RangeStart(ATL::CComVariant(m_RangeStartDate.m_dt, VT_DATE));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_RequestType == rtDateDate)
      {
         hr = m_spRequest->put_RangeEnd(ATL::CComVariant(m_RangeEndDate.m_dt, VT_DATE));
      }
      else
      {
         hr = m_spRequest->put_RangeEnd(ATL::CComVariant(m_edRangeEnd.GetValue()));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_IncludeEnd(boolToVarBool(m_IncludeEnd));
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get historical period
      eHistoricalPeriod historicalPeriod = 
            static_cast<eHistoricalPeriod>(m_cmbHistorical.GetItemData(m_cmbHistorical.GetCurSel()));

      if (historicalPeriod == hpUndefined)
      {
         hr = m_spRequest->put_IntradayPeriod(m_edIntraday.GetValue());
      }
      else
      {
         hr = m_spRequest->put_HistoricalPeriod(historicalPeriod);
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      eTickFilter tickFilter = 
         static_cast<eTickFilter>(m_cmbTickFilter.GetItemData(m_cmbTickFilter.GetCurSel()));
      hr = m_spRequest->put_TickFilter(tickFilter);
      Common::ThrowOnCOMError(m_spRequest, hr);

      eDailyBarClose dailyBarClose =
         static_cast<eDailyBarClose>(m_cmbDailyBarClose.GetItemData(m_cmbDailyBarClose.GetCurSel()));
      hr = m_spRequest->put_DailyBarClose(dailyBarClose);
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get continuation type
      eTimeSeriesContinuationType continuationType = 
         static_cast<eTimeSeriesContinuationType>(m_cmbContinuation.GetItemData(m_cmbContinuation.GetCurSel()));

      hr = m_spRequest->put_Continuation(continuationType);
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_EqualizeCloses(boolToVarBool(m_EqualizeCloses));
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_DaysBeforeExpiration(m_edDaysBeforeExp.GetValue());
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_UpdatesEnabled(boolToVarBool(m_UpdatesEnabled));
      Common::ThrowOnCOMError(m_spRequest, hr);
      hr = m_spRequest->put_IgnoreEventsOnHistoricalBars(boolToVarBool(m_NoHistorical));
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_CustomSession)
      {
         hr = m_spRequest->put_SessionsFilter(ATL::CComVariant(m_CustomSessionsFilter));
      }
      else
      {
         hr = m_spRequest->put_SessionsFilter(ATL::CComVariant(m_edSessionsFilter.GetValue()));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_SessionFlags(GetSessionFlags());
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Close request window
      OnOK();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsReqDlg::InitializeHistoricalCombo()
{
   int row = m_cmbHistorical.AddString(_T("Undefined"));
   m_cmbHistorical.SetItemData(row, hpUndefined);

   row = m_cmbHistorical.AddString(_T("Daily"));
   m_cmbHistorical.SetItemData(row, hpDaily);

   row = m_cmbHistorical.AddString(_T("Weekly"));
   m_cmbHistorical.SetItemData(row, hpWeekly);

   row = m_cmbHistorical.AddString(_T("Monthly"));
   m_cmbHistorical.SetItemData(row, hpMonthly);

   row = m_cmbHistorical.AddString(_T("Quarterly"));
   m_cmbHistorical.SetItemData(row, hpQuarterly);

   row = m_cmbHistorical.AddString(_T("Semiannual"));
   m_cmbHistorical.SetItemData(row, hpSemiannual);

   row = m_cmbHistorical.AddString(_T("Yearly"));
   m_cmbHistorical.SetItemData(row, hpYearly);
}

void CTimedBarsReqDlg::InitializeTickFilterCombo()
{
   int row = m_cmbTickFilter.AddString(_T("All"));
   m_cmbTickFilter.SetItemData(row, tfAll);

   row = m_cmbTickFilter.AddString(_T("Ask"));
   m_cmbTickFilter.SetItemData(row, tfAsk);

   row = m_cmbTickFilter.AddString(_T("AskLow"));
   m_cmbTickFilter.SetItemData(row, tfAskLow);

   row = m_cmbTickFilter.AddString(_T("Bid"));
   m_cmbTickFilter.SetItemData(row, tfBid);

   row = m_cmbTickFilter.AddString(_T("BidHigh"));
   m_cmbTickFilter.SetItemData(row, tfBidHigh);

   row = m_cmbTickFilter.AddString(_T("Default"));
   m_cmbTickFilter.SetItemData(row, tfDefault);

   row = m_cmbTickFilter.AddString(_T("Settlement"));
   m_cmbTickFilter.SetItemData(row, tfSettlement);

   row = m_cmbTickFilter.AddString(_T("Trade"));
   m_cmbTickFilter.SetItemData(row, tfTrade);
}

void CTimedBarsReqDlg::InitializeDailyBarCloseCombo()
{
   int row = m_cmbDailyBarClose.AddString(_T("dbcDefault"));
   m_cmbDailyBarClose.SetItemData(row, dbcDefault);

   row = m_cmbDailyBarClose.AddString(_T("dbcLastQuote"));
   m_cmbDailyBarClose.SetItemData(row, dbcLastQuote);

   row = m_cmbDailyBarClose.AddString(_T("dbcLastQuoteOrSettlement"));
   m_cmbDailyBarClose.SetItemData(row, dbcLastQuoteOrSettlement);

   row = m_cmbDailyBarClose.AddString(_T("dbcSettlementOrLastQuote"));
   m_cmbDailyBarClose.SetItemData(row, dbcSettlementOrLastQuote);
}

void CTimedBarsReqDlg::InitializeContinuationCombo()
{
   int row = m_cmbContinuation.AddString(_T("NoContinuation"));
   m_cmbContinuation.SetItemData(row, tsctNoContinuation);

   row = m_cmbContinuation.AddString(_T("Standard"));
   m_cmbContinuation.SetItemData(row, tsctStandard);

   row = m_cmbContinuation.AddString(_T("StandardByMonth"));
   m_cmbContinuation.SetItemData(row, tsctStandardByMonth);

   row = m_cmbContinuation.AddString(_T("Active"));
   m_cmbContinuation.SetItemData(row, tsctActive);

   row = m_cmbContinuation.AddString(_T("ActiveByMonth"));
   m_cmbContinuation.SetItemData(row, tsctActiveByMonth);

   row = m_cmbContinuation.AddString(_T("Adjusted"));
   m_cmbContinuation.SetItemData(row, tsctAdjusted);

   row = m_cmbContinuation.AddString(_T("AdjustedByMonth"));
   m_cmbContinuation.SetItemData(row, tsctAdjustedByMonth);
}

void CTimedBarsReqDlg::OnBnClickedRdRequestType()
{
   try
   {
      RequestTypeChanged();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsReqDlg::OnCbnSelChangeCmbHistorical()
{
   try
   {
      // Get historical period
      eHistoricalPeriod historicalPeriod = 
         static_cast<eHistoricalPeriod>(m_cmbHistorical.GetItemData(m_cmbHistorical.GetCurSel()));

      m_edIntraday.EnableWindow(historicalPeriod == hpUndefined);
      m_spnIntraday.EnableWindow(historicalPeriod == hpUndefined);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsReqDlg::OnDeltaposSpnIntraday(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edIntraday.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CTimedBarsReqDlg::OnDeltaposSpnRangeStart(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRangeStart.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CTimedBarsReqDlg::OnDeltaposSpnRangeEnd(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRangeEnd.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CTimedBarsReqDlg::OnDeltaposSpnDaysBeforeExp(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edDaysBeforeExp.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CTimedBarsReqDlg::OnDeltaposSpnSessionsFilter(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edSessionsFilter.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CTimedBarsReqDlg::OnBnClickedChkCustomSession()
{
   try
   {
      UpdateData();

      m_spnSessionsFilter.ShowWindow(m_CustomSession ? SW_HIDE : SW_SHOW);
      m_edSessionsFilter.ShowWindow(m_CustomSession ? SW_HIDE : SW_SHOW);

      m_edCustomSessionsFilter.ShowWindow(m_CustomSession ? SW_SHOW : SW_HIDE);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsReqDlg::OnCbnSelChangeCmbContinuation()
{
   try
   {
      // Get continuation type
      eTimeSeriesContinuationType continuationType = 
         static_cast<eTimeSeriesContinuationType>(m_cmbContinuation.GetItemData(m_cmbContinuation.GetCurSel()));

      switch (continuationType)
      {
      case tsctNoContinuation:
      case tsctStandard:
      case tsctStandardByMonth:
         m_chkEqualizeCloses.EnableWindow(FALSE);
         m_edDaysBeforeExp.EnableWindow(FALSE);
         m_spnDaysBeforeExp.EnableWindow(FALSE);
         break;
      case tsctActive:
      case tsctActiveByMonth:
         m_chkEqualizeCloses.EnableWindow(TRUE);
         m_edDaysBeforeExp.EnableWindow(FALSE);
         m_spnDaysBeforeExp.EnableWindow(FALSE);
         break;
      default:
         m_chkEqualizeCloses.EnableWindow(TRUE);
         m_edDaysBeforeExp.EnableWindow(TRUE);
         m_spnDaysBeforeExp.EnableWindow(TRUE);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTimedBarsReqDlg::RequestTypeChanged()
{
   UpdateData();

   m_dtpRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_HIDE : SW_SHOW);
   m_edRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_SHOW : SW_HIDE);
   m_spnRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_SHOW : SW_HIDE);

   m_dtpRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_SHOW : SW_HIDE);
   m_edRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_HIDE : SW_SHOW);
   m_spnRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_HIDE : SW_SHOW);

   m_chkIncludeEnd.EnableWindow(m_RequestType == rtDateDate);
}

void CTimedBarsReqDlg::SetSessionFlags(
                              eSessionFlag sessionFlag)
{
   UpdateData();
   m_DailyFromIntraday = ((sessionFlag & sfDailyFromIntraday) == sfDailyFromIntraday);
   UpdateData(FALSE);
}

eSessionFlag CTimedBarsReqDlg::GetSessionFlags()
{
   UpdateData();

   int sessionFlag = sfUndefined;

   if (m_DailyFromIntraday)
   {
      sessionFlag = (sessionFlag | sfDailyFromIntraday);
   }

   return (eSessionFlag)sessionFlag;
}

void CTimedBarsReqDlg::OnBnClickedChkUpdatesEnabled()
{
   UpdateData();
   m_chkNoHistorical.EnableWindow(m_UpdatesEnabled);
}
