// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include "TradableInstruments.h"
#include "TradableInstrumentsDlg.h"
#include "Common.h"

void CTradableInstrumentsDlg::InitializeControls()
{
   OnDataConnectionStatusChanged(csConnectionDown);
   OnGWConnectionStatusChanged(csConnectionDown);

   m_cbFutures.SetCheck(1);
   m_cbPutOptions.SetCheck(0);
   m_cbCallOptions.SetCheck(0);
   m_cbTreasuries.SetCheck(0);
   m_cbOther.SetCheck(0);

   m_btnRemove.EnableWindow(FALSE);
   m_btnRemoveAll.EnableWindow(FALSE);

   m_lvAccounts.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_CHECKBOXES);
   m_lvAccounts.InsertColumn(0, _T("Check"), LVCFMT_LEFT, 20);
   m_lvAccounts.InsertColumn(1, _T("Name"), LVCFMT_LEFT, 130);
   m_lvAccounts.InsertColumn(2, _T("Abr."), LVCFMT_LEFT, 50);
}

void CTradableInstrumentsDlg::EnableControls(
   BOOL enable/* = TRUE*/)
{
   VARIANT_BOOL isStarted;
   isStarted = VARIANT_FALSE;

   HRESULT hr;
   if (m_spCQGCEL != NULL)
   {
      hr = m_spCQGCEL->get_IsStarted(&isStarted);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);
   }

   if (isStarted == VARIANT_FALSE)
   {
      m_btnGetCommodities.EnableWindow(FALSE);
      m_btnGetExchanges.EnableWindow(FALSE);
      m_btnGetFullnames.EnableWindow(FALSE);
      m_btnRemove.EnableWindow(FALSE);
      m_btnRemoveAll.EnableWindow(FALSE);
      return;
   }

   // Check connection status
   ATL::CComPtr<ICQGEnvironment> spEnv;
   hr = m_spCQGCEL->get_Environment(&spEnv);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   eConnectionStatus connStatus;
   hr = spEnv->get_DataConnectionStatus(&connStatus);
   Common::ThrowOnCOMError(spEnv, hr);

   bool connectionIsUp = (connStatus == csConnectionUp);

   bool enableGetButtons(enable && (m_lvAccounts.GetItemCount() > 0)
                         && (GetCheckedAccountsCount() > 0) && connectionIsUp);

   m_btnGetCommodities.EnableWindow(enableGetButtons);
   m_btnGetExchanges.EnableWindow(enableGetButtons);

   if (m_trFullNames.GetCount() > 0)
   {
      HTREEITEM hTrItem = m_trFullNames.GetSelectedItem();

      bool isRootnode = m_trFullNames.GetParentItem(hTrItem) == NULL;
      m_btnRemove.EnableWindow(enable && isRootnode && connectionIsUp);
      m_btnRemoveAll.EnableWindow(enable && connectionIsUp);
   }
   else
   {
      m_btnRemove.EnableWindow(FALSE);
      m_btnRemoveAll.EnableWindow(FALSE);
   }}

void CTradableInstrumentsDlg::InitializeCQGCEL()
{
   // create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   // Change configuration in order not to check Ready status, 
   // and let CQCCEL know that it is used from ATL client.
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   Common::ThrowOnCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   CString str;
   str.Format(_T("InitializeCQGCEL: %x \n"), static_cast<ICQGCEL*>(m_spCQGCEL));
   OutputDebugString(str);

   hr = m_spCQGCEL->Startup();
   Common::ThrowOnCOMError(m_spCQGCEL, hr);
}

BOOL CTradableInstrumentsDlg::IsDataConnectionUp() const
{
   // Check connection status   
   ATL::CComPtr<ICQGEnvironment> spEnv;
   HRESULT hr = m_spCQGCEL->get_Environment(&spEnv);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   eConnectionStatus connStatus;
   hr = spEnv->get_DataConnectionStatus(&connStatus);
   Common::ThrowOnCOMError(spEnv, hr);

   return (connStatus == csConnectionUp);
}

BOOL CTradableInstrumentsDlg::IsGWConnectionUp() const
{
   // Connection status can be accessed via environment.
   // So at first we need to get it from CQGCEL.
   ATL::CComPtr<ICQGEnvironment> spEnv;
   HRESULT hr = m_spCQGCEL->get_Environment(&spEnv);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   eConnectionStatus connStatus;
   hr = spEnv->get_GWConnectionStatus(&connStatus);
   Common::ThrowOnCOMError(spEnv, hr);

   // Check connection status
   return (connStatus == csConnectionUp);
}

CString CTradableInstrumentsDlg::ConnectionStatusToString(eConnectionStatus status)
{
   CString  statusString;

   switch (status)
   {
   case csConnectionDelayed:
      statusString = _T("delayed");
      break;
   case csConnectionDown:
      statusString = _T("down");
      break;
   case csConnectionUp:
      statusString = _T("up");
      break;
   case csConnectionTrouble:
      statusString = _T("trouble");
      break;
   case csConnectionNotLoggedOn:
      statusString = _T("not logged on");
      break;
   default:
      //assert(!"Unknown connection status");
      CString str;
      str.Format(_T("\nUnknown connection status: %d"), status);
      OutputDebugString(str);
   }

   return statusString;
}
void CTradableInstrumentsDlg::ShutdownCQGCEL()
{
   try
   {
      if (m_spCQGCEL != NULL)
      {
         //we need to shut it down before we can change the DOMUpdateMode
         HRESULT hr = ICQGCELDispEventImpl::DispEventUnadvise(m_spCQGCEL);
         Common::ThrowOnCOMError<ICQGCEL>(m_spCQGCEL, hr);

         hr = m_spCQGCEL->Shutdown();
         Common::ThrowOnCOMError<ICQGCEL>(m_spCQGCEL, hr);

         m_spCQGCEL.Release();
      }
   }
   catch (std::runtime_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}
void CTradableInstrumentsDlg::InsertAccount(
                                 const ATL::CComPtr<ICQGAccount>& spAccount)
{
   // Create string for specified account to add to the list
   ATL::CComBSTR accName;
   HRESULT hr = spAccount->get_GWAccountName(&accName);
   Common::ThrowOnCOMError(spAccount, hr);

   long accID;
   hr = spAccount->get_GWAccountID(&accID);
   Common::ThrowOnCOMError(spAccount, hr);

   CString accIDString;
   accIDString.Format(_T("%d"), accID);

   // Get last row to insert new account name to the end 
   int row = m_lvAccounts.GetItemCount();

   m_lvAccounts.InsertItem(row, _T(""));
   m_lvAccounts.SetItemText(row, 1, ATL::CW2T(accName));
   m_lvAccounts.SetItemText(row, 2, accIDString);
}

void CTradableInstrumentsDlg::CheckAllAccounts(
                                 bool check)
{
   int count = m_lvAccounts.GetItemCount();
   for (int i = 0; i < count; ++i)
   {
      m_lvAccounts.SetCheck(i, check);
   }
}

int CTradableInstrumentsDlg::GetCheckedAccountsCount() const
{
   int count = m_lvAccounts.GetItemCount();
   int checkedCount = 0;
   for (int i = 0; i < count; ++i)
   {
      if (m_lvAccounts.GetCheck(i))
      {
         ++checkedCount;
      }
   }
   return checkedCount;
}

void CTradableInstrumentsDlg::GetTypes(long& instrumentTypes) const
{
   // NOTE: instrument type is a bit mask, here we collect all types
   // which user has selected to request for commodity.
   // User can select tradable only check box to get only instruments which
   // are tradable. This requires connection with Gateway server.

   instrumentTypes = itUndefined;

   if (m_cbCallOptions.GetCheck() && m_cbPutOptions.GetCheck() &&
         m_cbFutures.GetCheck() &&  m_cbTreasuries.GetCheck() && m_cbOther.GetCheck())
   {
      instrumentTypes = itAllInstruments;
      return;
   }

   if (m_cbCallOptions.GetCheck()) 
      instrumentTypes |= itOptionCall;

   if (m_cbPutOptions.GetCheck())
      instrumentTypes |= itOptionPut;

   if (m_cbFutures.GetCheck())
      instrumentTypes |=  itFuture;

   if (m_cbTreasuries.GetCheck()) 
      instrumentTypes |=  itTreasure;

   if (m_cbOther.GetCheck()) 
      instrumentTypes |=  itOther;   
}

void CTradableInstrumentsDlg::AddInstrumentTypesToNode(
                                 HTREEITEM htvNode,
                                 eInstrumentType type)
{
   // NOTE: Instrument type is a bit mask
   if ((type & itAllInstruments) == itAllInstruments)
   {
      m_trFullNames.InsertItem(_T("All instruments"), htvNode);
      return ;
   }

   CString typeString;
   if ((type & itAllOptions) == itAllOptions)
   {
      m_trFullNames.InsertItem(_T("All options"), htvNode);
   }
   else
   {
      if ((type & itOptionCall) == itOptionCall)
         m_trFullNames.InsertItem(_T("Calls"), htvNode);

      if ((type & itOptionPut) == itOptionPut)
         m_trFullNames.InsertItem(_T("Puts"), htvNode);
   }

   if ((type & itFuture) == itFuture)
      m_trFullNames.InsertItem(_T("Futures"), htvNode);

   if ((type & itStock) == itStock)
      m_trFullNames.InsertItem(_T("Stocks"), htvNode);

   if ((type & itTreasure) == itTreasure)
      m_trFullNames.InsertItem(_T("Fixed Income"), htvNode);

   if ((type & itOther) == itOther)
      m_trFullNames.InsertItem(_T("Other"), htvNode);

   return;
}

void CTradableInstrumentsDlg::AddInstrumentNamesToNode(
                                 HTREEITEM htvNode,
                                 ICQGCommodityInstruments* instrumentNames)
{
   try
   {
      HRESULT hr;
      // Add all resolved instruments as subnodes to list
      long count;
      hr = instrumentNames->get_Count(&count);
      Common::ThrowOnCOMError<ICQGCommodityInstruments>(instrumentNames, hr);

      for (long i = 0; i < count; ++i)
      {
         ATL::CComBSTR instrumentName;
         hr = instrumentNames->get_Item(i, &instrumentName);
         Common::ThrowOnCOMError<ICQGCommodityInstruments>(instrumentNames, hr);

         m_trFullNames.InsertItem(ATL::CW2T(instrumentName), htvNode);
      }

      // Enable remove all button
      m_btnRemoveAll.EnableWindow(TRUE);
   }
   catch (std::runtime_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
   return;
}

void CTradableInstrumentsDlg::ReportError(ICQGError* err)
{
   // Showing message box with error description which 
   // occurs during exchanges or commdities request process
   long errCode;
   HRESULT hr = err->get_Code(&errCode);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   ATL::CComBSTR errDesc;
   hr = err->get_Description(&errDesc);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   // Create error message
   CString codeString;
   codeString.Format(_T("%d"), errCode);

   CString errMsg(_T("Error "));
   errMsg += _T(" : ");
   errMsg += ATL::CW2T(errDesc);
   AfxMessageBox(errMsg, MB_ICONASTERISK);
}