// EventHandlersCQGCEl.cpp : implementation file
//

#include "stdafx.h"
#include "TradableInstruments.h"
#include "TradableInstrumentsDlg.h"
#include "Common.h"
#include <vector>


STDMETHODIMP CTradableInstrumentsDlg::OnCELStarted()
{
   // Disable controls until data connection goes up
   EnableControls(FALSE);

   m_CELIsStarted = true;
   OutputDebugString(_T("OnCELStarted\n"));
   return S_OK;
}

STDMETHODIMP CTradableInstrumentsDlg::OnDataError(
   LPDISPATCH /*obj*/,
   BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CTradableInstrumentsDlg::OnDataConnectionStatusChanged(
   eConnectionStatus newStatus)
{
   CString status;
   switch (newStatus)
   {
   case csConnectionDelayed:
      status = _T("delayed");
      break;
   case csConnectionDown:
      status = _T("down");
      break;
   case csConnectionUp:
      status = _T("up");
      break;
   //default:
      //AfxMessageBox(!"Unexpected Data connection status");
   }

   m_lblDataConnection.SetWindowText(_T("Data connection ") + status);

   // Enable/disable buttons
   m_btnGetFullnames.EnableWindow((m_edSymbol.GetWindowTextLength() > 0) && 
      (newStatus == csConnectionUp));
   m_edSymbol.EnableWindow(newStatus == csConnectionUp);

   return S_OK;
}

STDMETHODIMP CTradableInstrumentsDlg::OnGWConnectionStatusChanged(
   eConnectionStatus newStatus)
{
   m_lblGWConnection.SetWindowText(_T("GW connection : ") + ConnectionStatusToString(newStatus));

   if (newStatus == csConnectionUp)
   {
      m_spCQGCEL->put_AccountSubscriptionLevel(aslAccountsOnly);
   }

   EnableControls();

   return S_OK;
}

STDMETHODIMP CTradableInstrumentsDlg::OnLineTimeChanged(
   DATE new_line_time)
{
   try
   {
      m_lblLineTime.SetWindowText(DateTimeToString(new_line_time,false,true,true));
   } 
   catch (std::runtime_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
      return E_FAIL;
   }
   return S_OK;
}

STDMETHODIMP CTradableInstrumentsDlg::OnAccountChanged(
   eAccountChangeType change, 
   ICQGAccount* /*account*/, 
   ICQGPosition* /*position*/)
{
   try
   {
      if (change != actAccountsReloaded)
      {
         return S_OK;
      }

      ATL::CComPtr<ICQGAccounts> spAccounts;
      HRESULT hr = m_spCQGCEL->get_Accounts(&spAccounts);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      long count;
      hr = spAccounts->get_Count(&count);
      Common::ThrowOnCOMError(spAccounts, hr);

      if (count == 0)
      {
         // No accounts
         return S_OK;
      }

      // Iterate over accounts collection and add them to list
      ATL::CComPtr<IEnumVARIANT> spEnumVariant;
      hr = spAccounts->get__NewEnum(&spEnumVariant);
      Common::ThrowOnCOMError(spAccounts, hr);

      std::vector<ATL::CComVariant> pItems(count);
      ULONG celtFetched;
      hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
      Common::ThrowOnCOMError(spEnumVariant, hr);

      for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
      {
         ATLASSERT(it->vt == VT_DISPATCH);

         ATL::CComPtr<ICQGAccount> spAccount;
         hr = it->pdispVal->QueryInterface(__uuidof(ICQGAccount), (void** )&spAccount);
         Common::ThrowOnCOMError<IDispatch>(it->pdispVal, hr);

         // Add current account to list
         InsertAccount(spAccount);
      }
   }
   catch (std::runtime_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTradableInstrumentsDlg::OnTradableCommoditiesResolved(
   long gwAccountID, 
   ICQGCommodities* commodityNames, 
   ICQGError* err)
{
   try
   {
      if (err != NULL)
      {
         ReportError(err);
         // Note, that CQGCEL is not concerned with errors in the events
         // so we always return S_OK
         return S_OK;
      }
      InsertTradableObjectTreeNode(gwAccountID, commodityNames, m_trCommodities, m_spCQGCEL);
   }
   catch (const std::runtime_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
   return S_OK;
}

STDMETHODIMP CTradableInstrumentsDlg::OnTradableExchangesResolved(
   long gwAccountID, 
   ICQGExchanges* exchangesNames, 
   ICQGError* err)
{
   try
   {
      if (err != NULL)
      {
         ReportError(err);
         // Note, that CQGCEL is not concerned with errors in the events
         // so we always return S_OK
         return S_OK;
      }
      InsertTradableObjectTreeNode(gwAccountID, exchangesNames, m_trExchanges, m_spCQGCEL);
   }
   catch (const std::runtime_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
   return S_OK;
}

STDMETHODIMP CTradableInstrumentsDlg::OnCommodityInstrumentsResolved(
   BSTR commodityName, 
   eInstrumentType instrumentTypes, 
   ICQGCommodityInstruments* instrumentNames)
{
   try
   {
      // Create node with commodity name
      CString nodeName;
      nodeName = ATL::CW2A(commodityName);

      // Add node
      HTREEITEM hTrItem = m_trFullNames.InsertItem(nodeName);
      AddInstrumentTypesToNode(hTrItem,instrumentTypes);
      AddInstrumentNamesToNode(hTrItem,instrumentNames);
      m_trFullNames.Expand(hTrItem,TVE_EXPAND);
   }
   catch (std::runtime_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
   return S_OK;
}
