// SymbolResolutionDlg.h : header file
//

#pragma once

class CTradableInstrumentsDlg;
/**
\typedef ATL::IDispEventImpl<1, CTradableInstrumentsDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CTradableInstrumentsDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

/// \brief CTradableInstrumentsDlg dialog
class CTradableInstrumentsDlg : public CDialog, public ICQGCELDispEventImpl
{
public:

   CTradableInstrumentsDlg(
                     CWnd* pParent = NULL);

   /// \brief Dialog Data
   enum { IDD = IDD_TRADEABLEINSTRUMENTS_DIALOG };

   /// \brief Closes dialog with IDCANCEL result
   afx_msg HCURSOR OnQueryDragIcon();
   afx_msg void OnCheckBoxClick();
   afx_msg void OnPaint();
   afx_msg void OnDestroy();
   afx_msg void OnBnClickedBtnGetCommodities();
   afx_msg void OnBnClickedBtnGetexchanges();
   afx_msg void OnBnClickedBtnGetFullnames();
   afx_msg void OnBnClickedBtnRemove();
   afx_msg void OnBnClickedBtnRemoveAll();
   afx_msg void OnBnClickedBtnSelectAllDSs();
   afx_msg void OnBnClickedBtnUnselectAllDSs();
   afx_msg void OnEnChangeEdSymbol();
   afx_msg void OnLvnItemChangedLsvAccounts(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnTvnSelChangedTrCommodities(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnTvnSelChangedTrFullNames(NMHDR *pNMHDR, LRESULT *pResult);
   DECLARE_MESSAGE_MAP()

protected:

   /// \brief Icon on the form
   HICON m_hIcon;

   /// \brief Generated message map functions
   virtual void DoDataExchange(CDataExchange* pDX);   // DDX/DDV support

   /// \brief The system calls this function to obtain the cursor to display while the user drags
   /// \brief the minimized window.
   virtual BOOL OnInitDialog();

   /// \brief Override to catch Enter for edit controls
   virtual BOOL PreTranslateMessage(MSG* pMsg);

   /// \brief Used to unadvise from CQCEL events and shutdown it
   virtual void OnCancel();

   /// \brief Override to prevent dialog closing on Enter
   virtual void OnOK();

private:

   /// \brief Get Commodities button
   CButton m_btnGetCommodities;
   /// \brief Get Exchanges button
   CButton m_btnGetExchanges;
   /// \brief Get Fullnames button
   CButton m_btnGetFullnames;
   /// \brief Remove button
   CButton m_btnRemove;
   /// \brief Remove all button
   CButton m_btnRemoveAll;
   /// \brief Futures instruments type checkbox state
   CButton m_cbFutures;
   /// \brief Put options instruments type checkbox state
   CButton m_cbPutOptions;
   /// \brief Call Options instruments type checkbox state
   CButton m_cbCallOptions;
   /// \brief Treasuries instruments type checkbox state
   CButton m_cbTreasuries;
   /// \brief Other instruments checkbox state
   CButton m_cbOther;
   /// \brief Symbol edit box
   CEdit m_edSymbol;
   /// \brief Data connection label
   CStatic m_lblDataConnection;
   /// \brief Gatewsy connection label
   CStatic m_lblGWConnection;
   /// \brief LineTime label
   CStatic m_lblLineTime;
   /// \brief Accounts tree
   CListCtrl m_lvAccounts;
   /// \brief Commodities tree
   CTreeCtrl m_trCommodities;
   /// \brief full name tree
   CTreeCtrl m_trFullNames;
   /// \brief Exchanges name tree
   CTreeCtrl m_trExchanges;

private:

   /// \brief Initialize all controls
   void InitializeControls();

   /// \brief Enable/disables controls. Connection affects controls status.
   /// \param enable - Controls status
   void EnableControls(BOOL enable = TRUE);

   /// \brief Initialize CQGCEL object, and starts the CQGCEL
   void InitializeCQGCEL();

   /// \brief Checks if the connection with CQG data line is up
   /// \return Returns true if connection is up
   BOOL IsDataConnectionUp() const;

   /// \brief Checks if the connection with CQG Gateway server is up
   /// \return Returns true if connection is up
   BOOL IsGWConnectionUp() const;

   /// \brief Returns string representation of the connection status.    
   /// \param status - connection status
   /// \return String representation of the connection status
   static CString ConnectionStatusToString(
                                 eConnectionStatus status);

   /// \brief disengage CQGCEL and shut it down.
   void ShutdownCQGCEL();

   /// \brief Adds account to the accounts list
   /// \param spAccount - Account to add
   void InsertAccount( const ATL::CComPtr<ICQGAccount>& spAccount );

   /// \brief change checked state according to parameter passed
   /// \param check
   void CheckAllAccounts( bool check);

   /// \brief get count of checked states
   int GetCheckedAccountsCount() const;

   void AddInstrumentTypesToNode(HTREEITEM htvNode,
                                 eInstrumentType type);

   void AddInstrumentNamesToNode(HTREEITEM htvNode,
                                 ICQGCommodityInstruments* instrumentNames);

   /// \brief Counts the checked accounts in the accounts list
   /// \param type - instrumentTypes
   void GetTypes(long& instrumentTypes) const;

   /// \brief Returns string representation of the specified instrument type
   /// \param type - instrument type
   /// \return Instrument type string
   static CString InstrumentTypeToString( eInstrumentType type);

   /// \brief Reports Error occurred during exchanges or commodities request process
   /// \param err - ICQGError pointer occurred during request process or 0 if no error occurred
   void ReportError(ICQGError* err);

public:

   /// \brief This map is used to declare handler function for event specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CTradableInstrumentsDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000000a, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000000b, OnCELStarted)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000003, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000002, OnGWConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000001, OnLineTimeChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000007, OnAccountChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000015, OnTradableCommoditiesResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000000d, OnCommodityInstrumentsResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000052, OnTradableExchangesResolved)
   END_SINK_MAP()

protected:

   /// \brief OnCELStarted is fired when CQGCEL object 
   /// \brief is successfully started up.
   STDMETHOD(OnCELStarted)();

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - String describing the error
   /// \return S_OK on success
   STDMETHOD(OnDataError)(
      LPDISPATCH obj, 
      BSTR errorDescription);

   /// \brief Fired when some changes occurred 
   /// \brief in the connection with the CQG data server.
   /// \param newStatus - New connection status
   /// \return S_OK on success
   STDMETHOD(OnDataConnectionStatusChanged)(eConnectionStatus newStatus);

   /// \brief Fired when some changes occurred
   /// \brief in the connection with the CQG Gateway is changed.
   /// \param newStatus - New connection status
   /// \return S_OK
   STDMETHOD(OnGWConnectionStatusChanged)(
                        eConnectionStatus newStatus);

   /// \brief OnCELStarted is fired when CQGCEL object 
   /// \brief is successfully started up.
   STDMETHOD(OnLineTimeChanged)(DATE new_line_time);

   /// \brief Fired when new account is added, changed or removed.
   /// \param change - Account change type
   /// \param Account - Changed account instance
   /// \param position - Changed position instance
   /// \return S_OK
   STDMETHOD(OnAccountChanged)(
                        eAccountChangeType change,
                        ICQGAccount* account,
                        ICQGPosition* position);

   /// \brief Fired when list of tradable commodities is resolved.
   /// \param GWAccountID - Account Gateway ID which commodities were requested
   /// \param commodityNames - Collection of resolved commodities
   /// \param err - CQGError instance representing the error occurred
   /// \param while processing the time series request
   /// \return S_OK
   STDMETHOD(OnTradableCommoditiesResolved)(
                        long gwAccountID,
                        ICQGCommodities* commodityNames,
                        ICQGError* err);

   /// \brief Fired when list of tradable exchanges is resolved.
   /// \param GWAccountID - Account Gateway ID which exchanges were requested
   /// \param exchangeNames - Collection of resolved exchanges
   /// \param err - CQGError instance representing the error occurred
   /// \param while processing the time series request
   /// \return S_OK
   STDMETHOD(OnTradableExchangesResolved)(
                         long gwAccountID,
                         ICQGExchanges* exchangeNames,
                         ICQGError* err);

   /// \brief Fired when instruments list for requested commodity is resolved.
   /// \param commodityName - Requested symbol
   /// \param instrumentTypes - Requested instruments types
   /// \param instrumentNames - Collection of resolved instruments
   /// \return S_OK
   STDMETHOD(OnCommodityInstrumentsResolved)(
                        BSTR commodityName,
                        eInstrumentType instrumentTypes,
                        ICQGCommodityInstruments* instrumentNames);

private:

   /// \brief CQGCEL object
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Selected data sources object.
   ATL::CComPtr<ICQGAccounts> m_spAccounts;

   /// \brief Selected data source symbols object.
   ATL::CComPtr<ICQGDataSourceSymbols> m_spAccountSymbols;

   bool m_CELIsStarted;
};

