// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include <cassert>
#include <algorithm>
#include "Common.h"
#include "Helper.h"
#include "TradingSystem.h"
#include "TradingSystemDlg.h"

void CTradingSystemDlg::InitializeCQGCEL()
{
   // Create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   Common::ThrowOnCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   Common::ThrowOnCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   Common::ThrowOnCOMError(m_spCQGCEL, hr);
}

void CTradingSystemDlg::InitializeControls()
{
   InitializeTab();
   InitializeTradesList(m_lvLastTrades);
   InitializeTradesList(m_lvTrades);

   InitializeStatisticsList(m_lvLastStatistics);
   InitializeStatisticsList(m_lvStatistics);

   InitializeRelationsList(m_lvLastTradeItems);
   InitializeRelationsList(m_lvTradeItems);

   ClearAllData();

   InitializeDefParametersList();
   InitializeDefTradesList();
   InitializeDefEntryExitsList();

   InitializeParametersList();

   OnDataConnectionStatusChanged(csConnectionDown);

   ShowListViews();
}

void CTradingSystemDlg::InitializeDefParametersList()
{
   m_lvDefParameters.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvDefParameters.ModifyStyle(0, LVS_SINGLESEL);

   m_lvDefParameters.InsertColumn(dpcName, _T("Name"), LVCFMT_LEFT, 100);
   m_lvDefParameters.InsertColumn(dpcComment, _T("Comment"), LVCFMT_LEFT, 160);
   m_lvDefParameters.InsertColumn(dpcType, _T("Type"), LVCFMT_LEFT, 50);
   m_lvDefParameters.InsertColumn(dpcDefaultValue, _T("Def. Value"), LVCFMT_LEFT, 70);

   COLORREF crBkColor = ::GetSysColor(COLOR_3DFACE);
   m_lvDefParameters.SetBkColor(crBkColor);
   m_lvDefParameters.SetTextBkColor(crBkColor);
}

void CTradingSystemDlg::InitializeDefTradesList()
{
   m_lvDefTrades.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvDefTrades.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   m_lvDefTrades.InsertColumn(dtcName, _T("Name"), LVCFMT_LEFT, 100);
   m_lvDefTrades.InsertColumn(dtcSide, _T("Side"), LVCFMT_LEFT, 100);

   COLORREF crBkColor = ::GetSysColor(COLOR_3DFACE);
   m_lvDefTrades.SetBkColor(crBkColor);
   m_lvDefTrades.SetTextBkColor(crBkColor);
}

void CTradingSystemDlg::InitializeDefEntryExitsList()
{
   m_lvDefEntryExits.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvDefEntryExits.ModifyStyle(0, LVS_SINGLESEL);

   m_lvDefEntryExits.InsertColumn(dteecName, _T("Name"), LVCFMT_LEFT, 200);
   m_lvDefEntryExits.InsertColumn(dteecValue, _T("Value"), LVCFMT_LEFT, 65);

   COLORREF crBkColor = ::GetSysColor(COLOR_3DFACE);
   m_lvDefEntryExits.SetBkColor(crBkColor);
   m_lvDefEntryExits.SetTextBkColor(crBkColor);
}

void CTradingSystemDlg::InitializeParametersList()
{
   m_lvParameters.InsertColumn(pcName, _T("Name"), LVCFMT_LEFT, 200);
   m_lvParameters.InsertColumn(pcValue, _T("Value"), LVCFMT_LEFT, 100);

   COLORREF crBkColor = ::GetSysColor(COLOR_3DFACE);
   m_lvParameters.SetBkColor(crBkColor);
   m_lvParameters.SetTextBkColor(crBkColor);
}

void CTradingSystemDlg::InitializeTradesList(
                        CListCtrl& listCtrl)
{
   listCtrl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   listCtrl.ModifyStyle(0, LVS_SINGLESEL);

   listCtrl.InsertColumn(trcIndex, _T("#"), LVCFMT_LEFT, 40);
   listCtrl.InsertColumn(trcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 150);
   listCtrl.InsertColumn(trcIsActive, _T("IsActive"), LVCFMT_LEFT, 100);
   listCtrl.InsertColumn(trcEntrySignal, _T("Entry Signal"), LVCFMT_LEFT, 100);
   listCtrl.InsertColumn(trcEntryPrice, _T("Entry Price"), LVCFMT_LEFT, 100);
   listCtrl.InsertColumn(trcEntryStopLimitPrice, _T("Entry SL Price"), LVCFMT_LEFT, 100);
   listCtrl.InsertColumn(trcEntryQuantity, _T("Entry Quantity"), LVCFMT_LEFT, 100);
}

void CTradingSystemDlg::InitializeStatisticsList(
                        CListCtrl& listCtrl)
{
   listCtrl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   listCtrl.ModifyStyle(0, LVS_SINGLESEL);

   listCtrl.InsertColumn(srcIndex, _T("#"), LVCFMT_LEFT, 40);
   listCtrl.InsertColumn(srcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 100);
}

void CTradingSystemDlg::InitializeRelationsList(
                        CListCtrl& listCtrl)
{
   listCtrl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   listCtrl.ModifyStyle(0, LVS_SINGLESEL);

   listCtrl.InsertColumn(rcTimestamp, _T("Timestamp"), LVCFMT_LEFT, 150);
   listCtrl.InsertColumn(rcName, _T("Name"), LVCFMT_LEFT, 140);
}

void CTradingSystemDlg::InitializeTab()
{
   m_tabTradeStatistic.InsertItem(toTrades, _T("Trades"));
   m_tabTradeStatistic.InsertItem(toStatistics, _T("Statistics"));
}

bool CTradingSystemDlg::IsTradingSystemSelected(
                        const ATL::CComPtr<ICQGTradingSystem>& spTradingSystem)
{
   if (!m_spTradingSystem)
   {
      return false;
   }

   // Get selected trading system ID
   ATL::CComBSTR selTradingSystemID;
   HRESULT hr = m_spTradingSystem->get_Id(&selTradingSystemID);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   // Get added trading system ID
   ATL::CComBSTR tradingSystemID;
   hr = spTradingSystem->get_Id(&tradingSystemID);
   Common::ThrowOnCOMError(spTradingSystem, hr);

   return (selTradingSystemID == tradingSystemID);
}

void CTradingSystemDlg::ClearAllData()
{
   m_lblStatus.SetWindowText(_T(""));

   ClearRequestParams();

   ClearTags();

   ClearIndices();

   m_cmbTrades.ResetContent();
   m_cmbLastTradeItems.ResetContent();
   m_cmbTradeItems.ResetContent();

   ClearRecords(m_lvLastTrades);
   ClearRecords(m_lvTrades);

   ClearTradeHeaders(m_lvLastTrades);
   ClearTradeHeaders(m_lvTrades);

   ClearRecords(m_lvLastStatistics);
   ClearRecords(m_lvStatistics);

   ClearStatisticsHeaders(m_lvLastStatistics);
   ClearStatisticsHeaders(m_lvStatistics);

   ClearRecords(m_lvLastTradeItems);
   ClearRecords(m_lvTradeItems);

   m_lblStatsAllRecsCount.SetWindowText(_T(""));
   m_lblStatsFltrRecsCount.SetWindowText(_T(""));

   m_lblTradesAllRecsCount.SetWindowText(_T(""));
   m_lblTradesFltrRecsCount.SetWindowText(_T(""));
}

void CTradingSystemDlg::ClearAllDefinitionsData()
{
   m_cmbDefinition.ResetContent();
   ClearDefinitionData();

   m_edDefinitionName.SetWindowText(_T(""));
}

void CTradingSystemDlg::ClearDefinitionData()
{
   m_lvDefParameters.DeleteAllItems();
   m_lvDefTrades.DeleteAllItems();
   m_lvDefEntryExits.DeleteAllItems();
   m_lblDefAbbreviation.SetWindowText(_T(""));
   m_lblDefCommission.SetWindowText(_T(""));
   m_lblDefCommissionType.SetWindowText(_T(""));
}

void CTradingSystemDlg::ClearRequestParams()
{
   m_lvParameters.DeleteAllItems();

   UpdateData();

   m_DefinitionName = _T("");
   m_BaseExpression = _T("");
   m_RangeStart = _T("");
   m_RangeEnd = _T("");
   m_IntradayPeriod = _T("");
   m_HistoricalPeriod = _T("");
   m_IncludeEnd = _T("");
   m_SessionsFilter = _T("");
   m_Continuation = _T("");
   m_EqualizeCloses = _T("");
   m_DaysBeforeExpiration = _T("");
   m_SessionFlags = _T("");
   m_SubscriptionLevel = _T("");
   m_RecalcPeriod = _T("");

   UpdateData(FALSE);

   m_lblError.SetWindowText(_T(""));
}

void CTradingSystemDlg::ClearTags()
{
   m_edTagName.SetWindowText(_T(""));
   m_edTagValue.SetWindowText(_T(""));
}

void CTradingSystemDlg::ClearTradeHeaders(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();

   const int columnCount = listCtrl.GetHeaderCtrl()->GetItemCount() - trcExitsStart;

   // Delete headers
   for (int i = 0; i < columnCount; ++i)
   {
      listCtrl.DeleteColumn(trcExitsStart);
   }

   m_TradeExitsMap.clear();
}

void CTradingSystemDlg::ClearRecords(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();
}

void CTradingSystemDlg::ShowDefinitions()
{
   // Fill up new data
   m_cmbDefinition.ResetContent();

   long count;
   HRESULT hr = m_spTradingSystemDefinitions->get_Count(&count);
   Common::ThrowOnCOMError(m_spTradingSystemDefinitions, hr);

   for (long i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGTradingSystemDefinition> spTradingSystemDefinition;
      hr = m_spTradingSystemDefinitions->get_Item(i, &spTradingSystemDefinition);
      Common::ThrowOnCOMError(m_spTradingSystemDefinitions, hr);

      ATL::CComBSTR name;
      hr = spTradingSystemDefinition->get_Name(&name);
      Common::ThrowOnCOMError(spTradingSystemDefinition, hr);

      const int row = m_cmbDefinition.AddString(ATL::CW2T(name));
      m_cmbDefinition.SetItemData(row, i);
   }

   if (0 < count)
   {
      // Select first definition
      m_cmbDefinition.SetCurSel(0);
      OnCbnSelchangeCmbDefinition();
   }
}

void CTradingSystemDlg::ShowDefinitionParameters(
                        const ATL::CComPtr<ICQGTradingSystemDefinition>& spTradingSystemDefinition)
{
   ATL::CComPtr<ICQGParameterDefinitions> spParameterDefinitions;
   HRESULT hr = spTradingSystemDefinition->get_ParameterDefinitions(&spParameterDefinitions);
   Common::ThrowOnCOMError(spTradingSystemDefinition, hr);

   long count;
   hr = spParameterDefinitions->get_Count(&count);
   Common::ThrowOnCOMError(spParameterDefinitions, hr);

   for (long i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGParameterDefinition> spParameterDefinition;
      hr = spParameterDefinitions->get_Item(i, &spParameterDefinition);
      Common::ThrowOnCOMError(spParameterDefinitions, hr);

      ATL::CComBSTR name;
      hr = spParameterDefinition->get_Name(&name);
      Common::ThrowOnCOMError(spTradingSystemDefinition, hr);

      ATL::CComBSTR comment;
      hr = spParameterDefinition->get_Comment(&comment);
      Common::ThrowOnCOMError(spParameterDefinition, hr);

      eUserFormulaParameterType parameterType;
      hr = spParameterDefinition->get_Type(&parameterType);
      Common::ThrowOnCOMError(spParameterDefinition, hr);

      ATL::CComVariant defaultValue;
      hr = spParameterDefinition->get_DefaultValue(&defaultValue);
      Common::ThrowOnCOMError(spParameterDefinition, hr);

      const int row = m_lvDefParameters.InsertItem(m_lvDefParameters.GetItemCount(), _T(""));
      m_lvDefParameters.SetItemText(row, dpcName, ATL::CW2T(name));
      m_lvDefParameters.SetItemText(row, dpcComment, ATL::CW2T(comment));
      m_lvDefParameters.SetItemText(row, dpcType, CHelper::ParameterTypeToString(parameterType));
      m_lvDefParameters.SetItemText(row, dpcDefaultValue, 
         Common::VariantToString(defaultValue, CHelper::PARAM_FRAC_PART_LENGTH));
   }
}

void CTradingSystemDlg::ShowDefinitionTrades(
                        const ATL::CComPtr<ICQGTradingSystemDefinition>& spTradingSystemDefinition)
{
   ATL::CComPtr<ICQGTradingSystemTradeDefinitions> spTradeDefinitions;
   HRESULT hr = spTradingSystemDefinition->get_TradeDefinitions(&spTradeDefinitions);
   Common::ThrowOnCOMError(spTradingSystemDefinition, hr);

   long count;
   hr = spTradeDefinitions->get_Count(&count);
   Common::ThrowOnCOMError(spTradeDefinitions, hr);

   for (long i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGTradingSystemTradeDefinition> spTradeDefinition;
      hr = spTradeDefinitions->get_Item(i, &spTradeDefinition);
      Common::ThrowOnCOMError(spTradeDefinitions, hr);

      ATL::CComBSTR name;
      hr = spTradeDefinition->get_Name(&name);
      Common::ThrowOnCOMError(spTradeDefinition, hr);

      eTradeSide tradeSide;
      hr = spTradeDefinition->get_Side(&tradeSide);
      Common::ThrowOnCOMError(spTradeDefinition, hr);

      const int row = m_lvDefTrades.InsertItem(m_lvDefTrades.GetItemCount(), _T(""));
      m_lvDefTrades.SetItemText(row, dtcName, ATL::CW2T(name));
      m_lvDefTrades.SetItemText(row, dtcSide, CHelper::TradeSideToString(tradeSide));
      m_lvDefTrades.SetItemData(row, i);
   }

   if (0 < count)
   {
      // Select first trade definition
      m_lvDefTrades.SetItemState(0, LVIS_SELECTED, LVIS_SELECTED);
   }
}

void CTradingSystemDlg::ShowDefinitionTradeEntryExits(
                        const ATL::CComPtr<ICQGTradingSystemTradeDefinition>& spTradeDefinition)
{
   // Show entry parameters
   ATL::CComPtr<ICQGTradeEntryDefinition> spTradeEntryDefinition;
   HRESULT hr = spTradeDefinition->get_Entry(&spTradeEntryDefinition);
   Common::ThrowOnCOMError(spTradeDefinition, hr);

   eOrderType orderType;
   hr = spTradeEntryDefinition->get_OrderType(&orderType);
   Common::ThrowOnCOMError(spTradeEntryDefinition, hr);

   VARIANT_BOOL allowEntryOnExit;
   hr = spTradeEntryDefinition->get_AllowEntryOnExit(&allowEntryOnExit);
   Common::ThrowOnCOMError(spTradeEntryDefinition, hr);

   VARIANT_BOOL allowPyramids;
   hr = spTradeEntryDefinition->get_AllowPyramids(&allowPyramids);
   Common::ThrowOnCOMError(spTradeEntryDefinition, hr);

   int row = m_lvDefEntryExits.InsertItem(m_lvDefEntryExits.GetItemCount(), _T(""));
   m_lvDefEntryExits.SetItemText(row, dteecName, _T("OrderType (Entry)"));
   m_lvDefEntryExits.SetItemText(row, dteecValue, CHelper::OrderTypeToString(orderType));

   row = m_lvDefEntryExits.InsertItem(m_lvDefEntryExits.GetItemCount(), _T(""));
   m_lvDefEntryExits.SetItemText(row, dteecName, _T("AllowEntryOnExit (Entry)"));
   m_lvDefEntryExits.SetItemText(row, dteecValue, GetValueAsString(allowEntryOnExit, VT_BOOL));

   row = m_lvDefEntryExits.InsertItem(m_lvDefEntryExits.GetItemCount(), _T(""));
   m_lvDefEntryExits.SetItemText(row, dteecName, _T("AllowPyramids (Entry)"));
   m_lvDefEntryExits.SetItemText(row, dteecValue, GetValueAsString(allowPyramids, VT_BOOL));

   // Show exits parameters
   ATL::CComPtr<ICQGTradeExitDefinitions> spTradeExitDefinitions;
   hr = spTradeDefinition->get_Exits(&spTradeExitDefinitions);
   Common::ThrowOnCOMError(spTradeDefinition, hr);

   long exitsCount;
   hr = spTradeExitDefinitions->get_Count(&exitsCount);
   Common::ThrowOnCOMError(spTradeExitDefinitions, hr);

   for (long i = 0; i < exitsCount; ++i)
   {
      ATL::CComPtr<ICQGTradeExitDefinition> spTradeExitDefinition;
      hr = spTradeExitDefinitions->get_Item(i, &spTradeExitDefinition);
      Common::ThrowOnCOMError(spTradeExitDefinitions, hr);

      ATL::CComBSTR name;
      hr = spTradeExitDefinition->get_Name(&name);
      Common::ThrowOnCOMError(spTradeExitDefinition, hr);

      hr = spTradeExitDefinition->get_OrderType(&orderType);
      Common::ThrowOnCOMError(spTradeExitDefinition, hr);

      VARIANT_BOOL allowExitOnEntry;
      hr = spTradeExitDefinition->get_AllowExitOnEntry(&allowExitOnEntry);
      Common::ThrowOnCOMError(spTradeExitDefinition, hr);

      VARIANT_BOOL allowMoreThanOnce;
      hr = spTradeExitDefinition->get_AllowMoreThanOnce(&allowMoreThanOnce);
      Common::ThrowOnCOMError(spTradeExitDefinition, hr);

      CString exitName;
      row = m_lvDefEntryExits.InsertItem(m_lvDefEntryExits.GetItemCount(), _T(""));
      exitName.Format(_T("OrderType (%s)"), static_cast<LPCTSTR>(ATL::CW2T(name)));
      m_lvDefEntryExits.SetItemText(row, dteecName, exitName);
      m_lvDefEntryExits.SetItemText(row, dteecValue, CHelper::OrderTypeToString(orderType));

      row = m_lvDefEntryExits.InsertItem(m_lvDefEntryExits.GetItemCount(), _T(""));
      exitName.Format(_T("AllowExitOnEntry (%s)"), static_cast<LPCTSTR>(ATL::CW2T(name)));
      m_lvDefEntryExits.SetItemText(row, dteecName, exitName);
      m_lvDefEntryExits.SetItemText(row, dteecValue, GetValueAsString(allowExitOnEntry, VT_BOOL));

      row = m_lvDefEntryExits.InsertItem(m_lvDefEntryExits.GetItemCount(), _T(""));
      exitName.Format(_T("AllowMoreThanOnce (%s)"), static_cast<LPCTSTR>(ATL::CW2T(name)));
      m_lvDefEntryExits.SetItemText(row, dteecName, exitName);
      m_lvDefEntryExits.SetItemText(row, dteecValue, GetValueAsString(allowMoreThanOnce, VT_BOOL));
   }
}

void CTradingSystemDlg::ShowAllData()
{
   // Show request parameters
   ShowRequestParams();

   // Add trades and show first trade's headers
   AddTradesToCombo();
   OnCbnSelchangeCmbTrades();

   // Add all selected statistics to the map 
   AddStatisticsToMap();

   // Show statistic headers
   ShowStatisticsHeaders(m_lvLastStatistics);
   ShowStatisticsHeaders(m_lvStatistics);

   // If request is succeeded show records
   eRequestStatus status;
   HRESULT hr = m_spTradingSystem->get_Status(&status);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   m_lblStatus.SetWindowText(CHelper::RequestStatusToString(status));

   if (rsSuccess == status)
   {
      // Fills start and end induces for the filtering
      FillIndices();

      // Show last trades and records
      ShowLastTrades();
      ShowLastStatistics();
   }

   ShowTradeItemRelations(m_lvLastTrades, m_lvLastTradeItems, m_cmbLastTradeItems);
}

void CTradingSystemDlg::ShowLastStatistics()
{
   // Clear records
   ClearRecords(m_lvLastStatistics);

   // Get statistics count
   ATL::CComPtr<ICQGTradingSystemStatisticsCollection> spStatisticsCollection;
   HRESULT hr = m_spTradingSystem->get_Statistics(&spStatisticsCollection);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   long count;
   hr = spStatisticsCollection->get_Count(&count);
   Common::ThrowOnCOMError(spStatisticsCollection, hr);

   // Show last statistics
   const long startIndex = (std::max<long>)(count - SHOW_COUNT + 1, 1);
   ShowStatisticRecords(startIndex, count, m_lvLastStatistics);

   // Show statistics all records count
   m_lblStatsAllRecsCount.SetWindowText(GetValueAsString(count));
}

void CTradingSystemDlg::ShowLastTrades()
{
   // Clear records
   ClearRecords(m_lvLastTrades);

   // Get trades rows count
   ATL::CComPtr<ICQGTradingSystemTradesRows> spTradesRows;
   HRESULT hr = m_spTradingSystem->get_TradesRows(&spTradesRows);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   long count;
   hr = spTradesRows->get_Count(&count);
   Common::ThrowOnCOMError(spTradesRows, hr);

   // Show last trade records
   const long startIndex = (std::max<long>)(count - SHOW_COUNT + 1, 1);
   ShowTradeRecords(startIndex, count, m_lvLastTrades);

   // Show trades all records count
   m_lblTradesAllRecsCount.SetWindowText(GetValueAsString(count));

   ShowTradeItemRelations(m_lvLastTrades, m_lvLastTradeItems, m_cmbLastTradeItems);
}

void CTradingSystemDlg::AddTradesToCombo()
{
   // Get trades' definitions
   m_cmbTrades.ResetContent();

   ATL::CComPtr<ICQGTradingSystemDefinition> spTradingSystemDefinition;
   HRESULT hr = m_spTradingSystem->get_Definition(&spTradingSystemDefinition);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   ATL::CComPtr<ICQGTradingSystemTradeDefinitions> spTradeDefinitions;
   hr = spTradingSystemDefinition->get_TradeDefinitions(&spTradeDefinitions);
   Common::ThrowOnCOMError(spTradingSystemDefinition, hr);

   // Get trades names and add them to combo
   long count;
   hr = spTradeDefinitions->get_Count(&count);
   Common::ThrowOnCOMError(spTradeDefinitions, hr);

   for (long i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGTradingSystemTradeDefinition> spTradeDefinition;
      hr = spTradeDefinitions->get_Item(i, &spTradeDefinition);
      Common::ThrowOnCOMError(spTradeDefinitions, hr);

      ATL::CComBSTR name;
      hr = spTradeDefinition->get_Name(&name);
      Common::ThrowOnCOMError(spTradeDefinition, hr);

      const int row = m_cmbTrades.AddString(ATL::CW2T(name));
      m_cmbTrades.SetItemData(row, i);
   }

   if (0 < count)
   {
      // Select first trade
      m_cmbTrades.SetCurSel(0);
   }
}

void CTradingSystemDlg::ShowRequestParams()
{
   UpdateData();

   // Get request from trading system collection
   ATL::CComPtr<ICQGTradingSystemRequest> spRequest;
   HRESULT hr = m_spTradingSystem->get_Request(&spRequest);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   ShowReqDefinitionParameters(spRequest);

   // Show request parameters
   ATL::CComBSTR baseExpression;
   hr = spRequest->get_BaseExpression(&baseExpression);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeStart;
   hr = spRequest->get_RangeStart(&rangeStart);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant rangeEnd;
   hr = spRequest->get_RangeEnd(&rangeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   long intradayPeriod;
   hr = spRequest->get_IntradayPeriod(&intradayPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   eHistoricalPeriod historicalPeriod;
   hr = spRequest->get_HistoricalPeriod(&historicalPeriod);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL includeEnd;
   hr = spRequest->get_IncludeEnd(&includeEnd);
   Common::ThrowOnCOMError(spRequest, hr);

   ATL::CComVariant sessionsFilter;
   hr = spRequest->get_SessionsFilter(&sessionsFilter);
   Common::ThrowOnCOMError(spRequest, hr);

   eTimeSeriesContinuationType continuation;
   hr = spRequest->get_Continuation(&continuation);
   Common::ThrowOnCOMError(spRequest, hr);

   VARIANT_BOOL equalizeCloses;
   hr = spRequest->get_EqualizeCloses(&equalizeCloses);
   Common::ThrowOnCOMError(spRequest, hr);

   long daysBeforeExpiration;
   hr = spRequest->get_DaysBeforeExpiration(&daysBeforeExpiration);
   Common::ThrowOnCOMError(spRequest, hr);

   eSessionFlag sessionFlags;
   hr = spRequest->get_SessionFlags(&sessionFlags);
   Common::ThrowOnCOMError(spRequest, hr);
   
   eTimeSeriesSubscriptionLevel subscriptionLevel;
   hr = spRequest->get_SubscriptionLevel(&subscriptionLevel);
   Common::ThrowOnCOMError(spRequest, hr);
   
   long recalcPeriod;
   hr = spRequest->get_RecalcPeriod(&recalcPeriod);
   Common::ThrowOnCOMError(spRequest, hr);
   
   m_BaseExpression = ATL::CW2T(baseExpression);
   m_RangeStart = GetValueAsString(rangeStart);
   m_RangeEnd = GetValueAsString(rangeEnd);
   m_IntradayPeriod = GetValueAsString(intradayPeriod);
   m_HistoricalPeriod = CHelper::HistoricalPeriodToString(historicalPeriod);
   m_IncludeEnd = GetValueAsString(includeEnd, VT_BOOL);

   CString filter = GetValueAsString(sessionsFilter);
   if (filter == "31")
   {
      filter = "All sessions (31)";
   }
   else if (filter == "0")
   {
      filter = "Primary session only (0)";
   }
   m_SessionsFilter = filter;

   m_Continuation = CHelper::ContinuationTypeToString(continuation);
   m_EqualizeCloses = GetValueAsString(equalizeCloses, VT_BOOL);
   m_DaysBeforeExpiration = GetValueAsString(daysBeforeExpiration);
   m_SessionFlags = CHelper::SessionFlagToString(sessionFlags);
   m_SubscriptionLevel = CHelper::SubscriptionLevelToString(subscriptionLevel);
   if (subscriptionLevel == tslEndOfBarAndPeriod)
   {
      m_RecalcPeriod = GetValueAsString(recalcPeriod);
   }
   
   UpdateData(FALSE);

   // Get error from trading system collection and show it if exist
   ATL::CComPtr<ICQGError> spError;
   hr = m_spTradingSystem->get_LastError(&spError);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   VARIANT_BOOL isValid;
   hr = m_spCQGCEL->IsValid(ATL::CComVariant(spError), &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      ATL::CComBSTR errDescription;
      hr = spError->get_Description(&errDescription);
      Common::ThrowOnCOMError(spError, hr);

      m_lblError.SetWindowText(ATL::CW2T(errDescription));
   }
}

void CTradingSystemDlg::ShowReqDefinitionParameters(
                        const ATL::CComPtr<ICQGTradingSystemRequest>& spTradingSystemRequest)
{
   // Get definition from request
   ATL::CComPtr<ICQGTradingSystemDefinition> spTradingSystemDefinition;
   HRESULT hr = spTradingSystemRequest->get_Definition(&spTradingSystemDefinition);
   Common::ThrowOnCOMError(spTradingSystemRequest, hr);

   ATL::CComBSTR definitionName;
   hr = spTradingSystemDefinition->get_Name(&definitionName);
   Common::ThrowOnCOMError(spTradingSystemDefinition, hr);

   m_DefinitionName = ATL::CW2T(definitionName);

   // Fill up parameters in the list control
   m_lvParameters.DeleteAllItems();

   ATL::CComPtr<ICQGParameterDefinitions> spParameterDefinitions;
   hr = spTradingSystemDefinition->get_ParameterDefinitions(&spParameterDefinitions);
   Common::ThrowOnCOMError(spTradingSystemDefinition, hr);

   long count;
   hr = spParameterDefinitions->get_Count(&count);
   Common::ThrowOnCOMError(spParameterDefinitions, hr);

   for (long i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGParameterDefinition> spParameterDefinition;
      hr = spParameterDefinitions->get_Item(i, &spParameterDefinition);
      Common::ThrowOnCOMError(spParameterDefinitions, hr);

      ATL::CComBSTR name;
      hr = spParameterDefinition->get_Name(&name);
      Common::ThrowOnCOMError(spTradingSystemDefinition, hr);

      ATL::CComVariant value;
      hr = spTradingSystemRequest->get_Parameter(ATL::CComVariant(name), &value);
      Common::ThrowOnCOMError(spTradingSystemRequest, hr);

      const int row = m_lvParameters.InsertItem(m_lvParameters.GetItemCount(), _T(""));
      m_lvParameters.SetItemText(row, pcName, ATL::CW2T(name));
      m_lvParameters.SetItemText(row, pcValue,
                  Common::VariantToString(value, CHelper::PARAM_FRAC_PART_LENGTH));
   }
}

void CTradingSystemDlg::ShowTradeHeaders(
                        CListCtrl& listCtrl)
{
   // Get selected trade definition
   ATL::CComPtr<ICQGTradingSystemDefinition> spTradingSystemDefinition;
   HRESULT hr = m_spTradingSystem->get_Definition(&spTradingSystemDefinition);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   ATL::CComPtr<ICQGTradingSystemTradeDefinitions> spTradeDefinitions;
   hr = spTradingSystemDefinition->get_TradeDefinitions(&spTradeDefinitions);
   Common::ThrowOnCOMError(spTradingSystemDefinition, hr);

   // Get selected trade index
   const long idx = static_cast<long>(m_cmbTrades.GetItemData(m_cmbTrades.GetCurSel()));
   ATL::CComPtr<ICQGTradingSystemTradeDefinition> spTradeDefinition;
   hr = spTradeDefinitions->get_Item(idx, &spTradeDefinition);
   Common::ThrowOnCOMError(spTradeDefinitions, hr);

   if (!spTradeDefinition)
   {
      return;
   }

   // Get trade's exits definitions and show their headers
   ATL::CComPtr<ICQGTradeExitDefinitions> spTradeExitDefinitions;
   hr = spTradeDefinition->get_Exits(&spTradeExitDefinitions);
   Common::ThrowOnCOMError(spTradeDefinition, hr);

   long count;
   hr = spTradeExitDefinitions->get_Count(&count);
   Common::ThrowOnCOMError(spTradeExitDefinitions, hr);

   for (long i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGTradeExitDefinition> spTradeExitDefinition;
      hr = spTradeExitDefinitions->get_Item(i, &spTradeExitDefinition);
      Common::ThrowOnCOMError(spTradeExitDefinitions, hr);

      ATL::CComBSTR name;
      hr = spTradeExitDefinition->get_Name(&name);
      Common::ThrowOnCOMError(spTradeExitDefinition, hr);

      // Calculate exit's column
      const int exitCol = trcExitsStart + ecCount * i;

      CString columnHeader;
      columnHeader.Format(_T("Signal (%s)"), static_cast<LPCTSTR>(ATL::CW2T(name)));
      listCtrl.InsertColumn(exitCol + ecSignal, columnHeader, LVCFMT_LEFT, 100);

      columnHeader.Format(_T("Price (%s)"), static_cast<LPCTSTR>(ATL::CW2T(name)));
      listCtrl.InsertColumn(exitCol + ecPrice, columnHeader, LVCFMT_LEFT, 100);

      columnHeader.Format(_T("SL Price (%s)"), static_cast<LPCTSTR>(ATL::CW2T(name)));
      listCtrl.InsertColumn(exitCol + ecStopLimitPrice, columnHeader, LVCFMT_LEFT, 100);

      columnHeader.Format(_T("Quantity (%s)"), static_cast<LPCTSTR>(ATL::CW2T(name)));
      listCtrl.InsertColumn(exitCol + ecQuantity, columnHeader, LVCFMT_LEFT, 100);

      // Add exit's column to the map
      m_TradeExitsMap.insert(std::pair<ATL::CComBSTR, int>(name, exitCol));
   }
}

void CTradingSystemDlg::ShowStatisticsHeaders(
                        CListCtrl& listCtrl)
{
   // Clearing headers
   ClearStatisticsHeaders(listCtrl);

   for(StatisticMap::const_iterator it = m_StatisticsMap.begin(); it != m_StatisticsMap.end(); ++it)
   {
      const CString columnHeader = CHelper::TradingSystemStatisticToString(it->first);
      listCtrl.InsertColumn(srcStatsStart + it->second, columnHeader, LVCFMT_LEFT, 100);
   }
}

void CTradingSystemDlg::ClearStatisticsHeaders(
                        CListCtrl& listCtrl)
{
   listCtrl.DeleteAllItems();

   const int columnCount = listCtrl.GetHeaderCtrl()->GetItemCount() - srcStatsStart;

   // Delete headers
   for (int i = 0; i < columnCount; ++i)
   {
      listCtrl.DeleteColumn(srcStatsStart);
   }
}

void CTradingSystemDlg::AddStatisticsToMap()
{
   m_StatisticsMap.clear();

   // Get request from trading system collection
   ATL::CComPtr<ICQGTradingSystemRequest> spTradingSystemRequest;
   HRESULT hr = m_spTradingSystem->get_Request(&spTradingSystemRequest);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   int colNumber = 0;

   AddStatisticToMap(tssTotalNetProfit, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssClosedNetProfit, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssTotalTradeCount, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssOpenPosition, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssPercentLong, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssAverageDuration, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssAverageProfit, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssAverageWin, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssMaximumWin, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssAverageLoss, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssMaximumLoss, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssMaxClosedDraw, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssMaxDrawAmount, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssMaxDrawDuration, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssMaxConsecWins, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssCurConsecWins, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssMaxConsecLosses, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssCurConsecLosses, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssProfitToMaxDraw, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssProfitLossRatio, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssPercentWinners, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssRemoveToNeutral, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssTimePercentage, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssCurDrawDown, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssReturnRetracement, spTradingSystemRequest, colNumber);
   AddStatisticToMap(tssLinearRegression, spTradingSystemRequest, colNumber);
}

void CTradingSystemDlg::AddStatisticToMap(
                        eTradingSystemStatistic tradingSystemStatistic,
                        const ATL::CComPtr<ICQGTradingSystemRequest>& spTradingSystemRequest,
                        int& colNumber)
{
   VARIANT_BOOL isStatisticIncluded;
   HRESULT hr = spTradingSystemRequest->IsStatisticIncluded(tradingSystemStatistic, &isStatisticIncluded);
   Common::ThrowOnCOMError(spTradingSystemRequest, hr);

   // If statistic is included add it's column to the map
   if (isStatisticIncluded)
   {
      m_StatisticsMap.insert(std::pair<eTradingSystemStatistic, int>(tradingSystemStatistic, colNumber++));
   }
}

void CTradingSystemDlg::ShowStatisticRecords(
                        long startIndex,
                        long endIndex,
                        CListCtrl& listCtrl)
{
   for (long i = startIndex; i <= endIndex; ++i)
   {
      AddStatisticRecord(i, listCtrl);
   }
}

void CTradingSystemDlg::AddStatisticRecord(
                        long recordIndex,
                        CListCtrl& listCtrl)
{
   // Add empty row
   const int row = listCtrl.InsertItem(listCtrl.GetItemCount(), _T(""));

   ShowStatisticRecord(recordIndex, listCtrl, row);
}

void CTradingSystemDlg::ShowStatisticRecord(
                        long recordIndex,
                        CListCtrl& listCtrl,
                        long row)
{
   ATL::CComPtr<ICQGTradingSystemStatisticsCollection> spStatisticsCollection;
   HRESULT hr = m_spTradingSystem->get_Statistics(&spStatisticsCollection);
   Common::ThrowOnCOMError(m_spTradingSystem, hr); 

   long count;
   hr = spStatisticsCollection->get_Count(&count);
   Common::ThrowOnCOMError(spStatisticsCollection, hr);

   if (count == 0)
   {
      return;
   }

   ATL::CComPtr<ICQGTradingSystemStatistics> spStatistics;
   hr = spStatisticsCollection->get_Item(recordIndex - 1, &spStatistics);
   Common::ThrowOnCOMError(spStatisticsCollection, hr); 

   // Get timestamp to show it
   DATE timestamp;
   hr = spStatistics->get_Timestamp(&timestamp);
   Common::ThrowOnCOMError(spStatistics, hr);

   listCtrl.SetItemText(row, srcIndex, GetValueAsString(recordIndex));
   listCtrl.SetItemText(row, srcTimestamp, GetValueAsString(timestamp, VT_DATE, true));

   for(StatisticMap::const_iterator it = m_StatisticsMap.begin(); it != m_StatisticsMap.end(); ++it)
   {
      double value;
      hr = spStatistics->get_Value(it->first, &value);
      Common::ThrowOnCOMError(spStatistics, hr);

      listCtrl.SetItemText(row, srcStatsStart + it->second, GetValueAsString(value));
   }
}

void CTradingSystemDlg::ShowTradeRecords(
                        long startIndex,
                        long endIndex,
                        CListCtrl& listCtrl)
{
   for (long i = startIndex; i <= endIndex; ++i)
   {
      AddTradeRecord(i, listCtrl);
   }
}

void CTradingSystemDlg::AddTradeRecord(
                        long recordIndex,
                        CListCtrl& listCtrl)
{
   // Add empty row
   const int row = listCtrl.InsertItem(listCtrl.GetItemCount(), _T(""));

   ShowTradeRecord(recordIndex, listCtrl, row);
}

void CTradingSystemDlg::ShowTradeRecord(
                        long recordIndex,
                        CListCtrl& listCtrl,
                        long row)
{
   // Get trade row
   ATL::CComPtr<ICQGTradingSystemTradesRows> spTradesRows;
   HRESULT hr = m_spTradingSystem->get_TradesRows(&spTradesRows);
   Common::ThrowOnCOMError(m_spTradingSystem, hr); 

   ATL::CComPtr<ICQGTradingSystemTradesRow> spTradesRow;
   hr = spTradesRows->get_Item(recordIndex - 1, &spTradesRow);
   Common::ThrowOnCOMError(spTradesRows, hr);

   // Get selected row
   const long selRow = m_cmbTrades.GetCurSel();
   if (selRow == -1)
   {
      return;
   }

   // Get selected trade
   CString tradeName;
   m_cmbTrades.GetLBText(selRow, tradeName);

   ATL::CComPtr<ICQGTradingSystemTrade> spTrade;
   hr = spTradesRow->get_ItemByName(ATL::CComBSTR(tradeName), &spTrade);
   Common::ThrowOnCOMError(spTradesRow, hr);

   // Get timestamp and show it
   DATE timestamp;
   hr = spTradesRow->get_Timestamp(&timestamp);
   Common::ThrowOnCOMError(spTradesRow, hr);

   listCtrl.SetItemText(row, trcIndex, GetValueAsString(recordIndex));
   listCtrl.SetItemText(row, trcTimestamp, GetValueAsString(timestamp, VT_DATE, true));

   if (spTrade)
   {
      // Get IsActive from trade and show it
      VARIANT_BOOL isActive;
      hr = spTrade->get_IsActive(&isActive);
      Common::ThrowOnCOMError(spTrade, hr);
      listCtrl.SetItemText(row, trcIsActive, GetValueAsString(isActive, VT_BOOL));

      // Get entry from trade and show it
      ATL::CComPtr<ICQGTradeEntry> spTradeEntry;
      hr = spTrade->get_TradeEntry(&spTradeEntry);
      Common::ThrowOnCOMError(spTrade, hr);

      ShowTradeRecordEntry(spTradeEntry, listCtrl, row);

      // Get exits from trade and show them
      ATL::CComPtr<ICQGTradeExits> spTradeExits;
      hr = spTrade->get_TradeExits(&spTradeExits);
      Common::ThrowOnCOMError(spTrade, hr);

      long exitsCount;
      hr = spTradeExits->get_Count(&exitsCount);
      Common::ThrowOnCOMError(spTradeExits, hr);

      for (long i = 0; i < exitsCount; ++i)
      {
         ATL::CComPtr<ICQGTradeExit> spTradeExit;
         hr = spTradeExits->get_Item(i, &spTradeExit);
         Common::ThrowOnCOMError(spTradeExits, hr);

         ShowTradeRecordExit(spTradeExit, listCtrl, row);
      }
   }
}

void CTradingSystemDlg::ShowTradeRecordEntry(
                        const ATL::CComPtr<ICQGTradeEntry>& spTradeEntry,
                        CListCtrl& listCtrl,
                        long row)
{
   // Get entry's parameters and show them
   VARIANT_BOOL signal;
   HRESULT hr = spTradeEntry->get_Signal(&signal);
   Common::ThrowOnCOMError(spTradeEntry, hr);

   double price;
   hr = spTradeEntry->get_Price(&price);
   Common::ThrowOnCOMError(spTradeEntry, hr);
   
   double stopLimitPrice;
   hr = spTradeEntry->get_StopLimitPrice(&stopLimitPrice);
   Common::ThrowOnCOMError(spTradeEntry, hr);
   
   long quantity;
   hr = spTradeEntry->get_Quantity(&quantity);
   Common::ThrowOnCOMError(spTradeEntry, hr);

   listCtrl.SetItemText(row, trcEntrySignal, GetValueAsString(signal, VT_BOOL));
   listCtrl.SetItemText(row, trcEntryPrice, GetValueAsString(price));
   listCtrl.SetItemText(row, trcEntryStopLimitPrice, GetValueAsString(stopLimitPrice));
   listCtrl.SetItemText(row, trcEntryQuantity, GetValueAsString(quantity));
}

void CTradingSystemDlg::ShowTradeRecordExit(
                        const ATL::CComPtr<ICQGTradeExit>& spTradeExit,
                        CListCtrl& listCtrl,
                        long row)
{
   // Get exit's name
   ATL::CComPtr<ICQGTradeExitDefinition> spExitDefinition;
   HRESULT hr = spTradeExit->get_Definition(&spExitDefinition);
   Common::ThrowOnCOMError(spTradeExit, hr);

   ATL::CComBSTR name;
   spExitDefinition->get_Name(&name);
   Common::ThrowOnCOMError(spExitDefinition, hr);

   // Get exit's column in the list by name
   const TradeExitMap::const_iterator it = m_TradeExitsMap.find(name);
   
   if (it == m_TradeExitsMap.end())
   {
      assert(!"Exit's name does not exist");
   }

   const long exitCol = it->second;

   // Get exit's parameters and show them
   VARIANT_BOOL signal;
   hr = spTradeExit->get_Signal(&signal);
   Common::ThrowOnCOMError(spTradeExit, hr);

   double price;
   hr = spTradeExit->get_Price(&price);
   Common::ThrowOnCOMError(spTradeExit, hr);

   double stopLimitPrice;
   hr = spTradeExit->get_StopLimitPrice(&stopLimitPrice);
   Common::ThrowOnCOMError(spTradeExit, hr);

   long quantity;
   hr = spTradeExit->get_Quantity(&quantity);
   Common::ThrowOnCOMError(spTradeExit, hr);

   listCtrl.SetItemText(row, exitCol + ecSignal, GetValueAsString(signal, VT_BOOL));
   listCtrl.SetItemText(row, exitCol + ecPrice, GetValueAsString(price));
   listCtrl.SetItemText(row, exitCol + ecStopLimitPrice, GetValueAsString(stopLimitPrice));
   listCtrl.SetItemText(row, exitCol + ecQuantity, GetValueAsString(quantity));
}

void CTradingSystemDlg::ClearIndices()
{
   m_edStart.SetValue(0);
   m_edEnd.SetValue(0);
}

void CTradingSystemDlg::FillIndices()
{
   if (!m_spTradingSystem)
   {
      return;
   }

   eRequestStatus status;
   HRESULT hr = m_spTradingSystem->get_Status(&status);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   if (rsSuccess != status)
   {
      return;
   }

   // Fill start and end indices controls.
   m_edStart.SetValue(1);

   long count;

   if (toStatistics == m_tabTradeStatistic.GetCurSel())
   {
      ATL::CComPtr<ICQGTradingSystemStatisticsCollection> spStatisticsCollection;
      hr = m_spTradingSystem->get_Statistics(&spStatisticsCollection);
      Common::ThrowOnCOMError(m_spTradingSystem, hr);

      hr = spStatisticsCollection->get_Count(&count);
      Common::ThrowOnCOMError(spStatisticsCollection, hr);
   }
   else
   {
      ATL::CComPtr<ICQGTradingSystemTradesRows> spTradesRows;
      hr = m_spTradingSystem->get_TradesRows(&spTradesRows);
      Common::ThrowOnCOMError(m_spTradingSystem, hr);

      hr = spTradesRows->get_Count(&count);
      Common::ThrowOnCOMError(spTradesRows, hr);
   }

   m_edEnd.SetValue(count);
}

void CTradingSystemDlg::ShowListViews()
{
   int tradesShowStatus = SW_SHOW;
   int statsShowStatus = SW_HIDE;

   if (toStatistics == m_tabTradeStatistic.GetCurSel())
   {
      tradesShowStatus = SW_HIDE;
      statsShowStatus = SW_SHOW;
   }

   m_lvStatistics.ShowWindow(statsShowStatus);
   m_lvLastStatistics.ShowWindow(statsShowStatus);
   m_lblStatsAllRecsCount.ShowWindow(statsShowStatus);
   m_lblStatsFltrRecsCount.ShowWindow(statsShowStatus);

   m_cmbTrades.ShowWindow(tradesShowStatus);
   m_lvTrades.ShowWindow(tradesShowStatus);
   m_lvLastTrades.ShowWindow(tradesShowStatus);
   m_lvTradeItems.ShowWindow(tradesShowStatus);
   m_lvLastTradeItems.ShowWindow(tradesShowStatus);
   m_lblTradesAllRecsCount.ShowWindow(tradesShowStatus);
   m_lblTradesFltrRecsCount.ShowWindow(tradesShowStatus);
   m_cmbLastTradeItems.ShowWindow(tradesShowStatus);
   m_cmbTradeItems.ShowWindow(tradesShowStatus);

   m_lblLastRelations.ShowWindow(tradesShowStatus);
   m_lblRelations.ShowWindow(tradesShowStatus);
}

void CTradingSystemDlg::AddTradeItemsNames(
                        CComboBox& cmbItemNames)
{
   cmbItemNames.ResetContent();

   // Get definition form trading system
   ATL::CComPtr<ICQGTradingSystemDefinition> spTSDefinition;
   HRESULT hr = m_spTradingSystem->get_Definition(&spTSDefinition);
   Common::ThrowOnCOMError(m_spTradingSystem, hr);

   // Get trade definitions form trading system definition
   ATL::CComPtr<ICQGTradingSystemTradeDefinitions> spTSTradeDefs;
   hr = spTSDefinition->get_TradeDefinitions(&spTSTradeDefs);
   Common::ThrowOnCOMError(spTSDefinition, hr);

   // Get trade definition from trade definitions
   const long idx = static_cast<long>(m_cmbTrades.GetItemData(m_cmbTrades.GetCurSel()));
   ATL::CComPtr<ICQGTradingSystemTradeDefinition> spTSTradeDef;
   hr = spTSTradeDefs->get_Item(idx, &spTSTradeDef);
   Common::ThrowOnCOMError(spTSTradeDef, hr);

   if (!spTSTradeDef)
   {
      return;
   }

   // Get trade name
   ATL::CComBSTR tradeName("");
   hr = spTSTradeDef->get_Name(&tradeName);
   Common::ThrowOnCOMError(spTSTradeDef, hr);

   // Add data to combo box
   long index = 0;
   int row = cmbItemNames.AddString(ATL::CW2T(tradeName));
   cmbItemNames.SetItemData(row, index);

   // Get exit definitions
   ATL::CComPtr<ICQGTradeExitDefinitions> spTradeExitDefs;
   hr = spTSTradeDef->get_Exits(&spTradeExitDefs);
   Common::ThrowOnCOMError(spTSTradeDef, hr);

   // Get exit definitions count
   long count;
   hr = spTradeExitDefs->get_Count(&count);
   Common::ThrowOnCOMError(spTradeExitDefs, hr);

   for (long ix = 0; ix < count; ++ix)
   {
      // Get trade exit definition by index
      ATL::CComPtr<ICQGTradeExitDefinition> spTradeExitDef;
      hr = spTradeExitDefs->get_Item(ix, &spTradeExitDef);
      Common::ThrowOnCOMError(spTradeExitDefs, hr);

      // Get trade exit definition name
      ATL::CComBSTR exitDefName;
      hr = spTradeExitDef->get_Name(&exitDefName);
      Common::ThrowOnCOMError(spTradeExitDef, hr);

      ATL::CComBSTR itemName(tradeName);
      itemName += ATL::CComBSTR(NAME_SPLITTER);
      itemName += exitDefName;

      // Add data to combo box
      row = cmbItemNames.AddString(ATL::CW2T(itemName));
      cmbItemNames.SetItemData(row, ++index);
   }

   if (count > 0)
   {
      // Select first item
      cmbItemNames.SetCurSel(0);
   }
}

void CTradingSystemDlg::ShowTradeItemRelations(
                        CListCtrl& lvTrades, 
                        CListCtrl& lvRelations, 
                        CComboBox& cmbItemNames)
{
   try
   {
      ClearRecords(lvRelations);

      int selectedCount = lvTrades.GetSelectedCount();
      int selectedRow = -1;

      // Get selected item index
      if (selectedCount == 1)
      {
         selectedRow = lvTrades.GetNextItem(selectedRow, LVNI_SELECTED);
      }

      if (selectedRow == -1 || cmbItemNames.GetCurSel() == -1)
      {
         return;
      }

      const CString strIndex = lvTrades.GetItemText(selectedRow, trcIndex);
      const int row = _ttoi(static_cast<LPCTSTR>(strIndex));

      // Get trades rows
      ATL::CComPtr<ICQGTradingSystemTradesRows> spTradesRows;
      HRESULT hr = m_spTradingSystem->get_TradesRows(&spTradesRows);
      Common::ThrowOnCOMError(m_spTradingSystem, hr);

      // Get currently selected trade row
      ATL::CComPtr<ICQGTradingSystemTradesRow> spTradesRow;
      hr = spTradesRows->get_Item(row - 1, &spTradesRow);
      Common::ThrowOnCOMError(spTradesRows, hr);

      if (spTradesRow == NULL)
      {
         return;
      }

      // Get selected trade name
      CString tradeName;
      m_cmbTrades.GetLBText(m_cmbTrades.GetCurSel(), tradeName);

      // Get currently selected trade from the row
      ATL::CComPtr<ICQGTradingSystemTrade> spTrade;
      hr = spTradesRow->get_ItemByName(ATL::CComBSTR(tradeName), &spTrade);
      Common::ThrowOnCOMError(spTradesRow, hr);

      // Get selected item name
      CString itemName;
      cmbItemNames.GetLBText(cmbItemNames.GetCurSel(), itemName);
      int splitterIndex = itemName.Find(NAME_SPLITTER);

      if (splitterIndex == -1)
      {
         // Get trade entry
         ATL::CComPtr<ICQGTradeEntry> spTradeEntry;
         hr = spTrade->get_TradeEntry(&spTradeEntry);
         Common::ThrowOnCOMError(spTrade, hr);

         // Get trade entry related exits
         ATL::CComPtr<ICQGTradeExits> spTradeExits;
         hr = spTradeEntry->get_RelatedExits(&spTradeExits);
         Common::ThrowOnCOMError(spTradeEntry, hr);

         // Get exits count
         long count;
         hr = spTradeExits->get_Count(&count);
         Common::ThrowOnCOMError(spTradeExits, hr);

         for (long ix = 0; ix < count; ++ix)
         {
            // Get exit
            ATL::CComPtr<ICQGTradeExit> spTradeExit;
            hr = spTradeExits->get_Item(ix, &spTradeExit);
            Common::ThrowOnCOMError(spTradeExits, hr);

            // Get trade
            ATL::CComPtr<ICQGTradingSystemTrade> spTSTrade;
            hr = spTradeExit->get_Trade(&spTSTrade);
            Common::ThrowOnCOMError(spTradeExit, hr);

            // Get exit definition
            ATL::CComPtr<ICQGTradeExitDefinition> spExitDef;
            hr = spTradeExit->get_Definition(&spExitDef);
            Common::ThrowOnCOMError(spTradeExit, hr);

            // Get exit definition name
            ATL::CComBSTR exitDefName;
            hr = spExitDef->get_Name(&exitDefName);
            Common::ThrowOnCOMError(spExitDef, hr);

            ATL::CComBSTR name(NAME_SPLITTER);
            name += exitDefName;
            ShowRelations(lvRelations, spTSTrade, name);
         }
      }
      else
      {
         // Get exit name
         CString exitName = itemName.Right(itemName.GetLength() - splitterIndex - NAME_SPLITTER.GetLength());

         // Get Trade exits
         ATL::CComPtr<ICQGTradeExits> spTradeExits;
         hr = spTrade->get_TradeExits(&spTradeExits);
         Common::ThrowOnCOMError(spTrade, hr);

         // Get exit by name
         ATL::CComPtr<ICQGTradeExit> spTradeExit;
         hr = spTradeExits->get_ItemByName(ATL::CComBSTR(exitName), &spTradeExit);

         // Get related entries
         ATL::CComPtr<ICQGTradeEntries> spTradeEntries;
         hr = spTradeExit->get_RelatedEntries(&spTradeEntries);
         Common::ThrowOnCOMError(spTradeExit, hr);

         // Get entries count
         long count;
         hr = spTradeEntries->get_Count(&count);
         Common::ThrowOnCOMError(spTradeEntries, hr);

         for (long ix = 0; ix < count; ++ix)
         {
            // Get entry
            ATL::CComPtr<ICQGTradeEntry> spTradeEntry;
            hr = spTradeEntries->get_Item(ix, &spTradeEntry);
            Common::ThrowOnCOMError(spTradeEntries, hr);

            // Get trade
            ATL::CComPtr<ICQGTradingSystemTrade> spTSTrade;
            hr = spTradeEntry->get_Trade(&spTSTrade);
            Common::ThrowOnCOMError(spTradeEntry, hr);

            ShowRelations(lvRelations, spTSTrade, ATL::CComBSTR(""));
         }
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CTradingSystemDlg::ShowRelations(
                        CListCtrl& listCtrl,
                        const ATL::CComPtr<ICQGTradingSystemTrade>& spTrade,
                        const ATL::CComBSTR& name)
{
   // Get trade timestamp
   DATE timestamp;
   HRESULT hr = spTrade->get_Timestamp(&timestamp);
   Common::ThrowOnCOMError(spTrade, hr);

   // Get trade definition 
   ATL::CComPtr<ICQGTradingSystemTradeDefinition> spTradeDef;
   hr = spTrade->get_Definition(&spTradeDef);
   Common::ThrowOnCOMError(spTrade, hr);

   // Get definition name
   ATL::CComBSTR defName;
   hr = spTradeDef->get_Name(&defName);
   Common::ThrowOnCOMError(spTradeDef, hr);
   defName += name;

   const int row = listCtrl.InsertItem(listCtrl.GetItemCount(), _T(""));
   listCtrl.SetItemText(row, rcTimestamp, GetValueAsString(timestamp, VT_DATE, true));
   listCtrl.SetItemText(row, rcName, ATL::CW2T(defName));
}

void CTradingSystemDlg::EnableControls()
{
   bool isDataConnectionUp = false;

   HRESULT hr;
   if (m_spCQGCEL)
   {
      VARIANT_BOOL isStarted;
      hr = m_spCQGCEL->get_IsStarted(&isStarted);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      if (isStarted)
      {
         ATL::CComPtr<ICQGEnvironment> spEnvironment;
         hr = m_spCQGCEL->get_Environment(&spEnvironment);
         Common::ThrowOnCOMError(m_spCQGCEL, hr);

         eConnectionStatus dataConnectionStatus;
         hr = spEnvironment->get_DataConnectionStatus(&dataConnectionStatus);
         Common::ThrowOnCOMError(spEnvironment, hr);

         isDataConnectionUp = (csConnectionUp == dataConnectionStatus);
      }
   }

   m_btnRequestDefinition.EnableWindow(isDataConnectionUp);
   m_btnRequest.EnableWindow(isDataConnectionUp);

   const bool isTradingSystemSelected = (m_spTradingSystem != NULL);

   m_btnRemove.EnableWindow(isTradingSystemSelected && isDataConnectionUp);
   m_btnRemoveAll.EnableWindow(isTradingSystemSelected && isDataConnectionUp);

   m_edTagName.EnableWindow(isTradingSystemSelected);
   m_edTagValue.EnableWindow(isTradingSystemSelected);

   bool isTradingSystemSucceeded = false;
   if (isTradingSystemSelected)
   {
      eRequestStatus status;
      hr = m_spTradingSystem->get_Status(&status);
      Common::ThrowOnCOMError(m_spTradingSystem, hr);

      isTradingSystemSucceeded = (rsSuccess == status);
   }

   m_edStart.EnableWindow(isTradingSystemSucceeded);
   m_edEnd.EnableWindow(isTradingSystemSucceeded);
   m_spnStart.EnableWindow(isTradingSystemSucceeded);
   m_spnEnd.EnableWindow(isTradingSystemSucceeded);

   m_btnShow.EnableWindow(isTradingSystemSucceeded);

   const bool enableCmbControls = (m_cmbTrades.GetCount() > 0);
   m_cmbTrades.EnableWindow(enableCmbControls);
   m_cmbLastTradeItems.EnableWindow(enableCmbControls);
   m_cmbTradeItems.EnableWindow(enableCmbControls);
}

bool CTradingSystemDlg::GetIndices(
                        long& startIndex,
                        long& endIndex)
{
   // Get records count
   long count;

   if (toStatistics == m_tabTradeStatistic.GetCurSel())
   {
      ATL::CComPtr<ICQGTradingSystemStatisticsCollection> spStatisticsCollection;
      HRESULT hr = m_spTradingSystem->get_Statistics(&spStatisticsCollection);
      Common::ThrowOnCOMError(m_spTradingSystem, hr);

      hr = spStatisticsCollection->get_Count(&count);
      Common::ThrowOnCOMError(spStatisticsCollection, hr);
   }
   else
   {
      ATL::CComPtr<ICQGTradingSystemTradesRows> spTradesRows;
      HRESULT hr = m_spTradingSystem->get_TradesRows(&spTradesRows);
      Common::ThrowOnCOMError(m_spTradingSystem, hr);

      hr = spTradesRows->get_Count(&count);
      Common::ThrowOnCOMError(spTradesRows, hr);
   }

   CString msgText = _T("");

   startIndex = m_edStart.GetValue();
   endIndex = m_edEnd.GetValue();

   // Check for valid indices
   if (endIndex < startIndex)
   {
      msgText = _T("Start index must be smaller than end index.");
   }
   else if (1 > startIndex)
   {
      msgText = _T("Start index must be greater than zero.");
   }
   else if (endIndex > count)
   {
      msgText = _T("End index must be smaller than records count.");
   }
   else if (MAX_INTERVAL <= endIndex - startIndex)
   {
      msgText.Format(_T("Range must indicate to no more than %d records."), MAX_INTERVAL);
   }

   if (msgText.GetLength() == 0)
   {
      return true;
   }

   AfxMessageBox(msgText, MB_ICONWARNING);
   return false;
}

CString CTradingSystemDlg::GetValueAsString(
                        ATL::CComVariant value,
                        VARENUM valueType/*= VT_EMPTY*/,
                        bool withMilliseconds /*= false*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   Common::ThrowOnCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return Common::VariantToString(value, withMilliseconds);
   }
   return CHelper::N_A;
}
