// TradingSystem.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include <cassert>
#include "Helper.h"

const CString CHelper::N_A = _T("N/A");
const CString CHelper::DATE_FORMAT = _T("dd/MM/yyyy HH:mm");
const int CHelper::PARAM_FRAC_PART_LENGTH = 3;

void CHelper::SetComboBoxItem(
                        CComboBox& comboBoxCtrl,
                        DWORD_PTR itemData)
{
   for (int i = 0; i < comboBoxCtrl.GetCount(); ++i)
   {
      if (comboBoxCtrl.GetItemData(i) == itemData)
      {
         comboBoxCtrl.SetCurSel(i);
         return;
      }
   }

   assert(!"Invalid item data");
}

int CHelper::CompareDates(
                        const COleDateTime& date1,
                        const COleDateTime& date2)
{
   // Check if dates are equal (seconds are ignored)
   if (date1.GetYear() == date2.GetYear() && date1.GetMonth() == date2.GetMonth() &&
      date1.GetDay() == date2.GetDay() && date1.GetHour() == date2.GetHour() &&
      date1.GetMinute() == date2.GetMinute())
   {
      return 0;
   }

   return (date1 < date2 ? -1 : 1);
}

CString CHelper::ConnectionStatusToString(
                        eConnectionStatus connectionStatus)
{
   CString connectionStatusStr;

   switch (connectionStatus)
   {
   case csConnectionDelayed:
      connectionStatusStr = _T("Delayed");
      break;
   case csConnectionDown:
      connectionStatusStr = _T("Down");
      break;
   case csConnectionUp:
      connectionStatusStr = _T("Up");
      break;
   case csConnectionTrouble:
      connectionStatusStr = _T("Trouble");
      break;
   case csConnectionNotLoggedOn:
      connectionStatusStr = _T("Not Logged On");
      break;
   default:
      assert(!"Unknown connection status");
   }

   return connectionStatusStr;
}

CString CHelper::ParameterTypeToString(
                        eUserFormulaParameterType parameterType)
{
   CString parameterTypeStr;

   switch (parameterType)
   {
   case ufptInt:
      parameterTypeStr = _T("Int");
      break;
   case ufptDouble:
      parameterTypeStr = _T("Double");
      break;
   default:
      assert(!"Unknown parameter type");
   }

   return parameterTypeStr;
}

CString CHelper::RequestStatusToString(
                        eRequestStatus requestStatus)
{
   CString requestStatusStr;

   switch (requestStatus)
   {
   case rsInProgress:
      requestStatusStr = _T("InProgress");
      break;
   case rsSuccess:
      requestStatusStr = _T("Success");
      break;
   case rsCanceled:
      requestStatusStr = _T("Canceled");
      break;
   case rsFailed:
      requestStatusStr = _T("Failed");
      break;
   default:
      assert(!"Unknown request status");
   }

   return requestStatusStr;
}

CString CHelper::HistoricalPeriodToString(
                        eHistoricalPeriod historicalPeriod)
{
   CString historicalPeriodStr;

   switch (historicalPeriod)
   {
   case hpUndefined:
      historicalPeriodStr = _T("Undefined");
      break;
   case hpDaily:
      historicalPeriodStr = _T("Daily");
      break;
   case hpWeekly:
      historicalPeriodStr = _T("Weekly");
      break;
   case hpMonthly:
      historicalPeriodStr = _T("Monthly");
      break;
   case hpQuarterly:
      historicalPeriodStr = _T("Quarterly");
      break;
   case hpSemiannual:
      historicalPeriodStr = _T("Semiannual");
      break;
   case hpYearly:
      historicalPeriodStr = _T("Yearly");
      break;
   default:
      assert(!"Unknown historical period");
   }

   return historicalPeriodStr;
}

CString CHelper::ContinuationTypeToString(
                        eTimeSeriesContinuationType continuationType)
{
   CString continuationTypeStr;

   switch (continuationType)
   {
   case tsctNoContinuation:
      continuationTypeStr = _T("NoContinuation");
      break;
   case tsctStandard:
      continuationTypeStr = _T("Standard");
      break;
   case tsctStandardByMonth:
      continuationTypeStr = _T("StandardByMonth");
      break;
   case tsctActive:
      continuationTypeStr = _T("Active");
      break;
   case tsctActiveByMonth:
      continuationTypeStr = _T("ActiveByMonth");
      break;
   case tsctAdjusted:
      continuationTypeStr = _T("Adjusted");
      break;
   case tsctAdjustedByMonth:
      continuationTypeStr = _T("AdjustedByMonth");
      break;
   default:
      assert(!"Unknown continuation type");
   }

   return continuationTypeStr;
}

CString CHelper::SessionFlagToString(
                        eSessionFlag sessionFlag)
{
   CString sessionFlagStr = _T("Undefined");

   if (sfDailyFromIntraday == (sessionFlag & sfDailyFromIntraday))
   {
      sessionFlagStr = _T("DailyFromIntraday");
   }

   return sessionFlagStr;
}

CString CHelper::TradeSideToString(
                        eTradeSide tradeSide)
{
   CString tradeSideStr;

   switch (tradeSide)
   {
   case tsBuy:
      tradeSideStr = _T("Buy");
      break;
   case tsSell:
      tradeSideStr = _T("Sell");
      break;
   case tsOff:
      tradeSideStr = _T("Off");
      break;
   default:
      assert(!"Unknown trade side");
   }

   return tradeSideStr;
}

CString CHelper::OrderTypeToString(
                        eOrderType orderType)
{
   CString orderTypeStr;

   switch (orderType)
   {
   case otUndefined:
      orderTypeStr = _T("Undefined");
      break;
   case otMarket:
      orderTypeStr = _T("Market");
      break;
   case otLimit:
      orderTypeStr = _T("Limit");
      break;
   case otStop:
      orderTypeStr = _T("Stop");
      break;
   case otStopLimit:
      orderTypeStr = _T("Stop Limit");
      break;
   default:
      assert(!"Unknown order type");
   }

   return orderTypeStr;
}

CString CHelper::TradingSystemStatisticToString(
                        eTradingSystemStatistic tradingSystemStatistic)
{
   CString tradingSystemStatisticStr;

   switch (tradingSystemStatistic)
   {
   case tssTotalNetProfit:
      tradingSystemStatisticStr = _T("TotalNetProfit");
      break;
   case tssClosedNetProfit:
      tradingSystemStatisticStr = _T("ClosedNetProfit");
      break;
   case tssTotalTradeCount:
      tradingSystemStatisticStr = _T("TotalTradeCount");
      break;
   case tssOpenPosition:
      tradingSystemStatisticStr = _T("OpenPosition");
      break;
   case tssPercentLong:
      tradingSystemStatisticStr = _T("PercentLong");
      break;
   case tssAverageDuration:
      tradingSystemStatisticStr = _T("AverageDuration");
      break;
   case tssAverageProfit:
      tradingSystemStatisticStr = _T("AverageProfit");
      break;
   case tssAverageWin:
      tradingSystemStatisticStr = _T("AverageWin");
      break;
   case tssMaximumWin:
      tradingSystemStatisticStr = _T("MaximumWin");
      break;
   case tssAverageLoss:
      tradingSystemStatisticStr = _T("AverageLoss");
      break;
   case tssMaximumLoss:
      tradingSystemStatisticStr = _T("MaximumLoss");
      break;
   case tssMaxClosedDraw:
      tradingSystemStatisticStr = _T("MaxClosedDraw");
      break;
   case tssMaxDrawAmount:
      tradingSystemStatisticStr = _T("MaxDrawAmount");
      break;
   case tssMaxDrawDuration:
      tradingSystemStatisticStr = _T("MaxDrawDuration");
      break;
   case tssMaxConsecWins:
      tradingSystemStatisticStr = _T("MaxConsecWins");
      break;
   case tssCurConsecWins:
      tradingSystemStatisticStr = _T("CurConsecWins");
      break;
   case tssMaxConsecLosses:
      tradingSystemStatisticStr = _T("MaxConsecLosses");
      break;
   case tssCurConsecLosses:
      tradingSystemStatisticStr = _T("CurConsecLosses");
      break;
   case tssProfitToMaxDraw:
      tradingSystemStatisticStr = _T("ProfitToMaxDraw");
      break;
   case tssProfitLossRatio:
      tradingSystemStatisticStr = _T("ProfitLossRatio");
      break;
   case tssPercentWinners:
      tradingSystemStatisticStr = _T("PercentWinners");
      break;
   case tssRemoveToNeutral:
      tradingSystemStatisticStr = _T("RemoveToNeutral");
      break;
   case tssTimePercentage:
      tradingSystemStatisticStr = _T("TimePercentage");
      break;
   case tssCurDrawDown:
      tradingSystemStatisticStr = _T("CurDrawDown");
      break;
   case tssReturnRetracement:
      tradingSystemStatisticStr = _T("ReturnRetracement");
      break;
   case tssLinearRegression:
      tradingSystemStatisticStr = _T("LinearRegression");
      break;
   default:
      assert(!"Unknown trading system statistic");
   }

   return tradingSystemStatisticStr;
}

CString CHelper::TradingSystemCommissionTypeToString(
                        eTradingSystemCommissionType tradingSystemCommissionType)
{
   CString tradingSystemCommissionTypeStr;

   switch (tradingSystemCommissionType)
   {
   case tscmFixed:
      tradingSystemCommissionTypeStr = _T("Fixed");
      break;
   case tscmPerContract:
      tradingSystemCommissionTypeStr = _T("Per Contract");
      break;
   default:
      assert(!"Unknown trading system commission type");
   }

   return tradingSystemCommissionTypeStr;
}

CString CHelper::SubscriptionLevelToString(
                        eTimeSeriesSubscriptionLevel subscriptionLevel)
{
   CString subscriptionLevelStr;

   switch (subscriptionLevel)
   {
   case tslEachBar:
      subscriptionLevelStr = _T("EachBar");
      break;
   case tslEachTick:
      subscriptionLevelStr = _T("EachTick");
      break;
   case tslEndOfBarAndPeriod:
      subscriptionLevelStr = _T("EndOfBarAndPeriod");
      break;
   default:
      assert(!"Unknown subscription level");
   }

   return subscriptionLevelStr;
}