/*
* =====================================================================
*  Filename: Helper.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

class CHelper
{

private:

   /// \brief CHelper constructor.
   CHelper();

public:

   /// \brief Converts enum of connection status to appropriate string.
   /// \param connectionStatus - enum which will be converted
   /// \return String which describes connection status.
   static CString ConnectionStatusToString(
						eConnectionStatus connectionStatus);

   /// \brief Converts enum of parameter type to appropriate string.
   /// \param parameterType - enum which will be converted
   /// \return String which describes parameter type.
   static CString ParameterTypeToString(
                  eUserFormulaParameterType parameterType);

   /// \brief Converts enum of request status to appropriate string.
   /// \param requestStatus - enum which will be converted
   /// \return String which describes request status.
   static CString RequestStatusToString(
						eRequestStatus requestStatus);

   /// \brief Converts enum of historical period to appropriate string.
   /// \param historicalPeriod - enum which will be converted
   /// \return String which describes historical period.
   static CString HistoricalPeriodToString(
						eHistoricalPeriod historicalPeriod);

   /// \brief Converts enum of continuation type to appropriate string.
   /// \param continuationType - enum which will be converted
   /// \return String which describes continuation type.
   static CString ContinuationTypeToString(
						eTimeSeriesContinuationType continuationType);

   /// \brief Converts bitmap of session flag to appropriate string.
   /// \param sessionFlag - bitmap which will be converted
   /// \return String which describes session flag.
   static CString SessionFlagToString(
						eSessionFlag sessionFlag);
						
   /// \brief Converts enum of subscription level to appropriate string.
   /// \param subscriptionLevel - enum which will be converted
   /// \return String which describes subscription level.
   static CString SubscriptionLevelToString(
						eTimeSeriesSubscriptionLevel continuationType);

   /// \brief Converts enum of trading system commission type to appropriate string.
   /// \param tradingSystemCommissionType - enum which will be converted
   /// \return String which describes trading system commission type.
   static CString TradingSystemCommissionTypeToString(
                  eTradingSystemCommissionType tradingSystemCommissionType);
   
   /// \brief Converts enum of trading system statistic to appropriate string.
   /// \param tradingSystemStatistic - enum which will be converted
   /// \return String which describes trading system statistic.
   static CString TradingSystemStatisticToString(
                  eTradingSystemStatistic tradingSystemStatistic);
   
   /// \brief Converts enum of order type to appropriate string.
   /// \param orderType - enum which will be converted
   /// \return String which describes order type.
   static CString OrderTypeToString(
                  eOrderType orderType);
   
   /// \brief Converts enum of trade side to appropriate string.
   /// \param tradeSide - enum which will be converted
   /// \return String which describes trade side.
   static CString TradeSideToString(
                  eTradeSide tradeSide);

   /// \brief Compares two dates : COleDateTime comparison is not enough,
   /// \brief because we need to ignore seconds.
   /// \param date1 - first date
   /// \param date2 - second date
   /// \return Integer which means :
   /// \return -1 : first date is less than second,
   /// \return  0 : dates are equal,
   /// \return  1 : first date is greater than second.
   static int CompareDates(
                  const COleDateTime& date1,
                  const COleDateTime& date2);


public:

   /// \brief Selects an item of the specified combo box, which item data matches with given.
   /// \param comboBoxCtrl - combo box, which item will be selected
   /// \param itemData - item data, which must match with selected item data
   static void SetComboBoxItem(
						CComboBox& comboBoxCtrl,
						DWORD_PTR itemData);
public:

   /// \brief Custom date format.
   static const CString DATE_FORMAT;

   /// \brief Not available string.
   static const CString N_A;

   /// \brief Parameter fractional part length.
   static const int PARAM_FRAC_PART_LENGTH;
};