/*
* =====================================================================
*  Filename: TradingSystemDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

#include "afxwin.h"
#include "afxcmn.h"
#include "afxdtctl.h"
#include <map>
#include "NumEdit.h"


class CTradingSystemDlg;

/**
\typedef ATL::IDispEventImpl<1, CTradingSystemDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CTradingSystemDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

// CTradingSystemDlg dialog
class CTradingSystemDlg : public CDialog, public ICQGCELDispEventImpl
{

public:

   /// \brief CTradingSystemDlg constructor.
   /// \param pParent - parent window
   explicit CTradingSystemDlg(
                  CWnd* pParent = NULL); 

   /// \enum Dialog Data
   enum { IDD = IDD_TRADING_SYSTEM_DIALOG };

   /// \brief Dialog icon.
   HICON m_hIcon;

private:

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
                  CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter
   /// \brief and catch Enters in tag name and tag value edit controls.
   virtual void OnOK();

private:

   /// \brief Prepares the container for painting.
   afx_msg void OnPaint();

   /// \brief The system calls this function to obtain the cursor to
   /// \brief display while the user drags the minimized window.
   afx_msg HCURSOR OnQueryDragIcon();

   /// \brief Used to unadvise from CQGCEL events and shutdown it.
   afx_msg void OnDestroy();

   /// \brief Changes selected trading system object, and shows its data.
   afx_msg void OnCbnSelChangeCmbTradingSystem();

   /// \brief Requests trading system definitions.
   afx_msg void OnBnClickedBtnRequestDefinition();

   /// \brief Updates definition data and parameters on the form.
   afx_msg void OnCbnSelchangeCmbDefinition();

   /// \brief Shows selected trades entry and exits definitions.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnLvnItemchangedLsvDefTrades(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Opens request settings form to fill up request parameters.
   afx_msg void OnBnClickedBtnRequest();

   /// \brief Removes selected trading system from the CQGCEL and the combo box control.
   afx_msg void OnBnClickedBtnRemove();

   /// \brief Removes all trading system from the CQGCEL and the combo box control.
   afx_msg void OnBnClickedBtnRemoveAll();

   /// \brief Used to increment or decrement start range value for filtering.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnStart(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Used to increment or decrement end range value for filtering.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnEnd(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Shows filtered trading system.
   afx_msg void OnBnClickedBtnShow();

   /// \brief Shows error message.
   afx_msg void OnStnDblclickLblError();

   /// \brief Shows statistics or trades list views according to selected tab.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnTcnSelchangeTabTradeStatistic(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Shows corresponding trades in the list views.
   afx_msg void OnCbnSelchangeCmbTrades();

   /// \brief Changes to show the selected entry/exit relations.
   afx_msg void OnCbnSelchangeCmbLastTradeItems();

   /// \brief Changes to show the selected entry/exit relations.
   afx_msg void OnCbnSelchangeCmbTradesItems();

   /// \brief Shows selected row trades relations.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnLvnItemchangedLsvLastTrades(
                  NMHDR *pNMHDR, 
                  LRESULT *pResult);

   /// \brief Shows selected row trades relations.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnLvnItemchangedLsvTrades(
                  NMHDR *pNMHDR, 
                  LRESULT *pResult);

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CTradingSystemDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 10, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 3, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 40, OnTradingSystemDefinitionsResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 49, OnTradingSystemResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 50, OnTradingSystemAddNotification)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 51, OnTradingSystemUpdateNotification)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 79, OnTradingSystemInsertNotification)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 80, OnTradingSystemRemoveNotification)
   END_SINK_MAP()

private:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - string describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                  LPDISPATCH obj,
                  BSTR errorDescription);

   /// \brief Fired when some changes occurred in the connection with the CQG data server.
   /// \param newStatus - new connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when the definitions of tradingSystems are resolved
   /// \brief or when some error occurred during request processing.
   /// \param cqg_trading_system_definitions - collection of trading system definitions
   /// \param cqg_error - CQGError object describing last error occurred during the request processing
   /// \return S_OK
   STDMETHOD(OnTradingSystemDefinitionsResolved)(
                  ICQGTradingSystemDefinitions* cqg_trading_system_definitions, 
                  ICQGError* cqg_error);

   /// \brief Fired when the trading system (CQGTradingSystem) is resolved
   /// \brief or when some error occurred during trading system request processing.
   /// \param cqg_trading_system - reference to resolved CQGTradingSystem
   /// \param cqg_error - CQGError object describing last error occurred during processing the request.
   /// \return S_OK
   STDMETHOD(OnTradingSystemResolved)(
                  ICQGTradingSystem* cqg_trading_system,
                  ICQGError* cqg_error);

   /// \brief Fired when trade row and/or statistics is added to trading system.
   /// \param cqg_trading_system - reference to changed CQGTradingSystem
   /// \param cqg_trading_system_add_info - object containing information about the additions
   /// \return S_OK
   STDMETHOD(OnTradingSystemAddNotification)(
                  ICQGTradingSystem* cqg_trading_system,
                  ICQGTradingSystemAddInfo* cqg_trading_system_add_info);

   /// \brief Fired when trade row and/or statistics is updated in a trading system.
   /// \param cqg_trading_system - reference to changed CQGTradingSystem
   /// \param cqg_trading_system_update_info - object containing information about the updates
   /// \return S_OK
   STDMETHOD(OnTradingSystemUpdateNotification)(
                  ICQGTradingSystem* cqg_trading_system,
                  ICQGTradingSystemUpdateInfo* cqg_trading_system_update_info);
                  
   /// \brief Fired when trade row and/or statistics is inserted into a trading system.
   /// \param cqg_trading_system - reference to changed CQGTradingSystem
   /// \param cqg_trading_system_insert_info - object containing information about the insertions
   /// \return S_OK
   STDMETHOD(OnTradingSystemInsertNotification)(
                  ICQGTradingSystem* cqg_trading_system,
                  ICQGTradingSystemInsertInfo* cqg_trading_system_insert_info);
      
   /// \brief Fired when trade row and/or statistics is removed from a trading system.
   /// \param cqg_trading_system - reference to changed CQGTradingSystem
   /// \param cqg_trading_system_remove_info - object containing information about the deletions
   /// \return S_OK
   STDMETHOD(OnTradingSystemRemoveNotification)(
                  ICQGTradingSystem* cqg_trading_system,
                  ICQGTradingSystemRemoveInfo* cqg_trading_system_remove_info);

private:

   /// \enum eTradeRecordColumns. Represents trades columns in the list.
   enum eTradeRecordColumns;

   /// \enum eStatisticRecordColumns. Represents statistic columns in the list.
   enum eStatisticRecordColumns;

   /// \enum eExitColums. Represents exits columns in the list.
   enum eExitColums;

   /// \enum eDefintionParametersColumns. Represents definition parameters columns in the list.
   enum eDefintionParametersColumns;

   /// \enum eDefintionTradesColumns. Represents definition trades columns in the list.
   enum eDefintionTradesColumns;

   /// \enum eDefintionTradesEntryExitsColumns. Represents definition trades' entry, exits columns in the list.
   enum eDefintionTradesEntryExitsColumns;

   /// \enum eParametersColumns. Represents parameters columns in the list.
   enum eParametersColumns;

   /// \enum eTabsOutputs. Represents tab order.
   enum eTabsOutputs;

   /// \enum eRelationCloumns. Represents columns in trades relation's list.
   enum eRelationColumns;

private:

   /// \brief Initializes CQGCEL object, and starts the CQGCEL.
   void InitializeCQGCEL();

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes definition parameters list control's columns and styles.
   void InitializeDefParametersList();

   /// \brief Initializes definition trades list control's columns and styles.
   void InitializeDefTradesList();

   /// \brief Initializes definition trades' entry and exits list control's columns and styles.
   void InitializeDefEntryExitsList();

   /// \brief Initializes parameters list control's columns and styles.
   void InitializeParametersList();

   /// \brief Initializes specified list control's columns and styles.
   /// \param listCtrl - list control which columns will be configured
   static void InitializeTradesList(
                  CListCtrl& listCtrl);

   /// \brief Initializes specified list control's columns and styles.
   /// \param listCtrl - list control which columns will be configured
   static void InitializeStatisticsList(
                  CListCtrl& listCtrl);

   /// \brief Initializes specified list control's columns and styles.
   /// \param listCtrl - list control which columns will be configured
   static void InitializeRelationsList(
                  CListCtrl& listCtrl);

   /// \brief Initializes trades and statistics tab.
   void InitializeTab();

   /// \brief Checks if specified trading system is selected.
   /// \param spTradingSystem - trading system which will be checked
   /// \return Returns true if specified trading system is selected.
   bool IsTradingSystemSelected(
                  const ATL::CComPtr<ICQGTradingSystem>& spTradingSystem);

   
   /// \brief Clears trade exits' headers from the specified list control.
   /// \param listCtrl - list control which headers will be cleared
   void ClearTradeHeaders(
                  CListCtrl& listCtrl);

   /// \brief Clears statistics' headers from the specified list control.
   /// \param listCtrl - list control which headers will be cleared
   void ClearStatisticsHeaders(
                  CListCtrl& listCtrl);

   /// \brief Clears all records from the specified list.
   /// \param listCtrl - list control which will be cleared
   static void ClearRecords(
                  CListCtrl& listCtrl);

   /// \brief Clears all data connected to trading system definitions.
   void ClearAllDefinitionsData();

   /// \brief Clears all data connected to trading system definition.
   void ClearDefinitionData();

   /// \brief Adds all definitions to the combo
   void ShowDefinitions();

   /// \brief Shows specified definition's all trades in the list.
   /// \param spTradingSystemDefinition - definition which trades will be shown
   void ShowDefinitionTrades(
                  const ATL::CComPtr<ICQGTradingSystemDefinition>& spTradingSystemDefinition);
   
   /// \brief Shows specified trades entry and exits definitions in the list.
   /// \param spTradeDefinition - trade definition which entry and exits will be shown
   void ShowDefinitionTradeEntryExits(
                  const ATL::CComPtr<ICQGTradingSystemTradeDefinition>& spTradeDefinition);

   /// \brief Shows definition's all parameters in the list.
   /// \param spTradingSystemDefinition - definition which parameters will be shown
   void ShowDefinitionParameters(
                  const ATL::CComPtr<ICQGTradingSystemDefinition>& spTradingSystemDefinition);

   /// \brief Clears all data(parameters\outputs\records) from the form.
   void ClearAllData();

   /// \brief Clears request parameters from the form.
   void ClearRequestParams();

   /// \brief Clears request tags.
   void ClearTags();

   /// \brief Shows all data(parameters\outputs\records) connected
   /// \brief with selected request.
   void ShowAllData();

   /// \brief Shows selected request parameters on the form.
   void ShowRequestParams();

   /// \brief Shows specified request's definition parameters on the form.
   /// \param spTradingSystemRequest - request which definition parameters will be shown
   void ShowReqDefinitionParameters(
                  const ATL::CComPtr<ICQGTradingSystemRequest>& spTradingSystemRequest);

   /// \brief Shows trade exits headers in the specified list control.
   /// \param listCtrl - list control in which headers will be shown
   void ShowTradeHeaders(
                  CListCtrl& listCtrl);

   /// \brief Shows statistics headers in the specified list control.
   /// \param listCtrl - list control in which headers will be shown
   void ShowStatisticsHeaders(
                  CListCtrl& listCtrl);

   /// \brief Adds all trading system trades to the combo box.
   void AddTradesToCombo();

   /// \brief Adds all included statistics to the map.
   void AddStatisticsToMap();

   /// \brief Adds specified statistic to the map if it is included to the request.
   /// \param tradingSystemStatistic - statistic which will be checked and added
   /// \param spTradingSystemRequest - CQGTradingSystemRequest object
   /// \param colNumber [in, out] - current column number
   void AddStatisticToMap(
                  eTradingSystemStatistic tradingSystemStatistic,
                  const ATL::CComPtr<ICQGTradingSystemRequest>& spTradingSystemRequest,
                  int& colNumber);

   /// \brief Shows last statistics in the list control.
   void ShowLastStatistics();

   /// \brief Shows last trades in the list control.
   void ShowLastTrades();

   /// \brief Shows given range records in the specified list control.
   /// \param startIndex - range start index
   /// \param endIndex - range end index
   /// \param listCtrl - list control in which records will be shown
   void ShowTradeRecords(
                  long startIndex,
                  long endIndex,
                  CListCtrl& listCtrl);

   /// \brief Adds record with given index into the end of specified list control.
   /// \param recordIndex - record index which will be added
   /// \param listCtrl - list control in which record will be added
   void AddTradeRecord(
                  long recordIndex,
                  CListCtrl& listCtrl);

   /// \brief Shows record with given index into the specified row of the specified list control.
   /// \param recordIndex - record index which will be shown
   /// \param listCtrl - list control in which record will be shown
   /// \param row - row of the list control in which record will be shown
   void ShowTradeRecord(
                  long recordIndex,
                  CListCtrl& listCtrl,
                  long row);

   /// \brief Shows specified entry's information into the specified row of the specified list control.
   /// \param spTradeEntry - CQGTradeEntry object which information will be shown
   /// \param listCtrl - list control in which information will be shown
   /// \param row - row of the list control in which information will be shown
   void ShowTradeRecordEntry(
                  const ATL::CComPtr<ICQGTradeEntry>& spTradeEntry,
                  CListCtrl& listCtrl,
                  long row);

   /// \brief Shows specified exit's information into the specified row of the specified list control.
   /// \param spTradeExit - CQGTradeExit object which information will be shown
   /// \param listCtrl - list control in which information will be shown
   /// \param row - row of the list control in which information will be shown
   void ShowTradeRecordExit(
                  const ATL::CComPtr<ICQGTradeExit>& spTradeExit,
                  CListCtrl& listCtrl,
                  long row);

   /// \brief Shows given range statistics records in the specified list control.
   /// \param startIndex - range start index
   /// \param endIndex - range end index
   /// \param listCtrl - list control in which records will be shown
   void ShowStatisticRecords(
                  long startIndex,
                  long endIndex,
                  CListCtrl& listCtrl);

   /// \brief Adds statistic record with given index into the end of specified list control.
   /// \param recordIndex - record index which will be added
   /// \param listCtrl - list control in which record will be added
   void AddStatisticRecord(
                  long recordIndex,
                  CListCtrl& listCtrl);

   /// \brief Shows statistic record with given index into the specified row of the specified list control.
   /// \param recordIndex - record index which will be shown
   /// \param listCtrl - list control in which record will be shown
   /// \param row - row of the list control in which record will be shown
   void ShowStatisticRecord(
                  long recordIndex,
                  CListCtrl& listCtrl,
                  long row);

   /// \brief Adds trades names to ComboBox control.
   /// \param cmbItemNames - ComboBox to add the entry/exits names
   void AddTradeItemsNames(
                  CComboBox& cmbItemNames);

   /// \brief Shows relations of the selected trade entry/exit.
   /// \param lvTrades - list control which row is selected
   /// \param lvRelations - list control where relations should be shown
   /// \param cmbItemNames - CComboBox control which item is selected
   void ShowTradeItemRelations(
                  CListCtrl& lvTrades, 
                  CListCtrl& lvRelations, 
                  CComboBox& cmbItemNames);

   /// \brief Shows trade relations in the specified list control.
   /// \param listCtrl - list control in which relation will be shown
   /// \param spTrade - trading system trade 
   /// \param name - relation name
   void ShowRelations(
                  CListCtrl& listCtrl,
                  const ATL::CComPtr<ICQGTradingSystemTrade>& spTrade,
                  const ATL::CComBSTR& name);

   /// \brief Gets start and end indices and validates them to show records.
   /// \param startIndex [out] - range start index
   /// \param endIndex [out] - range end index
   /// \return Returns true if range is valid.
   bool GetIndices(
                  long& startIndex,
                  long& endIndex);

   /// \brief Clears indices, which are used for filtering.
   void ClearIndices();

   /// \brief Fills indices for filtering.
   void FillIndices();

   /// \brief Used for getting CQGCEL properties values string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - variant value
   /// \param valueType - variant type
   /// \param withMilliseconds - add milliseconds to the string, only for VT_DATE
   /// \return String representation of the value.
   CString GetValueAsString(
      ATL::CComVariant value,
      VARENUM valueType = VT_EMPTY,
      bool withMilliseconds = false) const;

   /// \brief Shows statistics or trades list views according to selected tab.
   void ShowListViews();

   /// \brief Enables/disables controls on the form based on connection status and other parameters.
   void EnableControls();

   /// \brief Shows tag value for specified name.
   void OnEnterEdTagName();

   /// \brief Sets tag value for specified name.
   void OnEnterEdTagValue();

private:

   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Selected trading system object.
   ATL::CComPtr<ICQGTradingSystem> m_spTradingSystem;

   /// \brief Resolved trading system definitions object.
   ATL::CComPtr<ICQGTradingSystemDefinitions> m_spTradingSystemDefinitions;

   /// \brief Map of trading system statistics and columns in the list box.
   typedef std::map<eTradingSystemStatistic, int> StatisticMap;
   StatisticMap m_StatisticsMap;

   /// \brief Map of trading system exits' names and columns in the list box.
   typedef std::map<ATL::CComBSTR, int> TradeExitMap;
   TradeExitMap m_TradeExitsMap;

private:

   /// \brief Trading system count that will be shown in the last chunk list.
   static const long SHOW_COUNT;

   /// \brief Max indices interval for filtering.
   static const long MAX_INTERVAL;

   /// \brief Specifies the trade name and exit name splitter
   static const CString NAME_SPLITTER;

private:

   /// \brief Data connection indicator Label control.
   CStatic m_lblDataConnection;

   /// \brief Request trading system definitions Button control.
   CButton m_btnRequestDefinition;

   /// \brief Trading system definitions ComboBox control.
   CComboBox m_cmbDefinition;

   /// \brief Definition parameters ListView control.
   CListCtrl m_lvDefParameters;

   /// \brief Definition trades ListView control.
   CListCtrl m_lvDefTrades;

   /// \brief Definition trades' entry and exits ListView control.
   CListCtrl m_lvDefEntryExits;

   /// \brief Definition abbreviation Label control.
   CStatic m_lblDefAbbreviation;

   /// \brief Definition commission Label control.
   CStatic m_lblDefCommission;

   /// \brief Definition commission type Label control.
   CStatic m_lblDefCommissionType;

   /// \brief Active trading system ComboBox control.
   CComboBox m_cmbTradingSystem;

   /// \brief Definition name EditBox control.
   CEdit m_edDefinitionName;

   /// \brief Request trading system Button control.
   CButton m_btnRequest;

   /// \brief Remove selected trading system Button control.
   CButton m_btnRemove;

   /// \brief Remove all trading system Button control.
   CButton m_btnRemoveAll;

   /// \brief Last error Label control.
   CStatic m_lblError;

   /// \brief Request status Label control.
   CStatic m_lblStatus;

   /// \brief Show filtered trading system Button control.
   CButton m_btnShow;

   /// \brief Tag name EditBox control.
   CEdit m_edTagName;

   /// \brief Tag value EditBox control.
   CEdit m_edTagValue;

   /// \brief All statistics records count Label control.
   CStatic m_lblStatsAllRecsCount;

   /// \brief Filtered statistics records count Label control.
   CStatic m_lblStatsFltrRecsCount;

   /// \brief All trades records count Label control.
   CStatic m_lblTradesAllRecsCount;

   /// \brief Filtered trades records count Label control.
   CStatic m_lblTradesFltrRecsCount;

   /// \brief Trades or statistic Tab control.
   CTabCtrl m_tabTradeStatistic;

   /// \brief Trades ComboBox control.
   CComboBox m_cmbTrades;

   /// \brief Last chunk of trades ListView control.
   CListCtrl m_lvLastTrades;

   /// \brief Last chunk of statistics ListView control.
   CListCtrl m_lvLastStatistics;
   
   /// \brief Filtering by indices range start EditBox control.
   CNumEdit m_edStart;

   /// \brief Filtering by indices range start SpinButton control.
   CSpinButtonCtrl m_spnStart;

   /// \brief Filtering by indices range end EditBox control.
   CNumEdit m_edEnd;

   /// \brief Filtering by indices range end SpinButton control.
   CSpinButtonCtrl m_spnEnd;

   /// \brief Filtered trades ListView control.
   CListCtrl m_lvTrades;

   /// \brief Filtered statistics ListView control.
   CListCtrl m_lvStatistics;

   /// \brief Request's parameters  ListView control.
   CListCtrl m_lvParameters;

   /// \brief Relations ListView control.
   CListCtrl m_lvLastTradeItems;

   /// \brief Filtered relation parameters ListView control.
   CListCtrl m_lvTradeItems;

   /// \brief  Entry/exits  ComboBox control.
   CComboBox m_cmbLastTradeItems;

   /// \brief Filtered entry/exits ComboBox control.
   CComboBox m_cmbTradeItems;

   /// \brief Relations Label control.
   CStatic m_lblLastRelations;

   /// \brief Filtered relations Label control.
   CStatic m_lblRelations;

   /// \brief String which describes request definition name.
   CString m_DefinitionName;
   
   /// \brief String which describes request base expression.
   CString m_BaseExpression;

   /// \brief String which describes request range start.
   CString m_RangeStart;

   /// \brief String which describes request range end.
   CString m_RangeEnd;

   /// \brief String which describes request intra day period.
   CString m_IntradayPeriod;

   /// \brief String which describes request historical period.
   CString m_HistoricalPeriod;

   /// \brief String which describes request include end.
   CString m_IncludeEnd;

   /// \brief String which describes request sessions filter.
   CString m_SessionsFilter;

   /// \brief String which describes request continuation.
   CString m_Continuation;

   /// \brief String which describes request equalize closes.
   CString m_EqualizeCloses;

   /// \brief String which describes request days before expiration.
   CString m_DaysBeforeExpiration;

   /// \brief String which describes request session flags.
   CString m_SessionFlags;
   
   /// \brief String which describes subscription level.
   CString m_SubscriptionLevel;
   
   /// \brief String which describes recalc period.
   CString m_RecalcPeriod;
};

enum CTradingSystemDlg::eTradeRecordColumns
{
   trcIndex,
   trcTimestamp,
   trcEntrySignal,
   trcIsActive,
   trcEntryPrice,
   trcEntryStopLimitPrice,
   trcEntryQuantity,
   trcExitsStart
};

enum CTradingSystemDlg::eStatisticRecordColumns
{
   srcIndex,
   srcTimestamp,
   srcStatsStart
};

enum CTradingSystemDlg::eExitColums
{
   ecSignal,
   ecPrice,
   ecQuantity,
   ecStopLimitPrice,
   ecCount
};

enum CTradingSystemDlg::eDefintionParametersColumns
{
   dpcName,
   dpcComment,
   dpcType,
   dpcDefaultValue
};

enum CTradingSystemDlg::eDefintionTradesColumns
{
   dtcName,
   dtcSide
};

enum CTradingSystemDlg::eDefintionTradesEntryExitsColumns
{
   dteecName,
   dteecValue
};

enum CTradingSystemDlg::eParametersColumns
{
   pcName,
   pcValue
};

enum CTradingSystemDlg::eTabsOutputs
{
   toStatistics,
   toTrades
};

enum CTradingSystemDlg::eRelationColumns
{
   rcTimestamp,
   rcName
};